/*
 *  Copyright 2002 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *  
 */
/* "@(#) DSP/BIOS 4.61.12 05-31-02 (barracuda-j21)" */
/*
 *  ======== dax.c ========
 *  "generic" D/A A/D driver.
 *
 *
 *  This driver will work for most A/D D/A devices (e.g. LSI AMELIA daughter
 *  modules) or serial port devices.  It requires a "controller" module
 *  which consists of bind(), start(), stop(), and unbind() functions which
 *  are called by DAX to bind a controller and to start and stop the flow of
 *  data to/from that controller and to unbind the controller.
 *
 *  A typical controller will enable and disable the particular A/D or D/A
 *  interrupt in its start() and stop() function respectively.  Each
 *  controller will have (at least one) hardware ISR which will call
 *  DAX_nextFrame() at frame boundaries.
 *
 *  The controller uses the external DAX_Obj structure and DAX_nextFrame()
 *  to interface efficiently with DAX.
 *
 *  This driver uses curJob and nextJob to keep track of the current and
 *  next I/O job.  This makes the ISR simpler and more efficient at the
 *  expense of a little more complexity at the task level.
 */

#include <std.h>

#include <dev.h>
#include <sem.h>
#include <sys.h>
#include <mem.h>
#include <que.h>
#include <hwi.h>

#include <dax.h>

static Int      DAX_close(DEV_Handle device);
static Int      DAX_ctrl(DEV_Handle device, Uns cmd, Arg arg);
static Int      DAX_idle(DEV_Handle device, Bool flush);
static Int      DAX_issue(DEV_Handle device);
static Int      DAX_open(DEV_Handle device, String name);
static Bool     DAX_ready(DEV_Handle device, SEM_Handle sem);
static Int      DAX_reclaim(DEV_Handle device);

/*
 *  Driver function table.
 */
DEV_Fxns DAX_FXNS = {
    DAX_close,          /* close */
    DAX_ctrl,           /* ctrl */
    DAX_idle,           /* idle */
    DAX_issue,          /* issue */
    DAX_open,           /* open */
    DAX_ready,          /* ready */
    DAX_reclaim         /* reclaim */
};

static Void load(DAX_Handle port, QUE_Handle queue);
static DAX_Handle mkPort(DEV_Handle device, String name);
static Void rmPort(DAX_Handle port, DAX_Params *params);

/*
 *  ======== load ========
 *  The curJob and nextJob structures are filled using frames from the
 *  given queue (device->todevice).  These structures are used by the
 *  hardware ISR as a source of data to transmit (when the device is in
 *  DEV_OUTPUT mode) or a buffer to fill with data (when the device is in
 *  DEV_INPUT mode).
 *
 *  Uses HWI_disable()/HWI_restore() to update curJob and nextJob since
 *  these fields are also modified by the HW ISRs.
 *
 *  NOTE: This function increases the system's interrupt latency since
 *        it disables all interrupts!
 *
 *        For systems that have another interrupt with a tighter
 *        interrupt response requirement, you may
 *        replace:
 *
 *              oldMask = HWI_disable();
 *
 *              ...
 *
 *              HWI_restore(oldMask);
 *
 *        with:
 *
 *              key = TSK_disable(); 
 *              oldMask = XXX_disable(1 << INTRID);
 *
 *              ...
 *
 *              XXX_enable(oldMask);
 *              TSK_enable(key);
 *
 *        This will allow *OTHER* interrupt routines to run but adds
 *        the overhead of the kernel entry/exit in TSK_enable().
 */
static Void load(DAX_Handle port, QUE_Handle queue)
{
    DEV_Frame   *frame;
    Uns         imask;

    /*
     * Disable all interrupts. This also disables MP Floating Point
     * operations for the MVP, but we don't need to do any during this time.
     */
    imask = HWI_disable();              /* DISABLE */

    while (port->nextFrame == NULL && !QUE_empty(queue)) {
        /*
         * 'queue' is device->todevice which contains empty frames
         * for input devices and full frames for output devices.
         */
        /*
         * QUE_dequeue() is the same as QUE_get() except that
         * it does not disable interrupts.  It is not a docemented
         * part of the interface.  Nevertheless, it is used here for
         * efficiency since interrupts are already disabled!
         */
        frame = (DEV_Frame *)QUE_dequeue(queue);

        /*
         *  update the Job structures
         */
        if (port->curFrame == NULL) {
            port->curJob.addr = frame->addr;
            port->curJob.size = frame->size;

            port->curFrame = frame;
        }
        else { /* port->nextFrame == NULL */
            port->nextJob.addr = frame->addr;
            port->nextJob.size = frame->size;

            port->nextFrame = frame;
        }

        /*
         *  Enable/disable interrupts - This is done so that
         *  if the loop is executed twice we don't leave interrupts
         *  disabled for too long.
         */
        HWI_restore(imask);             /* ENABLE */
        imask = HWI_disable();          /* DISABLE */
    }

    HWI_restore(imask);                 /* ENABLE */
}

/*
 *  ======== mkPort ========
 *  Make DAX port object.  This object (and the DAX_nextFrame() function)
 *  are used to interface DAX with its partner controller.
 */
static DAX_Handle mkPort(DEV_Handle device, String name)
{
    DAX_Params  *params = (DAX_Params *)device->params;
    DAX_Handle  port;

    /* allocate port object */
    if ((port = MEM_alloc(0, sizeof(DAX_Obj), 0)) == MEM_ILLEGAL) {
        return (NULL);
    }

    port->errCnt = 0;           /* real-time error count */
    
    /*
     * Create semaphore. The initial value of the semaphore is equal
     * to the number of frames on the fromdevice queue.
     *
     * Tasks will pend on port->sem if there are no available frames on
     * the fromdevice queue.
     */
    port->sem = SEM_create(0, NULL);

    if (device->mode == DEV_INPUT) {
        /* don't delay input */
        port->delay = 0;
    }
    else {
        /*
         * don't start output until outputdelay+1 buffers
         * have been queued for output
         */
        port->delay = params->outputdelay;
    }

    /* make sure SEM_create() succeeded ... */
    if (port->sem == NULL) {
        MEM_free(0, port, sizeof(DAX_Obj));     /* free port object */
        return (NULL);
    }

    port->ready = NULL;         /* used by DAX_ready() */

    port->curJob.size = port->nextJob.size = 0;

    port->curFrame = port->nextFrame = NULL;

    /*
     * Hang on to device object.  It is referenced in DAX_nextFrame()
     * and may also be referenced by the controller.
     */
    port->device = device;

    /* bind the controller */
    if ((*params->bind)(port, name) != SYS_OK) {
        rmPort(port, params);

        return (NULL);
    }

    return (port);
}

/*
 *  ======== rmPort ========
 *  Remove port object.
 */
static Void rmPort(DAX_Handle port, DAX_Params *params)
{
    if(params->unbind != NULL) {
        (*(params->unbind))(port);
    }

    /* the semaphore ... */
    SEM_delete(port->sem);

    /* and finally the object */
    MEM_free(0, port, sizeof(DAX_Obj));
}

/*
 *  ======== DAX_close ========
 *  DAX_idle() should be called before DAX_close().
 */
static Int DAX_close(DEV_Handle device)
{
    DAX_Params  *params = (DAX_Params *)device->params;
    DAX_Handle  port = (DAX_Handle)device->object;

    /*
     * The controller's stop function should have already
     * been called by DAX_idle().
     */

    rmPort(port, params);

    return (SYS_OK);
}

/*
 *  ======== DAX_ctrl ========
 *  Simply call down to the controller.
 *  DAX_ctrl() used to return the error count (no matter what the cmd is!)
 */
static Int DAX_ctrl(DEV_Handle device, Uns cmd, Arg arg)
{
    DAX_Params  *params = (DAX_Params *)device->params;

    (*params->ctrl)(device, cmd, arg);

    return (SYS_OK);
}

/*
 *  ======== DAX_issue ========
 */
static Int DAX_issue(DEV_Handle device)
{
    DAX_Handle  port = (DAX_Handle)device->object;
    DAX_Params  *params = (DAX_Params *)device->params;

    /*
     * don't load curJob and nextJob or start the device until
     * port->delay+1 buffers have been queued for I/O (i.e. this
     * function has been called delay+1 times with non-zero-length
     * buffers).
     *
     * For output devices, the stream may be primed with 0-length
     * buffers which just get transferred to the fromdevice.  These
     * shouldn't be counted in the delay buffers.
     *
     * NOTE: for input devices, port->delay is initialized to 0 so
     * the else block is always executed.
     */
    if (port->delay > 0) {
        if (((DEV_Frame *)QUE_head(device->todevice))->size > 0) {
            port->delay--;
        }
        else {
            QUE_put(device->fromdevice, QUE_get(device->todevice));
            SEM_post(port->sem);
        }
    }
    else {
        load(port, device->todevice);

        if (port->delay == 0) {         /* only call start() once */
            (*params->start)(port);
            port->delay = -1;           /* start() won't be called next time */
        }
    }

    return SYS_OK;
}

/*
 *  ======== DAX_reclaim ========
 */
static Int DAX_reclaim(DEV_Handle device)
{
    DAX_Handle  port = (DAX_Handle)device->object;

    /*
     * Wait here if there are no buffers on the device->fromdevice
     * queue.
     */
    if (SEM_pend(port->sem, device->timeout)) {
        return SYS_OK;
    }
    else {
        return SYS_ETIMEOUT;
    }
}

/*
 *  ======== DAX_idle ========
 *  DAX_idle() puts the device back to the state it was in just after
 *  DAX_open() was called.
 *
 *  The controller's stop function is called at different times depending
 *  on the device's mode.   For input devices, the stop function is called
 *  immediately and frames are recycled to the device->todevice  queue.
 *  For output devices, DAX_idle() waits until all the output has finished
 *  before calling the stop function.
 */
static Int DAX_idle(DEV_Handle device, Bool flush)
{
    DAX_Handle  port = (DAX_Handle)device->object;
    DAX_Params  *params = (DAX_Params *)device->params;
    Uns         pendCount = 0;

    if (device->mode == DEV_INPUT) {
        /*
         * stop input immediately
         */
        (*params->stop)(port);

        /* put "in process" frames back on todevice queue */
        if (port->curFrame) {
            QUE_put(device->todevice, port->curFrame);
        }
        if (port->nextFrame) {
            QUE_put(device->todevice, port->nextFrame);
        }

        /*
         * Discard input by putting all frames
         * on fromdevice queue.  Wait until after retrieving the
         * frames from curFrame and nextFrame since we need to
         * retain frame ordering for ISSUERECLAIM.
         */
        while (!QUE_empty(device->todevice)) {
            QUE_put(device->fromdevice, QUE_get(device->todevice));
            SEM_post(port->sem);
        }

        /* reset input delay so controller can get started again */
        port->delay = 0;
    }
    else {      /* device->mode == DEV_OUPUT */
        if (!flush) {
            /*
             * Make sure device has been started.  This is usually done
             * in DAX_issue() but it's possible for output to be queued
             * and the device not started.
             */
            load(port, device->todevice);
            (*params->start)(port);

            /* Wait for output to drain. */
            while (port->curFrame) {
                SEM_pend(port->sem, SYS_FOREVER);
                pendCount++;
            }

            (*params->stop)(port);

            while (pendCount--) {
                SEM_post(port->sem);
            }
        }
        else {
            /*
             * This case could probably be collapsed into the DEV_INPUT
             * case above, but the code would be difficult to understand.
             */
            (*params->stop)(port);

            /*
             * put "in process" frames back on todevice queue
             */
            if (port->curFrame) {
                QUE_put(device->todevice, port->curFrame);
            }
            if (port->nextFrame) {
                QUE_put(device->todevice, port->nextFrame);
            }

            /*
             * discard output! - put all frames on fromdevice queue
             */
            while (!QUE_empty(device->todevice)) {
                QUE_put(device->fromdevice, QUE_get(device->todevice));
                SEM_post(port->sem);
            }
        }

        /* reset output buffer delay */
        port->delay = params->outputdelay;
    }

    port->curJob.size = port->nextJob.size = 0;

    port->curFrame = port->nextFrame = NULL;

    return (SYS_OK);
}

/*
 *  ======== DAX_open ========
 */
static Int DAX_open(DEV_Handle device, String name)
{
    if ((device->object = (Ptr)mkPort(device, name)) != NULL) {
        return (SYS_OK);
    }
    else {
        SYS_error("DAX", SYS_EBADIO);

        return (SYS_EBADIO);
    }
}

/*
 *  ======== DAX_ready ========
 */
static Bool DAX_ready(DEV_Handle device, SEM_Handle sem)
{
    DAX_Handle  port = (DAX_Handle)device->object;

    port->ready = sem;

    return (!(QUE_empty(device->fromdevice)));
}

/*
 *  ======== DAX_nextFrame ========
 *  DAX_nextFrame() is called by the controller's HW ISR when an input
 *  frame  has been filled or an output frame has been emptied.  This
 *  function should only be called after interrupts have been reenabled.
 */
Void DAX_nextFrame(DAX_Handle port)
{
    DEV_Handle  device = port->device;

    /*
     * Put completed frame on fromdevice queue and post semaphore.
     * This will wake up the task waiting in DAX_reclaim().
     */
    QUE_put(device->fromdevice, port->curFrame);
        
    if (device->mode == DEV_OUTPUT) {
        port->curFrame->size = 0;
    }

    port->curFrame = port->nextFrame;
    port->nextFrame = NULL;

    SEM_post(port->sem);

    /* load next job */
    load(port, device->todevice);

    /*
     * If semaphore was registered with DAX_ready(), port->ready will
     * be non-NULL.  In this case, SIO_select() is probably waiting for
     * this semaphore to be posted by the first ready device.
     */
    if (port->ready) {
        SEM_post(port->ready);
    }
}
