/*****************************************************************/
/* Copyright (c) Texas Instruments, Incorporated  2000           */
/*****************************************************************/
/*****************************************************************************/
/*****************************************************************************/
/*  BOARD.H - Header file for board.c                                        */
/*                                                                           */
/*                                                                           */
/*  FUNCTIONS:                                                               */
/*                                                                           */
/*  brd_init()                                                               */
/*  brd_init_bios()                                                          */
/*  brd_fifo_read()                                                          */
/*  brd_fifo_write()                                                         */
/*  brd_led_enable()                                                         */
/*  brd_led_disable()                                                        */
/*  brd_led_toggle()                                                         */
/*                                                                           */
/*                                                                           */
/*  STATIC FUNCTIONS:                                                        */
/*    None                                                                   */
/*                                                                           */
/*                                                                           */
/*  GLOBAL VARIABLES DEFINED                                                 */
/*                                                                           */
/*                                                                           */ 
/*****************************************************************************/
#ifndef _BOARD_H
#define _BOARD_H

#include <type.h>

#undef  OK
#define OK                       0

#undef  ERROR
#define ERROR                   -1

//port declarations used to access CPLD registers
volatile ioport u16 port0;
volatile ioport u16 port1;
volatile ioport u16 port2;
volatile ioport u16 port3;
volatile ioport u16 port4;
volatile ioport u16 port5;
volatile ioport u16 port6;
volatile ioport u16 port7;

/*----------------------------------------------------------------------------*/
/* MACRO DEFINITIONS                                                          */
/*----------------------------------------------------------------------------*/
#define MIN_CPU_FREQ             10
#define MAX_CPU_FREQ             100
#define DELAY_TIMER_PORT         0

#define LED0_MASK                0x1
#define LED1_MASK                0x2
#define LED2_MASK                0x4

#define CPLD_CTRL1_DEFAULT       0x10 //NMIEN=1
#define CPLD_DMCTRL_DEFAULT      0x00
#define CPLD_DBIO_DEFAULT        0x30
#define CPLD_CTRL2_DEFAULT       0x03
#define CPLD_SEM0_DEFAULT        0x00
#define CPLD_SEM1_DEFAULT        0x00
#define CPLD_SLIC_DEFAULT        0x00

#define CPLD_CTRL1_REG           port0
#define CPLD_STAT_REG            port1
#define CPLD_DMCTRL_REG          port2
#define CPLD_DBIO_REG            port3
#define CPLD_CTRL2_REG           port4
#define CPLD_SEM0_REG            port5
#define CPLD_SEM1_REG            port6
#define CPLD_SLIC_REG            port7

//CPLD register addresses used the C-callable portio routines
//DO NOT use the above defines with the c-callable portio
//routines prototyped in portio.h

#ifdef C5410
typedef enum
{
    CpldCtrl1Reg   = 0x00,
    CpldStatReg    = 0x01,
    CpldDmctrlReg  = 0x02,
    CpldDbioReg    = 0x03,
    CpldCtrl2Reg   = 0x04,
    CpldSem0Reg    = 0x05,
    CpldSem1Reg    = 0x06,
    CpldSlicReg    = 0x07
} CpldReg, *PCpldReg;
#else
typedef enum
{
    CpldCtrl1Reg   = 0x00,
    CpldStatReg    = 0x01,
    CpldDmctrlReg  = 0x02,
    CpldDbioReg    = 0x03,
    CpldCtrl2Reg   = 0x04,
    CpldSem0Reg    = 0x05,
    CpldSem1Reg    = 0x06
} CpldReg, *PCpldReg;
#endif

//dsk semaphores
typedef enum
{
    BRD_SEM0,
    BRD_SEM1
} BrdSem;

typedef enum
{
    BRD_HINT,
     BRD_HINT2
} BrdIntr;

typedef enum
{
    BRD_LED0,
    BRD_LED1,
    BRD_LED2
} BrdLed;

typedef void (*Fp)(void);

/*****************************************************************************/
/* FUNCTIONS                                                                 */
/*****************************************************************************/

/******************************************************************************/
/* brd_init(u16 freq) - Initialize DSK board for use                          */
/*                                                                            */
/*  Parameters:                                                               */
/*      - freq: Desired board operating frequency.                            */
/*                                                                            */ 
/*  Return:                                                                   */
/*  - OK success                                                              */
/*  - ERROR failure                                                           */
/*                                                                            */ 
/*  Notes:                                                                    */
/*                                                                            */ 
/*  Board frequencies must be a multiple of 10 with the lowest being 20 MHz   */
/*  and the highest being 100 MHz.  Board frequencies supported are:	      */
/*  20,30,40,50,60,70,                                                        */ 
/*                                                                            */ 
/*  WARNING: This function is not compatible with DSP/BIOS                    */
/*                                                                            */
/******************************************************************************/
s16 brd_init(unsigned int cpuFreq);


/******************************************************************************/
/* brd_init_bios() - Initialize DSK board for use with DSP BIOS               */
/*                                                                            */
/*  Parameters:  none                                                         */
/*                                                                            */ 
/*  Return:                                                                   */
/*  - OK success                                                              */
/*  - ERROR failure                                                           */
/*                                                                            */ 
/*                                                                            */ 
/******************************************************************************/
s16 brd_init_bios(void);

/*****************************************************************************/
/*  s16 brd_interrupt_host(EvmInt intr)                                      */
/*                                                                           */ 
/*  This routine interrupts the host application .                           */
/*                                                                           */
/*  Parameters:                                                              */
/*      - intr: specified interrupt                                          */
/*  Return:                                                                  */
/*  - OK success                                                             */
/*  - ERROR failure                                                          */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_interrupt_host(BrdIntr intr);

/*****************************************************************************/
/*  s16 brd_led_enable(BrdLed led)                                            */
/*                                                                           */ 
/*  This routine enables the dsk led.             .                          */
/*                                                                           */
/*  Parameters:                                                              */
/*      - led - specific led                                                 */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - OK success                                                             */
/*  - ERROR failure                                                          */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_led_enable(BrdLed led);

/*****************************************************************************/
/*  s16 brd_led_disable(BrdLed led)                                           */
/*                                                                           */ 
/*  This routine disables the dsk led.            .                          */
/*                                                                           */
/*  Parameters:                                                              */
/*      - led - specific led                                                 */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - OK success                                                             */
/*  - ERROR failure                                                          */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_led_disable(BrdLed led);

/*****************************************************************************/
/*  s16 brd_toggle_led(BrdLed led)                                            */
/*                                                                           */ 
/*  This routine enables the dsk led.             .                          */
/*                                                                           */
/*  Parameters:                                                              */
/*      - led - specific led                                                 */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - OK success                                                             */
/*  - ERROR failure                                                          */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_led_toggle(BrdLed led);

/*****************************************************************************/
/*  s16 brd_sem_get(void)                                                    */
/*                                                                           */ 
/*  This routine tries to obtain the specified user semaphore.               */
/*                                                                           */
/*                                                                           */
/*  Parameters:                                                              */
/*      - sem - desired semaphore                                            */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - OK success                                                             */
/*  - ERROR failure                                                          */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_sem_get(BrdSem sem);

/*****************************************************************************/
/*  s16 brd_sem_wait(void)                                                   */
/*                                                                           */ 
/*  This routine tries to obtain the specified user semaphore. It blocks till*/
/*  the semaphore is obtained.                                               */
/*                                                                           */
/*  Parameters:                                                              */
/*      - sem - desired semaphore                                            */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - OK success                                                             */
/*  - ERROR failure                                                          */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_sem_wait(BrdSem sem);

/*****************************************************************************/
/*  s16 brd_sem_release(void)                                                */
/*                                                                           */ 
/*  This routine releases the previously obtained semaphore.                 */
/*                                                                           */
/*  Parameters:                                                              */
/*      - sem - semaphore to release                                         */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - OK success                                                             */
/*  - ERROR failure                                                          */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_sem_release(BrdSem sem);

/*****************************************************************************/
/*  s16 brd_fifo_read (s8 *bufptr, u16 cnt)			            		     */
/*                                                                           */
/*  This routine begins a synchronous read from the fifo channel.            */
/*                                                                           */
/*  Parameters:                                                              */
/*  - p_buffer is the location of the data to be written to the FIFO         */
/*  - num_bytes is the number of bytes of data to be written to the FIFO,    */
/*                                                                           */
/*  Return:                                                                  */
/*  - returns OK or ERROR,												     */
/*      possible error conditions include: another send in progress,         */
/*      chan not open, no DMA chan available                                 */
/*                                                                           */
/*  Notes:                                                                   */
/*                                                                           */
/*****************************************************************************/
s16 brd_fifo_read(char *bufptr, u16 cnt);

/*****************************************************************************/
/*  s16 brd_fifo_write (s8 *bufptr, u16 cnt)   					             */
/*                                                                           */
/*  This routine begins a synchronous read from the fifo channel.            */
/*                                                                           */
/*  Parameters:                                                              */
/*  - p_buffer is the location of the data to be written to the FIFO         */
/*  - num_bytes is the number of bytes of data to be written to the FIFO,    */
/*                                                                           */
/*  Return:                                                                  */
/*  - returns OK or ERROR,													 */
/*                                                                           */
/*  Notes:                                                                   */
/*                                                                           */
/*****************************************************************************/
s16 brd_fifo_write(char *bufptr, u16 cnt);

/*****************************************************************************/
/*  s16 brd_delay_msec(u16 msec)                                             */
/*                                                                           */ 
/*  This routine delays execution by specified milliseconds.                 */
/*                                                                           */
/*  Parameters:                                                              */
/*      - msec - time delay in milliseconds.                                 */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - TRUE success                                                           */
/*  - FALSE failure                                                          */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*  The delay is implemented using timer 0 at port (0x24). Care must be      */ 
/*  taken not to interrupt timer operation by way of interrupts.             */
/*  Maximum supported delay is 6553 usec or 6.5 secs.                        */ 
/*                                                                           */ 
/*  WARNING: This function is not compatible with DSP/BIOS                   */
/*                                                                           */
/*****************************************************************************/
s16 brd_delay_msec(u16 msec);

/*****************************************************************************/
/*  s16 brd_delay_usec(u16 usec)                                             */
/*                                                                           */ 
/*  This routine delays execution by specified microseconds.                 */
/*                                                                           */
/*  Parameters:                                                              */
/*      - usec - time delay in microseconds.                                 */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - TRUE success                                                           */
/*  - FALSE failure                                                          */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*  The delay is implemented using timer 0 at port (0x24). Care must be      */ 
/*  taken not to interrupt timer operation by way of interrupts.             */
/*  Maximum supported delay is 65536 usec or 65.536 msecs.                   */ 
/*                                                                           */ 
/*  WARNING: This function is not compatible with DSP/BIOS                   */
/*                                                                           */
/*****************************************************************************/
s16 brd_delay_usec(u16 usec);

/*****************************************************************************/
/*  s16 brd_set_wait_states(u16 flashDbProg,                                 */
/*                          u16 data32_64,                                   */
/*                          u16 io)                                          */
/*                                                                           */ 
/*  This routine sets the wait states for program, data & I/O spaces         */
/*                                                                           */
/*  Parameters:                                                              */
/*      - extendedProg - dsk onboard external program memory wait states     */
/*      - flashDbProg - flash/daughter board memory wait states              */
/*      - data0_32 - first 32K data memory wait states                       */
/*      - data32_64 - Second 32K data memory wait states                     */
/*      - io - I/O wait states                                               */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - OK success                                                             */
/*  - ERROR failure                                                          */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*  WARNING: This function is not compatible with DSP/BIOS                   */
/*                                                                           */
/*****************************************************************************/
s16 brd_set_wait_states(u16 flashDbProg, 
                        u16 data32_64, 
                        u16 io);

/*****************************************************************************/
/*   brd_set_cpu_freq()                                                      */
/*                                                                           */
/*  This routine sets the CPU frequency                                      */
/*                                                                           */
/*  Parameters:                                                              */
/*  - freq frequency in MHz to set cpu clock                                 */
/*                                                                           */
/*  Return:                                                                  */
/*  - void                                                                   */
/*                                                                           */
/*  Notes:                                                                   */
/*                                                                           */
/*  Board frequencies must be a multiple of 10 with the lowest being 20 MHz  */
/*  and the highest being 100 MHz.  Board frequencies supported are:	     */
/*  20,30,40,50,60,70,                                                       */ 
/*                                                                           */
/*  WARNING: This function is not compatible with DSP/BIOS                   */
/*                                                                           */
/*****************************************************************************/
void brd_set_cpu_freq(u16 freq);

/*****************************************************************************/
/*   brd_get_cpu_freq()                                                      */
/*                                                                           */
/*  This routine returns the CPU frequency                                   */
/*                                                                           */
/*  Parameters:                                                              */
/*  - None                    					                             */
/*                                                                           */
/*  Return:                                                                  */
/*  - void                                                                   */
/*                                                                           */
/*  Notes:                                                                   */
/*****************************************************************************/
u16 brd_get_cpu_freq();

/*****************************************************************************/
/*   brd_io_redir()                                                          */
/*                                                                           */
/*  This routine redirects printfs to the host terminal.                     */
/*                                                                           */
/*  Parameters:                                                              */
/*  - None                    					                             */
/*                                                                           */
/*  Return:                                                                  */
/*  - void                                                                   */
/*                                                                           */
/*  Notes:                                                                   */
/*****************************************************************************/
void brd_io_redir(bool enable);

/*****************************************************************************/
/*  s16 brd_mem_transfer()                                                   */
/*                                                                           */ 
/*  This routine uses the DMA to transfer data from one mem location to the  */
/*  the other.                                                               */
/*                                                                           */ 
/*  Parameters:                                                              */
/*      - ch : DMA channel to use.                                           */
/*      - src_addr - source address.                                         */
/*      - src_space - source memory space. Values in dma5410.h               */
/*      - dst_addr - destination address.                                    */
/*      - dst_space - destination memory space. Values in dma5410.h          */
/*      - cnt - number of words to transfer.                                 */
/*      - sync - wait for transfer to complete.                              */
/*  Return:                                                                  */
/*  - OK success                                                             */
/*  - ERROR failure                                                          */
/*                                                                           */ 
/*  Notes:                                                                   */
/*  - If channel is busy call waits for channel to be available.             */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_mem_transfer(u16 ch, u16 *src_addr, u16 src_space, 
                             u16 *dst_addr, u16 dst_space, u16 cnt, bool sync);


#endif /* end of #ifndef _BOARD_H */
