/******************************************************************************/
/*  aic27ex.c - Codec sample application for TMS320VC5510 EVM                 */
/*				Plays continuous tone to left, right and then both            */
/*				speakers. First LED is on during play to left speaker,        */
/*				second LED is on during play to right speaker, and both are   */
/*				on during play to both speakers. All LEDs flash on success.   */ 
/*                                                                            */
/******************************************************************************/
#include <board.h>
#include <sig.h>
#include <aic27.h>
#include <dma55x.h> 
#include <intr.h>

//*****************************************************************************/
//* defines
//*****************************************************************************/
#define TONE_PB_SAMPLE_RATE         8000   
#define LOOP_SAMPLE_RATE            AIC27_48000HZ
#define BUF_SIZE                    1024        
#define NUM_BUFFS_TO_PLAY           25
#define TONE_FREQ                   500
#define N_CH						2
#define DELAY_TIME					100

#define MIC_INPUT                   1
#define LINE_INPUT                  0

#define FOREVER                     0xFFFFFFFFUL

//*****************************************************************************/
//* globals data
//*****************************************************************************/
s16      toneBuf[2][BUF_SIZE];
s16      *pBuf[2];
volatile u16 bufsPlayed;

//*****************************************************************************/
//* functions
//*****************************************************************************/
static s16 Aic27ExContTonePb( s16 ch, s16 duration );
static s16 Aic27ExLb(s16 micSel);
void blink_leds(u32 cnt);
void busy_delay(u16 cnt);

//*****************************************************************************/
//* MAIN
//*****************************************************************************/
void main()                
{
    // Initialize board
    brd_init(160); 
    
    INTR_GLOBAL_DISABLE;

    // play continuous tone to left, right and then both speakers
    CPLD_CTRL1_REG |= 0x1;
    Aic27ExContTonePb(LEFT, NUM_BUFFS_TO_PLAY );

    busy_delay(50);
    
	CPLD_CTRL1_REG |= 0x2;
    Aic27ExContTonePb(RIGHT, NUM_BUFFS_TO_PLAY );

    busy_delay(50);

    Aic27ExContTonePb(BOTH, NUM_BUFFS_TO_PLAY );
	
    busy_delay(50);
    brd_led_toggle(BRD_LED0);
    brd_led_toggle(BRD_LED1);

    // loop line or mic input to line output
    Aic27ExLb(LINE_INPUT);
   	//Aic27ExLb(MIC_INPUT);

    // Do nothing else during loopback
    blink_leds(FOREVER);
}

/*****************************************************************************/
/* Aic27ExContTonePb() - Play continuous tone                                */
/*   This function provides an example of playing back a buffer of data in   */
/*   continuous (dma autoinitialization) mode.                               */
/*                                                                           */
/* Note: The volume of the output from the front_mixer of the AIC27 codec    */
/*       should be set to mute, otherwise the audio will be heard.           */
/*                                                                           */                                                       
/*****************************************************************************/
static s16 Aic27ExContTonePb( s16 ch, s16 duration )
{
    s16 *status;
    volatile u16 bufNum = 0;  

    //initialize tone generator
    sig_init(TONE_FREQ, 8000);

	//synthesize tone
    sig(toneBuf[0], BUF_SIZE, N_CH, (SigCh)ch);
    sig(toneBuf[1], BUF_SIZE, N_CH, (SigCh)ch);

    //select stereo output
    CPLD_CTRL2_REG &= ~1;

    //open codec
    aic27_open();

    //set volume
    aic27_write_reg(MASTER_VOL, 0x0202, 0xFFFF);

    //set sample rate
	aic27_sample_rate(AUDIO_ADC_RRATE, AIC27_8000HZ);
    aic27_sample_rate(FRONT_DAC_SRATE, AIC27_8000HZ);

    //clear buffer counter
    bufsPlayed  = 0;

    //perform continuous play
    aic27_cont_play(toneBuf[0], toneBuf[1], BUF_SIZE, 0, &status);
	
    //wait until first buffer is completed
    while (*status);

    //loop through all buffers
    while (bufsPlayed++ < duration)
	{		        

        bufNum ^= 1;
		*status = 1;

		//wait for buffer to be empty
        while (*status);
	}
	
	// stop codec
	aic27_stop_play();
	
    //close codec
    aic27_close();
	
	return(BRD_OK);
}


/****************************************************************************/
/* Aic27ExLb -  CODEC Example: Loopback mode                                */
/*                                                                          */
/*   This function provides a loopback feature.								*/
/*                                                                          */
/****************************************************************************/
static s16 Aic27ExLb(s16 micSel)
{
    int *bufStatus1;
    int *bufStatus2;
    u16 cnt;   
    
	CPLD_CTRL2_REG &= ~1;

	brd_led_disable(BRD_LED0);
	brd_led_disable(BRD_LED1);
	brd_led_disable(BRD_LED2);

    aic27_open();

    //select stereo output
    CPLD_CTRL2_REG &= ~1;

    // set Front_mixer and Master volume
    aic27_write_reg(MASTER_VOL, 0x0202, 0xFFFF);
    aic27_write_reg(FRONT_MIXER_VOL, 0x0000, 0xFFFF);

    //set sample rate
    aic27_sample_rate(AUDIO_ADC_RRATE, LOOP_SAMPLE_RATE);
    aic27_sample_rate(FRONT_DAC_SRATE, LOOP_SAMPLE_RATE);

	if (micSel)
	{
		aic27_write_reg(REC_SEL, MIC_IN, 0xFFFF); //mic_in
	}
	else
	{
		aic27_write_reg(REC_SEL, LINE_IN, 0xFFFF); //line_in
	}
	
    // clear buffers
    for (cnt=0; cnt<BUF_SIZE; cnt++)
    {
        toneBuf[0][cnt] = 0;
        toneBuf[1][cnt] = 0;
    }

    // enable simultaneous capture and playback
    aic27_cont_capture(toneBuf[0],toneBuf[1],BUF_SIZE,0,&bufStatus1);
    aic27_cont_play   (toneBuf[1],toneBuf[0],BUF_SIZE,0,&bufStatus2);
	
	// close codec
	aic27_close();

	return (BRD_OK);
}

/****************************************************************************/
/* blink_leds(u32 cnt) : Toggles all the user leds for a duration of cnt.   */
/*                                                                          */
/****************************************************************************/
void blink_leds(u32 cnt)
{
    while (cnt)
	{
		brd_led_toggle(BRD_LED0);
		busy_delay(DELAY_TIME);
		brd_led_toggle(BRD_LED1);
		busy_delay(DELAY_TIME);
		brd_led_toggle(BRD_LED2);
		busy_delay(DELAY_TIME);

        if (cnt != FOREVER) cnt--;
	}
}

void busy_delay(u16 cnt)
{
	volatile u16 temp;

	while (cnt--)
	{
		temp = 0xffff;
		while(temp--);
	}
}  
