/****************************************************************************/
/*	xflash.c  - program to write and read flash memory. One of the LEDs     */
/*              flash whenever there is an unsuccessful write or            */
/*			    a write mismatch occurs. All LEDs flash on success.         */
/****************************************************************************/
#include <board.h>
#include <emif.h>
#include <flash.h>

/****************************************************************************/
/*	defines																	*/
/****************************************************************************/
#define SIZE	0x100ul

/****************************************************************************/
/*	Function Prototypes                                                     */
/****************************************************************************/
void blinkme(u32 cnt, u16 led);
void probeFlash();
void testFlash();


/******************************************************************************/
/* Global variables   														  */
/******************************************************************************/
u16 *pdst = (u16 *)FLASH_BASE_ADDR;
s16 result = BRD_OK;
u16 data = 0;
u32 i = 0;

/*****************************************************************************/
/* MAIN																		 */
/*****************************************************************************/
void main()
{    
	// initialize board
	brd_init(160);
	           
	// reset the flash memory
 	flash_reset();	

	//erase all of flash memory
	flash_erase((u16)-1);
      
	// write a block of data to flash memory
	probeFlash();

	// write a word at a time to flash memory
	testFlash();

	// blink all the LEDs
	blinkme(0xffffffffu, 0x7); 
}
 
 
/****************************************************************************/
/*   void probeFlash()                                                      */
/*                                                                          */
/*   This function writes a block of data into Flash memory.                */
/*   An unsuccessful write results in the blinking of the first LED.        */
/*   The second LED blinks if there is a write mismatch.                    */
/*                                                                          */
/*   Parameters:  None                                                      */
/*                                                                          */
/*   Return : None                                                          */
/*                                                                          */
/****************************************************************************/ 
void probeFlash()
{
	u16 pBuf[SIZE];

	pdst = (u16 *)(FLASH_BASE_ADDR + i);

	for (i = 0; i < SIZE; i++)
	{
		pBuf[i] = 0xa000 + (u16)i;                        		
	}

	result = flash_write(pBuf, (u16 *)FLASH_BASE_ADDR, SIZE);

	if (result == BRD_ERROR)
		blinkme(0xffu, 0x1);	// blink first LED for unsuccessful write        
		
	for (i = 0; i < SIZE; i++)
	{
        if (*pdst++ != pBuf[i])
        	blinkme(0xffu, 0x2);	// blink second LED for a write mismatch 
	}   
}


/****************************************************************************/
/*   void testFlash()                                                       */
/*                                                                          */
/*   This function writes one word at a time to Flash memory.               */
/*   An unsuccessful write results in the blinking of the first LED.        */
/*   The second LED blinks if there is a write mismatch.                    */
/*                                                                          */
/*   Parameters:  None                                                      */
/*                                                                          */
/*   Return : None                                                          */
/*                                                                          */
/****************************************************************************/
void testFlash()
{
 	CPLD_CTRL1_REG &= ~7;   // disable all LEDs
 	        
	for (i = 0; i < SIZE; i++)
	{
		data = (u16)i;                        
		
		pdst = (u16 *)(0x200000ul + i);   

		   // write one word at a time to flash
        if (flash_write(&data, pdst, 1) == BRD_ERROR)
        	blinkme(0xffu, 0x1);		// blink first LED on unsuccessful write              
	}
        
	for (i = 0; i < SIZE; i++)
	{
		pdst = (u16 *)(0x200000ul + i);
		
        if (*pdst != (u16)i)
        	blinkme(0xffu, 0x2);	// blink second LED on write mismatch
	}   
}


/****************************************************************************/
/*   void blinkme(u32 cnt, u16 led)                                         */
/*                                                                          */
/*   This function blinks the first, second , third or all LEDs             */
/*	 depending on parameter u16 led.                                        */
/*                                                                          */
/*   Parameters:                                                            */
/*         unsigned long cnt                                                */
/*         unsigned int led                                                 */
/*                                                                          */
/*   Return : None                                                          */
/*                                                                          */
/****************************************************************************/
void blinkme(u32 cnt, u16 led)
{
	while(cnt--)
	{
		CPLD_CTRL1_REG |= led;
		brd_busy_delay(100);
		CPLD_CTRL1_REG &= ~led;
		brd_busy_delay(100);
	}
}


