/****************************************************************************/
/*	mem.c  -  example to test internal SARAM and external SBSRAM            */
/*            with a data pattern. First LED blinks on failure of           */
/*			  SBSRAM test. Second LED blinks on failure of SARAM            */
/*			  test. All LEDs blink on success.                              */                                                   
/****************************************************************************/
#include <board.h>

/****************************************************************************/
/*	defines                                                                 */
/****************************************************************************/
#define SIZE	(0x80000ul>>1)
#define ADDR	SBSRAM_BASE_ADDR   

/****************************************************************************/
/*	functions                                                               */
/****************************************************************************/
void blinkme(u32 cnt, u16 led);
void onChipMem();
void offChipMem();

/****************************************************************************/
/*	MAIN                                                                    */
/****************************************************************************/
void main()
{           
    // initialize board
    brd_init(160);
	
    // perform off-chip memory test (SBSRAM)
    offChipMem();
	
    // perform on-chip memory test (SARAM)
    onChipMem();

    // blink all three user LEDs forever
    while (1)
    { 
        blinkme(0xff, 0x7);
    }
}


/****************************************************************************/
/*	void blinkme(u32 cnt, u16 led)                                          */
/*                                                                          */
/*	This function flashes first, second, third or all LEDs                  */
/*  depending on the parameter u16 led.                                     */
/*                                                                          */
/*	Parameters:                                                             */
/*        unsigned long cnt                                                 */
/*        unsigned int led                                                  */
/*																			*/
/*	Return:		None														*/
/****************************************************************************/
void blinkme(u32 cnt, u16 led)
{
	while(cnt--)
	{
		CPLD_CTRL1_REG |= led;     
		brd_busy_delay(100);
		CPLD_CTRL1_REG &= ~led;
		brd_busy_delay(100);
	}
}


/****************************************************************************/
/*	void onChipMem()                                                        */
/*																			*/
/*	This function performs a test on SARAM. Second LED is on while          */
/*  test is being performed. Second LED blinks on failure.                  */
/*																			*/
/*	Parameters:	None														*/
/*																			*/
/*	Return:		None														*/
/****************************************************************************/
void onChipMem()
{        
    u16 i;
    u16 *p = (u16 *)0x20000;
	               
    // light second LED
    CPLD_CTRL1_REG |= 2;

    // create test array
    for (i=0; i<0x1000; i++)
	{   
		p[i] = i;  
	}    
	
    // store test array to 0x20000 in SARAM
    for (i=0; i<0x1000; i++)
	{
        // compare data, if error flash second LED forever
        if (p[i] != i)
		{
              blinkme(0xff, 0x2);
		}
	}

    // turn off second LED 
    CPLD_CTRL1_REG &= ~2;
}


/****************************************************************************/
/*	void offChipMem()														*/
/*																			*/
/*	This function performs a test on SBSRAM. First LED is on while          */
/*  test is being performed. First LED blinks on failure.                   */
/*																			*/
/*	Parameters:	None														*/
/*																			*/
/*	Return:		None														*/
/*																			*/
/****************************************************************************/
void offChipMem()
{
    u32 *p;
    u32 i = 0, j = 0;

    // light first LED 
    CPLD_CTRL1_REG |= 1;

    // start of SBSRAM
    p = (u32 *)ADDR;

    for (i=0; i<SIZE; i++)
    {                       
        j = i % 0x8000;
        
        // check for end of block
        if (i && (j == 0))
        {
            p = (u32 *)((u32)p + 0x10000);   
        }
    
        // store data
        p[j] = i;
    }        
    
    // start of SBSRAM
    p = (u32 *)ADDR;
    
    for (i=0; i<SIZE; i++)
    {
        j = i % 0x8000;
        
        // check for end of block
        if (i && (j == 0))
        {
            p = (u32 *)((u32)p + 0x10000);   
        }     
        
        // compare data, if error blink first LED forever
        if (p[i] != i)
            blinkme(0xff, 0x1);
    } 

    // turn off LED 1
    CPLD_CTRL1_REG &= ~1;
}


