/****************************************************************************/
/*	post.c	- This example provides a power-on self test that can be        */
/*			  programmed into Flash memory and executed in 32-bit           */
/*			  asynchronous memory boot mode. It tests the CLOCKS, DSP       */
/*			  Memory, and AIC27 CODEC.                                      */
/*																			*/
/****************************************************************************/
#include <board.h>
#include <regs55x.h>
#include <mcbsp55x.h>
#include <intr.h>
#include <timer.h>
#include <aic27.h>
#include <sig.h>

#define DBG_PRINT                   0

#if DBG_PRINT
	#include <stdio.h>
#endif

/****************************************************************************/
/* enums & defines                                                          */
/****************************************************************************/
typedef enum
{
	POST_PASS   = 0x0,
    CLOCK_ERROR = 0x1,
    MEM_ERROR   = 0x2,
    AIC27_ERROR = 0x4
} PostResult;

//McBsp Settings
#define AIC_MCBSP   2

#define SPCR1_VAL   0
#define SPCR2_VAL   0

// set receive control reg values: receive data has single-phase
// frame with 16-bit word and 1-bit data delay
#define RCR1_VAL	(WORD_LENGTH_16<<5)
#define RCR2_VAL	((SINGLE_PHASE<<15) | DATA_DELAY1)

// set transmit control reg values: transmit data has single-phase
// frame with 16-bit word and 1-bit data delay
#define XCR1_VAL	(WORD_LENGTH_16<<5)
#define XCR2_VAL	((SINGLE_PHASE<<15) | DATA_DELAY1)

#define SRGR1_VAL	0
#define SRGR2_VAL	0

// set pin control reg values: transmit and receive frame-sync pulses are
// active high. Transmit data is clocked on rising edge of CLKX, while
// receive data is sampled on falling edge of CLKR.
#define PCR_VAL     ((FSYNC_POL_HIGH<<3)|(FSYNC_POL_HIGH<<2)|(CLKX_POL_RISING<<1)|(CLKR_POL_FALLING))

#define TONE_PB_SAMPLE_RATE         8000
#define BUF_SIZE                    1024        
#define NUM_BUFFS_TO_PLAY           100
#define TONE_FREQ                   1000
#define N_CH						2

#define BLK_SIZE					0x10000
#define CE0_SBSRAM_SIZE				(0x80000ul>>1)
#define CE0_SBSRAM_ADDR				SBSRAM_BASE_ADDR  

#define DELAY_TIME                  50

/****************************************************************************/
/* functions                                                                */
/****************************************************************************/
void processResult(PostResult ret);
interrupt void _serialPort2Isr(void);
interrupt void _timerIsr(void);
void setup_serial_port();

void captureTone();
void playTone();

typedef unsigned long datum;     //Set the data bus width to 16 bits

/******************************************************************************/
/* FUNCTIONS                                                                  */
/******************************************************************************/
#if !DBG_PRINT
	#define NULL  (void *) 0
#endif

datum memTestDataBus(u32 addr);
datum * memTestAddressBus(u32 addr, u32 nWords);
datum * memTestDevice(u32 addr, u32 nWords); 
u32 addr;

s16 aic27Test();
s16 clockTest();
s16 memTest();

void reportError();

/****************************************************************************/
/* Global variables                                                         */
/****************************************************************************/
volatile u16 cnt = 20;
extern u16 cpuFreq;

s32 expected_count;
s32 timer_count = 0;
s32 codec_count = 0;
volatile s16 clockStatus = -1;
volatile bool testDone = False; 
volatile u16 reg1 = 0;
volatile u16 reg2 = 0;

s16  toneBuf[2][BUF_SIZE];
u16 errorLog;

/****************************************************************************/
/* MAIN                                                                     */
/****************************************************************************/
void main()
{     
    // initialize board
    brd_init(160);
    
    // clear error log
    errorLog = 0;

#if 1
    if (clockTest())
		processResult(CLOCK_ERROR);
#endif

#if 1
	if (memTest())
		processResult(MEM_ERROR);
#endif

#if 1          
	if (aic27Test())
		processResult(AIC27_ERROR);
#endif

#if DBG_PRINT
	reportError();
#endif
	
    // all LEDs off
    CPLD_CTRL1_REG &= ~7;

    if (!errorLog)
    {
       // blink all 3 user LEDs forever if no errors
       while (1)
       {
           brd_led_toggle(BRD_LED0);
                   
           brd_busy_delay(DELAY_TIME);

           brd_led_toggle(BRD_LED1);

           brd_busy_delay(DELAY_TIME);

           brd_led_toggle(BRD_LED2);

           brd_busy_delay(DELAY_TIME);
       }
    }
    else
    {
       // blink all error code forever 
       while (1)
       {
           CPLD_CTRL1_REG |= errorLog;
           brd_busy_delay(DELAY_TIME);

           CPLD_CTRL1_REG &= ~errorLog;
           brd_busy_delay(DELAY_TIME);
       }
    }
}

/****************************************************************************/
#if DBG_PRINT
void reportError()
{
	char *pass = "PASSED";
	char *fail = "FAILED";

    printf("\nTMS320VC5510  EVM Test Results\n");
	printf("==========================================\n");

    printf("CPU Clock Test....................%s\n", (errorLog & (1<<CLOCK_ERROR)) ? Fail : Pass);
    printf("SBSRAM Test.......................%s\n", (errorLog & (1<<MEM_ERROR)) ? Fail : Pass);
    printf("AIC27 Codec Test..................%s\n", (errorLog & (1<<AIC27_ERROR)) ? Fail : Pass);
}      
#endif


/****************************************************************************/
/*	void processResult(PostResult error)								    */
/*																		    */
/*	This function blinks a respective LED depending on the error            */
/*	encountered. LED 1 blinks in case of CLOCK_ERROR, LED 2 blinks          */
/*	in case of MEM_ERROR, while LED 3 blinks in case of AIC27_ERROR.        */
/*                                                                          */
/****************************************************************************/
void processResult(PostResult error)
{	
    int flashCount;

    // log error
    errorLog |= error;

    // if fail, then flash error code
    for (flashCount=0; flashCount<5; flashCount++)
    // Flash test # that failed 5 times
    {
       CPLD_CTRL1_REG |= error;
       brd_busy_delay(DELAY_TIME);
       CPLD_CTRL1_REG &= ~error;
       brd_busy_delay(DELAY_TIME);
    }

    // all LEDs off
    CPLD_CTRL1_REG &= ~7;
}


/****************************************************************************/
/*	Function:	s16 memTest()   											*/
/*                                                                          */
/*	Description:	Tests the SBSRAM in the DSP Memory.                     */
/*                                                                          */
/*	Returns:	BRD_OK on success and BRD_ERROR on failure                  */
/*                                                                          */
/****************************************************************************/
s16 memTest()
{
    
    brd_led_disable(BRD_LED0);
    brd_led_enable(BRD_LED1);
    brd_led_disable(BRD_LED2);

    brd_busy_delay(400);

	if (memTestDataBus((u32)CE0_SBSRAM_ADDR) == (u16)BRD_ERROR)
	{   
		return BRD_ERROR;
	}	
	
	for (addr = CE0_SBSRAM_ADDR; addr < (u32)(CE0_SBSRAM_ADDR+CE0_SBSRAM_SIZE); addr += (u32)BLK_SIZE)
	{
		if ((memTestAddressBus(addr, BLK_SIZE>>1)) != NULL)
		{
			return BRD_ERROR;
		}
	}
	
	for (addr = CE0_SBSRAM_ADDR; addr < (u32)(CE0_SBSRAM_ADDR+CE0_SBSRAM_SIZE); addr += (u32)BLK_SIZE)
	{
		if ((memTestDevice(addr, BLK_SIZE>>1)) != NULL)
		{
			return BRD_ERROR;
		}
	}   
	
	return BRD_OK;
}


/****************************************************************************/
/*                                                                          */
/* Function:    memTestDataBus()                                            */
/*                                                                          */
/* Description: Test the data bus wiring in a memory region by              */
/*              performing a walking 1's test at a fixed address            */
/*              within that region.  The address (and hence the             */
/*              memory region) is selected by the caller.                   */
/*                                                                          */
/* Notes:                                                                   */
/*                                                                          */
/* Returns:     0 if the test succeeds.                                     */
/*              A non-zero result is the first pattern that failed.         */
/*                                                                          */
/****************************************************************************/
datum memTestDataBus(u32 addr)
{
    datum pattern;
	volatile datum *address = (volatile datum *)addr;
	
    //Perform a walking 1's test at the given address.
    for (pattern = 1; pattern != 0; pattern <<= 1)
    {
        //Write the test pattern.
        *address = pattern;
		
        //Read it back (immediately is okay for this test).
        if (*address != pattern) 
        {
            return (pattern);
        }
    }
	
    return (0);
}


/****************************************************************************/
/*                                                                          */                           
/* Function:    memTestAddressBus()                                         */
/*                                                                          */
/* Description: Test the address bus wiring in a memory region by           */
/*              performing a walking 1's test on the relevant bits          */
/*              of the address and checking for aliasing. This test         */
/*              will find single-bit address failures such as stuck         */
/*              -high, stuck-low, and shorted pins.  The base address       */
/*              and size of the region are selected by the caller.          */
/*                                                                          */
/* Notes:       For best results, the selected base address should          */
/*              have enough LSB 0's to guarantee single address bit         */
/*              changes.  For example, to test a 64-Kbyte region,           */
/*              select a base address on a 64-Kbyte boundary.  Also,        */
/*              select the region size as a power-of-two--if at all         */ 
/*              possible.                                                   */
/*                                                                          */
/* Returns:     NULL if the test succeeds.                                  */
/*              A non-zero result is the first address at which an          */
/*              aliasing problem was uncovered.  By examining the           */
/*              contents of memory, it may be possible to gather            */
/*              additional information about the problem.                   */
/*                                                                          */
/****************************************************************************/
datum *memTestAddressBus(u32 addr, u32 nWords)
{
	volatile datum *baseAddress = (volatile datum *)addr;
    u32 addressMask = (nWords - 1);
    u32 offset;
    u32 testOffset;
	
    datum pattern     = (datum) 0xAAAAAAAA;
    datum antipattern = (datum) 0x55555555;
	
	
    //Write the default pattern at each of the power-of-two offsets.
    for (offset = sizeof(datum); (offset & addressMask) != 0; offset <<= 1)
    {
        baseAddress[offset] = pattern;
    }
	
    //Check for address bits stuck high.
    testOffset = 0;
	
    baseAddress[testOffset] = antipattern;
	
    for (offset = sizeof(datum); (offset & addressMask) != 0; offset <<= 1)
    {
        if (baseAddress[offset] != pattern)
        {
            return ((datum *) &baseAddress[offset]);
        }
    }
	
    baseAddress[testOffset] = pattern;
	
    //Check for address bits stuck low or shorted.
    for (testOffset = sizeof(datum); (testOffset & addressMask) != 0; testOffset <<= 1)
    {
        baseAddress[testOffset] = antipattern;
		
        for (offset = sizeof(datum); (offset & addressMask) != 0; offset <<= 1)
        {
            if ((baseAddress[offset] != pattern) && (offset != testOffset))
            {
                return ((datum *) &baseAddress[testOffset]);
            }
        }
		
        baseAddress[testOffset] = pattern;
    }
	
    return (NULL);
}


/****************************************************************************/
/*                                                                          */
/* Function:    memTestDevice()                                             */
/*                                                                          */
/* Description: Test the integrity of a physical memory device by           */
/*              performing an increment/decrement test over the             */
/*              entire region.  In the process every storage bit            */
/*              in the device is tested as a zero and a one.  The           */
/*              base address and the size of the region are                 */
/*              selected by the caller.                                     */
/*                                                                          */
/* Notes:                                                                   */
/*                                                                          */
/* Returns:     NULL if the test succeeds.  Also, in that case, the         */
/*              entire memory region will be filled with zeros.             */
/*                                                                          */
/*              A non-zero result is the first address at which an          */
/*              incorrect value was read back.  By examining the            */
/*              contents of memory, it may be possible to gather            */
/*              additional information about the problem.                   */
/*                                                                          */
/****************************************************************************/
datum* memTestDevice(u32 addr, u32 nWords)       
{
    u32 offset;
	volatile datum *baseAddress = (volatile datum *)addr;
	
    datum pattern;
    datum antipattern;
	
	//Fill memory with a known pattern.
    for (pattern = 1, offset = 0; offset < nWords; pattern++, offset++)
    {
        baseAddress[offset] = pattern;
    }
	
	
	
    //Check each location and invert it for the second pass.
    for (pattern = 1, offset = 0; offset < nWords; pattern++, offset++)
    {
        if (baseAddress[offset] != pattern)
        {
            return ((datum *) &baseAddress[offset]);
        }
		
        antipattern = ~pattern;
		
        baseAddress[offset] = antipattern;
    }
	
	//Check each location for the inverted pattern and zero it.
    for (pattern = 1, offset = 0; offset < nWords; pattern++, offset++)
    {
        antipattern = ~pattern;
		
        if (baseAddress[offset] != antipattern)
        {
            return ((datum *) &baseAddress[offset]);
        }
		
        baseAddress[offset] = 0;
    }

    return (NULL);
}


/****************************************************************************/
/*	Function:	s16 aic27Test()												*/
/*																			*/
/*	Description:	Tests the AIC27 Codec.									*/
/*																			*/
/*	Returns:	BRD_OK on success and BRD_ERROR on failure.                 */
/*                                                                          */
/*  Note: The analog input from line_in and mic_in along with the output    */
/*       from the front_mixer of the AIC27 codec should be set to mute. The */
/*       mute bit can not be set for the current version of this codec, but */
/*       the vol has been reduced as much as possible. The user can hear    */
/*       the audio as the tones are being played.                           */                                        
/*																			*/
/****************************************************************************/
s16 aic27Test()
{
	s16 ret = BRD_OK;
	
    brd_led_disable(BRD_LED0);
    brd_led_disable(BRD_LED1);
    brd_led_enable(BRD_LED2);

	aic27_open();		// open AIC27 codec and connect codec to a McBSP
	
	aic27_write_reg(MASTER_VOL, 0x0202, 0xFFFF);	// write to an AIC27 control reg
	
	reg1 = aic27_read_reg(MASTER_VOL);		// read from the AIC27 control reg   
	
 
	playTone(RIGHT);	

	brd_busy_delay(10);

	playTone(LEFT);

	brd_busy_delay(10);

	playTone(BOTH);

	brd_busy_delay(10);

	captureTone();

	aic27_close();		// close the codec connection		
	
	return ret;
}


/****************************************************************************/
/*	Function:	void playTone(int ch)                                       */
/*																			*/
/*	Description:															*/
/*																			*/
/*	Returns:	none														*/
/*																			*/
/****************************************************************************/
void playTone(int ch)
{
	u16 i;
    int *bufStatus;
    bool bufNum = (bool)0;

	sig_init(TONE_FREQ,TONE_PB_SAMPLE_RATE);
	
	for (i=0; i<BUF_SIZE; i++)
	{
		toneBuf[0][i] = toneBuf[1][i] = 0;
		i++;
		toneBuf[0][i] = toneBuf[1][i] = 0;
	}

	//synthesize tone
	sig(toneBuf[0], BUF_SIZE, N_CH, (SigCh)ch);
	sig(toneBuf[1], BUF_SIZE, N_CH, (SigCh)ch);
	
	CPLD_CTRL2_REG &= ~1;

    aic27_write_reg(EXTD_AUDIO_STATCTRL, 0x0001, 0x0001);
    aic27_sample_rate(AUDIO_ADC_RRATE, AIC27_8000HZ);
    aic27_sample_rate(FRONT_DAC_SRATE, AIC27_8000HZ);

	reg2 = aic27_read_reg(AUDIO_ADC_RRATE);
            
    aic27_cont_play(toneBuf[0], toneBuf[1], BUF_SIZE, 0,&bufStatus);
                     
	//wait for first buffer to be done
    while (*bufStatus);

    i = 40;

	while(i--)
	{		        
		
        bufNum ^= 1;
		*bufStatus = 1;

		//wait for buffer to be empty
        while (*bufStatus);
	}
	
	aic27_stop_play();
}


/****************************************************************************/
/*	Function:	void captureTone()                                          */
/*																			*/
/*	Description:															*/
/*																			*/
/*	Returns:	None                                                        */
/*																			*/
/****************************************************************************/
void captureTone()
{
    int *bufStatus1;
    int *bufStatus2;
    int count;

	CPLD_CTRL2_REG &= ~1;

    aic27_sample_rate(AUDIO_ADC_RRATE, AIC27_48000HZ);
    aic27_sample_rate(FRONT_DAC_SRATE, AIC27_48000HZ);
    
	aic27_write_reg(FRONT_MIXER_VOL, 0x0000, 0xFFFF);
	aic27_write_reg(REC_SEL, MIC_IN, 0xffff); //mic

    aic27_write_reg(GP_REG, 0x0000, 0x0080);

	aic27_cont_capture(toneBuf[0],toneBuf[1],BUF_SIZE,0, &bufStatus1);
	
	while (*bufStatus1);

    aic27_cont_play(toneBuf[0],toneBuf[1],BUF_SIZE,0,&bufStatus2);

    // Delay for loopback
    for (count=0; count<10; count++)
	{
        brd_busy_delay(100);
    }  
}


/****************************************************************************/
/*	Function:	s16 clockTest()                                             */
/*																			*/
/*	Description:															*/
/*																			*/
/*	Returns:	0 on success and -1 on failure                              */
/*																			*/	
/****************************************************************************/
s16 clockTest()
{
	u16 time_ms = 100;
    u16 tddr;
    u16 cpuFreq;
	volatile u16 prd; 
    volatile u16 ctrl;
    volatile u16 ctrl1, ctrl2;
	
    brd_led_enable(BRD_LED0);
    brd_led_disable(BRD_LED1);
    brd_led_disable(BRD_LED2);

    brd_busy_delay(400);

    testDone = False;
	
	setup_serial_port();

    TIMER_HALT(TIMER_PORT1); //stop timer
	
	INTR_GLOBAL_DISABLE;

	// Disable interrupts
    IER0 &= ~(1<<RINT2);
    IER1 &= ~(1<<TINT1);
	
    // Clear pending interrupt flags (Note: Write 1 to clear)
    IFR0 |= (1<<RINT2);
    IFR1 |= (1<<TINT1);

	//setup interrupts
    hook_interrupt(RINT2_TRAP, _serialPort2Isr);
    IER0 |= (1<<RINT2);

    hook_interrupt(TINT1_TRAP, _timerIsr);
    IER1 |= (1<<TINT1);
	
    //clear codec count
    codec_count = 0;

    // default clock status to error
    clockStatus = -1;

    //expected count in 100 ms
	//expected_count = ( 48000 * time_ms ) / 1000;
	expected_count = ( 48 * time_ms );
	
    //set timer interrupt rate to 10000 per sec (every 100 us)
	//prd = ((f32)cpuFreq * 1000000) / ( (tddr+1) * 10000 ) - 1;
    cpuFreq = brd_get_cpu_freq();
    
    //initialize timer values
    tddr = 3;
    prd = ((cpuFreq * 100) >> 2) - 1;
    ctrl1 = 0x0F30;
    ctrl2 = 0x0B20;
	
    // 10 timer interrupts at 10000 Hz per millisecond
    timer_count = time_ms * 10;
	
    // initialize timer registers
    TCR(1)  = ctrl1;
    PRSC(1) = tddr;
    PRD(1)  = prd;
    TCR(1)  = ctrl2;

    MCBSP_ENABLE(AIC_MCBSP,MCBSP_RX);
    
	INTR_GLOBAL_ENABLE;

    // If test was not done, fail
    while (testDone == False) {};

    // Stop timer and serial port
    TIMER_HALT(TIMER_PORT1);
    MCBSP_RX_RESET(AIC_MCBSP);

    // Disable interrupts
    IER0 &= ~(1<<RINT2);
    IER1 &= ~(1<<TINT1);
    
	return clockStatus;
}

/****************************************************************************/
/*	Function: interrupt void _timerIsr()									*/
/*																			*/
/*	Description: Decrements counter and compares codec count				*/
/*														                    */
/****************************************************************************/
interrupt void _timerIsr(void)
{
    s32 count_diff;

    if (testDone == False)
    {
       if (--timer_count == 0)
	   {
          //NOTE: 240 is +/- 5% assuming 100 ms period
          count_diff = codec_count - expected_count;

          //Check if within range
          if ((count_diff > -240) && (count_diff <= 240))
          {
             clockStatus = 0;  
	  	  }

          // Set done flag
          testDone = True;
       }
	} 
}

/****************************************************************************/
/*	Function: interrupt void _serialPort2Isr()								*/
/*																			*/
/*	Description: Serial Port 2 Receive Interrupt Handler                    */
/*																			*/
/****************************************************************************/
interrupt void _serialPort2Isr(void)
{
	volatile u16 dummy;

    //increment codec count (48 kHz)
    codec_count++;

    //read data from serial port
    dummy = DRR2(2);
    dummy = DRR1(2);
}

/****************************************************************************/
/*	Function: void setup_serial_port()										*/
/*																			*/
/*	Descripton: setup serial port for specified codec                */
/******************************************************************************/
void setup_serial_port()
{
	McBspConfig bsp;

    MCBSP_TX_RESET(AIC_MCBSP);
    MCBSP_RX_RESET(AIC_MCBSP);

    bsp.port   = AIC_MCBSP;

	bsp.spcr1  = SPCR1_VAL;
    bsp.spcr2  = SPCR2_VAL;
    bsp.rcr1   = RCR1_VAL;
    bsp.rcr2   = RCR2_VAL;
    bsp.xcr1   = XCR1_VAL;
    bsp.xcr2   = XCR2_VAL;

    bsp.srgr1  = 0;
    bsp.srgr2  = 0;

    bsp.mcr1   = 0;
    bsp.mcr2   = 0;

    bsp.pcr    = PCR_VAL;

    bsp.rcera  = 0;
    bsp.xcera  = 0;
    bsp.rcerb  = 0;
    bsp.xcerb  = 0;
    bsp.rcerc  = 0;
    bsp.xcerc  = 0;
    bsp.rcerd  = 0;
    bsp.xcerd  = 0;
    bsp.rcere  = 0;
    bsp.xcere  = 0;
    bsp.rcerf  = 0;
    bsp.xcerf  = 0;
    bsp.rcerg  = 0;
    bsp.xcerg  = 0;
    bsp.rcerh  = 0;
    bsp.xcerh  = 0;
    
	/****************************************************************/
	/* Set values of all control registers                          */
	/****************************************************************/
	mcbsp_init(&bsp);
}
