;***************************************************************************
;**  $Id: $
;*****************************************************************************
;**
;**  Revision History
;**  ----------------
;**   $Log: $
;**
;*****************************************************************************
;**  DSP_XFER.ASM
;**  ------------
;**  Support Routines for the C5510 EVM Memory Transfer Module
;**
;**  _dspInit:         Initializes the DSP
;**  _processData:     Transfers blocks of data to memory
;**  _delay:           Implements a simple time delay 
;**
;*****************************************************************************/


;******************************************************************************
;**                       INCLUDE FILES
;******************************************************************************
        .mmregs

;******************************************************************************
;**  Data Transfer Definitions
;******************************************************************************
HDR_ADDR            .set (27FC0h)     ;Address of transfer header in memory
DATA_ADDR           .set (HDR_ADDR+6) ;Address of data xfer buffer in memory
TXFR_ADDR           .set (HDR_ADDR+2) ;Address of data xfer buffer in memory

;******************************************************************************
;**  DSP Control Register Definitions
;******************************************************************************
;**    Variables referenced 
;******************************************************************************

;******************************************************************************
;**    Functions declared
;******************************************************************************
    .global _processData
    .global _delay
    .global _dspInit
 
;******************************************************************************
;**    External functions used
;******************************************************************************
    .ref    _flash_write
    .ref    _flash_erase
    .ref    _flash_reset

;******************************************************************************
;**  Procedure: _dspInit
;**
;**  Arguments: None
;**
;**  Returns: None
;**
;**  Global Variables used: none
;**
;**  Subroutines: None
;**
;**  Description: This function initializes the C5510 DSP for the memory
;**               transfer kernel.
;**
;******************************************************************************
        .text

_dspInit:

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ;;   Initialize the Data transfer header for HPI access
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        xar3 = #HDR_ADDR				;Load the address of the xfer header  
        
        *ar3     = #0					;Clear out transfer count high
        *ar3(#1) = #0					;Clear out transfer count low
        *ar3(#2) = #0   				;Initialize transfer address, high word
        *ar3(#3) = #0					;Initialize transfer address, low word
        *ar3(#4) = #0					;Initialize transfer command, high
        *ar3(#5) = #0					;Initialize transfer command, low
 
        return              


;******************************************************************************
;** Procedure: _processData
;**
;**  Arguments: None
;**
;**  Returns: None
;**
;**  Global Variables used: 
;**
;**  Subroutines: None
;**
;**  Description: This function processes the data transfer header written to
;**               by the host through the HPI interface.  If a transfer is
;**               requested by the presence of a non-zero count, that transfer
;**               is executed and the count is then cleared to signal to the
;**               host that the transfer is complete.
;**
;**  Memory Transfer Format: (64 words total)
;**               HDR_ADDR      = Transfer Count (2 words)
;**               HDR_ADDR+2    = Source/Destination Address (2 words)
;**               HDR_ADDR+4	= Memory Transfer Select (settings are below)
;**               HDR_ADDR+6    = Data Buffer to Transfer (58 words)
;**
;**               Memory Transfer Select Options:
;**               0 - Write memory
;**               1 - Read memory/flash
;**               2 - Write Flash Memory
;**               3 - Erase Flash Memory
;**               4 - Reset Flash Memory
;**
;**               AR1 = Transfer Addresss
;**               AR2 = Data Address
;**               AR3 = Header Address
;**               CSR = Count - 1
;******************************************************************************
        .text

_processData:

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ;;   Check to see if a new transfer request is present
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        xar3 = #HDR_ADDR                                ;ar3 points to the xfer header
        
        ac0 = dbl(*ar3)                                 ;load the data transfer count
        if (ac0 == #0) return;                          ;done if no transfer count present

        *(#0x600000) |= #1                              ;turn on LED0
        
        t0 = ac0 - #1
        csr = t0                                        ;counter for repeat single

        xar1 = dbl(*ar3(#2))                            ;preserve the transfer address (b0-b23) in ar1

        ac0 = dbl(*ar3(#4))                             ;get the memory transfer select

        if (ac0 == #0) goto _dataWrite			;call data write handler if DATA_WRITE?

        ac0 -= #1                                       ;Is this a memory/flash transfer?
        if (ac0 == #0) goto _dataRead			;call data read handler if DATA_READ?

        ac0 -= #1                                       ;Is this a flash transfer?
        if (ac0 == #0) goto _flashWrite			;call data write handler if FLASH_WRITE?

        ac0 -= #1                                       ;Is this a flash erase?
        if (ac0 == #0) goto _flashErase			;call flash erase handler if FLASH_ERASE?

        ac0 -= #1                                       ;Is this a flash erase?
        if (ac0 == #0) goto _flashReset			;call flash erase handler if FLASH_ERASE?

        ac0 -= #1    							
        if (ac0 == #0) goto _dataDone			;Done if transfer select is invalid

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ;;   Data space transfer handler (configure CPLD and then xfer data)
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_dataWrite:

        xar2 = #DATA_ADDR

        repeat(csr)                             ; move data from off-chip to on-chip
        *ar1+ = *ar2+

        goto _dataDone


   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ;;   Data space read handler (configure CPLD and then read data)
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_dataRead:

        xar2 = #DATA_ADDR

        repeat(csr)                             ; move data from off-chip to on-chip
        *ar2+ = *ar1+

        goto _dataDone


   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ;;   Flash memory transfer handler
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_flashWrite:

        xar0 = mar(*(#DATA_ADDR))               ;1st arg is the src address
        xar1 = dbl(*ar3(#2))                    ;2st arg is the dst address
        ac0  = *ar3(#1)                         ;3rd arg is the count (64K max)

        call _flash_write                       ;Transfer the HPI buffer to Flash
		
        goto _dataDone

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ;;   Flash memory erase handler
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_flashErase:

        t0 = ar1                                        ;1st arg is the sector number

        call _flash_erase				;Erase all of the flash memory

        goto _dataDone

_flashReset:

        call _flash_reset  				;Reset the flash device

_dataDone:
        *(#HDR_ADDR) = #0                               ;Clear count to signal xfer complete
        *(#(HDR_ADDR+1)) = #0        
        return


;******************************************************************************
;** Procedure: _delay
;**
;** Arguments: None
;**
;** Returns: None
;**
;** Global Variables used: None
;**
;** Subroutines: executeCmd
;**
;** Description: This function is a simple time delay which uses code rather
;**              than a timer.  The delay implemented by this routine is
;**              1 millisecond on a 100MHz internal clock.
;**
;******************************************************************************
        .text
; delay (unsigned short cnt)

_delay:

        brc0 = t0                        ;Set repeat block count

        blockrepeat {
             repeat(#997)                ;Nested loop to cut down code size
             nop                         ;Fill instruction for delay loop
        }

        return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Global Variable Declarations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        .data 
        

;******************************************************************************
;******************************************************************************
        .end

