/******************************************************************************/
/******************************************************************************/
/*  aic27.c - TLV320AIC27 Audio Codec Driver Routines                         */
/*                                                                            */
/*   This module is the device driver library for the TLV320AIC27 Audio Codec */
/*   on the 5510 EVM board.                                                   */
/*                                                                            */
/*  FUNCTIONS:                                                                */
/*                                                                            */
/*    aic27_open() - Open codec connection to McBSP with default reg values   */
/*    aic27_close() - Close codec connection to McBSP                         */
/*    aic27_init() - Initializes AIC27 control registers.                     */
/*    aic27_reset() - Reset the codec to the default configuration.           */
/*    aic27_read_reg() - Read an AIC27 control register                       */
/*    aic27_write_reg() - Write an AIC27 control register                     */
/*    aic27_sample_rate() - Configures the specified channel's sampling rate  */
/*    aic27_cont_play() - Sets continuous play (D/A) mode                     */
/*    aic27_cont_capture() - Sets continuous capture (A/D) mode               */
/*    aic27_stop_play() - Stops sending data to AIC27                         */
/*    aic27_stop_capture() - Stops receiving data from AIC27                  */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/* INCLUDES                                                                   */
/******************************************************************************/
//EVM Driver Include Files
#include  <stddef.h>
#include  <type.h>          //typedefs
#include  <aic27.h>         //AIC27 Codec public header file
#include  <board.h>         //board definitions
#include <regs55x.h>
#include <mcbsp55x.h>
#include <dma55x.h>
#include <intr.h>

/******************************************************************************/
/* DEFINITIONS                                                                */
/******************************************************************************/
//AIC27 timeslot valid bits
#define AIC27_CODEC_READY   	0x8000ul
#define AIC27_VALID_FRAME   	0x8000ul
#define AIC27_VALID_REGADDR_CH	0x4000ul //SLOT 1
#define AIC27_VALID_REGDATA_CH	0x2000ul //SLOT 2
#define AIC27_LEFT_CH			0x1000ul //Play data to left channel only
#define AIC27_RIGHT_CH   		0x0800ul //Play data to Right channel only
#define AIC27_BOTH_CH	 		0x1800ul //Play data to both channels
#define AIC27_DATA_REQ          0x0C00ul //On-demand data requests bit for channels 3 & 4
#define AIC27_SLOT3_REQ         0x0800ul //On-demand data request bit for channel 3
#define AIC27_SLOT4_REQ         0x0400ul //On-demand data request bit for channel 4

#define REG_READ_REQUEST        0x80000ul // b19 in composite 20-bit word

//DMA settings
#define AIC27TXCH				4		//DMA Ch 4
#define AIC27_TXDMA_FLAG		DMAC4
#define AIC27_TXDMA_TRAP		DMAC4_TRAP

#define AIC27RXCH				5		//DMA Ch 5
#define AIC27_RXDMA_FLAG		DMAC5
#define AIC27_RXDMA_TRAP		DMAC5_TRAP

#define DMA_BUFSIZE				5

//McBsp setting for AC'97 compliant codec in dual mode
#define WD_PER_FRAME_1			0x0
#define WD_PER_FRAME_4			0x3

#define RCR1_VAL				((WD_PER_FRAME_1 << 8) | (WORD_LENGTH_16 << 5))

#define RCR2_VAL 				((DUAL_PHASE        << 15) |\
								(WD_PER_FRAME_4     << 8)  |\
								(WORD_LENGTH_20     << 5)  |\
								(NO_COMPAND_MSB_1ST << 3)  |\
								(NO_FRAME_IGNORE    << 2)  |\
								(DATA_DELAY1        << 0))

#define XCR1_VAL				((WD_PER_FRAME_1 << 8) | (WORD_LENGTH_16 << 5))

#define XCR2_VAL				((DUAL_PHASE        << 15) |\
								(WD_PER_FRAME_4     << 8)  |\
								(WORD_LENGTH_20     << 5)  |\
								(NO_COMPAND_MSB_1ST << 3)  |\
								(NO_FRAME_IGNORE    << 2)  |\
								(DATA_DELAY1        << 0))

#define PCR_VAL					((FSYNC_POL_HIGH<<3)|(FSYNC_POL_HIGH<<2)|(CLKX_POL_RISING<<1)|(CLKR_POL_FALLING))

#define SPCR1_VAL               0
#define SPCR2_VAL               0

#define SRGR1_VAL               0
#define SRGR2_VAL               0

/******************************************************************************/
/* ENUMs                                                                      */
/******************************************************************************/
/*****************************************************************************/
/* AIC27 state                                                               */
/*****************************************************************************/
typedef enum
{
   AIC27_CODEC_CLOSED    = 0,
   AIC27_CODEC_OPENED    = 1
} AIC27State, *PAIC27State;

/******************************************************************************/
/* AIC27 codec structure  													  */
/******************************************************************************/
typedef struct _AIC27packet
{ 
    s16        status;        //complete (1) or not complete (0)
    u16        size;          //buffer size (64K max)
    u16        indx;          //sample count
    bool       bufi;          //selects ping-pong buffer
    s16        *pBuf[2];      //pointers to ping-pong buffers
    Fp         callback;      //callback function
} AIC27Packet, *PAIC27Packet;

typedef struct _AIC27Codec
{ 
    AIC27State state;		  //codec state
    AIC27Packet rxPkt;        //receive packet
    AIC27Packet txPkt;        //transmit packet
} AIC27Codec, *PAIC27Codec;

typedef struct _AIC27Reg
{
	u32  tag;
    u32  addr;       //20-bit reg address (slot 0)
    u32  data;       //20-bit reg value   (slot 1)
} AIC27Prog, *PAIC27Prog;

typedef struct _AIC27DmaStruct
{
	u32  tag;
    u32  addr;
    u32  data;
	u32  leftCh;
	u32  rightCh;
} AIC27DmaStruct, *PAIC27DmaStruct;

/******************************************************************************/
/* GLOBAL VARIABLES                                                           */
/******************************************************************************/
#pragma DATA_ALIGN(dmaTxBuf, 2 /*4*/)
#pragma DATA_ALIGN(dmaRxBuf, 2 /*4*/)

//pointer to DMA buffers allocated in ASM
volatile AIC27DmaStruct dmaTxBuf;
volatile AIC27DmaStruct dmaRxBuf;

volatile AIC27Prog aic27Status = {0,0,0};
volatile AIC27Prog aic27Cmd = {(u32)AIC27_VALID_FRAME,0,0};
/******************************************************************************/
/* FILE LOCAL (STATIC) VARIABLES                                              */
/******************************************************************************/
static volatile AIC27Codec cdx = {AIC27_CODEC_CLOSED, 0, 0};

static volatile bool writeDone = True;
static volatile bool readDone = True;

static volatile bool rx_stop = False;
static volatile bool tx_stop = False;

/******************************************************************************/
/* FILE LOCAL (STATIC) PROTOTYPES                                             */
/******************************************************************************/
static void _serial_reset();
static void _setup_aic27_serial_port();
static void _reset_aic27_pkts();
static void _setup_aic27_registers();
static void _setup_aic27_dma();
static void _enable_aic27_interrupt();
static void _disable_aic27_interrupt();
static void enable_codec_comm();
static void disable_codec_comm();
static void reset_dma();
static u16 setMemSpace(u32 addr);

// McBSP Interrupt Handler
interrupt void _aic27_rxdma_isr(void);

/******************************************************************************/
/* aic27_open() - Open AIC27 codec and connect codec to McBSP.                */
/*																			  */
/*                                                                            */
/*    Parameters - None                                                       */
/*                                                                            */
/*    Returns - BRD_OK on success, BRD_ERROR on failure.                      */
/*                                                                            */
/******************************************************************************/
int aic27_open()
{
    // If codec is already open return BRD_OK
    if (cdx.state == AIC27_CODEC_OPENED)
		return BRD_OK;

    // Set flag to opened
    cdx.state = AIC27_CODEC_OPENED;
		
    // Initialize AIC27
    aic27_init();
	
    // Return true
    return BRD_OK;
}

/******************************************************************************/
/* aic27_close() - Closes codec connection									  */
/*                                                                            */
/*    Parameters - None                                                       */
/*                                                                            */
/*    Returns - BRD_OK on success, BRD_ERROR on failure.                      */
/*                                                                            */
/******************************************************************************/
int aic27_close()
{
    // If codec not opened, then just return
    if (cdx.state == AIC27_CODEC_CLOSED)
		return BRD_OK;
	
    // Disable codec interrupt
	disable_codec_comm();

    cdx.state = AIC27_CODEC_CLOSED;
	
    return BRD_OK;
}

/******************************************************************************/
/* aic27_init() - Initialize AIC27 codec by setting up the registers          */
/*                                                                            */
/*    Parameters - None                                                       */
/*                                                                            */
/*    Returns - None                                                          */
/*                                                                            */
/******************************************************************************/
void aic27_init()
{
	rx_stop = tx_stop = False;

    //Enable codec communications
    enable_codec_comm();

	//Setup AIC27 codec registers
	_setup_aic27_registers();
}

/******************************************************************************/
/* aic27_reset() - Reset the codec to the default configuration.              */
/*                                                                            */
/*   This function resets the AIC27 codec by sending it a software reset.     */
/*   All registers are configured to their default values.                    */
/*                                                                            */
/*    Parameters - None                                                       */
/*                                                                            */
/*    Returns - BRD_OK on success, BRD_ERROR on failure.                      */
/*                                                                            */
/******************************************************************************/
int aic27_reset()
{
	//reset aic27 codec by writing any value to register the reset reg (addr 0x00)
	if (aic27_write_reg(RESET, 0xffff, 0xffff) == (u16)BRD_ERROR)
		return BRD_ERROR;

	disable_codec_comm();

    cdx.state = AIC27_CODEC_OPENED;

    return BRD_OK;
}

/******************************************************************************/
/*    aic27_read_reg() - Read an AIC27 control register                       */
/*                                                                            */
/*    Parameters -                                                            */
/*     reg - an element of an enumeration indicating the register to read.    */
/*                                                                            */
/*    Returns - Content of the indicated register                             */
/*                                                                            */
/******************************************************************************/
u16 aic27_read_reg(AIC27Reg reg)
{
    if (cdx.state == AIC27_CODEC_CLOSED)
		return (u16)BRD_ERROR;

    //disable receive DMA interrupt
    IER0 &= (u16)(~(1<<AIC27_RXDMA_FLAG));

    //assemble cmd addr
	//b19 in composite 20-bit word is 1
	//aic27Cmd.addrL = ((reg & 0xe) << 12);
	//aic27Cmd.addrH = REG_READ_REQUEST | ((reg & 0x70) >> 4);
    aic27Cmd.addr = REG_READ_REQUEST | ((u32)((u32)reg & 0x7e) << 12);
	
    //data must be 0 for read
    aic27Cmd.data = 0;

    //set tag for next output frame (indicate addr/data slots are valid)
    aic27Cmd.tag |= (AIC27_VALID_REGADDR_CH | AIC27_VALID_REGDATA_CH);
	aic27Status.tag &= ~(AIC27_VALID_REGADDR_CH | AIC27_VALID_REGDATA_CH);

    //clear read flag
    readDone = False;

    //enable receive DMA interrupt
    IER0 |= (u16)(1<<AIC27_RXDMA_FLAG);

    //wait until register data is available
    while (readDone == False);

    // return value of register
    return (u16)(aic27Status.data >> 4);
}

/******************************************************************************/
/*    aic27_write_reg() - Write an AIC27 control register                     */
/*                                                                            */
/*    Parameters -                                                            */
/*     reg - an element of an enumeration indicating the register to write.   */
/*     value - a number containing the bits to by written, already in their   */
/*             correct field locations.                                       */
/*     mask - a number indicating the position of the bits in "value" to be   */
/*            written.                                                        */
/*                                                                            */
/*    Returns - BRD_OK on success, BRD_ERROR on failure.                      */
/*                                                                            */
/******************************************************************************/
u16 aic27_write_reg(AIC27Reg reg, u16 value, u16 mask)
{
    // If codec closed, just return
    if (cdx.state == AIC27_CODEC_CLOSED)
		return (u16)BRD_ERROR;

	// Mask register and get write value
    if (mask != 0xFFFF)
    {
       value |= (aic27_read_reg(reg) & ~mask);
    }

    //disable receive DMA interrupt
    IER0 &= (u16)(~(1<<AIC27_RXDMA_FLAG));

    //assemble cmd addr
	//b19 in composite 20-bit word is 0
	//aic27Cmd.addrL = ((reg & 0xe) << 12);
	//aic27Cmd.addrH = ((reg & 0x70) >> 4);
	aic27Cmd.addr = ((u32)((u32)reg & 0x7e) << 12);
	
	//assemble cmd data
	aic27Cmd.data = value;
	aic27Cmd.data <<= 4;

    //set tag to indicate valid addr/data on next frame
    aic27Cmd.tag |= (AIC27_VALID_REGADDR_CH | AIC27_VALID_REGDATA_CH);

    //clear write flag
    writeDone = False;

    //enable receive DMA interrupt
    IER0 |= (u16)(1<<AIC27_RXDMA_FLAG);

    // Wait for write to complete
    while (writeDone == False);

	return (u16)BRD_OK;
}

/******************************************************************************/
/*    aic27_sample_rate() - Set the specified channel sample rate.            */
/*                                                                            */
/*    Parameters -                                                            */
/*     reg - an element of an enumeration indicating the pertinent register.  */
/*     freq - an element of an enumeration indicating the frequency.          */
/*                                                                            */
/*    Returns - BRD_OK on success, BRD_ERROR on failure.                      */
/*                                                                            */
/******************************************************************************/
int aic27_sample_rate(AIC27Reg reg, u16 freq)
{
	if ((reg == FRONT_DAC_SRATE) ||
		(reg == REAR_DAC_SRATE)  ||
		(reg == LFE_DAC_SRATE)   ||
		(reg == AUDIO_ADC_RRATE) ||
		(reg == LINE1_SRATE)     ||
		(reg == LINE2_SRATE))
	{
        return aic27_write_reg(reg, freq, 0xFFFF);
	}

	return BRD_ERROR;
}
/******************************************************************************/
/* aic27_cont_play()                                                          */
/*                                                                            */
/*    This function is used to continuously send data to the codec.           */
/*                                                                            */
/*    Parameters -                                                            */
/*         pingBuf - a buffer containing data to play                         */
/*         pongBuf - a second buffer containing data to play                  */
/*         size - the number of samples in each buffer                        */
/*         pfunc - Pointer to function to run at end of buffer                */
/*         bufStatus - pointer to transfer status.                            */
/*                     1 = transfer in progress, 0 = transfer complete        */
/*                                                                            */
/*    Returns - BRD_OK on success, BRD_ERROR on failure.                      */
/*                                                                            */
/******************************************************************************/
int aic27_cont_play(int *pingBuf,
                    int *pongBuf,
                    unsigned int size,
                    Fp pfunc,
                    int **bufStatus)
{
    // Codec must be opened first
    if (cdx.state == AIC27_CODEC_CLOSED)
        return BRD_ERROR;
	
    //disable receive DMA interrupt
    IER0 &= (u16)(~(1<<AIC27_RXDMA_FLAG));
	
    // Setup transmit pack
    cdx.txPkt.size = size;
    cdx.txPkt.indx = 0;
    cdx.txPkt.bufi = (bool)0;
    *bufStatus = (int *)&cdx.txPkt.status;
    cdx.txPkt.status = 1;
    cdx.txPkt.callback = pfunc;
    cdx.txPkt.pBuf[0] = pingBuf;
    cdx.txPkt.pBuf[1] = pongBuf;
	
    //enable receive DMA interrupt
    IER0 |= (u16)(1<<AIC27_RXDMA_FLAG);
	
    return BRD_OK;
}

/******************************************************************************/
/* aic27_cont_capture()                                                       */
/*                                                                            */
/*    This function is used to capture data from the codec                    */
/*                                                                            */
/*    Parameters -                                                            */
/*         pingBuf - a buffer to contain data read                            */
/*         pongBuf - a second buffer to contain data read                     */
/*         size - the number of samples in each buffer                        */
/*         pfunc - Pointer to function to run at end of buffer                */
/*         bufStatus - pointer to transfer status.                            */
/*                     1 = transfer in progress, 0 = transfer complete        */
/*                                                                            */
/*    Returns - BRD_OK on success, BRD_ERROR on failure.                      */
/*                                                                            */
/*                                                                            */
/******************************************************************************/
int aic27_cont_capture(int *pingBuf,
                       int *pongBuf,
                       unsigned int size,
                       Fp pfunc,
                       int **bufStatus)
{
    // Codec must be opened first
    if (cdx.state == AIC27_CODEC_CLOSED)
        return BRD_ERROR;
	
    //disable receive DMA interrupt
    IER0 &= (u16)(~(1<<AIC27_RXDMA_FLAG));
    
    /* Setup receive packet */
    cdx.rxPkt.size = size;
    cdx.rxPkt.indx = 0;
    cdx.rxPkt.bufi = (bool)0;
    *bufStatus = (int *)&cdx.rxPkt.status;
    cdx.rxPkt.status = 1;
    cdx.rxPkt.callback = pfunc;
    cdx.rxPkt.pBuf[0] = pingBuf;
    cdx.rxPkt.pBuf[1] = pongBuf;
	
    //enable receive DMA interrupt
    IER0 |= (u16)(1<<AIC27_RXDMA_FLAG);
	
    return BRD_OK;
}

/******************************************************************************/
/*    aic27_stop_play() - Stops sending data to AIC27                         */
/*                                                                            */
/*    Parameters -  None                                                      */
/*                                                                            */
/*    Returns - None                                                          */
/*                                                                            */
/******************************************************************************/
void aic27_stop_play()
{
    cdx.txPkt.callback = 0;
    cdx.txPkt.pBuf[0] = NULL;
    cdx.txPkt.pBuf[1] = NULL;
}

/******************************************************************************/
/*    aic27_stop_capture() - Stops capturing data from AIC27.                 */
/*                                                                            */
/*    Parameters - None                                                       */
/*                                                                            */
/*    Returns -  None                                                         */
/*                                                                            */
/******************************************************************************/
void aic27_stop_capture()
{
    cdx.rxPkt.status = 1;
    cdx.rxPkt.callback = 0;
    cdx.rxPkt.pBuf[0] = NULL;
    cdx.rxPkt.pBuf[1] = NULL;
}

/******************************************************************************/
/* _reset_aic27_pkts() - reset codec packets								  */
/******************************************************************************/
static void _reset_aic27_pkts()
{
	//reset AIC27 input and output frames
	aic27Status.tag = 0;
    aic27Cmd.tag = 0;
	aic27Status.addr = aic27Cmd.addr = 0;
	aic27Status.data = aic27Cmd.data = 0;
	
    // Initialize buffer pointers and callback function
    cdx.rxPkt.size = 0;
    cdx.rxPkt.indx = 0;
    cdx.rxPkt.bufi = (bool)0;
    cdx.rxPkt.pBuf[0]  = NULL;
    cdx.rxPkt.pBuf[1]  = NULL;
    cdx.rxPkt.callback = NULL;
	
    cdx.txPkt.size = 0;
    cdx.txPkt.indx = 0;
    cdx.txPkt.bufi = (bool)0;
    cdx.txPkt.pBuf[0]  = NULL;
    cdx.txPkt.pBuf[1]  = NULL;
    cdx.txPkt.callback = NULL;

	dmaTxBuf.tag = dmaRxBuf.tag = 0;
	dmaTxBuf.data = dmaRxBuf.data = 0;
	dmaTxBuf.addr = dmaRxBuf.addr = 0;
	dmaTxBuf.leftCh = dmaRxBuf.leftCh = 0;
	dmaTxBuf.rightCh = dmaRxBuf.rightCh = 0;
}

/******************************************************************************/
/* _enable_aic27_interrupt() - RX interrupt for AIC27 codec                   */
/******************************************************************************/
static void _enable_aic27_interrupt()
{
    // Clear any pending interrupt
    IFR0 |= (u16)(1<<AIC27_RXDMA_FLAG);

    // Hook and enable McBSP receive interrupt
    hook_interrupt(AIC27_RXDMA_TRAP, _aic27_rxdma_isr);
    IER0 |= (u16)(1<<AIC27_RXDMA_FLAG);

    // Make sure global interrupts are enabled
    INTR_GLOBAL_ENABLE;
}

/******************************************************************************/
/* _disable_aic27_interrupt() - disable RX interrupt for AIC27 codec          */
/******************************************************************************/
static void _disable_aic27_interrupt()
{
    // Disable interrupt
    IER0 &= (u16)(~(1<<AIC27_RXDMA_FLAG));

    // Unhook interrupt handler
    unhook_interrupt(AIC27_RXDMA_TRAP);
}

/******************************************************************************/
/* _setup_aic27_dma() - setup DMA ch 4 & 5 for AIC27 codec                    */
/* TX - Ch 4, RX - Ch 5                                                       */
/******************************************************************************/
static void _setup_aic27_dma()
{
    u32 addr = 0;
    u16 mSpace = 0;

    DMGCR = 0x0000; //initialize gcr

    // clear all DMA registers
    reset_dma();

    /*****************************************************************/
    /* Transmit DMA setup                                            */
    /*****************************************************************/
    DMCCR(AIC27TXCH)  = (DM_NOMOD     << DM_DST_AMODE)  | 
                        (DM_POST_INCR << DM_SRC_AMODE)  | 
                        (1 << DM_AUTOINIT)              |  // Requires ENDPROG
                        (1 << DM_PRIO)                  | 
                        (DMSYNC_XEVT2 << DM_SYNC);

    // Determine memory type of source data
    mSpace = setMemSpace((u32)&dmaTxBuf);

    DMCSDP(AIC27TXCH)  = (DM_RHEA << DM_DST) |
                         (mSpace << DM_SRC)  |
                         (DM_DTYPE_32 << DM_DATA_TYPE);

    // Transmit interrupt is not used (only receive)
    DMCICR(AIC27TXCH)  = 0;

    // set src address (byte address)
    addr = ((u32)&dmaTxBuf) << 1;
    DMCSSAL(AIC27TXCH) = (u16)addr;
    DMCSSAU(AIC27TXCH) = (u16)((addr & 0xff0000)>>16);

    // set dst address (byte address)
    addr = ((u32)DXR2_ADDR(AIC27_PORT)) << 1;
    DMCDSAL(AIC27TXCH) = (u16)addr;
    DMCDSAU(AIC27TXCH) = (u16)((addr & 0xff0000)>>16);
    
    DMCEN(AIC27TXCH)   = DMA_BUFSIZE;
    DMCFN(AIC27TXCH)   = 1;
    DMCEI(AIC27TXCH)   = 0;
    DMCFI(AIC27TXCH)   = 0;

    /*****************************************************************/
    // Receive DMA setup
    /*****************************************************************/
    DMCCR(AIC27RXCH)  = (DM_POST_INCR << DM_DST_AMODE)  | 
                        (DM_NOMOD << DM_SRC_AMODE)      | 
                        (1 << DM_REPEAT)                | 
                        (1 << DM_AUTOINIT)              | 
                        (1 << DM_PRIO)                  | 
                        (DMSYNC_REVT2 << DM_SYNC);

    // Determine memory type of destination
    mSpace = setMemSpace((u32)&dmaRxBuf);

    DMCSDP(AIC27RXCH)  = (mSpace << DM_DST)  |
                         (DM_RHEA << DM_SRC) |
                         (DM_DTYPE_32 << DM_DATA_TYPE);

    DMCICR(AIC27RXCH)  = (1 << DM_FRAME);

    // set src address
    addr = ((u32)DRR2_ADDR(AIC27_PORT)) << 1;
    DMCSSAL(AIC27RXCH) = (u16)addr;
    DMCSSAU(AIC27RXCH) = (u16)((addr & 0xff0000)>>16);

    // set dst address
    addr = ((u32)&dmaRxBuf) << 1;
    DMCDSAL(AIC27RXCH) = (u16)addr;
    DMCDSAU(AIC27RXCH) = (u16)((addr & 0xff0000)>>16);
    
    DMCEN(AIC27RXCH)   = DMA_BUFSIZE;
    DMCFN(AIC27RXCH)   = 1;
    DMCEI(AIC27RXCH)   = 0;
    DMCFI(AIC27RXCH)   = 0;
}

static void reset_dma()
{
    //Stop any current transfers
    DMCCR(AIC27TXCH)   = 0;
    DMCCR(AIC27RXCH)   = 0;

    //reset Tx DMA ch
    DMCSDP(AIC27TXCH)  = 0;
    DMCICR(AIC27TXCH)  = 0;

    DMCSSAL(AIC27TXCH) = 0;
    DMCSSAU(AIC27TXCH) = 0;
    DMCDSAL(AIC27TXCH) = 0;
    DMCDSAU(AIC27TXCH) = 0;
    
    DMCEN(AIC27TXCH)   = 0;
    DMCFN(AIC27TXCH)   = 0;
    DMCEI(AIC27TXCH)   = 0;
    DMCFI(AIC27TXCH)   = 0;

    //reset Rx DMA ch
    DMCSDP(AIC27RXCH)  = 0;
    DMCICR(AIC27RXCH)  = 0;

    DMCSSAL(AIC27RXCH) = 0;
    DMCSSAU(AIC27RXCH) = 0;
    DMCDSAL(AIC27RXCH) = 0;
    DMCDSAU(AIC27RXCH) = 0;
    
    DMCEN(AIC27RXCH)   = 0;
    DMCFN(AIC27RXCH)   = 0;
    DMCEI(AIC27RXCH)   = 0;
    DMCFI(AIC27RXCH)   = 0;
}

/******************************************************************************/
/* _setup_aic27_serial_port() - setup serial port for AIC27 codec             */
/******************************************************************************/
static void _setup_aic27_serial_port()
{
    SPCR1(AIC27_PORT) = SPCR1_VAL;
    SPCR2(AIC27_PORT) = SPCR2_VAL;

    RCR1(AIC27_PORT)  = RCR1_VAL;
    RCR2(AIC27_PORT)  = RCR2_VAL;

    XCR1(AIC27_PORT)  = XCR1_VAL;
    XCR2(AIC27_PORT)  = XCR2_VAL;

    SRGR1(AIC27_PORT) = 0;
    SRGR2(AIC27_PORT) = 0;

    MCR1(AIC27_PORT)  = 0;
    MCR2(AIC27_PORT)  = 0;

    PCR(AIC27_PORT)   = PCR_VAL; 
    
    RCERA(AIC27_PORT) = 0;
    RCERB(AIC27_PORT) = 0;
    RCERC(AIC27_PORT) = 0;
    RCERD(AIC27_PORT) = 0;
    RCERE(AIC27_PORT) = 0;
    RCERF(AIC27_PORT) = 0;
    RCERG(AIC27_PORT) = 0;
    RCERH(AIC27_PORT) = 0;
    
    XCERA(AIC27_PORT) = 0;
    XCERB(AIC27_PORT) = 0;
    XCERC(AIC27_PORT) = 0;
    XCERD(AIC27_PORT) = 0;
    XCERE(AIC27_PORT) = 0;
    XCERF(AIC27_PORT) = 0;
    XCERG(AIC27_PORT) = 0;
    XCERH(AIC27_PORT) = 0;
    
    readDone = writeDone = True;

    //now start McBsp and DMA transfers
    DMA_ENABLE(AIC27RXCH);
    DMA_ENABLE(AIC27TXCH);

    // Enable serial port (both TX/RX)
    MCBSP_ENABLE(AIC27_PORT, MCBSP_BOTH);

    //wait until codec is ready (b15 of slot 1 must be '1')
    while (!(aic27Status.tag & AIC27_CODEC_READY));
}


/******************************************************************************/
/* _serial_reset() - Resets the device's serial port                          */
/******************************************************************************/
static void _serial_reset()
{
    // Reset McBSP
    MCBSP_TX_RESET(AIC27_PORT);
    MCBSP_RX_RESET(AIC27_PORT);
    
    // Disable interrupts
    IER0 &= (u16)(~(1<<AIC27_RXDMA_FLAG));

    // Clear pending interrupt flags
    IFR0 |= (u16)(1<<AIC27_RXDMA_FLAG);
}

/******************************************************************************/
/* _setup_aic27_registers() - Sets up the AIC27 codec registers               */
/******************************************************************************/
static void _setup_aic27_registers()
{
    //enable variable rate audio
    aic27_write_reg(EXTD_AUDIO_STATCTRL, 0x0001, 0xffff);

    aic27_write_reg(MASTER_VOL, 0x0000, 0xffff);
    aic27_write_reg(MIC_VOL, 0x0000, 0xffff);
    aic27_write_reg(LINEIN_VOL, 0x0000, 0xffff);

    //set pcmout volume to about -13dB gain
    aic27_write_reg(PCMOUT_VOL, 0x0808, 0xffff);

    aic27_write_reg(REC_SEL, MIC_IN, 0xffff); //mic
    aic27_write_reg(REC_GAIN, 0x0000, 0xffff);

    //mute front mixer vol to disable loopback path
    aic27_write_reg(FRONT_MIXER_VOL, 0x8000, 0xffff);

    aic27_sample_rate(AUDIO_ADC_RRATE, AIC27_8000HZ);
    aic27_sample_rate(FRONT_DAC_SRATE, AIC27_8000HZ);
}

/******************************************************************************/
/* _aic27_rxdma_isr() - ISR for DMA ch 5 block interrupt                      */
/* NOTE: The transmit handling is done here also after the receive handling   */
/*       since rx/tx are synchronized and transmit must occur after receive   */
/*       to support dynamic sample rates.                                     */
/******************************************************************************/
interrupt void _aic27_rxdma_isr(void)
{
    volatile int dummy;
    volatile u32 slotReq;
    int *pRx = cdx.rxPkt.pBuf[cdx.rxPkt.bufi];
    int *pTx = cdx.txPkt.pBuf[cdx.txPkt.bufi];
    
    //clear interrupt conditions
    dummy = DMCSR(AIC27RXCH);
    dummy = DMCSR(AIC27TXCH);

//*********************************************************************/
//Receive Processing
//*********************************************************************/

    //*********************************************************************/
    //read 16-bit Input Frame TAG
    //*********************************************************************/
    aic27Status.tag = dmaRxBuf.tag;

    //*********************************************************************/
    //read 20-bit reg address
    //*********************************************************************/
    aic27Status.addr = dmaRxBuf.addr;

    //*********************************************************************/
    //read 20-bit reg value
    //*********************************************************************/
    aic27Status.data = dmaRxBuf.data;
    
    if (aic27Status.tag & AIC27_VALID_REGDATA_CH)
    {
        readDone = True;
        aic27Cmd.tag &= ~(AIC27_VALID_REGADDR_CH | AIC27_VALID_REGDATA_CH);
    }
    
    //*********************************************************************/
    //Only read PCM left channel into buffer if buffer address is non-zero
    //*********************************************************************/
    if ((pRx != NULL) && (aic27Status.tag & AIC27_LEFT_CH))
    {
        pRx[cdx.rxPkt.indx++] = (u16)(dmaRxBuf.leftCh >> 4);
    }
    
    //*********************************************************************/
    //Only read PCM Right channel into buffer if buffer address is non-zero
    //*********************************************************************/
    if ((pRx != NULL) && (aic27Status.tag & AIC27_RIGHT_CH))
    {
        pRx[cdx.rxPkt.indx++] = (u16)(dmaRxBuf.rightCh >> 4);
    }
    
    //*********************************************************************/
    //Ping-pong Rx buffers if necessary
    //*********************************************************************/
    if (pRx != NULL)
    {
        if (cdx.rxPkt.indx == cdx.rxPkt.size)
        {
            cdx.rxPkt.bufi ^= 1;  //swap buffers
            cdx.rxPkt.indx = 0;   //reset buffer index
            cdx.rxPkt.status = 0; //indicate transfer completion
            
            // Invoke callback routine if provided
            if (cdx.rxPkt.callback != NULL)
                (*cdx.rxPkt.callback)(); //invoke rx callback routine
        }
    }

//*********************************************************************/
//Transmit Processing
//*********************************************************************/

    //*********************************************************************/
    //write 16-bit output Frame TAG
    //*********************************************************************/
    slotReq = ((aic27Status.addr & AIC27_DATA_REQ) ^ AIC27_DATA_REQ) << 1;
    dmaTxBuf.tag = aic27Cmd.tag | slotReq;

    //if at least one slot valid, indicate valid frame
    if (dmaTxBuf.tag)
    {
        dmaTxBuf.tag |= AIC27_VALID_FRAME;
    }
    
    //*********************************************************************/
    //write 20-bit reg address
    //*********************************************************************/
    dmaTxBuf.addr = aic27Cmd.addr;
    
    //*********************************************************************/
    //write 20-bit reg value
    //*********************************************************************/
    dmaTxBuf.data = aic27Cmd.data;
    
    //clear cmd and data port
    aic27Cmd.tag &= 0;  // ~(AIC27_VALID_REGADDR_CH | AIC27_VALID_REGDATA_CH);
    aic27Cmd.data = aic27Cmd.addr = 0;
    writeDone = True;

    //*********************************************************************/
    //Only write PCM left channel from buffer if buffer address is non-zero
    //and AIC27 has requested data.
    //*********************************************************************/
    if ((pTx != NULL) && (!(aic27Status.addr & AIC27_SLOT3_REQ)))
    {
        //get left channel data and shift to left by 4 to fill 20-bit slot
        dmaTxBuf.leftCh = (u32)pTx[cdx.txPkt.indx++] << 4;
    }
    else
    {
        //no data was requested
        dmaTxBuf.leftCh = 0;
    }
    
    //*********************************************************************/
    //Only write PCM Right channel from buffer if buffer address is non-zero
    //and AIC27 has requested data.
    //*********************************************************************/
    if ((pTx != NULL) && (!(aic27Status.addr & AIC27_SLOT4_REQ)))
    {
        //get right channel data and shift to left by 4 to fill 20-bit slot
        dmaTxBuf.rightCh = (u32)pTx[cdx.txPkt.indx++] << 4;
    }
    else
    {
        dmaTxBuf.rightCh = 0x0;
    }

    // Allow TX DMA to auto-init now with valid tag data
    DMCCR(AIC27TXCH) |= (1<<DM_END_PROG);
    DMCCR(AIC27TXCH) &= (~(1<<DM_END_PROG));

    //*********************************************************************/
    //Ping-pong Tx buffers if necessary
    //*********************************************************************/
    if (pTx != NULL)
    {
        if (cdx.txPkt.indx == cdx.txPkt.size)
        {
            cdx.txPkt.bufi ^= 1;  //swap buffers
            cdx.txPkt.indx = 0;   //reset buffer index
            cdx.txPkt.status = 0; //indicate transfer completion
            
            // Invoke callback routine if provided
            if (cdx.txPkt.callback != NULL)
                (*cdx.txPkt.callback)(); //invoke tx callback routine
        }
    }
}

static void enable_codec_comm()
{
	_reset_aic27_pkts();

    _serial_reset();
	
	_setup_aic27_dma();

    _enable_aic27_interrupt();

    //Setup McBSP
    _setup_aic27_serial_port();
}

static void disable_codec_comm()
{
	rx_stop = tx_stop = True;

	dma_reset(AIC27TXCH); //reset Tx DMA ch
	dma_reset(AIC27RXCH); //reset Rx DMA ch

    _disable_aic27_interrupt();

	_reset_aic27_pkts();

    _serial_reset();

	cdx.state = AIC27_CODEC_OPENED;
}

static u16 setMemSpace(u32 addr)
{
	if (addr < 0x8000ul)
		return DM_DARAM;
	else if (addr < 0x28000ul)
		return DM_SARAM;
	else
		return DM_EMIF;
}
