/****************************************************************************/
/****************************************************************************/
/*  flash.c                                                                 */
/*                                                                          */
/*      This module is the device library of driver functions for the		*/
/*      for the on-board Flash memory                                       */
/*                                                                          */
/*  MACRO FUNCTIONS:                                                        */
/*                                                                          */
/*  FUNCTIONS:                                                              */
/*                                                                          */
/*  flash_reset(void)                                                       */
/*  flash_erase(u16 sector)                                                 */
/*  flash_write(u16 *source, u16 *dest, u32 length)                         */
/*  flash_read(u16 *source, u16 *dest, u32 length)                          */
/*  flash_write_32(u32 *source, u32 *dest, u32 length)                      */
/*  flash_read_32(u32 *source, u32 *dest, u32 length)                       */
/*  flash_checksum(u32 source, u32 length)                                  */
/****************************************************************************/

/****************************************************************************/
/* INCLUDES                                                                 */
/****************************************************************************/
//EVM Driver Include Files   
#include  "board.h"         // Include board header file            
#include  "flash.h"         // Flash public header file
							//  includes i/o space access macros

/******************************************************************************/
/* defines                                                                    */
/******************************************************************************/
#define FCP5            (FLASH_BASE_ADDR+(0x5555ul<<1))     //Flash command port 5
#define FCPA            (FLASH_BASE_ADDR+(0x2AAAul<<1))     //Flash command port A
#define	FCW5			0x55555555ul
#define FCWA            0xAAAAAAAAul
#define	FWORDWRITE		0xA0A0A0A0ul
#define	FRESET			0xF0F0F0F0ul
#define	FCHIPERASE0		0x80808080ul
#define	FCHIPERASE1		0x10101010ul
#define	FSECTORERASE0	0x80808080ul
#define	FSECTORERASE1	0x30303030ul
#define	FCHIPID			0x90909090ul

#define	DQ7				(1ul<<7)
#define	DQ6				(1ul<<6)
#define	DQ5				(1ul<<5)
#define	DQ3				(1ul<<3)
#define	DQ2				(1ul<<2)

#define	DQ7_32BIT		((DQ7<<16) | DQ7)
#define	DQ6_32BIT		((DQ6<<16) | DQ6)
#define	DQ5_32BIT		((DQ5<<16) | DQ5)
#define	DQ3_32BIT		((DQ3<<16) | DQ3)
#define	DQ2_32BIT		((DQ2<<16) | DQ2)

/******************************************************************************/
/*  Global Variables                                                          */
/******************************************************************************/
static u32 fcwa[2] = {(FCWA & 0xffff0000), (FCWA & 0xffff)};
static u32 fcw5[2] = {(FCW5 & 0xffff0000), (FCW5 & 0xffff)};
static u32 fwordwrite[2] = {(FWORDWRITE & 0xffff0000), (FWORDWRITE & 0xffff)};

/******************************************************************************/
/* FUNCTION PROTOTYPES                                                        */
/******************************************************************************/
u32* getBlockAddress(u16 sector);
int waitFor16bitCompletion(volatile u16 *s, u16 value);
int waitFor32bitCompletion(volatile u32 *s, u32 value);
int flash_erase_sector(u16 sector);
int chip_erase(void);

/******************************************************************************/
/* PUBLIC FUNCTIONS                                                           */
/******************************************************************************/
/****************************************************************************/
/*  void flash_reset()		         										*/
/*																			*/
/*  This function resets the flash device              .					*/
/*                                                                          */
/* Parameters:                                                              */
/*      none														        */
/*                                                                          */ 
/*  Return:                                                                 */
/*  - BRD_OK success                                                            */
/*  - BRD_ERROR failure                                                         */
/*                                                                          */ 
/*  Notes:                                                                  */
/*                                                                          */ 
/****************************************************************************/
void flash_reset()
{
	*(volatile u32 *)FLASH_BASE_ADDR = FRESET;

	brd_busy_delay(2);
}

/*****************************************************************************/
/*  int flash_erase(int sector)                                              */
/*                                                                           */ 
/*  This function erases one or more segments of Flash.                      */
/*                                                                           */
/*  Parameters:                                                              */
/*      int sector //from params 0-10 or -1 for ALL 						 */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - BRD_OK success                                                             */
/*  - BRD_ERROR failure                                                          */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
int flash_erase(u16 sector)
{
	int ret = BRD_ERROR;
	
    if (sector == 0xFFFFu)
	{				
		// -1 = erase all
        ret = chip_erase();
    } 
	else 
	{
        sector &= 0xF;
		
		if (sector < MAX_SECTOR)
		{
			ret = flash_erase_sector(sector);	//erase just one
		} 
	}
	
	return ret;
}

/****************************************************************************/
/*  int flash_write(u16 *source, u16 *dest, u32 length)                     */
/*                                                                          */
/*  This function writes a block of data into Flash memory. The Flash is    */
/*	addressed with a linear address scheme that addresses the words of the	*/
/*  Flash memory as an offset from the start of the Flash chip. In other    */
/*	words, a dest of 0 will result in a write to sector 0, page 0, address	*/
/*  as viewed by the CPU, will be 0x400000 (0x200000 word address).         */
/*                                                                          */
/*  Parameters:                                                             */
/*      u16 *source                                                         */
/*      u16 *dest                                                           */
/*      u32 length                                                          */
/*                                                                          */
/*  Return:                                                                 */
/*  - BRD_OK success                                                            */
/*  - BRD_ERROR failure                                                         */
/*                                                                          */
/*  Notes:                                                                  */
/*                                                                          */
/****************************************************************************/
int flash_write(u16 *source, u16 *dest, u32 length)
{
    u16 *pdst = dest;
    u16 *psrc = source;
    u32 i,len = length;

	for(i = 0; i < len; i++) 
	{
        /* NOTE: Writes below use masks to only put one of Flash devices */
        /*       in program mode.                                        */
        *(u32 *)FCP5 = fcwa[((u32)pdst) & 1];
		
		*(u32 *)FCPA = fcw5[((u32)pdst) & 1];
		
		*(u32 *)FCP5 = fwordwrite[((u32)pdst) & 1];
		
		*pdst = *psrc;
		
        if (waitFor16bitCompletion((volatile u16 *)pdst++, *psrc++) == BRD_ERROR)
			return BRD_ERROR;
	}
    return BRD_OK;
}

/****************************************************************************/
/*  int flash_read(u16 *source, u16 *dest, u32 length)                      */
/*                                                                          */
/*  This function reads a block of data from Flash memory. The addressing   */
/*	is the same as in the functions above.					                */
/*                                                                          */
/*  Parameters:                                                             */
/*      u16 *source                                                         */
/*      u16 *dest                                                           */
/*      u32 length                                                          */
/*                                                                          */
/*  Return:                                                                 */
/*  - data is read from the Flash and written into the memory addressed by  */
/*		dest																*/
/*  - always returns the length of the block read.							*/
/*		there are no documented errors; however, it is possible to overrun	*/
/*      the border of the Flash page window. No checking is done at this    */
/*		time.		                                                        */
/*                                                                          */
/*  Notes:                                                                  */
/*                                                                          */
/****************************************************************************/
int flash_read(u16 *source, u16 *dest, u32 length)
{
	u32	i;
	
	for(i = 0; i < length; i++)
		dest[i] = source[i];
	
	return length;
}

/****************************************************************************/
/*  int flash_write_32(u32 *source, u32 *dest, u32 length)                  */
/*                                                                          */
/*  This function writes a block of data into Flash memory. The Flash is    */
/*	addressed with a linear address scheme that addresses the words of the	*/
/*  Flash memory as an offset from the start of the Flash chip. In other    */
/*	words, a dest of 0 will result in a write to sector 0, page 0, address	*/
/*  as viewed by the CPU, will be 0x400000 (0x200000 word address).         */
/*                                                                          */
/*  Parameters:                                                             */
/*		u32 *source															*/
/*		u32	*dest															*/
/*      u32 length                                                          */
/*                                                                          */
/*  Return:                                                                 */
/*  - BRD_OK success                                                            */
/*  - BRD_ERROR failure                                                         */
/*                                                                          */
/*  Notes:                                                                  */
/*                                                                          */
/****************************************************************************/
int flash_write_32(u32 *source, u32 *dest, u32 length)
{
    u32 *pdst = dest;
    u32 *psrc = source;
    u32 i,len = length;

	for(i = 0; i < len; i++) 
	{
        *(u32 *)FCP5 = FCWA;
		
        *(u32 *)FCPA = FCW5;
		
        *(u32 *)FCP5 = FWORDWRITE;
		
		*pdst = *psrc;
		
        if (waitFor32bitCompletion((volatile u32 *)pdst++, *psrc++) == BRD_ERROR)
			return BRD_ERROR;
	}
    return BRD_OK;
}

/****************************************************************************/
/*  int flash_read_32(u32 *source, u32 *dest, u32 length)                   */
/*                                                                          */
/*  This function reads a block of data from Flash memory. The addressing   */
/*	is the same as in the functions above.					                */
/*                                                                          */
/*  Parameters:                                                             */
/*      u32 *source                                                         */
/*      u32 *dest                                                           */
/*      u32 length                                                          */
/*                                                                          */
/*  Return:                                                                 */
/*  - data is read from the Flash and written into the memory addressed by  */
/*		dest																*/
/*  - always returns the length of the block read.							*/
/*		there are no documented errors; however, it is possible to overrun	*/
/*		the border of the flash page window. No checking is done at this	*/
/*		time.		                                                        */
/*                                                                          */
/*  Notes:                                                                  */
/*                                                                          */
/****************************************************************************/
int flash_read_32(u32 *source, u32 *dest, u32 length)
{
	u32	i;
	
	for(i = 0; i < length; i++)
		dest[i] = source[i];
	
	return length;
}

/****************************************************************************/
/*  int flash_checksum(u32 *source, u32 length)                             */
/*                                                                          */
/*  This function returns a checksum for the specified memory range         */
/*                                                                          */
/*  Parameters:                                                             */
/*      u32 *source;        //linear memory address within the flash        */
/*      u32 length;                                                         */
/*                                                                          */
/*  Return:                                                                 */
/*  - checksum value                                                        */
/*  - no errors documented                                                  */
/*                                                                          */
/*  Notes:                                                                  */
/*                                                                          */
/****************************************************************************/
u32 flash_checksum(u32 *source, u32 length)
{
    u32	cs, i;
    
    for(cs = 0, i = 0; i < length; i++)
        cs += source[i];
    
    return (cs);
}

/******************************************************************************/
/* PRIVATE FUNCTIONS                                                          */
/******************************************************************************/

/****************************************************************************/
/*  int waitFor16bitCompletion(volatile u16 *s, u16 value)   	       		*/
/*                                                                          */ 
/*  This function waits for the Flash operation to complete.                */
/*                                                                          */
/*  Parameters:                                                             */
/*      s - pointer to location to compare to value 					    */
/*      value - value to compare to data stored in location pointedt to by s*/
/*                                                                          */ 
/*  Return:                                                                 */
/*  - BRD_OK success                                                            */
/*  - BRD_ERROR failure                                                         */
/*                                                                          */ 
/*  Notes:                                                                  */
/*                                                                          */ 
/****************************************************************************/
int waitFor16bitCompletion(volatile u16 *s, u16 value)
{
	while (1)
	{
		if ((*s & DQ7) == (value & DQ7))
			return BRD_OK;
		
		if (*s & DQ5)
		{
			return ((*s & DQ7) == (value & DQ7)) ? BRD_OK : BRD_ERROR;
		}
	}
}

/****************************************************************************/
/*  int waitFor32bitCompletion(volatile u32 *s, u32 value)             		*/
/*                                                                          */ 
/*  This function waits for the flash operation to complete.                */
/*                                                                          */
/*  Parameters:                                                             */
/*      s - pointer to location to compare to value 					    */
/*      value - value to compare to data stored in location pointedt to by s*/
/*                                                                          */ 
/*  Return:                                                                 */
/*  - BRD_OK success                                                            */
/*  - BRD_ERROR failure                                                         */
/*                                                                          */ 
/*  Notes:                                                                  */
/*                                                                          */ 
/****************************************************************************/
int waitFor32bitCompletion(volatile u32 *s, u32 value)
{
	while (1)
	{
		if ((*s & DQ7_32BIT) == (value & DQ7_32BIT))
			return BRD_OK;
		
		if ((*s & DQ5_32BIT) == DQ5_32BIT)
		{
			return ((*s & DQ7_32BIT) == (value & DQ7_32BIT)) ? BRD_OK : BRD_ERROR;
		}
	}
}

/****************************************************************************/
/*  int flash_erase_sector(u16 sector)										*/
/*                                                                          */ 
/*  This function erases one sector of the Flash.                           */
/*                                                                          */
/*  Parameters:                                                             */
/*      int sector		//valid values are from 0-11	 					*/
/*                                                                          */ 
/*  Return:                                                                 */
/*  - BRD_OK success                                                            */
/*  - BRD_ERROR failure                                                         */
/*                                                                          */ 
/*  Notes:                                                                  */
/*                                                                          */ 
/****************************************************************************/
int flash_erase_sector(u16 sector)
{
	volatile u32 *s = (volatile u32 *)getBlockAddress(sector);

	//first command cycle
	*(u32 *)FCP5 = FCWA;
	*(u32 *)FCPA = FCW5;
	*(u32 *)FCP5 = FSECTORERASE0;

	//second command cycle
	*(u32 *)FCP5 = FCWA;
	*(u32 *)FCPA = FCW5;

    //write to first address in sector
    *s = FSECTORERASE1;

	//wait for command sequence to be accepted
	while ((*s & DQ7_32BIT) == DQ7_32BIT);

	//wait for erase cycle to begin
	while ((*s & DQ3) == 0x00000000);

    return waitFor32bitCompletion(s, 0xFFFFFFFF);
}

/****************************************************************************/
/*  u32* getBlockAddress(int sector)									    */
/*                                                                          */ 
/*  This function returns address of block to erase.		                */
/*                                                                          */
/*  Parameters:                                                             */
/*      int sector      //valid values are from 0-10                        */
/*                                                                          */ 
/*  Return:                                                                 */
/*  - address                                                               */
/*                                                                          */ 
/*  Notes:                                                                  */
/*                                                                          */ 
/****************************************************************************/
u32* getBlockAddress(u16 sector)
{
    //Default to sector 0 (even if invalid sector > 10)
    u32 base = FLASH_BASE_ADDR;

    //Get 32-bit address of sector
    if (sector < 8)
        base += ((u32)sector << 16);
    else if (sector == 8)
        base += 0x78000ul;
    else if (sector == 9)
        base += 0x7A000ul;
    else if (sector == 10)
        base += 0x7C000ul;

    return (u32 *)base;
}

/*****************************************************************************/
/*  int chip_erase()												         */
/*                                                                           */ 
/*  This function erases the entire Flash memory.                            */
/*                                                                           */
/*  Parameters:                                                              */
/*      none																 */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - BRD_OK success                                                             */
/*  - BRD_ERROR failure                                                          */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
int chip_erase(void)
{
	volatile u32 *s = (volatile u32 *)FLASH_BASE_ADDR;

	*(u32 *)FCP5 = FCWA;
	*(u32 *)FCPA = FCW5;
	*(u32 *)FCP5 = FCHIPERASE0;

	*(u32 *)FCP5 = FCWA;
	*(u32 *)FCPA = FCW5;
	*(u32 *)FCP5 = FCHIPERASE1;

	return waitFor32bitCompletion(s, 0xffffffff);
}
