/*****************************************************************************/
/*****************************************************************************/
/*  led.c - TMS320VC5510 EVM LED Support Library Functions                   */
/*                                                                           */
/*  This module provides the routines used to manipulate the board user LEDs */
/*                                                                           */
/*                                                                           */
/*  FUNCTIONS:                                                               */
/*                                                                           */
/*  brd_led_enable() - enables one of the three user LEDs                    */
/*  brd_led_disable() - disables one of the three user LEDs                  */
/*  brd_led_toggle() - togles one of the three user LEDs                     */
/*                                                                           */
/*  STATIC FUNCTIONS:                                                        */
/*                                                                           */
/*                                                                           */
/*  GLOBAL VARIABLES DEFINED                                                 */
/*                                                                           */
/*                                                                           */ 
/*****************************************************************************/

/*---------------------------------------------------------------------------*/
/* INCLUDES AND LOCAL DEFINES                                                */
/*---------------------------------------------------------------------------*/
#include <board.h>

/*---------------------------------------------------------------------------*/
/* typedefs                                                                  */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* DEFINES                                                                   */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* PRAGMAS                                                                   */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* GLOBAL VARIABLES                                                          */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* EXTERNAL FUNCTION PROTOTYPES                                              */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* FILE LOCAL (STATIC) VARIABLES                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* FILE LOCAL (STATIC) PROTOTYPES                                            */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* FUNCTIONS                                                                 */
/*---------------------------------------------------------------------------*/

/*****************************************************************************/
/*  s16 brd_led_disable(BrdLed led)                                          */
/*                                                                           */ 
/*  This routine disables the specified user-defined LED.                    */
/*                                                                           */
/*  Parameters:                                                              */
/*      - led - indicates which led to disable                               */
/*                                                                           */ 
/*  Return:                                                                  */
/*		BRD_OK on success                                                    */
/*		BRD_ERROR on failure                                                 */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_led_disable(BrdLed led)
{
    switch (led)
    {
    case BRD_LED0:
        CPLD_CTRL1_REG &= ~LED0_MASK;
        break;
        
    case BRD_LED1:
        CPLD_CTRL1_REG &= ~LED1_MASK;
        break;
        
    case BRD_LED2:
        CPLD_CTRL1_REG &= ~LED2_MASK;
        break;
        
    default:
        return BRD_ERROR;
    }
    
    return (BRD_OK);
}

/*****************************************************************************/
/*  s16 brd_led_enable(BrdLed led)                                           */
/*                                                                           */ 
/*  This routine enables specified user-defined LED.                         */
/*                                                                           */
/*  Parameters:                                                              */
/*      - led - indicates which led to enable                                */
/*                                                                           */ 
/*  Return:                                                                  */
/*		BRD_OK on success                                                    */
/*		BRD_ERROR on failure                                                 */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_led_enable(BrdLed led)
{
    switch (led)
    {
    case BRD_LED0:
        CPLD_CTRL1_REG |= LED0_MASK;
        break;
        
    case BRD_LED1:
        CPLD_CTRL1_REG |= LED1_MASK;
        break;
        
    case BRD_LED2:
        CPLD_CTRL1_REG |= LED2_MASK;
        break;
        
    default:
        return BRD_ERROR;
    }
    
    return (BRD_OK);
}

/*****************************************************************************/
/*  s16 brd_toggle_led(BrdLed led)                                           */
/*                                                                           */ 
/*  This routine toggles the specified user-defined LED.                     */
/*                                                                           */
/*  Parameters:                                                              */
/*      - led - indicates which led to toggle                                */
/*                                                                           */ 
/*  Return:                                                                  */
/*		BRD_OK on success                                                    */
/*		BRD_ERROR on failure                                                 */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_led_toggle(BrdLed led)
{
    int ledMask;
    
    switch (led)
    {
    case BRD_LED0:
        ledMask = LED0_MASK;
        break;
        
    case BRD_LED1:
        ledMask = LED1_MASK;
        break;
        
    case BRD_LED2:
        ledMask = LED2_MASK;
        break;
        
    default:
        return BRD_ERROR;
    }
    
    ( CPLD_CTRL1_REG & ledMask ) ? brd_led_disable(led) : brd_led_enable(led);
    
    return (BRD_OK);
}
