/****************************************************************/
/* THS1206 driver version 2.0 for C5400                         */
/* $Revision: 5 $                         $JustDate: 24.11.00 $ */
/****************************************************************/

#undef _INLINE

#include "dc_conf.h"
#include "t1206_fn.h"

/****************************************************************/
/* define and initialize data converter objects                 */
/****************************************************************/
&&DEFINE_OBJECT&&


/****************************************************************/
/* DSP FUNCTIONS                                                */
/****************************************************************/

TTIDCSTATUS ths1206_configure(void *pDC)
{
    TTHS1206 *pTHS = pDC;

    /* set reset bit in control register 1   */
    PORTADDR = 0x401 << pTHS->data_shift;        

    /* clear reset bit in control register 1 */
    PORTADDR = 0x400 << pTHS->data_shift;    

    /* write PlugIn generated control register 0 value in ADC   */
    PORTADDR = pTHS->creg0.value << pTHS->data_shift; 
    
    /* write PlugIn generated control register 1 value in ADC   */    
    PORTADDR = (pTHS->creg1.value | 0x400) << pTHS->data_shift;

    /* reset FIFO */
    PORTADDR = (pTHS->creg1.value | T1206_FRST | 0x400)
        << pTHS->data_shift;                           

    
    return TIDC_NO_ERR;
}


/****************************************************************/
/* ths1206_power                                                */
/* This sets the ths1206 in powerdown mode                      */
/* Parameters:    pointer to DC object                          */
/* Return Values: NONE                                          */
/****************************************************************/
void ths1206_power(void *pDC, int bDown)
{
    TTHS1206 *pTHS = pDC;

    if (bDown)
    {
        PORTADDR = (pTHS->creg0.value | T1206_PD) 
            << pTHS->data_shift;
    }
    else
    {
        PORTADDR = (pTHS->creg0.value & !T1206_PD) 
            << pTHS->data_shift;
    }
}


/****************************************************************/
/* ths1206_read                                                 */
/* This function reads the (FIFO) data                          */
/* Parameters:    pointer to DC object                          */
/* Return Values: one sample value, not shifted!                */
/****************************************************************/
long ths1206_read(void *pDC)
{
    return (PORTADDR);
}
 

/****************************************************************/
/* ths1206_write                                                */
/* This function writes data to ADC                             */
/* Parameters:    pointer to DC object                          */
/* Return Values: one sample value, not shifted!                */
/****************************************************************/
void ths1206_write(void *pDC, long lData)
{
    TTHS1206 *pTHS = pDC;

    PORTADDR = (lData) << pTHS->data_shift;     
}


/****************************************************************/
/* ths1206_rblock                                               */
/* NOT IMPLEMENTED                                              */
/****************************************************************/
void ths1206_rblock(void *pDC, void *pData, 
                    unsigned long ulCount, 
                    void (*callback) (void *))
{
}

/****************************************************************/
/* ths1206_wblock                                               */
/* NOT IMPLEMENTED                                              */
/****************************************************************/
void ths1206_wblock(void *pDC, void *pData, 
                    unsigned long ulCount, 
                    void (*callback) (void *))
{
}    

/****************************************************************/
/* ths1206_init                                                 */
/* This function initializes the control registers              */
/* zero and one of the THS1206.                                 */
/* Parameters:    pointer to DC object                          */
/*                int sel: -1 init all registers                */
/*                          0 init CR0 only                     */
/*                          1 init CR1 only                     */
/* Return Values: NONE                                          */
/****************************************************************/
void ths1206_init (void *pDC, int sel)
{
    TTHS1206 *pAdc = pDC;

    switch (sel)
    {
        case -1:
            PORTADDR = pAdc->creg0.value 
                << pAdc->data_shift;
            PORTADDR = (pAdc->creg1.value | 0x400) 
                << pAdc->data_shift;
            break;
        
        case  0:
            PORTADDR = pAdc->creg0.value << pAdc->data_shift;
            break;
        
        case  1:
            PORTADDR = (pAdc->creg1.value | 0x400) 
                << pAdc->data_shift;
            break;
     
        default:
            PORTADDR = pAdc->creg0.value 
                << pAdc->data_shift;
            PORTADDR = (pAdc->creg1.value | 0x400) 
                << pAdc->data_shift;
            break;
     }
}

/****************************************************************/
/* ths1206_reset                                                */
/* This function resets the ths1206 control register            */
/* Parameters:    pointer to DC object                          */
/* Return Values: NONE                                          */
/****************************************************************/
void ths1206_reset(void *pDC)
{
    TTHS1206 *pAdc = pDC;

    PORTADDR = 0x401 << pAdc->data_shift;
    PORTADDR = 0x400 << pAdc->data_shift;
}

/****************************************************************/
/* ths1206_freset                                               */
/* This function resets just the FIFO of the THS1206            */
/* Parameters:    pointer to DC object                          */
/* Return Values: NONE                                          */
/****************************************************************/
void ths1206_freset(void *pDC)
{
    TTHS1206 *pAdc = pDC;

    PORTADDR = (pAdc->creg1.value | T1206_FRST | 0x400)
        << pAdc->data_shift;
}

/****************************************************************/
/* ths1206_init_obj                                             */
/* This function initializes the data converter object          */
/* Calling this function is always necessary as a first step    */
/* Parameters:    pointer to DC object                          */
/*                adc_mem should contain the address of the DC  */
/*                uintcreg0 contains the init value for creg0   */
/*                uintcreg1 contains the init value for creg1   */
/*                shift contains the init value for data_shift  */
/* Return Values: NONE                                          */
/****************************************************************/
void ths1206_init_obj(void           *pDC,
                      unsigned int   uintcreg0,
                      unsigned int   uintcreg1,
                      unsigned short shift)
{
    TTHS1206 *pAdc = pDC;

    pAdc->creg0.value = uintcreg0;
    pAdc->creg1.value = uintcreg1;
    pAdc->data_shift = shift;
}

/****************************************************************/
/* ths1206_read_CR                                              */
/* This function reads the control registers of the device      */
/* and return the data. No changes to the object data are made  */
/* Parameters:    pointer to DC object                          */
/*                2 pointers to unsigned int variable           */
/* Return Values: updates pointer contents                      */
/****************************************************************/
void ths1206_read_CR(void         *pDC,
                     unsigned int *uintcreg0,
                     unsigned int *uintcreg1)
{
    TTHS1206 *pAdc = pDC;

    PORTADDR = (pAdc->creg1.value | T1206_DBG) 
        << pAdc->data_shift;
    
    *uintcreg0 = PORTADDR >> pAdc->data_shift;
    *uintcreg1 = PORTADDR >> pAdc->data_shift;
}

/****************************************************************/
/* ths1206_powerdown                                            */
/* This sets the ths1206 in powerdown mode                      */
/* Parameters:    pointer to DC object                          */
/* Return Values: NONE                                          */
/****************************************************************/
void ths1206_powerdown(void *pDC)
{
    TTHS1206 *pAdc = pDC;

    PORTADDR = (pAdc->creg0.value | T1206_PD) 
        << pAdc->data_shift;
}

/****************************************************************/
/* ths1206_powerwake                                            */
/* This sets the ths1206 in active mode (wakeup)                */
/* Parameters:    pointer to DC object                          */
/* Return Values: NONE                                          */
/****************************************************************/
void ths1206_powerwake(void *pDC)
{
    TTHS1206 *pAdc = pDC;

    PORTADDR = (pAdc->creg0.value & !T1206_PD) 
        << pAdc->data_shift;
}
