/******************************************************************************/
/*     File:    evm55xtest.c                                                  */
/*                                                                            */
/*     Purpose: Main module for the Confidence Test for the TMX320VC5510 EVM. */
/*                                                                            */
/******************************************************************************/

#include <windows.h>
#include <process.h>
#include <stdio.h>
#include <conio.h>

#include "evm55xdll.h"
#include "_evm55xdll.h"
#include "_evm55xtest.h"


char helpTxt[] = 
"TMX320VC5510 Prototype Board Confidence Test Utility, Version 2.04\n\n\
Syntax:\n\
  evm55xtest [-b num] [-c] [-d] [-i] [-n] [-p] [-u] [-?] [logfilename]\n\n\
  -?|h          Display summary of command syntax and options.\n\n\
  -b num        Specifies board to test.\n\
                PCI: 0 (for board 1) to n - 1.\n\n\
  -c            Perform continuous loop of confidence tests.\n\n\
  -d            Enable verbose mode for debugging.\n\n\
  -i            Enable information-only mode.\n\n\
  -n            No time delay between screen displays.\n\n\
  -p            Enable low-level driver error messages.\n\n\
  -u            Enable interactive confidence test.\n\n\
  -x            Continue testing after failure.\n";


/*--------------------------------------------------------------------------*
 *  File-local (static) variables
 *--------------------------------------------------------------------------*/
static FILE     *pLogFile = NULL;
static LPVOID   hHpi = NULL;   // HPI interface handle
static HANDLE   hBoard;
static BOOL     bStopOnError = FALSE;
static EVM55X_BOARD_TYPE  tBoardType;
static ULONG    ulRevId;    
static UINT     ind = 0;
static BOOL     DebugFlag = FALSE;
static BOOL     Map1Flag = TRUE;
static char     stringBuffer[STRING_BUFFER_SIZE];
static BOOL     userIn = FALSE;
static BOOL     bStopAtEnd = TRUE;
static BOOL     noDelay = FALSE;
static BOOL     LoopCount = 0;
static BOOL     ContLoop = FALSE;
static ULONG    TotalErrors = 0;


/*--------------------------------------------------------------------------*
 *  Global variables
 *--------------------------------------------------------------------------*/

/****************************************************************************/
/* main() - the startup routine                                             */
/*                                                                          */
/*  This routine excercises and verifies proper operation of the pci2040dll */
/*  calls and the low level driver calls.  With the exception of the mapTBC */
/*  low level driver call, DLL functions utilize driver calls, thus test    */
/*  both the DLL and the low level driver functionality.                    */
/*                                                                          */
/****************************************************************************/
void main( int argc, char *argv[], char *envp[] )
{
   ULONG               nBoardIndex= 0;
   int                 i;
   int                 infoOnly= FALSE;
   int                 boardCommError= FALSE;
   char				   *stp = "";
   char                *sLogFile = NULL;
   Register            reg = {0, 0};

   // Install Ctrl-C handler
   SetConsoleCtrlHandler( ControlHandler, TRUE );

   // Disable low-level driver printf's by default
   evm55x_printf_control(FALSE,NULL);

   memset (stringBuffer, '\0', STRING_BUFFER_SIZE);
   
   for ( i=1; argc > i; i++ )
   {
      if (( argv[i][0] != '-' ) && ( argv[i][0] != '/'))
      {
         sLogFile = argv[i];
      }
      else
      {
         switch ( tolower(argv[i][1]) )
         {

		 case 'b':
            if (argv[i][2] == '\0')
            {
               if (argv[i][3] != '-')
               {
                  nBoardIndex = strtoul(&argv[i][3],&stp,0);
                  ++i;
               }
               else
               {
                  nBoardIndex = 0;
               }
            }
            else
            {
               nBoardIndex = strtoul(&argv[i][2],&stp,0);
            }            
            break;

		 case 'c':
			ContLoop = TRUE;
			bStopOnError = FALSE;
			break;

         case 'd':
            DebugFlag = TRUE;
            break;

         case 'i' :
            infoOnly= TRUE;
            break;

		 case 'n' :
			noDelay = TRUE;
			break;

		 case 'p' :
			evm55x_printf_control(TRUE,NULL);
			break;

		 case 'x' :
            bStopOnError = FALSE;
            break;

         case 'u' :
            userIn = TRUE;
            break;

		 case 's':
			bStopAtEnd = TRUE;
			break;

         case '?': 
		 case 'h':
            help();
            break;

         default:
            break;
         }
      }
   }
   
   //open log file 
   if ( sLogFile != NULL )
   {
      pLogFile = fopen( sLogFile, "w" );
      
      if ( pLogFile == NULL )
      {
         LogPrint("====================================================================\n",0,0,0,0);
         LogPrint("= TMX320VC5510 Prototype Board Confidence Test, Version 2.04       =\n",0,0,0,0);
		 LogPrint("====================================================================\n",0,0,0,0);
		 printf( "WARNING: Unable to open log file \"%s\".\n\n", sLogFile );
         ProgramExit(-1);;
      }
   }


   LogPrint("====================================================================\n",0,0,0,0);
   LogPrint("= TMX320VC5510 Prototype Board Confidence Test, Version 2.04       =\n",0,0,0,0);
   LogPrint("=                                                                  =\n",0,0,0,0);

   
   //Display Board Index and Open board
   LogPrint( "= Board Index: %-2d                                                  =\n", nBoardIndex,0,0,0 );
   
   if ( DebugFlag ) PressAnyKey();
   
   if (!nBoardIndex) 
	   nBoardIndex = DEFAULT_PCI_PORT;

   if ( (hBoard = evm55x_open( nBoardIndex, TRUE )) == INVALID_HANDLE_VALUE )
   {
	   LogPrint( "\nERROR: evm55x_open() failed for board %d, exiting.\n",
		   nBoardIndex,0,0,0 );
	   LogPrint( "Windows driver or board #%d is not installed.\n",
		   nBoardIndex,0,0,0 );
	   if ( pLogFile != NULL ) 
	   { 
		   fclose( pLogFile ); 
	   }
	   ProgramExit(-1);
   }

   if ( (hHpi = evm55x_hpi_open( hBoard )) == INVALID_HANDLE_VALUE )
   {
      LogPrint( "ERROR: evm55x_hpi_open() failed for board %d, exiting.\n",
         nBoardIndex,0,0,0 );
      if ( pLogFile != NULL ) { fclose( pLogFile ); }
      ProgramExit(-1);
   }
   
   //reset the board
   evm55x_reset_board(hBoard);

   //Retrieve board type & revision and display
   if ( !evm55x_board_info( hBoard, &tBoardType, &ulRevId) )
   {
      LogPrint( "ERROR: evm55x_board_type() failed.\n", 0,0,0,0 );
      boardCommError = TRUE;
   }
   else
   {
      LogPrint( "= Board Type: ", 0,0,0,0 );
       switch (tBoardType)
       {
         case TYPE_PCI2040_C5510_EVM:   
            LogPrint("TMX320VC5510 Prototype Board", 0,0,0,0 );  
            break;

      default:
         LogPrint( "UNKNOWN", 0,0,0,0 ); 
         break;
       }

      LogPrint( ", Rev %-2d                 =\n", ulRevId, 0,0,0 );
   }

   if ( DebugFlag ) PressAnyKey();

   LogPrint("====================================================================\n",0,0,0,0);

   if (pLogFile != NULL)
   {
       fprintf (pLogFile,"Test Started: ",0,0,0,0);

       GetTimeFormat( LOCALE_SYSTEM_DEFAULT, 0, NULL,
            "hh':'mm':'ss tt", stringBuffer, STRING_BUFFER_SIZE );
       fprintf(pLogFile,stringBuffer);
       fprintf(pLogFile," on ");
       GetDateFormat( LOCALE_SYSTEM_DEFAULT, 0, NULL,
            "MMM dd',' yyyy", stringBuffer, STRING_BUFFER_SIZE );
       fprintf(pLogFile,stringBuffer);
	   fprintf(pLogFile,"\n\n");
   }

   if (!infoOnly)
   {
      LogPrint("Beginning Confidence Test ...\n",0,0,0,0);
	  
	  if (userIn)
		  PressAnyKey(); // To much data to display on DOS shell
	  else
		  Sleep (1000);
      
      if (!boardCommError)
      {
         ULONG revMajor, revMinor, buildNum;

		 // Get DLL revision information
		 evm55x_dll_revision (&revMajor,&revMinor,&buildNum);

		 LogPrint("\nWin32 DLL (evm55x.dll) Revision: %d.%d - Build %d\n",revMajor,revMinor,buildNum,0);
         LogPrint("\nTesting Win32 DLL and Low-Level Driver access................PASSED.\n\n",0,0,0,0);
      }
      else
      {
         LogPrint("\nTesting Win32 DLL and Low-Level Drivers access...............FAILED.\n\n",0,0,0,0);
      }

#if 1
      if (do_cpld_tests())
      {        
		 LogPrint("\nERROR: CPLD tests failed. \n\n",0,0,0,0);
         if ( bStopOnError ) { ProgramExit(-1); }
      }

		if (userIn)
			PressAnyKey();
		else
		{
			if (!noDelay)
			{
				LogPrint("\nWill proceed in 5 seconds....\n",0,0,0,0);
			    Sleep(5000);
			}
		}
#endif


#if 1
	  if (do_PCI2040_tests())
	  {
		  LogPrint("\nERROR: PCI2040 PCI Bridge tests failed. \n\n",0,0,0,0);
		  if ( bStopOnError ) { ProgramExit(-1); }
	  }
#endif
 
// Loop entry if continous mode selected	  
LoopEntry:

#if 1
	  if (do_jtag_tests())
	  {
		  LogPrint("\nERROR: JTAG TBC tests failed. \n\n",0,0,0,0);
		  if ( bStopOnError ) { ProgramExit(-1); }
	  }
#endif

#if 1
	  if (do_memory_tests())
	  {
		  LogPrint("\nERROR: Prototype Board Memory tests failed. \n\n",0,0,0,0);
		  if ( bStopOnError ) { ProgramExit(-1); }
	  }
#endif
	  
#if 1
      if (do_clock_tests())
      {        
		 LogPrint("\nERROR: DSP CPU Clock test failed. \n\n",0,0,0,0);
         if ( bStopOnError ) { ProgramExit(-1); }
      }
#endif

#if 1
	  if (do_dsprev_tests())
	  {
		  LogPrint("\nERROR: DSP revision test failed. \n\n",0,0,0,0);
		  if ( bStopOnError ) { ProgramExit(-1); }
	  }
#endif

#if 1
      if (do_led_tests())
      {
         LogPrint("\nERROR: LED tests failed. \n\n",0,0,0,0);
         if ( bStopOnError ) { ProgramExit(-1); }
      }
#endif

#if 1
      if (do_sem_tests())
      {
         LogPrint("\nERROR: Semaphore tests failed. \n\n",0,0,0,0);
         if ( bStopOnError ) { ProgramExit(-1); }
      }
#endif

#if 1
      if (do_interrupt_tests())
      {
         LogPrint("\nERROR: DSP INTERRUPT tests failed. \n\n",0,0,0,0);
         if ( bStopOnError ) { ProgramExit(-1); }
      }
#endif

#if 1
	  if (do_aic27_tests())
	  {
		  LogPrint("\nERROR: AIC27 codec tests failed. \n\n",0,0,0,0);
		  if ( bStopOnError ) { ProgramExit(-1); }
	  }
#endif

	  
#if 1
	if (!ContLoop)
	{	  
	  LogPrint( "====================================================================\n",0,0,0,0);
	  LogPrint( "===========         Flash Memory Confidence Tests         ==========\n",0,0,0,0);
	  LogPrint( "====================================================================\n",0,0,0,0);
	  
	  if ( DebugFlag ) {
	      LogPrint( "\nTesting and Erasing Flash Memory...\n\n", 0,0,0,0 ); 	  
		  PressAnyKey();
	  }
			  
	  if (do_flash_tests())
	  {
		  LogPrint("\nERROR: Flash memory tests failed. \n\n",0,0,0,0);
		  if ( bStopOnError ) { ProgramExit(-1); }
	  }
	}  // run only if not continuous loop

#endif

   } // close infoOnly


   // Check if in continuous loop mode
   if (ContLoop) 
   {
	  LoopCount++;

	  LogPrint( "\n====================================================================\n",0,0,0,0);
	  LogPrint(   "Continuous Loop Mode Status:\n",0,0,0,0);
	  LogPrint(   "   Loop Count          = %d\n",LoopCount,0,0,0);
	  LogPrint(   "   Total Test Failures = %d\n",TotalErrors,0,0,0);
	  LogPrint( "====================================================================\n\n",0,0,0,0);
	  
	  LogPrint( "Tests will re-start in 5 seconds...\n\n",0,0,0,0);
	  LogPrint( "Press Ctrl-C to quit testing at any time.\n\n",0,0,0,0);
	
	  if (!noDelay)  Sleep(5000);

	  goto LoopEntry;
   }


   /****************************************************************************/
   /* Cleanup
   /****************************************************************************/
   // re-enable default printf for driver functions
   evm55x_printf_control(TRUE,NULL); 
   
   if (!bStopOnError)
   {
       LogPrint("\n\nTotal Test Failures = %d\n\n",TotalErrors,0,0,0);
   }
   
   if ( DebugFlag ) 
   {
        LogPrint( "***** Cleaning up.\n", 0,0,0,0 ); 
        PressAnyKey();
   }
   
   //close logfile
   if ( DebugFlag )
      LogPrint( "Closing log file.\n", 0,0,0,0);
      
   if ( DebugFlag )
      LogPrint( "Closing board %d.\n", nBoardIndex,0,0,0 );
   
   evm55x_hpi_close( hHpi );
   
   evm55x_close( hBoard );
   
   if (!infoOnly)
   {
	   LogPrint( "====================================================================\n",0,0,0,0);
	   LogPrint( "====   TMX320VC5510 Prototype Board Confidence Test Complete   =====\n",0,0,0,0);
	   LogPrint( "====================================================================\n",0,0,0,0);
   }
   
   if ( pLogFile != NULL ) 
        fclose( pLogFile );

   if (bStopAtEnd) 
   {
	  char c;

      printf( "\nPress any key to return to the control panel." );
   
      c = _getch();
   }
}

/****************************************************************************/
/* Wrapper for printf
/****************************************************************************/
void LogPrint( char *format, ULONG parm1, ULONG parm2,
                     ULONG parm3, ULONG parm4 )
{
   if ( pLogFile != NULL )
   {
      fprintf( pLogFile, format, parm1, parm2, parm3, parm4 );
   }
   
   printf( format, parm1, parm2, parm3, parm4 );
}


/****************************************************************************/
/* Press Any Key routine
/****************************************************************************/
void PressAnyKey( void )
{
   int     c;
   
   printf( "Press any key to continue (Ctrl-C to abort)." );
   
   c = _getch();
   
   printf( "\n" );
   
   if ( c == 0x03 ) 
   {
      ControlHandler (CTRL_C_EVENT);
   }
   
   return;
} /* end of PressAnyKey() */


/****************************************************************************/
int LoadAicTests (void)
{

    if ( DebugFlag )
    {
       BLANK(stringBuffer);
       sprintf(stringBuffer,"Loading AIC27 codec test on DSP %d........\n",ind);
       LogPrint(stringBuffer,0,0,0,0);
    }

	// Load AIC test application on DSP
	if (Load_Dsp_Test_Application("aic27tst.out",ind) == EVM_ERROR)
	   return (EVM_ERROR);
         
    if ( DebugFlag ) 
    {
       LogPrint( "Loaded DSP executable \'aic27tst.out\' is now running!\n",0,0,0,0 ); 
       PressAnyKey();
    }

	Sleep(SLEEP_TIME);

    return (EVM_OK);
}


/****************************************************************************/
/* do_aic27_tests routine
/****************************************************************************/
int do_aic27_tests(void)
{
   short command[2];
   int status = EVM_OK;
   
   LogPrint( "====================================================================\n",0,0,0,0);
   LogPrint( "===========         AIC27 Codec Confidence Tests          ==========\n",0,0,0,0);
   LogPrint( "====================================================================\n",0,0,0,0);

   if ( DebugFlag ) PressAnyKey();
   
#if 1
   /*************************************************************************/
   /* Begin Tests
   /*************************************************************************/
   for (ind=0; ind<MAX_PCI2040_DSPS; ind++)
   {
      if (evm55x_dsp_present(hBoard, ind))
      {
         // Load AIC test application
         if (LoadAicTests() == EVM_ERROR)
		 {
            return(EVM_ERROR);
		 }

         /*************************************************************************/
         /* Left Channel Test                                                     */
         /*************************************************************************/
         command[0] = LEFT_CHAN_TONE_NUM;
         command[1] = DONT_WAIT_FOR_INPUT;

         LogPrint("DSP %d: AIC27 Codec Left Channel Tone Test....................",ind,0,0,0);

         if ( DebugFlag ) PressAnyKey();
         
         // set timeout for test to 6 seconds
		 evm55x_set_timeout(hBoard,6000);

         status = execute_test(command,2,NULL,0,ind,WAIT);

		 // restore default timeout
		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);         

		 // check test result
         if ( status != EVM_OK )
         {
			if (status != EVM_FIFO_ERROR)
			{
			   LogPrint("FAILED.\n",0,0,0,0);
			}
            if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
         }
         else
         {
            LogPrint("PASSED.\n",0,0,0,0);
         }

		 // If FIFO error, reload application
		 if (status == EVM_FIFO_ERROR)
		 {
			// Load AIC test application
			if (LoadAicTests() == EVM_ERROR)
			{
				return(EVM_ERROR);
			}
		 }

         /*************************************************************************/
         /* Right Channel Test                                                    */
         /*************************************************************************/
         command[0] = RIGHT_CHAN_TONE_NUM;
         command[1] = DONT_WAIT_FOR_INPUT;
   
         LogPrint("DSP %d: AIC27 Codec Right Channel Tone Test...................",ind,0,0,0);
         if ( DebugFlag ) PressAnyKey();
         
         // set timeout for test to 6 seconds
		 evm55x_set_timeout(hBoard,6000);

         status = execute_test(command,2,NULL,0,ind,WAIT);

		 // restore default timeout
		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);   
		 
		 // check test result
		 if ( status != EVM_OK )
         {
 			if (status != EVM_FIFO_ERROR)
			{
			   LogPrint("FAILED.\n",0,0,0,0);
			}
			 if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
         }
         else
         {
            LogPrint("PASSED.\n",0,0,0,0);
         }

		 // If FIFO error, reload application
		 if (status == EVM_FIFO_ERROR)
		 {
			// Load AIC test application
			if (LoadAicTests() == EVM_ERROR)
			{
				return(EVM_ERROR);
			}
		 }

         /*************************************************************************/
         /* Both Channel Test                                                     */
         /*************************************************************************/
         command[0] = BOTH_CHAN_TONE_NUM;
         command[1] = DONT_WAIT_FOR_INPUT;
   
         LogPrint("DSP %d: AIC27 Codec Left/Right Channel Tone Test..............",ind,0,0,0);
         if ( DebugFlag ) PressAnyKey();
         
          // set timeout for test to 6 seconds
		 evm55x_set_timeout(hBoard,6000);

         status = execute_test(command,2,NULL,0,ind,WAIT);

		 // restore default timeout
		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);   
		 
		 // check test result
         if ( status != EVM_OK )
         {
 			if (status != EVM_FIFO_ERROR)
			{
			   LogPrint("FAILED.\n",0,0,0,0);
			}
			
			if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
         }
         else
         {
            LogPrint("PASSED.\n",0,0,0,0);
         }

		 // If FIFO error, reload application
		 if (status == EVM_FIFO_ERROR)
		 {
			// Load AIC test application
			if (LoadAicTests() == EVM_ERROR)
			{
				return(EVM_ERROR);
			}
		 }

         /*************************************************************************/
         /* Codec LINE loopback test.                                             */
         /*************************************************************************/
         command[0] = LINE_LOOPBACK_NUM;
         command[1] = WAIT_FOR_INPUT;
   
         LogPrint("DSP %d: AIC27 Codec Line In Loopback Test.....................",ind,0,0,0);
         if ( DebugFlag ) PressAnyKey();
         
         // set timeout for test to 12 seconds
		 evm55x_set_timeout(hBoard,12000);

         status = execute_test(command,2,NULL,0,ind,WAIT);

		 // restore default timeout
		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);   	 
		 
		 // check test result
		 if ( status != EVM_OK )
         {
 			if (status != EVM_FIFO_ERROR)
			{
			   LogPrint("FAILED.\n",0,0,0,0);
			}           
			 
			if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
         }
         else
         {
            LogPrint("PASSED.\n",0,0,0,0);
         }
 
		 // If FIFO error, reload application
		 if (status == EVM_FIFO_ERROR)
		 {
			// Load AIC test application
			if (LoadAicTests() == EVM_ERROR)
			{
				return(EVM_ERROR);
			}
		 }
		 
         /*************************************************************************/
         /* Codec MIC loopback test.                                              */
         /*************************************************************************/
         command[0] = MIC_LOOPBACK_NUM;
         command[1] = WAIT_FOR_INPUT;
   
         LogPrint("DSP %d: AIC27 Codec Mic In Loopback Test......................",ind,0,0,0);
         if ( DebugFlag ) PressAnyKey();
         
         // set timeout for test to 12 seconds
		 evm55x_set_timeout(hBoard,12000);

         status = execute_test(command,2,NULL,0,ind,WAIT);

		 // restore default timeout
		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);   
		 
		 // check test result
		 if ( status != EVM_OK )
         {		 
			 if (status != EVM_FIFO_ERROR)
			 {
		        LogPrint("FAILED.\n",0,0,0,0);
			 }

		     if ( bStopOnError ) return(EVM_ERROR);
	   	     TotalErrors++;
         }
         else
         {
            LogPrint("PASSED.\n\n",0,0,0,0);
         }
      }
   }

#endif
   return(EVM_OK);
}


/*************************************************************************/
/*  Interrupt Tests                                                      */
/*************************************************************************/
int do_interrupt_tests(void)
{
   short command[2];
   int status = EVM_OK;

   int dspInt[4] = {INTDSP0,INTDSP0,INTDSP2,INTDSP3};
   PCI2040_HOST_EVENT dspEventNum[4] = {DSP0_INT, DSP1_INT, DSP2_INT, DSP3_INT};
   HANDLE hintEvHandle = NULL;
   
   char *filename = "misctst.out";

#if 1
 
   LogPrint( "====================================================================\n",0,0,0,0);
   LogPrint( "===========       DSP Interrupt Confidence Tests          ==========\n",0,0,0,0);
   LogPrint( "====================================================================\n",0,0,0,0);
   
   if ( DebugFlag ) PressAnyKey();
   
   /*************************************************************************/
   /* HOST-to-DSP Interrupt Tests                                           */
   /*************************************************************************/
   command[0] = DSPINT_TEST_NUM;
   command[1] = DONT_WAIT_FOR_INPUT;
   
   for (ind=0; ind<MAX_PCI2040_DSPS; ind++)
   {
      if (evm55x_dsp_present(hBoard, ind))
      {
         if ( DebugFlag ) 
         {
             BLANK(stringBuffer);
             sprintf(stringBuffer,"Loading Interrupt test on DSP %d..............\n",ind);
             LogPrint(stringBuffer,0,0,0,0);
         }
 
	     // Load test application on DSP
      	 if (Load_Dsp_Test_Application(filename,ind) == EVM_ERROR)
	        return (EVM_ERROR);		 
		 
         if ( DebugFlag ) 
         {
            sprintf(stringBuffer, "Loaded DSP executable \'%s\' is now running!\n", filename ); 
            LogPrint(stringBuffer,0,0,0,0);
			PressAnyKey();
         }
   
         Sleep(SLEEP_TIME);
         
         /*************************************************************************/
         /* HOST-to-DSP Interrupt Tests                                           */
         /*************************************************************************/
         LogPrint("DSP %d: Host-to-DSP Interrupt Tests...........................",ind,0,0,0);
         if ( DebugFlag ) PressAnyKey();
         
         //generate all HOST-to-DSP interrupts
              	 
         if (!evm55x_generate_int(hHpi, NMI_INT, ind))
         {
            LogPrint("Unable to generate NMI interrupt on DSP %d.\n\n",ind,0,0,0);
            if ( bStopOnError ) return (EVM_ERROR);
			TotalErrors++;
         }

         if (!evm55x_generate_int(hHpi, DSP_INT, ind))
         {
            LogPrint("Unable to generate DSPINT interrupt on DSP %d.\n\n",ind,0,0,0);
            if ( bStopOnError ) return (EVM_ERROR);
			TotalErrors++;
         }

		 if (!evm55x_generate_int(hHpi, DSP_INT5, ind))
         {
            LogPrint("Unable to generate DSPINT5 interrupt on DSP %d.\n\n",ind,0,0,0);
            if ( bStopOnError ) return (EVM_ERROR);
			TotalErrors++;
         }

		 // set timeout for test to 3 seconds
		 evm55x_set_timeout(hBoard,3000);

         status = execute_test(command,2,NULL,0,ind,WAIT);

		 // restore timeout
		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);
    
		 
		 /* NOTE: DSPINT errors are not reported due to possible silicon issue at this time. */
		 if (status & 2)
		 {
            status = EVM_OK;
		 }

		 if ( status != EVM_OK )
         {
			if (status != EVM_FIFO_ERROR)
			{
			   LogPrint("FAILED.\n",0,0,0,0); 
			   
			   if (DebugFlag)
			      LogPrint("ERROR: Interrupt test status = 0x%X.\n",status,0,0,0);

			   if (status & 0x1)
			   {
			       TotalErrors++;
				   LogPrint("ERROR: NMI interrupt on DSP %d FAILED.\n",ind,0,0,0);
			   }
            
               if (status & 0x2)
			   {
			       TotalErrors++; 
				   LogPrint("ERROR: DSPINT interrupt on DSP %d FAILED.\n",ind,0,0,0);
			   }

			   if (status & 0x8)
			   {  
			       TotalErrors++;
				   LogPrint("ERROR: DSPINT interrupt seen by DSP, but ISR didn't handle it.\n",0,0,0,0);
				   LogPrint("ERROR: DSPINT interrupt on DSP %d FAILED.\n",ind,0,0,0);
			   }

			   if (status & 0x4)
			   {  
			       TotalErrors++;
				   LogPrint("ERROR: DSPINT5 interrupt on DSP %d FAILED.\n",ind,0,0,0);
			   }

			   if (status & 0x10)
			   {  
			       TotalErrors++;
				   LogPrint("ERROR: DSPINT5 interrupt seen by DSP, but ISR didn't handle it.\n",0,0,0,0);
				   LogPrint("ERROR: DSPINT5 interrupt on DSP %d FAILED.\n",ind,0,0,0);
			   }
                 
			   if ( bStopOnError ) return(EVM_ERROR);			
			}
         }
         else
         {
            LogPrint("PASSED.\n",0,0,0,0);
         }

		 //DSP-to-HOST interrupts only supported for PCI
		 /*************************************************************************/
		 /* DSP-to-HOST Interrupt Tests                                           */
		 /*************************************************************************/
		 command[0] = HOSTINT_TEST_NUM;
		 command[1] = DONT_WAIT_FOR_INPUT;
		 
		 LogPrint("DSP %d: DSP-to-Host Interrupt Tests...........................",ind,0,0,0);
		 if ( DebugFlag ) PressAnyKey();
		 
		 hintEvHandle = evm55x_get_event_handle(hBoard, dspEventNum[ind]);
		 if (hintEvHandle == NULL) return (EVM_ERROR);
		 
		 if (!evm55x_enable_int(hHpi, dspInt[ind]))
		 {
			 printf( "evm55x_enable_int failed for DSP%d_INT!", ind );
			 if ( bStopOnError ) return(EVM_ERROR);
			 TotalErrors++;
		 }
		 
		 //reset any pending events
		 ResetEvent(hintEvHandle);
		 

		 // set timeout for test to 2 seconds
		 evm55x_set_timeout(hBoard,2000);

		 status = execute_test(command,2,NULL,0,ind, WAIT);

		 // restore timeout 
		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);
		 
		 if (status != EVM_OK)
		 {
 			if (status != EVM_FIFO_ERROR)
			{
			   LogPrint("FAILED.\n",0,0,0,0);
			}   			 
			
			LogPrint("No ACK from DSP %d.\n",ind,0,0,0);
			if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
		 }
		 
		 status = waitForInterrupt(hintEvHandle);
		 
		 if ( status == EVM_OK )
		 {
			 LogPrint("PASSED.\n\n",0,0,0,0);
		 }
		 else
		 {
			 LogPrint("FAILED.\n",0,0,0,0);

			 if ( bStopOnError ) 
				 return status;
			 TotalErrors++;
		 }
      }
   }
      
#endif 
   
   return(status);
}

/*************************************************************************/
/*  Semaphore Tests                                                      */
/*************************************************************************/
int do_sem_tests(void)
{
   short command[2];
   int status = EVM_OK, ret = EVM_OK;
   char *filename = "misctst.out";

#if 1
 
   LogPrint( "====================================================================\n",0,0,0,0);
   LogPrint( "===========          Semaphore Confidence Tests           ==========\n",0,0,0,0);
   LogPrint( "====================================================================\n",0,0,0,0);
   
   if ( DebugFlag ) PressAnyKey();
   
   /*************************************************************************/
   /* HOST-to-DSP Semaphore Tests                                           */
   /*************************************************************************/
   command[0] = HOSTOWNED_SEM_TEST_NUM;
   command[1] = DONT_WAIT_FOR_INPUT;
   
   for (ind=0; ind<MAX_PCI2040_DSPS; ind++)
   {
      if (evm55x_dsp_present(hBoard, ind))
      {  
         if (DebugFlag)
         {
             BLANK(stringBuffer);
             sprintf(stringBuffer,"Loading Semaphore test on DSP %d..............\n",ind);
             LogPrint(stringBuffer,0,0,0,0);
         }

	     // Load test application on DSP
      	 if (Load_Dsp_Test_Application(filename,ind) == EVM_ERROR)
	        return (EVM_ERROR);	

         if ( DebugFlag ) 
         {
            sprintf(stringBuffer, "Loaded DSP executable \'%s\' is now running!\n",filename ); 
            LogPrint(stringBuffer,0,0,0,0);
			PressAnyKey();
         }
   
         Sleep(SLEEP_TIME);
         
         LogPrint("DSP %d: Semaphore Test (Host-Owned)...........................",ind,0,0,0);

		 if ( DebugFlag ) PressAnyKey();

         //release semaphores
         evm55x_user_semaphore_release(hBoard, PCI2040_SEM0);
         evm55x_user_semaphore_release(hBoard, PCI2040_SEM1);

         //obtain semaphore 0 & 1
         if (!evm55x_user_semaphore_wait(hBoard, PCI2040_SEM0))
         {
			LogPrint( "FAILED.\n",0,0,0,0);
            LogPrint( "ERROR:Host could not obtain semaphore 0!\n", 0,0,0,0 );
            if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
         }
         
         if (!evm55x_user_semaphore_wait(hBoard, PCI2040_SEM1))
         {
			LogPrint("FAILED.\n",0,0,0,0);
            LogPrint( "ERROR:Host could not obtain semaphore 1!\n", 0,0,0,0 );
            if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
         }

		 evm55x_set_timeout(hBoard,3000);

         status = execute_test(command,2,NULL,0,ind,WAIT);

         evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);

         if (status != EVM_OK)
         {
 			if (status != EVM_FIFO_ERROR)
			{
			   LogPrint("FAILED.\n",0,0,0,0);
			}   

            if (status & 1)
            {
                ret++;
                LogPrint("ERROR:DSP %d was able to obtain semaphore 0 when the host owned it!\n",ind,0,0,0);
            }
            if (status & 2)
            {
                ret++;
                LogPrint("ERROR:DSP %d was able to obtain semaphore 1 when the host owned it!\n",ind,0,0,0);
            }

            if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;	
         }
		 else
		 {
			 LogPrint("PASSED.\n",0,0,0,0);
		 }

         //release semaphores
         if (!evm55x_user_semaphore_release(hBoard, PCI2040_SEM0))
         {
            LogPrint( "\nERROR:Host could not release semaphore 0!\n", 0,0,0,0 );
            if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
            continue;
         }         
         
         if (!evm55x_user_semaphore_release(hBoard, PCI2040_SEM1))
         {
            LogPrint( "\nERROR:Host could not release semaphore 1!\n", 0,0,0,0 );
            if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
            continue;
         }

         /***********************************************************/
         /*Now test host trying to get semaphore while DSP has it
         /***********************************************************/
         command[0] = DSPOWNED_SEM_TEST_NUM;
         command[1] = DONT_WAIT_FOR_INPUT;
         
         status = 0;

         LogPrint("DSP %d: Semaphore Test (DSP-Owned)............................",ind,0,0,0);

		 evm55x_set_timeout(hBoard,3000);

         status = execute_test(command,2,NULL,0,ind,WAIT);

		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);

         if(status != EVM_OK)
         {
 			 if (status != EVM_FIFO_ERROR)
			 {
			   LogPrint("FAILED.\n",0,0,0,0);
			 }   

             if ((status & 1) == 1)
             {
                 LogPrint( "ERROR:DSP could not obtain semaphore 0!\n", 0,0,0,0 );
                 ret++;
                 if ( bStopOnError ) return(EVM_ERROR);
				 TotalErrors++;
             }
             if ((status & 2) == 2)
             {
                 LogPrint( "ERROR:DSP could not obtain semaphore 1!\n", 0,0,0,0 );
                 ret++;
                 if ( bStopOnError ) return(EVM_ERROR);
				 TotalErrors++;
             }
         }

         if ( DebugFlag ) 
         {
            LogPrint("Checking to make sure that DSP has semaphores\n",0,0,0,0); 
            PressAnyKey();
         }
   
         //now try to get semaphore
         if (evm55x_user_semaphore_get(hBoard, PCI2040_SEM0))
         {
             LogPrint( "\nERROR:Host acquired semaphore 0, but DSP owned it!\n", 0,0,0,0 );
             ret++;
			 TotalErrors++;
         }
         
         if (evm55x_user_semaphore_get(hBoard, PCI2040_SEM1))
         {
             LogPrint( "\nERROR:Host acquired semaphore 1, but DSP owned it!\n", 0,0,0,0 );
             ret++;
			 TotalErrors++;
         }
         
         if (ret)
         {
            if ( bStopOnError ) return(EVM_ERROR);
         }
         else
         {
            LogPrint("PASSED.\n\n",0,0,0,0);
         }
      }
   }
      
#endif 
   
   return(status);
}

/*************************************************************************/
/*  CPU Clock Tests                                                      */
/*************************************************************************/
int do_clock_tests(void)
{
   short command[2];
   int status = EVM_OK;
   
#if 1
 
   LogPrint( "====================================================================\n",0,0,0,0);
   LogPrint( "===========         DSP Clocks Confidence Test            ==========\n",0,0,0,0);
   LogPrint( "====================================================================\n",0,0,0,0);
   
   if ( DebugFlag ) PressAnyKey();
   
   command[0] = CPU_CLOCK_TEST_NUM;
   command[1] = DONT_WAIT_FOR_INPUT;
   
   for (ind=0; ind<MAX_PCI2040_DSPS; ind++)
   {
      if (evm55x_dsp_present(hBoard, ind))
      {
		 if (DebugFlag)
		 {
			 BLANK(stringBuffer);
			 sprintf(stringBuffer,"Loading Clock test on DSP %d..............\n",ind);
			 LogPrint(stringBuffer,0,0,0,0);
		 }

	     // Load test application on DSP
      	 if (Load_Dsp_Test_Application("misctst.out",ind) == EVM_ERROR)
	        return (EVM_ERROR);			 

         if ( DebugFlag ) 
         {
            sprintf(stringBuffer, "Loaded DSP executable \'misctst.out\' is now running!\n" ); 
            LogPrint(stringBuffer,0,0,0,0);
			PressAnyKey();
         }
   
         Sleep(SLEEP_TIME);
         
         LogPrint("DSP %d: Clocks Test (150 MHz).................................",ind,0,0,0);
         if ( DebugFlag ) PressAnyKey();

         // set timeout for test to 3 seconds
		 evm55x_set_timeout(hBoard,3000);

         status = execute_test(command,2,NULL,0,ind,WAIT);

		 // restore default timeout
		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);

         if ( status != EVM_OK )
         {
			if (status != EVM_FIFO_ERROR)
			{
			   LogPrint("FAILED.\n",0,0,0,0);		
			}
			if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
         }
		 else
		 {
			 LogPrint("PASSED.\n\n",0,0,0,0);
             //LogPrint("Clock Difference = %d\n",status,0,0,0);
		 }
      }
   }
      
#endif 
   
   return(status);
}


/*************************************************************************/
/*  DSP Revision Test                                                    */
/*************************************************************************/
int do_dsprev_tests(void)
{
   USHORT command[2];
   USHORT revData[8];
   int status = EVM_OK;
   
#if 1
 
   LogPrint( "====================================================================\n",0,0,0,0);
   LogPrint( "===========         DSP Revision Confidence Test          ==========\n",0,0,0,0);
   LogPrint( "====================================================================\n",0,0,0,0);
   
   if ( DebugFlag ) PressAnyKey();
   
   command[0] = DSP_REV_TEST_NUM;
   command[1] = DONT_WAIT_FOR_INPUT;
   
   for (ind=0; ind<MAX_PCI2040_DSPS; ind++)
   {
      if (evm55x_dsp_present(hBoard, ind))
      {
		 if (DebugFlag)
		 {
			 BLANK(stringBuffer);
			 sprintf(stringBuffer,"Loading DSP Rev test on DSP %d............\n",ind);
			 LogPrint(stringBuffer,0,0,0,0);
		 }

	     // Load test application on DSP
      	 if (Load_Dsp_Test_Application("misctst.out",ind) == EVM_ERROR)
	        return (EVM_ERROR);	

         if ( DebugFlag ) 
         {
            sprintf(stringBuffer, "Loaded DSP executable \'misctst.out\' is now running!\n" ); 
            LogPrint(stringBuffer,0,0,0,0);
			PressAnyKey();
         }
   
         Sleep(SLEEP_TIME);
         
         LogPrint("DSP %d:DSP Revision Test......................................",ind,0,0,0);

         if ( DebugFlag ) PressAnyKey();

         // set timeout for test to 5 seconds
		 evm55x_set_timeout(hBoard,5000);

         status = execute_test(command,2,revData,5,ind,WAIT);

		 // restore default timeout
		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);

         if ( status != EVM_OK )
         {
			if (status != EVM_FIFO_ERROR)
			{
			   LogPrint("FAILED.\n",0,0,0,0);
			}
			if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
         }
		 else
		 {
			 LogPrint("PASSED.\n",0,0,0,0);
			 sprintf(stringBuffer,"      TMX320VC5510 Die ID = 0x%04X%04X%04X%04X\n",
				     revData[0],revData[1],revData[2],revData[3]);
			 LogPrint(stringBuffer,0,0,0,0);
			 sprintf(stringBuffer,"      TMX320VC5510 Rev ID = 0x%04X\n\n",revData[4]);
			 LogPrint(stringBuffer,0,0,0,0);
		 }
      }
   }
      
#endif 
   
   return(status);
}

/*************************************************************************/
/*  LED Tests                                                            */
/*************************************************************************/
int do_led_tests(void)
{
   short command[2];
   int status = EVM_OK;
   char *filename = "misctst.out";
   
#if 1

   LogPrint( "====================================================================\n",0,0,0,0);
   LogPrint( "===========             LED Confidence Test               ==========\n",0,0,0,0);
   LogPrint( "====================================================================\n",0,0,0,0);
   
   if ( DebugFlag ) PressAnyKey();
   
   /*************************************************************************/
   /* DSP Led Tests                                                         */
   /*************************************************************************/
   command[0] = LED_TEST_NUM;
   command[1] = DONT_WAIT_FOR_INPUT;
   
   //flash all LEDs on the boards
   for (ind=0; ind<MAX_PCI2040_DSPS; ind++)
   {
      if (evm55x_dsp_present(hBoard, ind))
      {
         if (DebugFlag)
         {
             BLANK(stringBuffer);
             sprintf(stringBuffer,"Loading LED test on DSP %d..............\n",ind);
             LogPrint(stringBuffer,0,0,0,0);
         }
         
	     // Load test application on DSP
      	 if (Load_Dsp_Test_Application(filename,ind) == EVM_ERROR)
	        return (EVM_ERROR);	

         if ( DebugFlag ) 
         {
            printf( "Loaded DSP executable \'%s\' is now running!\n", filename); 
            PressAnyKey();
         }
   
         Sleep(SLEEP_TIME);
         
         LogPrint("DSP %d: Observe that the three user LEDs are blinking.........",ind,0,0,0);
         if ( DebugFlag ) PressAnyKey();

         status = execute_test(command,2,NULL,0,ind, WAIT);

         if ( status != EVM_OK )
         {
			if (status != EVM_FIFO_ERROR)
			{
               LogPrint("FAILED.\n",0,0,0,0);
			}
            if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
         }
         else
         {
             LogPrint("PASSED.\n\n",0,0,0,0);
         }
      }
   }

#endif 
   
   return(status);
}

/*************************************************************************/
/* do_memory_tests routine performs all memory tests                     */
/*************************************************************************/
int do_memory_tests(void)
{
   short   command[2];
   USHORT  errorInfo[8];
   ULONG   errorLen;
   int     i = 0, status = EVM_OK;
   char *filename = "memtst.out";
   
   LogPrint( "====================================================================\n",0,0,0,0);
   LogPrint( "===========            Memory Confidence Tests            ==========\n",0,0,0,0);
   LogPrint( "====================================================================\n",0,0,0,0);
   
   if ( DebugFlag ) PressAnyKey();
   
#if 1

   // Set timeout for 20 seconds
   evm55x_set_timeout(hBoard,20000);

   for (ind=0; ind<MAX_PCI2040_DSPS; ind++)
   {
       if (evm55x_dsp_present(hBoard, ind))
       {
		  /*************************************************************************/
		  /* Internal DARAM Memory Test                                            */
		  /*************************************************************************/
		  if (DebugFlag)
          {
             BLANK(stringBuffer);
             sprintf(stringBuffer,"Running DARAM memory test (A5) on DSP %d......................\n",ind); 
			 LogPrint(stringBuffer,0,0,0,0);
          }

		  // Reset/Unreset DSP to put in idle state
		  evm55x_reset_dsp(hBoard,ind);
		  evm55x_unreset_dsp(hBoard,ind);
		  
		  LogPrint("DSP %d: Int. DARAM Memory Test (A5)...........................",ind,0,0,0);

          if (doOnChipTest(MEM_A5_TEST, 0x60, 0x7FA0, ind) == EVM_OK)
		  {
		     LogPrint("PASSED.\n",0,0,0,0);
		  }
		  else 
		  {
            if ( bStopOnError ) return(EVM_ERROR);
		  }

		  if (DebugFlag)
          {
			 PressAnyKey();
             BLANK(stringBuffer);
             sprintf(stringBuffer,"Running DARAM memory test (WALK) on DSP %d....................\n",ind); 
			 LogPrint(stringBuffer,0,0,0,0);
          }

          LogPrint("DSP %d: Int. DARAM Memory Test (WALK).........................",ind,0,0,0);

          if (doOnChipTest(MEM_WALK_TEST, 0x60, 0x7FA0, ind) == EVM_OK)
		  {
		     LogPrint("PASSED.\n",0,0,0,0);
		  }
		  else 
		  {
            if ( bStopOnError ) return(EVM_ERROR);
		  }   
		  
		  if (DebugFlag)
          {
			 PressAnyKey();
             BLANK(stringBuffer);
             sprintf(stringBuffer,"Running DARAM memory test (ADDR) on DSP %d....................\n",ind); 
			 LogPrint(stringBuffer,0,0,0,0);
          }

          LogPrint("DSP %d: Int. DARAM Memory Test (ADDR).........................",ind,0,0,0);

          if (doOnChipTest(MEM_ADDR_TEST, 0x60, 0x7FA0, ind) == EVM_OK)
		  {
		     LogPrint("PASSED.\n",0,0,0,0);
		  }
		  else 
		  {
            if ( bStopOnError ) return(EVM_ERROR);
		  }  

		  if (DebugFlag)
          {
			 PressAnyKey();
             BLANK(stringBuffer);
             sprintf(stringBuffer,"Running DARAM memory test (PULSE) on DSP %d...................\n",ind); 
			 LogPrint(stringBuffer,0,0,0,0);
          }

          LogPrint("DSP %d: Int. DARAM Memory Test (PULSE)........................",ind,0,0,0);

          if (doOnChipTest(MEM_PULSE_TEST, 0x60, 0x7FA0, ind) == EVM_OK)
		  {
		     LogPrint("PASSED.\n",0,0,0,0);
		  }
		  else 
		  {
            if ( bStopOnError ) return(EVM_ERROR);
		  }          		  
   	   
		  LogPrint("\n",0,0,0,0);
         
          if ( DebugFlag ) PressAnyKey();

		  /*************************************************************************/
		  /* Internal SARAM Memory Test                                            */
		  /*************************************************************************/
		  if (DebugFlag)
          {
             BLANK(stringBuffer);
             sprintf(stringBuffer,"Running SARAM memory test (A5) on DSP %d......................\n",ind); 
			 LogPrint(stringBuffer,0,0,0,0);
          }

          LogPrint("DSP %d: Int. SARAM Memory Test (A5)...........................",ind,0,0,0);

		  // Reset/Unreset DSP to put in idle state that supports full HPI access to SARAM
		  evm55x_reset_dsp(hBoard,ind);
		  evm55x_unreset_dsp(hBoard,ind);

          if (doOnChipTest(MEM_A5_TEST, 0x8000, 0x1FFFF, ind) == EVM_OK)
		  {
		     LogPrint("PASSED.\n",0,0,0,0);
		  }
		  else 
		  {
            if ( bStopOnError ) return(EVM_ERROR);
		  }

		  if (DebugFlag)
          {
			 PressAnyKey();
             BLANK(stringBuffer);
             sprintf(stringBuffer,"Running SARAM memory test (WALK) on DSP %d....................\n",ind); 
			 LogPrint(stringBuffer,0,0,0,0);
          }

          LogPrint("DSP %d: Int. SARAM Memory Test (WALK).........................",ind,0,0,0);

          if (doOnChipTest(MEM_WALK_TEST, 0x8000, 0x1FFFF, ind) == EVM_OK)
		  {
		     LogPrint("PASSED.\n",0,0,0,0);
		  }
		  else 
		  {
            if ( bStopOnError ) return(EVM_ERROR);
		  }   
		  
		  if (DebugFlag)
          {
			 PressAnyKey();
             BLANK(stringBuffer);
             sprintf(stringBuffer,"Running SARAM memory test (ADDR) on DSP %d....................\n",ind); 
			 LogPrint(stringBuffer,0,0,0,0);
          }

          LogPrint("DSP %d: Int. SARAM Memory Test (ADDR).........................",ind,0,0,0);

          if (doOnChipTest(MEM_ADDR_TEST, 0x8000, 0x1FFFF, ind) == EVM_OK)
		  {
		     LogPrint("PASSED.\n",0,0,0,0);
		  }
		  else 
		  {
            if ( bStopOnError ) return(EVM_ERROR);
		  }          

		  if (DebugFlag)
          {
			 PressAnyKey();
             BLANK(stringBuffer);
             sprintf(stringBuffer,"Running SARAM memory test (PULSE) on DSP %d...................\n",ind); 
			 LogPrint(stringBuffer,0,0,0,0);
          }

          LogPrint("DSP %d: Int. SARAM Memory Test (PULSE)........................",ind,0,0,0);

          if (doOnChipTest(MEM_PULSE_TEST, 0x8000, 0x1FFFF, ind) == EVM_OK)
		  {
		     LogPrint("PASSED.\n",0,0,0,0);
		  }
		  else 
		  {
            if ( bStopOnError ) return(EVM_ERROR);
		  }          		  
   	   
		  LogPrint("\n",0,0,0,0);
         
          if ( DebugFlag ) PressAnyKey();		    
		   

		  /*************************************************************************/
		  /* Data Bus Test                                                         */
		  /*************************************************************************/
		  if (DebugFlag)
          {
             BLANK(stringBuffer);
             sprintf(stringBuffer,"Loading SBSRAM test on DSP %d...................\n",ind);
             LogPrint(stringBuffer,0,0,0,0);
          }
		  
	     // Load test application on DSP
      	 if (Load_Dsp_Test_Application(filename,ind) == EVM_ERROR)
	        return (EVM_ERROR);	
		  
          if ( DebugFlag ) 
          {
             printf( "Loaded DSP executable \'%s\' is now running!\n", filename ); 
             PressAnyKey();
          }
   
          Sleep(SLEEP_TIME);
         
          if ( DebugFlag ) PressAnyKey();
		  
		  
		  command[0] = DATA_BUS_TEST_NUM;
		  command[1] = DONT_WAIT_FOR_INPUT;

          LogPrint("DSP %d: SBSRAM Data Bus Test .................................",ind,0,0,0);

          // Set timeout for 2 seconds
		  evm55x_set_timeout(hBoard,2000);

          status = execute_test(command,2,NULL,0,ind, WAIT);

          if ( status != EVM_OK )
          {
             if (status != EVM_FIFO_ERROR)
			  {
				  LogPrint("FAILED.\n",0,0,0,0);

                  // Display error information
				  errorLen = 6;
				  if (!evm55x_fifo_read(hHpi, (PULONG)errorInfo, (PULONG)&errorLen, ind))
				  {
					 LogPrint("ERROR: evm55x_fifo_read() general failure during error information retrieval.\n", 0,0,0,0 );
					 LogPrint("ERROR: The DSP did not respond.\n",0,0,0,0);
				  }	 
				  else
				  {  // Got error info, now display it
		             sprintf(stringBuffer,"ERROR: Wrote 0x%04X%04X, but read back 0x%04X%04X at address 0x%04X%04X on DSP %d.\n", 
						     errorInfo[2], errorInfo[3], errorInfo[4], errorInfo[5], errorInfo[0], errorInfo[1],ind);
				     LogPrint(stringBuffer,0,0,0,0);
				  }  
				  // Restore default timeout
		          evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);
			  }
              if ( bStopOnError ) return(EVM_ERROR);
			  TotalErrors++;
          }
          else
          {
			  // Restore default timeout
		      evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);
              LogPrint("PASSED.\n",0,0,0,0);
          }

          Sleep(SLEEP_TIME);
         
          if ( DebugFlag ) PressAnyKey();

          /*********************************************************************/
          /* Address Bus Test                                                  */
          /*********************************************************************/
          command[0] = ADDRESS_BUS_TEST_NUM;
          command[1] = DONT_WAIT_FOR_INPUT;

          LogPrint("DSP %d: SBSRAM Address Bus Test ..............................",ind,0,0,0);

          // Set timeout for 2 seconds
		  evm55x_set_timeout(hBoard,2000);

          status = execute_test(command,2,NULL,0,ind, WAIT);

		  if (status != EVM_OK)
		  {
			  if (status != EVM_FIFO_ERROR)
			  {
				  LogPrint("FAILED.\n",0,0,0,0);

                  // Display error information
				  errorLen = 4;
				  if (!evm55x_fifo_read(hHpi, (PULONG)errorInfo, (PULONG)&errorLen, ind))
				  {
					 LogPrint("ERROR: evm55x_fifo_read() general failure during error information retrieval.\n", 0,0,0,0 );
					 LogPrint("ERROR: The DSP did not respond.\n",0,0,0,0);
				  }	 
				  else
				  {  // Got error info, now display it
		             sprintf(stringBuffer,"ERROR: Wrote 0x%04X, but read back 0x%04X at address 0x%04X%04X on DSP %d.\n", 
						     errorInfo[2], errorInfo[3], errorInfo[0], errorInfo[1],ind);
				     LogPrint(stringBuffer,0,0,0,0);
				  } 
				  // Restore default timeout
		          evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);
			  }
              if ( bStopOnError ) return(EVM_ERROR);
			  TotalErrors++;
          }
          else
          {
			  // Restore default timeout
		      evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);
              LogPrint("PASSED.\n",0,0,0,0);
          }

          /*********************************************************************/
          /* SBSRAM Device Test                                                */
          /*********************************************************************/
          command[0] = SBSRAM_DEVICE_TEST_NUM;
          command[1] = DONT_WAIT_FOR_INPUT;

          LogPrint("DSP %d: SBSRAM Device Test ...................................",ind,0,0,0);

          // Set timeout for 2 seconds
		  evm55x_set_timeout(hBoard,2000);

          status = execute_test(command,2,NULL,0,ind, WAIT);

		  if (status != EVM_OK)
		  {
			  if (status != EVM_FIFO_ERROR)
			  {
				  LogPrint("FAILED.\n",0,0,0,0);

                  // Display error information
				  errorLen = 4;
				  if (!evm55x_fifo_read(hHpi, (PULONG)errorInfo, (PULONG)&errorLen, ind))
				  {
					 LogPrint("ERROR: evm55x_fifo_read() general failure during error information retrieval.\n", 0,0,0,0 );
					 LogPrint("ERROR: The DSP did not respond.\n",0,0,0,0);
				  }	 
				  else
				  {  // Got error info, now display it
		             sprintf(stringBuffer,"ERROR: Wrote 0x%04X, but read back 0x%04X at address 0x%04X%04X on DSP %d.\n", 
						     errorInfo[2], errorInfo[3], errorInfo[0], errorInfo[1],ind);
				     LogPrint(stringBuffer,0,0,0,0);
				  }     
			  }

			  // Restore default timeout
		      evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);

              if ( bStopOnError ) return(EVM_ERROR);
			  TotalErrors++;
          }
          else
          {
			  // Restore default timeout
		      evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);
              LogPrint("PASSED.\n\n",0,0,0,0);
          }
       }
   }
   
#endif
   
   return(status);
}

#if 1
int doOnChipTest(MemTestNumber testNum, ULONG addr, ULONG len, ULONG dspNum)
{
    USHORT *writeBuf;
    USHORT *readBuf;
    ULONG i, cnt = len;

    if ((writeBuf = (PUSHORT) malloc (len * sizeof(short))) == NULL)
	{
       LogPrint("FAILED.\n",0,0,0,0);
	   LogPrint( "ERROR: Host memory could not be allocated for the write buffer.\n", 0,0,0,0 );
	   TotalErrors++;
	   return (EVM_ERROR);
	}

    if ((readBuf = (PUSHORT) malloc (len * sizeof(short))) == NULL)
	{
       free(writeBuf);
	   LogPrint("FAILED.\n",0,0,0,0);
	   LogPrint( "ERROR: Host memory could not be allocated for the read buffer.\n", 0,0,0,0 );
	   TotalErrors++;
	   return (EVM_ERROR);		
	}

    //generate buffer based on test
    switch (testNum)
    {
    case MEM_A5_TEST:
        for (i=0; i<cnt; i++)
        {
            writeBuf[i] = (i&0x1) ? 0xaaaa : 0x5555;
            readBuf[i] = 0;
        }
        break;

    case MEM_WALK_TEST:
        for (i=0; i<cnt; i++)
        {
            writeBuf[i] = (i&0x1) ? 0xaaaa : 0x5555;
            readBuf[i] = 0;
        }
        break;
     
    case MEM_ADDR_TEST:
        for (i=0; i<cnt; i++)
        {
            writeBuf[i] = (USHORT)(addr + i);
            readBuf[i] = 0;
        }
        break;

    case MEM_PULSE_TEST:
        for (i=0; i<cnt; i++)
        {
            writeBuf[i] = (i&0x1) ? 0x0000 : 0xffff;
            readBuf[i] = 0;
        }
        break;

    default:
        return (EVM_OK);
    }

    if (!evm55x_hpi_write(hHpi, (PULONG)writeBuf, &cnt, addr, dspNum))
	{
	    LogPrint("FAILED.\n",0,0,0,0);
		LogPrint( "ERROR: evm55x_hpi_write() failed for DSP %d.\n", dspNum,0,0,0 );
	    TotalErrors++;
		return EVM_ERROR;
	}

    if (!evm55x_hpi_read(hHpi, (PULONG)readBuf, &cnt, addr, dspNum))
	{
	    LogPrint("FAILED.\n",0,0,0,0);
		LogPrint( "ERROR: evm55x_hpi_read() failed for DSP %d.\n", dspNum,0,0,0 );
		TotalErrors++;
        return EVM_ERROR;
	}
    
    for (i=0; i<len; i++)
    {
        if (writeBuf[i] != readBuf[i])
		{
	        LogPrint("FAILED.\n",0,0,0,0);
		    LogPrint( "ERROR: Wrote 0x%04X, but read back 0x%04X at address 0x%X on DSP %d.\n", writeBuf[i], readBuf[i],addr+i, dspNum );
            TotalErrors++;
			return (EVM_ERROR);
		}
    }

    // Free memory
	free(writeBuf);
	free(readBuf);

	return (EVM_OK);
}
#endif

/*****************************************************************************/
/* do_PCI2040_tests()
/*****************************************************************************/
int do_PCI2040_tests(void)
{
	int  status = EVM_OK;
	Register reg = {0, 0};
	UINT hsStat = 0;
	
	PCI2040_CONFIG_DATA pci_regs;
	
	LogPrint( "\n====================================================================\n",0,0,0,0);
	LogPrint( "===========  On-board PCI Controller (PCI2040) Settings   ==========\n",0,0,0,0);
	LogPrint( "====================================================================\n",0,0,0,0);
	
	if ( DebugFlag ) PressAnyKey();
	
	if ( !evm55x_get_pci_configuration(hBoard, &pci_regs) )
	{
		if (DebugFlag) printf("\n**Error: Failure getting PCI info\n"); 
		
		return(EVM_ERROR);
	}
	
	if (1)
	{
		LogPrint("VendorID            = 0x%X\n", pci_regs.VendorID, 0,0,0); 
		LogPrint("DeviceID            = 0x%X\n", pci_regs.DeviceID,0,0,0); 
		LogPrint("Command             = 0x%X\n", pci_regs.Command,0,0,0); 
		LogPrint("Status              = 0x%X\n", pci_regs.Status,0,0,0); 
		LogPrint("RevisionID          = 0x%X\n", pci_regs.RevisionID,0,0,0); 
		LogPrint("ProgIf              = 0x%X\n", pci_regs.VendorID,0,0,0); 
		LogPrint("SubClass            = 0x%X\n", pci_regs.SubClass,0,0,0); 
		LogPrint("BaseClass           = 0x%X\n", pci_regs.BaseClass,0,0,0); 
		LogPrint("CacheLineSize       = 0x%X\n", pci_regs.CacheLineSize,0,0,0); 
		LogPrint("LatencyTimer        = 0x%X\n", pci_regs.LatencyTimer,0,0,0); 
		LogPrint("HeaderType          = 0x%X\n", pci_regs.HeaderType,0,0,0); 
		LogPrint("BIST                = 0x%X\n", pci_regs.BIST,0,0,0); 
		LogPrint("HPICSRBar           = 0x%X\n", pci_regs.HPICSRBar,0,0,0); 
		LogPrint("CSpaceBar           = 0x%X\n", pci_regs.CSpaceBar,0,0,0); 
		LogPrint("GPBBar              = 0x%X\n", pci_regs.GPBBar,0,0,0); 
		LogPrint("SubVendorID         = 0x%X\n", pci_regs.SubVendorID,0,0,0); 
		LogPrint("SubSysID            = 0x%X\n", pci_regs.SubSysID,0,0,0); 
		LogPrint("CapPointer          = 0x%X\n\n", pci_regs.CapPointer,0,0,0);
		
		if (userIn)
			PressAnyKey();
		else
		{
			if (!noDelay)
			{
			   LogPrint("\nWill proceed in 5 seconds....\n",0,0,0,0);
			   Sleep(5000);
			}
		}
	
	    LogPrint( "\n====================================================================\n",0,0,0,0);
	    LogPrint( "===========  On-board PCI Controller (PCI2040) Settings   ==========\n",0,0,0,0);
	    LogPrint( "====================================================================\n",0,0,0,0);
	
		LogPrint("IRQLine             = 0x%X\n", pci_regs.IRQLine,0,0,0); 
		LogPrint("IRQPin              = 0x%X\n", pci_regs.IRQPin,0,0,0); 
		LogPrint("MinGnt              = 0x%X\n", pci_regs.MinGnt,0,0,0); 
		LogPrint("MaxLat              = 0x%X\n", pci_regs.MaxLat,0,0,0); 
		LogPrint("GPBSelect           = 0x%X\n", pci_regs.GPBSelect,0,0,0); 
		LogPrint("GPBInData           = 0x%X\n", pci_regs.GPBInData,0,0,0); 
		LogPrint("GPBOutData          = 0x%X\n", pci_regs.GPBOutData,0,0,0); 
		LogPrint("GPBInType           = 0x%X\n", pci_regs.GPBIntType,0,0,0); 
		LogPrint("MiscControl         = 0x%X\n", pci_regs.MiscControl,0,0,0); 
		LogPrint("Diagnostic          = 0x%X\n", pci_regs.Diag,0,0,0); 
		LogPrint("PMCapID             = 0x%X\n", pci_regs.PMCapID,0,0,0); 
		LogPrint("PMNext              = 0x%X\n", pci_regs.PMNext,0,0,0); 
		LogPrint("PMCap               = 0x%X\n", pci_regs.PMCap,0,0,0); 
		LogPrint("PMCntlStat          = 0x%X\n", pci_regs.PMCntlStat,0,0,0); 
		LogPrint("HPICSRIOBar         = 0x%X\n", pci_regs.HPICSRIOBar,0,0,0); 
		LogPrint("HSCapID             = 0x%X\n", pci_regs.HSCapID,0,0,0); 
		LogPrint("HSNext              = 0x%X\n", pci_regs.HSNext,0,0,0); 
		LogPrint("HSCSR               = 0x%X\n", pci_regs.HSCSR,0,0,0);

		if (userIn)
			PressAnyKey();
		else
		{ 
			if (!noDelay)
			{
			   LogPrint("\nWill proceed in 5 seconds....\n",0,0,0,0);
			   Sleep(5000);
			}
		}
		
	}
	
	
	if ( !evm55x_store_pci_configuration(hBoard, &pci_regs) )
	{
		if (DebugFlag) printf("\n**Error: Failure storing PCI info\n"); 
		
		return(EVM_ERROR);
	}
	
	return(status);
}

/*****************************************************************************/
/* do_jtag_tests()
/*****************************************************************************/
#define REG_NUM 2

int do_jtag_tests(void)
{
   int  status = EVM_OK,i;
   Register reg[REG_NUM] = {{0xc, 0}, {0xe,0}};
   USHORT mask[REG_NUM]  = {0x1234, 0x5678};

   LogPrint( "\n====================================================================\n",0,0,0,0);
   LogPrint( "===========              JTAG Confidence Test             ==========\n",0,0,0,0);
   LogPrint( "====================================================================\n",0,0,0,0);
   
   if ( DebugFlag ) PressAnyKey();
 
   LogPrint("JTAG Test Bus Controller Test................................",ind,0,0,0);

   Sleep(500);

   for (i=0; i<REG_NUM; i++)
   {
      //test TBC COUNTER1 UPDATE0 reg
      if (!verifyReg(&reg[i], mask[i]))
      {
         if (DebugFlag) 
         {
            printf("**Error: Failure writing or reading TBC register at offset %d\n", 
            reg[i].Regoff); 
         }
         
         return(EVM_ERROR);
      }
   }

   LogPrint("PASSED.\n\n",ind,0,0,0);

   return (status);
}


/****************************************************************************/
/* do_flash_tests routine
/****************************************************************************/
int do_flash_tests(void)
{
   short command[2];
   int status = EVM_OK;
   char *filename = "flashtst.out";
   BOOL flashFailed = FALSE;
      
#if 1
   /*************************************************************************/
   /* Begin Tests
   /*************************************************************************/
   for (ind=0; ind<MAX_PCI2040_DSPS; ind++)
   {
      if (evm55x_dsp_present(hBoard, ind))
      {
         if ( DebugFlag )
         {
             BLANK(stringBuffer);
             sprintf(stringBuffer,"Loading Flash test on DSP %d..............\n",ind);
             LogPrint(stringBuffer,0,0,0,0);
         }
         
	     // Load test application on DSP
      	 if (Load_Dsp_Test_Application(filename,ind) == EVM_ERROR)
	        return (EVM_ERROR);	

         if ( DebugFlag ) 
         {
            printf( "Loaded DSP executable \'%s\' is now running!\n", filename ); 
            PressAnyKey();
         }
   
         Sleep(SLEEP_TIME);


         /*************************************************************************/
         /* Store Flash image to SBSRAM                                           */
         /*************************************************************************/
		 command[0] = FLASH_SAVE_TEST;
         command[1] = DONT_WAIT_FOR_INPUT;
          
 		 LogPrint("DSP %d: Flash Storage to SBSRAM Test..........................",ind,0,0,0);
         if ( DebugFlag ) PressAnyKey();
                 
         // Set timeout for 10 seconds
		 evm55x_set_timeout(hBoard,10000);

         status= execute_test(command,2,NULL,0,ind,WAIT);

		 // Restore default timeout
		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);

         if ( status != EVM_OK )
         {
			flashFailed = TRUE;

            if (status != EVM_FIFO_ERROR)
			{
			   LogPrint(" FAILED.\n",0,0,0,0);
			}
            if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
         }
         else
         {
            LogPrint("PASSED.\n",0,0,0,0);
         }
   

         /*************************************************************************/
         /* Write an alternating A's and 5's pattern to each page of the FLASH    */
         /*************************************************************************/
         command[0] = FLASH_A5_TEST;
         command[1] = DONT_WAIT_FOR_INPUT;

         LogPrint("DSP %d: Flash Memory A5 Test..................................",ind,0,0,0);
         if ( DebugFlag ) PressAnyKey();
         
         // Set timeout for 10 seconds
		 evm55x_set_timeout(hBoard,10000);

         status= execute_test(command,2,NULL,0,ind,WAIT);

		 // Restore default timeout
		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);

         if ( status != EVM_OK )
         {
			flashFailed = TRUE;

            if (status != EVM_FIFO_ERROR)
			{
			   LogPrint(" FAILED.\n",0,0,0,0);
			}
            if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
         }
         else
         {
            LogPrint("PASSED.\n",0,0,0,0);
         }
   
         /*************************************************************************/
         /* Perform FLASH erase test                                              */
         /*************************************************************************/
         command[0] = FLASH_ERASE_TEST;
         command[1] = DONT_WAIT_FOR_INPUT;
  
         LogPrint("DSP %d: Flash Memory Erase Test...............................",ind,0,0,0);
         if ( DebugFlag ) PressAnyKey();

         // Set timeout for 10 seconds
		 evm55x_set_timeout(hBoard,10000);
         
         status= execute_test(command,2,NULL,0,ind,WAIT);

		 // Restore default timeout
		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);

         if ( status != EVM_OK )
		 {
			flashFailed = TRUE;

            if (status != EVM_FIFO_ERROR)
			{
			   LogPrint(" FAILED.\n",0,0,0,0);
			}            
            if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
         }
         else
         {
            LogPrint("PASSED.\n",0,0,0,0);
         }


         /*************************************************************************/
         /* Restore Flash image from SBSRAM                                       */
         /*************************************************************************/
         command[0] = FLASH_RESTORE_TEST;
         command[1] = DONT_WAIT_FOR_INPUT;
          
 		 LogPrint("DSP %d: Restoring Flash from SBSRAM Test......................",ind,0,0,0);
 
		 if ( DebugFlag ) PressAnyKey();
                 
         // Set timeout for 20 seconds
		 evm55x_set_timeout(hBoard,20000);

         status= execute_test(command,2,NULL,0,ind,WAIT);

		 // Restore default timeout
		 evm55x_set_timeout(hBoard,DEFAULT_TIMEOUT);

         if ( status != EVM_OK )
         {
			flashFailed = TRUE;

            if (status != EVM_FIFO_ERROR)
			{
			   LogPrint(" FAILED.\n",0,0,0,0);
			}
            if ( bStopOnError ) return(EVM_ERROR);
			TotalErrors++;
         }
         else
         {
            LogPrint("PASSED.\n",0,0,0,0);
         }
      }
   }

 
#endif
   if (!flashFailed) return(EVM_OK);
   else              return(EVM_ERROR);
}


/*****************************************************************************/
/* verifyReg()
/*****************************************************************************/
BOOL verifyReg(PRegister pReg, USHORT mask)
{
    ULONG newVal;
    int status = FALSE;
    
    //change register value
    newVal = pReg->DataVal = mask; 
    
    //update reg with new value
    if (!evm55x_write_tbc_reg(hBoard, pReg))
    {
        return (status);
    }

    pReg->DataVal = 0;
    
    //read reg
    if (!evm55x_read_tbc_reg(hBoard, pReg))
    {
        return (status);
    }
    
    if (newVal == pReg->DataVal)
        status = TRUE;
    
    return (status);
}

/*****************************************************************************
/* do_cpld_tests
/*****************************************************************************/
int do_cpld_tests()
{
   USHORT  regs[6];
   
   if (!evm55x_cpld_read_all(hBoard, regs))
   {
      LogPrint("ERROR: evm55x_cpld_read_all failed\n",0,0,0,0);
      return (ERROR);
   }

   LogPrint("====================================================================\n",0,0,0,0);
   LogPrint("=                       CPLD Registers Dump                        =\n",0,0,0,0);
   LogPrint("====================================================================\n",0,0,0,0);
   LogPrint( "CNTL          : 0x%02X\n", regs[0] & 0xff ,0,0,0);
   LogPrint( "STAT1         : 0x%02X\n", regs[1] & 0xff ,0,0,0);
   LogPrint( "STAT2         : 0x%02X\n", regs[2] & 0xff ,0,0,0);
   LogPrint( "CPLDREV       : 0x%02X   (%d)\n", regs[3] & 0xff , regs[3]& 0xff,0,0);

   LogPrint( "SEM0          : 0x%02X   ", regs[4] & 0xff,0,0,0);
   if (regs[4] & 1) 
      LogPrint("(Owned)\n",0,0,0,0);
   else             
      LogPrint("(Not Owned)\n",0,0,0,0);
   
   LogPrint( "SEM1          : 0x%02X   ", regs[5] & 0xff,0,0,0 );
   if (regs[5] & 1) 
      LogPrint("(Owned)\n",0,0,0,0);
   else             
      LogPrint("(Not Owned)\n",0,0,0,0);
   
   LogPrint("====================================================================\n",0,0,0,0);

   if (userIn)
		PressAnyKey();
   else
   {
	    if (!noDelay)
		{
		   LogPrint("\nWill proceed in 5 seconds....\n",0,0,0,0);
		   Sleep(5000);
		}
   }

   LogPrint("\n\n\n",0,0,0,0);
   LogPrint( "====================================================================\n",0,0,0,0);
   LogPrint( "===========                  Board State                  ==========\n",0,0,0,0);
   LogPrint( "====================================================================\n",0,0,0,0);

   if (regs[0] & 0x01)
   {
      LogPrint("DSP Interrupt Mask: YES\n",0,0,0,0);
   }
   else
   {
      LogPrint("DSP Interrupt Mask: NO\n",0,0,0,0);
   }

   if (regs[0] & 0x02)
   {
      LogPrint("DSP NMI           : YES\n",0,0,0,0);
   }
   else
   {
      LogPrint("DSP NMI           : NO\n",0,0,0,0);
   }

   if (regs[0] & 0x04)
   {
      LogPrint("TBC Reset         : YES\n",0,0,0,0);
   }
   else
   {
      LogPrint("TBC Reset         : NO\n",0,0,0,0);
   }

    if (regs[0] & 0x08)
   {
      LogPrint("DSP Reset         : YES\n",0,0,0,0);
   }
   else
   {
      LogPrint("DSP Reset         : NO\n",0,0,0,0);
   }

   if (regs[0] & 0x10)
   {
      LogPrint("HINT Mask         : YES\n",0,0,0,0);
   }
   else
   {
      LogPrint("HINT Mask         : NO\n",0,0,0,0);
   }
   
   if (regs[1] & 0x02)
   {
      LogPrint("User Switch 1     : ON\n",0,0,0,0);
   }
   else
   {
      LogPrint("User Switch 1     : OFF\n",0,0,0,0);
   }
   
   if (regs[1] & 0x01)
   {
      LogPrint("User Switch 0     : ON\n",0,0,0,0);
   }
   else
   {
      LogPrint("User Switch 0     : OFF\n",0,0,0,0);
   }


   if (regs[1] & 0x04)
   {
      LogPrint("DSP EHPIENA       : ON\n",0,0,0,0);
   }
   else
   {
      LogPrint("DSP EHPIENA       : OFF\n",0,0,0,0);
   }

   if (regs[1] & 0x08)
   {
      LogPrint("DSP Operating Mode: MP\n",0,0,0,0);
   }
   else
   {
      LogPrint("DSP Operating Mode: MC\n",0,0,0,0);
   }

   if (regs[1] & 0x10)
   {
      LogPrint("TBC Ready         : YES\n",0,0,0,0);
   }
   else
   {
      LogPrint("TBC Ready         : NO\n",0,0,0,0);
   }

   if (regs[2] & 0x01)
   {
      LogPrint("DSP Core Voltage  : BAD\n",0,0,0,0);
   }
   else
   {
      LogPrint("DSP Core Voltage  : OK\n",0,0,0,0);
   }

   if (regs[2] & 0x02)
   {
      LogPrint("Daughterboard     : INSTALLED\n",0,0,0,0);
   }
   else
   {
      LogPrint("Daughterboard     : NONE\n",0,0,0,0);
   }

   if (regs[2] & 0x04)
   {
      LogPrint("DSP-Host Interrupt: ACTIVE\n",0,0,0,0);
   }
   else
   {
      LogPrint("DSP-Host Interrupt: NOT ACTIVE\n",0,0,0,0);
   }

   if (regs[2] & 0x08)
   {
      LogPrint("DSP HREADY        : READY\n",0,0,0,0);
   }
   else
   {
      LogPrint("DSP HREADY        : NOT READY\n",0,0,0,0);
   }

   if (regs[4] & 0x01)
   {
      LogPrint("Semaphore 0       : OWNED\n",0,0,0,0);
   }
   else
   {
      LogPrint("Semaphore 0       : NOT OWNED\n",0,0,0,0);
   }
   
   if (regs[5] & 0x01)
   {
      LogPrint("Semaphore 1       : OWNED\n",0,0,0,0);
   }
   else
   {
      LogPrint("Semaphore 1       : NOT OWNED\n",0,0,0,0);
   }

   LogPrint("====================================================================\n",0,0,0,0);


   return (OK);
}

/*****************************************************************************/
/* execute_test() - Send command to EVM and return test result               */
/*****************************************************************************/
int execute_test(short *sendBuf, ULONG sendWords, 
				 short *rcvBuf,  ULONG rcvWords, UINT dspNum, int wait)
{
   
   int   returnVal = EVM_ERROR;
   ULONG sendLen = sendWords;
   ULONG rcvLen  = rcvWords;
   short ret = 0;
   ULONG i;
   
   /***************************************************************************/
   /* send command with handshaking acks                                      */
   /***************************************************************************/
   
   if ( !evm55x_fifo_write(hHpi, (PULONG)sendBuf, (PULONG)&sendLen, dspNum ))
   {
      LogPrint( "FAILED.\nERROR: evm55x_fifo_write() general failure.\n", 0,0,0,0 );
      return(EVM_FIFO_ERROR);
   }
   
   if ( DebugFlag ) 
   { 
      printf("Sent...\n"); 
      
      for (i=0; i<sendWords; i++)
      {
         printf("%X\n",sendBuf[i]);  
      }
      
      PressAnyKey();
   }
   
   if ( DebugFlag ) 
   { 
	   printf("Getting Test Result.\n");  
	   PressAnyKey(); 
   }
   
   Sleep(100);
   
   if (wait == WAIT)
   {
	   short * pReadBuf;
	   ULONG   ReadLen;

	   // Get response to test 
       pReadBuf = &ret;
	   ReadLen = 1;        

       if (!evm55x_fifo_read(hHpi, (PULONG)pReadBuf, (PULONG)&ReadLen, dspNum))
       {
           LogPrint("FAILED.\nERROR: evm55x_fifo_read() general failure.\n", 0,0,0,0 );
           LogPrint("ERROR: The DSP did not respond.\n",0,0,0,0);
           return(EVM_FIFO_ERROR);
       }
       
       // Get return value
	   returnVal = (USHORT)*pReadBuf;

       if ( DebugFlag ) 
       { 
           printf("Test Result = %d\n\n",returnVal); 
	   }


       // Check if test provides additional data
	   if ((rcvBuf != NULL) && rcvWords)
	   {
          // Buffer of additional data is requested
          pReadBuf = rcvBuf;
		  ReadLen = rcvWords;

		  // Get additional data
          if (!evm55x_fifo_read(hHpi, (PULONG)pReadBuf, (PULONG)&ReadLen, dspNum))
		  {
              LogPrint("FAILED.\nERROR: evm55x_fifo_read() general failure.\n", 0,0,0,0 );
              LogPrint("ERROR: The DSP did not respond.\n",0,0,0,0);
              return(EVM_FIFO_ERROR);
		  }	   

		  // Debug output
		  if (DebugFlag)
		  {
             ULONG i;
			  
	         for (i=0; i<rcvWords; i++)
			 {
                sprintf(stringBuffer,"Receive Data[%d] = 0x%04X\n",i,pReadBuf[i]);
				LogPrint(stringBuffer,0,0,0,0);
			 }
		     printf("\n");
		   
             PressAnyKey(); 
		  }
	   }
	   
	   return(returnVal);
   }
   else
   {
       return(EVM_ERROR);
   }
   
}

/*************************************************************************/
/* Waits for interrupts from the specified dsp                           */
/*************************************************************************/
int waitForInterrupt(HANDLE hint)
{
   //wait a couple of seconds for interrupts
   if (WaitForSingleObject(hint, 2000) == WAIT_OBJECT_0)
      return EVM_OK;

   return EVM_ERROR;
}

/*****************************************************************************/
/* help() -- Provide assistance to the naive, then exit.                     */
/*****************************************************************************/
static void help( void )
{
   printf(helpTxt);

   ProgramExit(1);
}


//
//  FUNCTION: ControlHandler ( DWORD dwCtrlType )
//
//  PURPOSE: Handled console control events
//
//  PARAMETERS:
//    dwCtrlType - type of control event
//
//  RETURN VALUE:
//    True - handled
//    False - unhandled
//
//  COMMENTS:
//
static BOOL WINAPI ControlHandler ( DWORD dwCtrlType )
{
    switch( dwCtrlType )
    {
        case CTRL_BREAK_EVENT:  // use Ctrl+C or Ctrl+Break to simulate
        case CTRL_C_EVENT:      // SERVICE_CONTROL_STOP in debug mode
			
			// record abort time
		    LogPrint("\n\nTesting Aborted: ",0,0,0,0);
            GetTimeFormat( LOCALE_SYSTEM_DEFAULT, 0, NULL,
                 "hh':'mm':'ss tt", stringBuffer, STRING_BUFFER_SIZE );
            LogPrint( stringBuffer, 0,0,0,0 );
            LogPrint( " on ", 0,0,0,0 );
            GetDateFormat( LOCALE_SYSTEM_DEFAULT, 0, NULL,
                 "MMM dd',' yyyy", stringBuffer, STRING_BUFFER_SIZE );
            LogPrint( stringBuffer, 0,0,0,0 );

		    // If continuous loops, identify loop count
			if (ContLoop)
			{
	           LogPrint( "\n====================================================================\n",0,0,0,0);
	           LogPrint(   "Continuous Loop Test Results:\n",0,0,0,0);
	           LogPrint(   "   Loop Count          = %d\n",LoopCount,0,0,0);
	           LogPrint(   "   Total Test Failures = %d\n",TotalErrors,0,0,0);
	           LogPrint( "====================================================================\n\n",0,0,0,0);	  
			}
			else
			{
               LogPrint("\n\nTotal Test Failures = %d\n\n",TotalErrors,0,0,0);
			}  
			
            // re-enable default printf for driver functions
			evm55x_printf_control(TRUE,NULL);

			// close file if opened
            if ( pLogFile != NULL ) 
			{
                // close log file
				fclose( pLogFile );
			}

			if (bStopAtEnd) 
			{
				char c;

				printf( "\nPress any key to return to the control panel." );
   
				c = _getch();
			}

			exit(1);
    }
    return FALSE;
}



static int Load_Dsp_Test_Application(char *app, int dsp)
{
	char strBuffer[255];
	char *evm55x_var;

    // Check for EVM55X environment variable
	if ((evm55x_var = getenv( "EVM55X_DIR" )) != NULL)
	{  // Environment variable, use comm55x.out in bin subdirectory
	   sprintf(strBuffer,"%s\\conftest\\%s",evm55x_var,app);
	}
	else 
	{  // Environment variable not set, use current directory
       sprintf(strBuffer,"%s",app);
	}

    if ( !evm55x_coff_load(hBoard, NULL, strBuffer, FALSE, FALSE, FALSE, ind) )
	{
	   sprintf(strBuffer,"\nERROR: evm55x_coff_load() failed loading \"%s\" on DSP %d.\n", app,dsp);
	   LogPrint(strBuffer,0,0,0,0 );
			 
	   return(EVM_ERROR);
	}

	return(EVM_OK);
}


static void ProgramExit(int exitcode)
{
    if (bStopAtEnd) 
	{
		char c;

		printf( "\nPress any key to return to the control panel." );
   
		c = _getch();
	}
    exit(exitcode);
}
