/* fifo_file.c */
/*----------------------------------------------------------------------------*/
/* This example shows how the host and DSP can use the software FIFOs in      */
/* DSP memory to transfer data to each other.  Data from a host file is       */
/* sent to the DSP from the host through the FIFO.  Data from the DSP to the  */
/* host is sent through the FIFO and stored to a file on the host.            */                                                                 
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* INCLUDES                                                                   */
/*----------------------------------------------------------------------------*/
#include <windows.h>
#include <stdio.h>
#include <malloc.h>
#include <io.h>
#include <fcntl.h>
#include "evm55xdll.h"
#include "fifo_file.h"


/*--------------------------------------------------------------------------*/
/*  Local #defines                                                          */
/*--------------------------------------------------------------------------*/
#define FILE_LEN                    256
#define READ_BUFFER_SIZE             32      
#define WRITE_BUFFER_SIZE            32      

/*--------------------------------------------------------------------------*/
/*  File-local (static) variables                                           */
/*--------------------------------------------------------------------------*/
static  HANDLE    hBd;
static 	LPVOID	  hHpi; 
static  char     *sDspAppName    = "fifo.out";
static  char     *sReadFileName  = "ToDsp.bin";
static  char     *sWriteFileName = "FromDsp.bin";
static  DWORD    readFileLen = 0;
static  FILE     *hReadFile = NULL;
static  FILE     *hWriteFile = NULL;

/*--------------------------------------------------------------------------*
*  File-local (static) function prototypes
*--------------------------------------------------------------------------*/
static void writeBin();
static void exitApp (int i);
static void doWriteFile(DWORD WriteFile16BitWords);
static void doReadFile(DWORD ReadFile16BitWords);


/*--------------------------------------------------------------------------*/
/* FUNCTIONS                                                                */
/*--------------------------------------------------------------------------*/


/******************************************************************************/
/*   main() - Supports simple file I/O to DSP using FIFOs                     */
/*                                                                            */
/*    This function handles opening of of play and record files, and starting */
/*    play and record threads.  The play and record processes may be opened   */
/*    and run independently or simultaneously.  The play process runs         */
/*    until data is exhausted or a <Ctrl-Z> is entered.  The record process   */
/*    runs indefinetly or until <Ctrl-Z> is enterned or until an optional     */
/*    time limit is reached or until an an optional file size limit is reached*/
/*                                                                            */
/******************************************************************************/
int main ()
{
	int         fh;
	
	
	printf("\nFIFO and File I/O Example\n");
	printf("=========================\n");

    /* always create binary input file (don't depend on it being there) */
	printf("Creating binary input file \"%s\"for FIFO transmit...\n",sDspAppName);
	writeBin();

    /* open DSP board */
	printf("Opening DSP board...\n");
    hBd = evm55x_open( 0,TRUE );

    if ( hBd == INVALID_HANDLE_VALUE )
    {
	   printf( "**Error: Board open failure.\n" );  
	   exit(-1);
	}

	/* open HPI */
	printf("Opening HPI...\n");
	hHpi = evm55x_hpi_open(hBd);
   
    if ( hHpi == NULL )
    {
	   printf( "**Error: evm55x_hpi_open() failed.\n" ); 
	   evm55x_close(hBd);
	   exit(-1);
    }
   	
	/* reset board */
	printf("Resetting board...\n");
	if ( !evm55x_reset_board(hBd) )
	{
	    printf( "**Error: evm55x_reset_board() failed.\n" );
		exitApp(-1);
	}

	/* download test FIFO routine to DSP */
	printf("Downloading DSP application (%s) to DSP\n",sDspAppName);
	if (!evm55x_coff_load(hBd,hHpi,sDspAppName,0,0,0,0))
	{
		printf( "\n**Error: Failed to open \"%s\" for reading.\n",
			    sDspAppName );
		exitApp(-1);
	}
	
	/* make sure file does not already exist */
	printf("Checking to see if binary output file from DSP already exists...\n");
	if ( (hWriteFile = fopen( sWriteFileName, "rb" )) != NULL )
	{
		char ow;

        fclose(hWriteFile);
				
		printf( "\n**Warning: Write file \"%s\" already exists.\n",
				sWriteFileName );
		printf( "Enter 'y' to overwrite: ");
				
		scanf("%c",&ow);
				
		if(!(ow == 'y' || ow == 'Y'))
		{
			exitApp(-1);
		}
	}
			
	/* open write file */
	printf("Creating new binary output file...\n");
	if ( (hWriteFile = fopen( sWriteFileName, "wb" )) == NULL )
	{
		printf( "\n**Error: Failed to open \"%s\" for writing.\n",
				sWriteFileName );
		exitApp(-1);
	}
		
	/* open read file */
	printf("Opening input binary file to send to DSP...\n");
	if ( (hReadFile = fopen( sReadFileName, "rb" )) == NULL )
	{
		printf( "\n**Error: Failed to open \"%s\" for reading.\n",
			    sReadFileName );
		exitApp(-1);
	}


	/* determine size of read file */
	// close file temporarily 
	fclose(hReadFile);
		
	// open binary file
	fh = _open( sReadFileName, _O_RDONLY | _O_BINARY );

	// Determine size of binary file in 16-bit words
	readFileLen = (ULONG)_lseek(fh,0,SEEK_END) >> 1;

	printf("File size in 16-bit words is %d.\n",readFileLen);

    // Close and reopen file
	_close(fh);
	hReadFile = fopen( sReadFileName, "rb");
		
    /* NOTE: You could have separate write file and read file threads
	to support concurrent FIFO and file operations.  However, for 
	simplicity and to focus on how to use the FIFOs, data is first sent
	to the DSP and then read back in a sequential manner. */
	
	/* read data from File and write to DSP */
	printf("Reading file \"%s\" and transmitting data to DSP...\n",sReadFileName);
	doReadFile(readFileLen);
	
	/* read data from DSP and write to file */
	printf("Reading DSP FIFO and storing to file \"%s\"\n",sWriteFileName);
	doWriteFile(readFileLen);
	
    /* exit application */
	printf("\nExample application completed successfully!\n");
	exitApp(0);

} /* end of main() */


/******************************************************************************/
/*   exitApp() - Closes files and handles                                     */
/******************************************************************************/
static void exitApp (int i)
{
	if (i != 0)
	{

       printf("\nExample application DID NOT complete successfully.\n");
	}

	/* clean up */
	if (hReadFile != NULL) fclose( hReadFile ); 
	if (hWriteFile != NULL)fclose( hWriteFile );

	evm55x_hpi_close(hHpi);
	evm55x_close(hBd);

	exit(i);
}

/****************************************************************************/
/*doWriteFile() - Read data from DSP and write to file
/*
/****************************************************************************/
static void doWriteFile(DWORD WriteFile16BitWords)
{
	ULONG       ulLength, byteLen;
	ULONG       ulTotalLength = 0;
    LPDWORD     lpWriteFileBuffer = NULL;


    // set timeout value for FIFO transfers
	evm55x_set_timeout(hBd,1000);

	// allocate write buffer
	lpWriteFileBuffer = (LPDWORD)malloc(WRITE_BUFFER_SIZE*sizeof(USHORT));
	if ( lpWriteFileBuffer == NULL )
	{
		printf( "\n**Error: malloc() failed.\n" );
		exitApp(-1);
	}
	
	while ( ulTotalLength < WriteFile16BitWords) 
	{
		/* determine the length to request */
		ulLength = WriteFile16BitWords - ulTotalLength;
		if ( ulLength > WRITE_BUFFER_SIZE )
		{   // limit to WRITE_BUFFER_SIZE maximum at each FIFO read
			ulLength = WRITE_BUFFER_SIZE;
		}

		/* request data from DSP */
		evm55x_fifo_read(hHpi, 
			(LPDWORD)lpWriteFileBuffer, 
			&ulLength,
			0);

        /* if data received from DSP... */
	    if (ulLength)
		{  
	 		/* write received data to file */
			byteLen = ulLength << 1;
			if ( byteLen > 0 )
			{
				size_t  temp;
				char    *curpos=(char *)lpWriteFileBuffer;

				temp = fwrite( curpos, sizeof(USHORT), ulLength, hWriteFile );
				if ( temp != ulLength )
				{
					/* error condition */
					free(lpWriteFileBuffer);
			        exitApp(-1);
				} 
			
				ulTotalLength += ulLength;
			}	
		}
	}

	free(lpWriteFileBuffer);

} /* end of doWriteFile() */


/****************************************************************************
/*doReadFile() - thread to handle data from file to DSP
/*
/*  This function ...
/****************************************************************************/
static void doReadFile(DWORD ReadFile16BitWords)
{
	ULONG       ulLength;
	ULONG       ulSentLen;
	ULONG       ulReadLen;
	ULONG       ulTotalLength = 0;
	unsigned    uRetCode = 1;
	ULONG       tempLen;
	unsigned    uReadSuccess = 1;
	LPDWORD     lpReadFileBuffer = NULL;

    // set timeout value for FIFO transfers
	evm55x_set_timeout(hBd,1000);

	/* malloc read buffer */
	lpReadFileBuffer = (LPDWORD)malloc(READ_BUFFER_SIZE*sizeof(USHORT));
		
	if ( lpReadFileBuffer == NULL )
	{
		printf( "\n**Error: malloc() failed.\n" );
		exitApp(-1);
	}

	/* get length to read from file*/
	ulReadLen = ReadFile16BitWords;

	/* write length to FIFO */
	/* NOTE: This is application dependent.  In this case, the DSP application
	         fifo.out expects a count before receiving data.  This may be useful
			 for your application, or you may just stream data without sending
			 the word count first (it is application-dependent). */

    tempLen = 1;
	if (!evm55x_fifo_write(hHpi,&ulReadLen,&tempLen,0))
	{
	   printf("\n**Error: Write of data length to DSP FIFO failed.\n");
	   free(lpReadFileBuffer);
       exitApp(-1);
	}	
	
    /* clear sent counter */
	ulSentLen = 0;
		
	while (ulReadLen > ulSentLen)
	{
		/* read data from file */
		if (uReadSuccess)
		{
		   ulLength = ulReadLen - ulSentLen;

		   // transfer in 32-word blocks
		   if (ulLength > READ_BUFFER_SIZE) ulLength = READ_BUFFER_SIZE;
		   ulLength = fread( lpReadFileBuffer, sizeof(USHORT), ulLength, hReadFile );;
		}
	
		/* send data to DSP */
		evm55x_fifo_write(hHpi, 
			(LPDWORD)((char *)lpReadFileBuffer), 
			&ulLength,
			0);

        /* get length of data transferred */
		if (ulLength)
		{				
			ulSentLen += ulLength;
		    ulTotalLength += ulSentLen;
			uReadSuccess = 1;
		}
		else 
		{
            uReadSuccess = 0;
		}
			
		
	   	/* done if did not read a full buffer */
		if ( ulLength != READ_BUFFER_SIZE )
		{
			break;
		}
		
	} /* end of while loop */
	
	// free buffer
    free(lpReadFileBuffer);
	
} /* end of doReadFile() */



/*--------------------------------------------------------------------------*/
// Test code to create binary file for testing
static void writeBin()
{
	FILE    *fp;
	USHORT  i;
	USHORT  len = FILE_LEN; 

	// Check to see if input file exists
	fp = fopen(sReadFileName, "rb" );
	if (fp != NULL)
	{
		printf("Input file \"%s\" already exists, so it will not be recreated.\n",sReadFileName);
		fclose(fp);
		return;
	}

	// File does not exist, create it.
	printf("Input file \"%s\" does not exist, so one will be created.\n",sReadFileName); 
	fp = fopen(sReadFileName, "wb");

	for (i=0; i<len; i++)
	   fwrite(&i, sizeof(USHORT), 1, fp);

	fclose(fp);
}

