/******************************************************************************/
/*     File:   evm55xdll.c                                                    */
/*                                                                            */
/*     Purpose: Main module for the DLL functions for the TMS320VC5510 EVM    */
/*                                                                            */
/*     Copyright (c) 1999-2001, DNA Enterprises, Inc. All rights reserved     */
/******************************************************************************/

/******************************************************************************/
/*                                                                            */
/*    Description here                                                        */
/*                                                                            */
/* TMS320VC5510 EVM DLL functions:                                            */
/*                                                                            */
/*    evm55x_open()              Open a driver connection to a board          */
/*    evm55x_pci_init()          Performs initialization of board             */
/*    evm55x_close()             Close a driver connection to a board         */
/*    evm55x_reset_board()       Completely reset a board                     */
/*    evm55x_reset_dsp()         Reset the DSP on a board                     */
/*    evm55x_unreset_dsp()       Release the DSP from it's reset state        */
/*    evm55x_run_dsp()           Run DSP core                                 */
/*    evm55x_board_info()        Return EVM board info                        */
/*    evm55x_get_pci_configuration() Returns the PCI configuration            */
/*    evm55x_store_pci_configuration() Writes the PCI configuration           */
/*    evm55x_coff_load()         Load a COFF image into DSP memory            */
/*    evm55x_coff_display()      Display section information for a COFF file  */
/*    evm55x_program_flash()     Program Flash memory with Motorola Hex file  */
/*    evm55x_reset_flash()       Reset Flash memory state machine             */
/*    evm55x_erase_flash()       Erases Flash memory device                   */
/*    evm55x_hpi_open()          Open the HPI for a DSP                       */
/*    evm55x_hpi_close()         Close the HPI for a DSP                      */
/*    evm55x_hpi_read()          Read DSP memory using the HPI                */
/*    evm55x_hpi_read_single()   Read single DSP memory location using HPI    */
/*    evm55x_hpi_write()         Write to DSP memory using HPI                */
/*    evm55x_hpi_write_single()  Write to single DSP memory location using HPI*/
/*    evm55x_hpi_fill()          Write a fixed pattern to DSP memory          */
/*    evm55x_fifo_read()         Read data from fifo                          */
/*    evm55x_fifo_write()        Write data to fifo                           */
/*    evm55x_generate_int()      Interrupt DSP                                */
/*    evm55x_enable_int()        Enable interrupt source from DSP             */
/*    evm55x_disable_int()       Disables interrupt source from DSP           */
/*    evm55x_printf_control()    Controls DLL printf output                   */
/*    evm55x_input_control()     Controls DLL input function                  */
/*    evm55x_dll_revision()      Returns DLL revision                         */
/*    evm55x_read_pci_reg()      Returns a PCI2040 register value             */
/*    evm55x_write_pci_reg()     Writes to a PCI2040 register                 */
/*    evm55x_read_cpld_reg()     Returns the specified CPLD register value    */
/*    evm55x_write_cpld_reg()    Writes to a CPLD register                    */
/*    evm55x_set_timeout()       Sets the FIFO timeout                        */ 
/*    evm55x_dsp_present()       Returns True if the specified DSP is present */
/*    evm55x_cpld_read_all()     Gets a dump of cpld registers                */                  
/*    evm55x_user_semaphore_get()Acquires the specified semaphore             */ 
/*    evm55x_user_semaphore_release() Releases the specified semaphore        */                   
/*    evm55x_user_semaphore_wait() Waits for the specified semaphore          */   
/*	  evm55x_get_event_handle()  Return handle of requested event             */  
/*    evm55x_get_base_address()  Returns a PCI base address pointer           */
/*    evm55x_read_hpi_reg()      Reads DSP HPI register                       */
/*    evm55x_write_hpi_reg()     Writes to DSP HPI register                   */
/*    evm55x_tbc_reset()         Holds the JTAG TBC in reset                  */
/*    evm55x_tbc_unreset()       Removes the JTAG TBC from reset			  */
/*    evm55x_read_tbc_reg()      Reads JTAG TBC register                      */
/*	  evm55x_write_tbc_reg()     Writes to a JTAG TBC register                */                            
/*                                                                            */
/* REVISION HISTORY                                                           */
/*                                                                            */
/*  15Jan99 T. Kyiamah    Original                                            */
/*  31Jan01 B. Carlson    Fixed bugs, added comments and updated for C55x EVM.*/  
/******************************************************************************/
// Static strings
static char copyright[] =
"Copyright (c) 2001, DNA Enterprises, Inc. All rights reserved";

/****************************************************************************/
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <winioctl.h>
#include <string.h>
#include <process.h>

#define evm55xdll_c

#include "ddpci2040.h"      // board driver header file
#include "cloadsup.h"		// suplemental cload routines header file
#include "evm55xdll.h"		// public header file used by higher level apps
#include "_evm55xdll.h"		// private header file 
/****************************************************************************/

/************************************************************************/
/* For Win95 & NT, the following variables need to be shared.  When the */
/* data_seg pragma is used, you must initialize the variables or else   */
/* the compiler will not place the data in the shared segment.  In      */
/* addition, you must use the following option with the linker:         */
/* /SECTION:shared,rws                                                  */
/************************************************************************/
#pragma data_seg("shared")
// Want to keep shared info about boards here
// this should be shared among all processes using this DLL
LONG  Process_count = 0;
#pragma data_seg()


/****************************************************************************/
/* Globals                                                                  */
/****************************************************************************/

/****************************************************************************/
/* private data                                                             */
/****************************************************************************/
static BoardStruct                 boards[MAX_PCI2040_BOARDS];
static BOOL                        boardsInitialized = FALSE;
static BOOL                        printf_enabled = TRUE;   // default to printf enabled
static BOOL                        input_enabled = TRUE;    // default to input enabled
static PPCI2040_PRINTF_FUNC        output_func    = NULL;   // default to stdio
static PPCI2040_INPUT_FUNC         input_func    = NULL;    // default to stdin
static HANDLE                      gMutex = INVALID_HANDLE_VALUE;

char   printf_str[PRINTF_STR_SIZE];                         // printf string
char   writeBuf[PRINTF_STR_SIZE];
char   commFile[COMM_FILE_SIZE];

static SpaceStruct Spaces[] = {
   {IOCTL_PCI2040_MAP_HPI_CSR_SPACES,IOCTL_PCI2040_UNMAP_HPI_CSR_SPACES,"HPI_CSR"},
      
   {IOCTL_PCI2040_MAP_DSP_HPI_SPACES,IOCTL_PCI2040_UNMAP_DSP_HPI_SPACES,"DSP_HPI"},
   
   {IOCTL_PCI2040_MAP_GPB_TBC_SPACES,IOCTL_PCI2040_UNMAP_GPB_TBC_SPACES,"GPB_TBC"},

   {0,0,NULL}
};

char    Eventnames[MAX_PCI2040_EVENTS][DEV_NAME_SIZE];
char    *_Eventnames[] = 
{
    "INTDSP0_%03d",
    "INTDSP1_%03d",
    "INTDSP2_%03d",
    "INTDSP3_%03d",
    "GPINT_%03d",
    "INTGPIO2_%03d",
    "INTGPIO3_%03d",
    "HPIERROR_%03d",
    "HPIERROR0_%03d",
    "HPIERROR1_%03d",
    "HPIERROR2_%03d",
    "HPIERROR3_%03d",
    0
};   

// Valid memory regions
EvmMemSpace validMem[] = {
	{DARAM_START_ADDR, DARAM_END_ADDR},
	{SARAM_START_ADDR, SARAM_END_ADDR},
	{SBSRAM_START_ADDR, SBSRAM_END_ADDR},
	{FLASH_START_ADDR, FLASH_END_ADDR},
	{DB_START_ADDR, DB_END_ADDR},
	{CPLD_START_ADDR, CPLD_END_ADDR},
	{0, 0}
};

// Reserved memory regions (not available)
EvmMemSpace rsvdMem[] = {
	{(SBSRAM_END_ADDR+1), (FLASH_START_ADDR-1)},
	{(FLASH_END_ADDR+1), (DB_START_ADDR-1)},
	{(CPLD_END_ADDR+1), (PDROM_START_ADDR-1)},
	{0, 0}
};

int		nt_os;
PExtMemDummyStruct pTxfr = (PExtMemDummyStruct)COMM_ADDR;

extern ULONG entry_point;
extern BOOL loadHex(BoardStruct *pBd, char *pHexFile, ULONG dspNum);
extern BOOL verifyBlock(BoardStruct *pBd, char *hexFile, ULONG dspNum);

BOOL kernelLoaded = FALSE;

static ULONG branchInstr;

/****************************************************************************/
/*  DllMain()                                                               */
/*                                                                          *
/*  The DllMain function is the entry point of the DLL.  This entry point   */
/*  is called on startup and shutdown of the DLL.  The dwReason parameter   */
/*  indicates what kind of startup or shutdown caused the current call.     */
/****************************************************************************/
BOOL APIENTRY DllMain( HANDLE hModule, DWORD dwReason, LPVOID lpReserved )
{
   BOOL            status = TRUE;
   OSVERSIONINFO   osinfo;
   
   osinfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
   GetVersionEx(&osinfo);
   nt_os = (osinfo.dwPlatformId == VER_PLATFORM_WIN32_NT) ? TRUE : FALSE;
   
   initEnvironment();
   
   switch( dwReason )
   {
   case DLL_PROCESS_ATTACH:
      InterlockedIncrement( &Process_count );
      break;
      
   case DLL_THREAD_ATTACH:
      break;
      
   case DLL_THREAD_DETACH:
      break;
      
   case DLL_PROCESS_DETACH:
      if (InterlockedDecrement( &Process_count ) == 0)
      {   
         hpi_close_all();
      }
      break;
   }
   
   return (status);
   
   // NOTE: a return of FALSE (i.e. 0) will cause the calling process to be terminated
}

/****************************************************************************** 
evm55x_open()                                                              

  The evm55x_open function opens a driver connection to a specific EVM board.  
  The returned HANDLE is used for all further accesses to the target board.  
  The port parameter is a zero based relative
  index.  Valid index values depend on the number of EVM boards
  in a system and will range from '0' to 'n-1' where 'n' is the number of
  boards.  The exclusive_flag parameter indicates an exclusive open
  request of the target board.  An exclusive open will fail if the target
  board is currently open and additional open requests will fail for
  a target board that has been opened exclusively.
/****************************************************************************/
DLL32_ENTRY HANDLE evm55x_open(int port, BOOL exclusive_flag)
{
   PBoardStruct   pBd = NULL;

   // Get mutex
   WaitForSingleObject(gMutex, INFINITE);
   
   // Check if any board handles are available
   if ((pBd = getUnusedBoard()) == INVALID_HANDLE_VALUE) 
   {
      RedirectPrintf(EVM55XDLL_ERROR, "All boards have already been opened!\n");
      ReleaseMutex(gMutex);
      return(NULL);
   }
   
   // Store port and exclusive flag
   pBd->index = port;
   pBd->excl  = exclusive_flag;

   // Set default FIFO timeout to infinite
   // User applications can set accordingly to activate timeout
   pBd->timeout = DEFAULT_TIMEOUT;

   // Supported board type is C5510 EVM
   pBd->boardType = TYPE_PCI2040_C5510_EVM;

   // Initialize board structure pointers to HPI functions
   setupHpiFunctions(pBd);

   pBd->hMutex = CreateMutex(NULL,FALSE,NULL);

   // Open PCI board
   pciOpen(pBd);

   // Create communications kernel COFF filename
   if (!setCommKernelFile())
	  return (NULL);
      
   // Release mutex
   ReleaseMutex(gMutex);

   if (pBd->hDevice == INVALID_HANDLE_VALUE)
   {
	   CloseHandle(pBd->hMutex);
	   return INVALID_HANDLE_VALUE;
   }

   pBd->state = OPEN;

   return (pBd->hDevice);
}

/****************************************************************************
evm55x_pci_init()                                                            

  The evm55x_pci_init function performs all board and DLL initialization and
  assigns all PCI2040 and CPLD registers to their default values.
  
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_pci_init(HANDLE hDevice, PPCI2040DLL_INIT_DATA pInitData)
{
    PBoardStruct pBd = NULL;
    USHORT cpld_stat = 0;
    BOOL status;
    
    // Get pointer to board structure 
	if ((pBd = getBoard(hDevice)) == NULL)
        return(FALSE);
    
	// Initialize with default values or passed values
	if (pInitData == NULL)
		status = setPCIRegs(hDevice, &pBd->initDefaults.pci_regs); 
	else
		status = setPCIRegs(hDevice, &pInitData->pci_regs); 
	
	if (!status) 
		return (FALSE);
	
	return (TRUE);
}

/****************************************************************************
evm55x_close()

  The evm55x_close function closes a previously opened driver connection
  to a board.  The returned value is TRUE for a successful operation.
  The hDevice parameter is the handle returned from a successful
  evm55x_open call.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_close(HANDLE hDevice)
{
	ULONG i;
	BOOL status = TRUE;
	PBoardStruct pBd = NULL;
	
	// Get pointer to device structure
	if ((pBd = getBoard(hDevice)) == NULL)
		return(FALSE);
	
	// Terminate event thread
	TerminateThread(pBd->outThr, 0); //terminate event-listening thread
		
	// Clear event
	evm55x_clear_event(pBd->hDevice, pBd->hEvent);
	CloseHandle(pBd->hEvent);
	
	// Close event handles
	for (i=0; i<MAX_PCI2040_EVENTS; i++)
	{
		CloseHandle(pBd->events[i]);
	}
	
	// Unmap PCI memory spaces
	status = unmap_spaces(pBd);
	
	// Close board handle
	CloseHandle(pBd->hDevice);
	   
	CloseHandle(pBd->hMutex);
	
	pBd->hDevice = INVALID_HANDLE_VALUE;
	
	pBd->state = CLOSED;
	pBd->inReset = TRUE;
	pBd->index = -1;
	pBd->boardType = -1;

	return (status);
}


/****************************************************************************
evm55x_reset_board()                                                      

  The evm55x_reset_board function causes a hardware reset on the target
  board. This function will fail unless the target board has been opened
  exclusively.

  The return value indicates the success of the function call.
  The hDevice parameter is the handle returned from a successful
  exclusive evm55x_open call.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_reset_board(HANDLE hDevice)
{
    PBoardStruct pBd = NULL;
    
    if ( ((pBd = getBoard(hDevice)) == NULL) || !pBd->excl)
        return (FALSE);
    
	// Board reset is bit 4 of CPLD host SEM0 register
	// NOTE: Board reset will be automatically cleared by on-board hardware after 200 ms.
	*(PUSHORT)((char *)pBd->mappedAddr[CPLD_BAR] + (CPLD_SEM0_OFFSET)) |= CPLD_BOARD_RESET;
	
    // Wait for board reset to complete
	Sleep(500);

	// NOTE: The DSP will still be held in reset after a board reset
	//       since the HPIC RESET bit must be set when EHPIEN = 1.

	// Set reset state flag in board structure
	pBd->inReset = TRUE;
	
	return(TRUE);  
}

/****************************************************************************
evm55x_reset_dsp()

  The evm55x_reset_dsp function causes a reset to only the DSP. To allow 
  the DSP to begin executing, use the evm55x_unreset_dsp function.
  The return value indicates the success of the function call.
  The hDevice parameter is the handle returned from a successful
  exclusive evm55x_open call.
  The mode parameter is the boot mode to reset the DSP into.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_reset_dsp(HANDLE hDevice, ULONG dspNum)
{
   ULONG len;
   PBoardStruct pBd = NULL;
   
   // Get pointer to board structure.  Board should be opened exclusively.
   if ( ((pBd = getBoard(hDevice)) == NULL) || !pBd->excl)
	   return (FALSE);
   
   // Verify that DSP is present on board
   if (!pBd->dsp[dspNum].isPresent)
   {
	   sprintf(printf_str,"DSP %d not present\n", dspNum);
	   RedirectPrintf(EVM55XDLL_ERROR, printf_str);
	   
	   return(FALSE);
   }
  
   // Call PCI2040 driver to reset DSP.  Sets reset bit in PCI2040 register.
   // NOTE: The DSP could also be reset using the host CPLD control register (bit 3).
   if (!DeviceIoControl(hDevice, 
	   (ULONG)IOCTL_PCI2040_DSP_RESET_HOLD, 
	   &dspNum, 
	   sizeof(dspNum),
	   NULL, 
	   0, 
	   &len, 
	   NULL))
   {
	   sprintf(printf_str,"IOCTL_PCI2040_DSP_RESET_HOLD failed, (%d)\n",
		   GetLastError());
	   RedirectPrintf(EVM55XDLL_ERROR, printf_str);
	   
	   return(FALSE);
   }  

   // Set reset state flag
   pBd->inReset = TRUE;
   
   return(TRUE);
}


/****************************************************************************
evm55x_unreset_dsp()                                                      

  The evm55x_unreset_dsp function deasserts the reset signal to the DSP that
  was asserted by the evm55x_reset_dsp function.  This function should only be 
  used in conjunction with a evm55x_reset_dsp call.

  NOTE: When EHPIENA is high (Dipswitch #8 off), the evm55x_run_dsp() 
  function must also be called for the DSP CPU to fetch its reset vector
  and start running.

  The return value indicates the success of the function call.
  The hDevice parameter is the handle returned from a successful
  exclusive evm55x_open call.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_unreset_dsp(HANDLE hDevice, ULONG dspNum)
{
   ULONG len;
   
   PBoardStruct pBd = NULL;
   
   // Get pointer to board structure.  Board should be opened exclusively.
   if ( ((pBd = getBoard(hDevice)) == NULL) || !pBd->excl)
      return (FALSE);
   
   // Verify that DSP is present on board
   if (!pBd->dsp[dspNum].isPresent)
   {
	   sprintf(printf_str,"DSP %d not present\n", dspNum);
	   RedirectPrintf(EVM55XDLL_ERROR, printf_str);
	   
	   return(FALSE);
   }
   
   // Call PCI2040 driver to unreset DSP.  Clears reset bit in PCI2040 register.
   if (!DeviceIoControl(hDevice, 
	   (ULONG)IOCTL_PCI2040_DSP_RESET_RELEASE, 
	   &dspNum, 
	   sizeof(dspNum),
	   NULL, 
	   0, 
	   &len, 
	   NULL))
   {
	   sprintf(printf_str,"IOCTL_PCI2040_DSP_RESET_RELEASE failed, (%d)\n",
		   GetLastError());
	   RedirectPrintf(EVM55XDLL_ERROR, printf_str);
	   
	   return(FALSE);
   }    
   
   // Make sure that CPLD DSP reset bit is also clear to prevent 
   // case where it would override the deasserted PCI2040 DSP reset signal.
   *(PUSHORT)((char *)(pBd->mappedAddr[CPLD_BAR]) + (CPLD_CTRL_OFFSET)) &= CPLD_DSP_UNRESET;

   // DSP still in reset until RESET bit written in HPIC. Set flag
   pBd->inReset = TRUE;
   
   return(TRUE);
}


/****************************************************************************
evm55x_run_dsp()                                                      

  The evm55x_run_dsp function releases the DSP core from the halted state
  invoked by the DSP being reset when the EHPIENA input is high.  
  This function should only be used after a evm55x_unreset_dsp call is made.

  The return value indicates the success of the function call.
  The hDevice parameter is the handle returned from a successful
  exclusive evm55x_open call.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_run_dsp(HANDLE hDevice, ULONG dspNum)
{
   PBoardStruct pBd = NULL;
   
   // Get pointer to board structure.  Board should be opened exclusively.
   if ( ((pBd = getBoard(hDevice)) == NULL) || !pBd->excl)
      return (FALSE);
   
   // Verify that DSP is present on board
   if (!pBd->dsp[dspNum].isPresent)
   {
	   sprintf(printf_str,"DSP %d not present\n", dspNum);
	   RedirectPrintf(EVM55XDLL_ERROR, printf_str);
	   
	   return(FALSE);
   }

   // Release DSP core from reset after RESET signal deassertion with
   // EHPIENA high by setting RESET bit in HPIC register.
   // NOTE: There is no reason to mask with current HPIC value in this situation.
   *pBd->dsp[dspNum].pHpic = HPIC_RESET;

   // Clear reset flag in board structure
   pBd->inReset = FALSE;
   
   return(TRUE);
}


/****************************************************************************
evm55x_board_info()                                                       

  The evm55x_board_info function gets the board type and revision of a
  target board as stored in the PCI configuration space of the board.
  The return value indicates the success of the function call.
  The hDevice parameter is the handle returned from a successful
  evm55x_open call.
  The p_board_info and p_rev_id parameters are pointers to the locations
  to place the requested information.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_board_info(HANDLE hDevice,
                                    PEVM55X_BOARD_TYPE p_board_type, 
                                    PULONG p_rev_id)
{
   PBoardStruct pBd = NULL;
   Register     Reg;
   
   // Default to unknown unless successful
   *p_board_type = TYPE_UNKNOWN;

   // Default to rev 0
   *p_rev_id = 0;
   
   // Get pointer to board structure
   if ((pBd = getBoard(hDevice)) == NULL)  
	   return(FALSE);
   
   // Get PCI revision register
   Reg.Regoff  = REVISIONID_OFFSET;
   Reg.DataVal = 0;
   if (!evm55x_read_pci_reg(hDevice, &Reg))
   {  // if failure, default to invalid
	  *p_rev_id = PCI_INVALID_REV_ID;
   }
   else
   {  // return PCI revision value
      *p_rev_id = Reg.DataVal;
   }

   // Return TMS320VC5510 EVM type
   *p_board_type = TYPE_PCI2040_C5510_EVM;
   
   return (TRUE);
}

/****************************************************************************
evm55x_get_pci_configuration()                                                       

  The evm55x_get_pci_configuration function retrieves the PCI configuration 
  information from the PCI2040 device.
  
    The function returns a FALSE upon failure and TRUE on success
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_get_pci_configuration(HANDLE hDevice,
										      PPCI2040_CONFIG_DATA pReg)
{
	ULONG   len;
	PBoardStruct pBd = NULL;
	
    // Get pointer to board structure
	if ((pBd = getBoard(hDevice)) == NULL)
		return(FALSE);
	
	// Call PCI2040 driver to get PCI configuration information
	if (!DeviceIoControl(hDevice, 
		(ULONG)IOCTL_PCI2040_DUMP_CONFIGURATION, 
		NULL, 
		0,
		(PPCI2040_DUMP_CONFIG_REGS)pReg, 
		sizeof(PCI2040_DUMP_CONFIG_REGS), 
		&len, 
		NULL))
	{
		sprintf(printf_str,"IOCTL_PCI2040_DUMP_CONFIGURATION failed, (%d)\n",
			GetLastError());
		
		RedirectPrintf(EVM55XDLL_ERROR, printf_str);
		
		return(FALSE);
	}
	
	return(TRUE);
}

/****************************************************************************
evm55x_store_pci_configuration()                                                       

  The evm55x_store_pci_configuration function writes the PCI configuration 
  information to the PCI2040 device.
  
    The function returns a FALSE upon failure and TRUE on success
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_store_pci_configuration(HANDLE hDevice,
												PPCI2040_CONFIG_DATA pReg)
{
	ULONG   len;
	PBoardStruct pBd = NULL;
	
    // Get pointer to board structure
	if ((pBd = getBoard(hDevice)) == NULL)
		return(FALSE);
	
    // Call PCI2040 driver to store PCI configuration
	if (!DeviceIoControl(hDevice, 
		(ULONG)IOCTL_PCI2040_STORE_CONFIGURATION, 
		(PPCI2040_DUMP_CONFIG_REGS)pReg, 
		sizeof(PCI2040_DUMP_CONFIG_REGS), 
		NULL, 
		0,
		&len, 
		NULL))
	{
		sprintf(printf_str,"IOCTL_PCI2040_STORE_CONFIGURATION failed, (%d)\n",
			GetLastError());
		
		RedirectPrintf(EVM55XDLL_ERROR, printf_str);
		
		return(FALSE);
	}
	
	return(TRUE);
}


/****************************************************************************
evm55x_coff_load()

  The evm55x_coff_load function reads a COFF image and writes the data
  to DSP memory using the HPI interface.  This function allows the user
  to load executable images from a COFF file to DSP memory.
    
    The return value indicates the success of the function call.
    The hDevice parameter is the handle returned from a successful
    evm55x_open call.
    The pBd parameter is either NULL or the handle returned from a
    successful evm55x_open_hpi call.  If this parameter is NULL, the
    function will call evm55x_open_hpi to get it's own handle to the HPI
    interface then close it before returning.  It the parameter is not
    NULL, the function will use the provided handle to the HPI interface
    and not close it before returning.  Please note that if the HPI
    interface is currently open, it can not be opened again since the HPI
    interface only supports one user at a time.
    The filename parameter is the filename of the COFF file to be
    processed.
    The verbose_flag parameter if TRUE will cause COFF file information
    to be sent to standard out during COFF file processing.
    The clear_bss_flag parameter if TRUE will cause the bss section to be
    set to zero.  This is not the default behavior of the DSP debugger.
    The dump_flag parameter if TRUE will cause the display to standard out
    of all the data being written to DSP memory.  This can be a very large
    amount of data.
    The dspNum parameter indicates which DSP to load with the COFF image

    NOTE: The COFF application is loaded and executed.

/****************************************************************************/
DLL32_ENTRY BOOL evm55x_coff_load( HANDLE hDevice, 
                                   LPVOID pBd_,
                                   char *filename, 
                                   BOOL verbose_flag,
                                   BOOL clear_bss_flag, 
                                   BOOL dump_flag,
                                   ULONG dspNum)
{
	BOOL  status = FALSE;
	FILE  *fin;
	PBoardStruct pBd = NULL;
	ULONG len = 1;
	
    // Get pointer to board structure
	if ((pBd = getBoard(hDevice)) == NULL)
		return(FALSE);
	
	// Verify that DSP is present on board
	if (!pBd->dsp[dspNum].isPresent)
	{
		sprintf(printf_str,"DSP %d not present\n", dspNum);
		RedirectPrintf(EVM55XDLL_ERROR, printf_str);
		
		return(FALSE);
	}
	
	// Open COFF file
	if( (fin = fopen( filename, "rb" )) == NULL )
	{
		sprintf(printf_str,"The file \"%s\" could not be opened for reading.\n",
			filename );
		
		RedirectPrintf(EVM55XDLL_ERROR, printf_str);
		
		return(FALSE);
	}


    // Load communications kernel to handle external mem accesses if not loaded already
	loadCommKernel(pBd);

	// NOTE: At this point, the DSP is actually running the communications kernel,
	//       so the COFF load can load anywhere in memory.  There is no limitation
	//       of loading only into SARAM as there is when the DSP is held in reset.
	//       The DSP will have to be reset after this to start the COFF application.
	
	// Load COFF application
	status = call_cload(hDevice, pBd, fin, TRUE, verbose_flag, 
		                clear_bss_flag, dump_flag, dspNum);
	
	// If COFF load successful write branch to entry point 
	if (status)
	{		
		// NOTE: entry_point global variable is set by COFF load function
		branchInstr = (C55X_BRANCH_OPCODE | (entry_point & 0x00ffffff));
	    // Swap high/low words since DSP low address is MSW.
		branchInstr = (branchInstr << 16) | (branchInstr  >> 16);
		
		// Write two words
		len = 2;

		// Write reset vector at HPI boot address
		if (!(*pBd->intWriteFunc)(pBd, (PULONG)&branchInstr, &len, C5510_HPI_BOOT_ADDR, 0))
		{
			RedirectPrintf(EVM55XDLL_ERROR, "Could not initiate code execution\n");
			kernelLoaded = FALSE;
		}
	}
	
	// Close COFF file
	fclose(fin);

	// Reset and unreset DSP, and then run DSP to execute COFF application
    evm55x_reset_dsp(hDevice,dspNum);
	Sleep(2);
	evm55x_unreset_dsp(hDevice,dspNum);
	Sleep(2);

	evm55x_run_dsp(hDevice,dspNum);
	
	return (status);
}

/****************************************************************************
evm55x_coff_display()

  The evm55x_coff_display function outputs COFF file information to
  standard out.  This information includes details for each section,
  including name, size and flags.
  
    The return value indicates the success of the function call.
    The filename parameter is the filename of the COFF file to be
    processed.
    The clear_bss_flag parameter if TRUE will cause the bss section to be
    set to zero.  This is not the default behavior of the DSP debugger.
    The dump_flag parameter if TRUE will cause the display to standard out
    of all the data being written to DSP memory.  This can be a very large
    amount of data.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_coff_display( char *filename,
                                      BOOL clear_bss_flag, 
                                      BOOL dump_flag)
{
   BOOL status = FALSE;
   FILE *fin;
   
   // Open COFF input file
   if( (fin = fopen( filename, "rb" )) == NULL )
   {
      sprintf(printf_str,"The file \"%s\" could not be opened for reading.\n",
         filename );
      
      RedirectPrintf(EVM55XDLL_ERROR, printf_str);
      
      return(FALSE);
   }
   
   // Call COFF load routine to display COFF information
   status = call_cload(NULL, NULL, fin, FALSE, TRUE, clear_bss_flag, dump_flag, 0);
   
   // Close COFF file
   fclose(fin);
   
   return (status);
}


/****************************************************************************
evm55x_program_flash()

   The evm55x_program_flash function programs the Flash memory with the
   passed Motorola S2 format hex file.

/****************************************************************************/
DLL32_ENTRY BOOL evm55x_program_flash(HANDLE hDevice, char *hexFile, ULONG dspNum)
{
	BOOL  status = FALSE;

	PBoardStruct pBd = NULL;

	
    // Get pointer to board structure
	if ((pBd = getBoard(hDevice)) == NULL)
		return(FALSE);
	
    // Check if DSP is present on board
	if (!pBd->dsp[dspNum].isPresent)
	{
		sprintf(printf_str,"DSP %d not present\n", dspNum);
		RedirectPrintf(EVM55XDLL_ERROR, printf_str);
		
		return(FALSE);
	}

	// Program Flash memory
	status =  loadHex(pBd, hexFile, dspNum);

	if (status == TRUE)
	{
		RedirectPrintf("", "Done!\n");
		RedirectPrintf(EVM55XDLL_INFO, "Verifying............");

		// Verify Flash memory to see if it programmed correctly
		if ((status = verifyBlock(pBd, hexFile, dspNum)) == FALSE)
			RedirectPrintf("", "FAIL\n");
		else
			RedirectPrintf("", "OK\n");
	}

	return (status);
}

/****************************************************************************
evm55x_reset_flash()

/****************************************************************************/
DLL32_ENTRY BOOL evm55x_reset_flash(HANDLE hDevice)
{
    PBoardStruct pBd = NULL;
    ExtMemStruct com; 
    ULONG len = 2,writeLen = 6;
	ULONG start = 1<<16;
    
    // Get pointer to board structure
    if ((pBd = getBoard(hDevice)) == NULL)
        return(FALSE);
  
	// Load communications kernel if not running
    loadCommKernel(pBd);

    // Initialize com structure
    com.len = 0;
    com.addr = 0;
    com.memSpace = EXT_FLASH_RESET << 16;
    
	// Write to com DSP application to reset Flash 
    (*pBd->intWriteFunc)(pBd, (PULONG)&com, &writeLen, COMM_ADDR, 0);
    (*pBd->intWriteFunc)(pBd, (PULONG)&start, &len, COMM_ADDR, 0);
    
	if (!waitForCompletion(pBd, DEFAULT_COMM_TIMEOUT))
		return FALSE;
    
    return TRUE;
}

/****************************************************************************
* evm55x_erase_flash(HANDLE hDevice, int sector)
*
*  The evm55x_erase_flash erases a sector of the Flash memory.
*  
*		hDevice - Board handle returned from the call to evm55x_open.
*		sector - specifies block of flash to erase (0-10 or -1 for entire chip
*
*    Returns TRUE on succes
*    FALSE on failure.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_erase_flash(HANDLE hDevice, int sector)
{
    PBoardStruct pBd = NULL;
    ExtMemStruct com; 
    ULONG len = 2,writeLen = 6;
	USHORT start = 1;
    
    // Get pointer to board structure
	if ((pBd = getBoard(hDevice)) == NULL)
        return(FALSE);
    
    // Load communications kernel if not running
	loadCommKernel(pBd);

    // Initialize com structure
    com.len = 0;
    com.addr = sector << 16;
    com.memSpace = EXT_FLASH_ERASE << 16;
   
    // Write to com DSP application to erase Flash sector
    (*pBd->intWriteFunc)(pBd, (PULONG)&com, &writeLen, (ULONG)COMM_ADDR, 0);
    (*pBd->intWriteFunc)(pBd, (PULONG)&start, &len, (ULONG)COMM_ADDR, 0);
    
	// Wait until erase operation completed or 4-second timeout
	if (!waitForCompletion(pBd, 4000))
		return FALSE;
    
    return TRUE;
}


/****************************************************************************
evm55x_hpi_open()

  The evm55x_hpi_open function establishes a single connection per target
  board to the HPI of a target board.  After the HPI 
  has been successfully opened, read and write operations can be
  performed to DSP memory.
  
    The return value is the handle to be used for HPI access.  A value of
    NULL indicates a failed attempt to open the HPI.
    The hDevice parameter is the handle returned from a successful
    evm55x_open call.
/****************************************************************************/
DLL32_ENTRY LPVOID evm55x_hpi_open(HANDLE hDevice)
{
   PBoardStruct pBd = NULL;
   
   // Get pointer to board structure
   if ((pBd = getBoard(hDevice)) == NULL)
      return((LPVOID)NULL);
   
   // Return pointer to board structure
   return((LPVOID)pBd);
}

/****************************************************************************
evm55x_hpi_close()

  The evm55x_hpi_close function closes an open HPI session started with
  a evm55x_hpi_open call.
  
    The return value indicates the success of the function call.
    The pBd parameter is the handle returned from a successful
    evm55x_hpi_open call.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_hpi_close(LPVOID pBd_)
{
   PBoardStruct   pBd = pBd_;
   
   // Only fails if invalid board
   if (!validBoard(pBd))
      return (FALSE);
   
   return (TRUE);
}


/*****************************************************************************
*  evm55x_hpi_read()
*
*	 Description: 
*		Reads data from the DSP memory.
*
*  Parameters:
*		h_hpi_map - A void pointer corresponding to the HPI map object.
*		p_buffer - Pointer to the buffer to read data into.
*		p_length - Number of 16 -bit words to read.
*		src_addr - DSP memory address to read from.
*   dspNum - Specifies the DSP.
*
*  Return:
*		TRUE - success.
*		FALSE - failure
*
*  Note: 
*
*******************************************************************************/
DLL32_ENTRY BOOL evm55x_hpi_read(LPVOID pBd_,
                                  PULONG p_buffer,
                                  PULONG p_len,
                                  ULONG src_addr,
                                  ULONG dspNum)
{
   PBoardStruct pBd = pBd_;
   BOOL  status = FALSE;
   int memLoc1;
   int memLoc2;
   
   // Make sure pointer to board is valid 
   if (!validBoard(pBd)) 
      return (FALSE);

   // Verify DSP is present
   if (!pBd->dsp[dspNum].isPresent)
   {
      sprintf(printf_str,"DSP %d not present\n", dspNum);
      RedirectPrintf(EVM55XDLL_ERROR, printf_str);
      
      return(FALSE);
   }
   
   // Determine location of memory start/end addresses 
   memLoc1 = mem5510Location(src_addr, pBd);
   memLoc2 = mem5510Location(src_addr + *p_len - 1, pBd);

   // Check if memory range is inside HPI visibility
   if ( (memLoc1 == HPI_VISIBLE) && (memLoc2 == HPI_VISIBLE) )
   {
	   status = (*pBd->intReadFunc)(pBd, p_buffer, p_len, src_addr, dspNum);
   }
   // Check if memory range is outside HPI visibility
   else if ( (memLoc1 == NOT_HPI_VISIBLE) && (memLoc2 == NOT_HPI_VISIBLE) )
   {
	   status = (*pBd->extReadFunc)(pBd, p_buffer, p_len, src_addr, dspNum);
   }
   else if ( (memLoc1 == HPI_VISIBLE) && (memLoc2 == NOT_HPI_VISIBLE) )
   {   
	   ULONG intLen  = HPI_END_ADDR - src_addr + 1;
	   ULONG extLen  = *p_len - intLen;
	   ULONG extAddr = src_addr + intLen;

	   // Read memory using HPI (internal memory + visible external memory)
	   status = (*pBd->intReadFunc)(pBd, p_buffer, &intLen, src_addr, dspNum);

	   // Read memory using communication kernel (non-visible external memory)

	   status = (*pBd->extReadFunc)(pBd, p_buffer, &extLen, extAddr, dspNum);

	   // Update p_len for user indication
	   *p_len = intLen + extLen;
   }
   else // invalid address
   {
	   RedirectPrintf(EVM55XDLL_ERROR, "Memory access involves invalid address!\n");
	   sprintf(printf_str,"Address: 0x%06x, Length: %d\n", src_addr, *p_len);
	   RedirectPrintf(EVM55XDLL_ERROR, printf_str);
	   status = FALSE;
   }
   
   return (status);
}

/*****************************************************************************
*  evm55x_hpi_read_single()
*
*	 Description: 
*		Reads a single 16-bit value from the DSP memory.
*
*  Parameters:
*		h_hpi_map - A void pointer corresponding to the HPI map object.
*		p_data - Pointer to the buffer to read data into.
*		src_addr - DSP memory address to read from.
*       dspNum - Specifies the DSP.
*
*  Return:
*		TRUE - success.
*       FALSE - failure
*
*******************************************************************************/
DLL32_ENTRY BOOL evm55x_hpi_read_single(LPVOID pBd_, 
                                           PULONG p_data,
                                           ULONG src_addr,
                                           ULONG dspNum)
{
	PBoardStruct pBd = pBd_;
	BOOL  status = FALSE;
	int memLoc;
	ULONG len = 1;
	
    // Make sure pointer to board is valid 
	if (!validBoard(pBd)) 
		return (FALSE);

	// Check if DSP present
	if (!pBd->dsp[dspNum].isPresent)
	{
		sprintf(printf_str,"DSP %d not present\n", dspNum);
		RedirectPrintf(EVM55XDLL_ERROR, printf_str);
		
		return(FALSE);
	}
	
	// Determine memory address location
	if ((memLoc = mem5510Location(src_addr, pBd) == HPI_VISIBLE))
	{
		// Visible to HPI
		status = (*pBd->intReadFunc)(pBd, p_data, &len, src_addr, dspNum);
	}
	else if (memLoc == NOT_HPI_VISIBLE)
	{
		// Not visible to HPI.  Use communications kernel.
		status = (*pBd->extReadFunc)(pBd, p_data, &len, src_addr, dspNum);
	}
	else
	{
		// Invalid (reserved) address
		sprintf(printf_str,"Invalid address: 0x%06x\n", src_addr);
		RedirectPrintf(EVM55XDLL_ERROR, printf_str);
		status = FALSE;
	}
	
	return (status);
}

/*****************************************************************************
*  evm55x_hpi_write()
*
*	 Description: 
*		Writes data to the DSP memory.
*
*  Parameters:
*		h_hpi_map - A void pointer corresponding to the HPI map object.
*		p_buffer - Pointer to the buffer to read data into.
*		p_length - Number of 16 -bit words to write.
*		src_addr - DSP memory address to read from.
*		dspNum - Specifies the DSP
*
*  Return:
*		TRUE - success.
*       FALSE - failure
*
*******************************************************************************/
DLL32_ENTRY BOOL evm55x_hpi_write(LPVOID pBd_, 
                                   PULONG  p_buffer,
                                   PULONG p_len, 
                                   ULONG  dest_addr,
                                   ULONG dspNum)
{
   PBoardStruct pBd = pBd_;
   BOOL  status = FALSE;
   int memLoc1 = NOT_HPI_VISIBLE;
   int memLoc2 = NOT_HPI_VISIBLE;

   // Make sure pointer to board is valid 
   if (!validBoard(pBd))
      return (FALSE);
   
   // Verify DSP is present
   if (!pBd->dsp[dspNum].isPresent)
   {
      sprintf(printf_str,"DSP %d not present\n", dspNum);
      RedirectPrintf(EVM55XDLL_ERROR, printf_str);
      
      return(FALSE);
   }

   // Determine location of memory start/end addresses 
   memLoc1 = mem5510Location(dest_addr, pBd);
   memLoc2 = mem5510Location(dest_addr + *p_len - 1, pBd);

   // Check if memory range is inside HPI visibility  
   if ( (memLoc1 == HPI_VISIBLE) && (memLoc2 == HPI_VISIBLE) )
   {
      status = (*pBd->intWriteFunc)(pBd, p_buffer, p_len, dest_addr, dspNum);
   }
   // Check if memory range is outside HPI visibility
   else if ( (memLoc1 == NOT_HPI_VISIBLE) && (memLoc2 == NOT_HPI_VISIBLE) )
   {
      status = (*pBd->extWriteFunc)(pBd, p_buffer, p_len, dest_addr, dspNum);
   }
   else if ( (memLoc1 == HPI_VISIBLE) && (memLoc2 == NOT_HPI_VISIBLE) )
   {   
	   ULONG intLen  = HPI_END_ADDR - dest_addr + 1;
	   ULONG extLen  = *p_len - intLen;
	   ULONG extAddr = dest_addr + intLen;

	   // Write memory using HPI (internal memory + visible external memory)
	   status = (*pBd->intWriteFunc)(pBd, p_buffer, &intLen, dest_addr, dspNum);

	   // Write memory using communication kernel (non-visible external memory)

	   status = (*pBd->extWriteFunc)(pBd, p_buffer, &extLen, extAddr, dspNum);

	   // Update p_len for user indication
	   *p_len = intLen + extLen;
   }
   else // invalid address
   {
	   RedirectPrintf(EVM55XDLL_ERROR, "Memory access spans an invalid area!\n");
	   sprintf(printf_str,"Address: 0x%06x, Length: %d\n", dest_addr, *p_len);
	   RedirectPrintf(EVM55XDLL_ERROR, printf_str);
	   status = FALSE;
   }

   return (status);
}

/*****************************************************************************
*  evm55x_hpi_write_single()
*
*	 Description: 
*		Writes a single 16-bit value to the DSP memory.
*
*  Parameters:
*		h_hpi_map - A void pointer corresponding to the HPI map object.
*		p_data - Pointer to the buffer to read data into.
*		dest_addr - DSP memory address to write to.
*     dspNum - Specifies the DSP.
*
*  Return:
*		TRUE - success.
*       FALSE - failure
*
*  Note: 
*
*******************************************************************************/
DLL32_ENTRY BOOL evm55x_hpi_write_single(LPVOID pBd_, 
                                           PULONG p_data,
                                           ULONG dest_addr,
                                           ULONG dspNum)
{
	PBoardStruct pBd = pBd_;
	BOOL  status = FALSE;
	int memLoc = NOT_HPI_VISIBLE;
	ULONG len = 1;
	
    // Make sure pointer to board is valid 
	if (!validBoard(pBd)) 
		return (FALSE);
	
	// Check if DSP present
	if (!pBd->dsp[dspNum].isPresent)
	{
		sprintf(printf_str,"DSP %d not present\n", dspNum);
		RedirectPrintf(EVM55XDLL_ERROR, printf_str);
		
		return(FALSE);
	}
	
	// Determine memory address location
	if ((memLoc = mem5510Location(dest_addr, pBd)) == HPI_VISIBLE)
	{
		// Visible to HPI
		status = (*pBd->intWriteFunc)(pBd, p_data, &len, dest_addr, dspNum);
	}
	else if (memLoc == NOT_HPI_VISIBLE)
	{
		// Not visible to HPI.  Use communications kernel.
		status = (*pBd->extWriteFunc)(pBd, p_data, &len, dest_addr, dspNum);
	}
	else
	{
		sprintf(printf_str,"Invalid address: 0x%06x\n", dest_addr);
		RedirectPrintf(EVM55XDLL_ERROR, printf_str);
		status = FALSE;
	}
	
	return (status);
}

/*****************************************************************************
*  evm55x_hpi_fill()
*
*	 Description: 
*		Fills DSP memory with a single value.
*
*  Parameters:
*		h_hpi_map - A void pointer corresponding to the HPI map object.
*		fill_value - 16-bit value to fill DSP memory with.
*		p_length - Number of 16 -bit words to write.
*		src_addr - DSP memory address to read from.
*       dspNum - Specifies the DSP.
*
*  Return:
*		TRUE - success.
*       FALSE - failure
*
*******************************************************************************/
DLL32_ENTRY BOOL evm55x_hpi_fill(LPVOID pBd_, 
                                  USHORT fill_value,
                                  PULONG p_len, 
                                  ULONG dest_addr,
                                  ULONG dspNum)
{
   ULONG i;
   BOOL status = TRUE;  
   PUSHORT p_buffer = (PUSHORT) malloc(*p_len*sizeof(USHORT));
   
   // Fill local buffer with fill value
   for (i=0; i<*p_len; i++)
      p_buffer[i] = fill_value;
   
   // Call HPI write routine to write fill data to DSP memory
   status = evm55x_hpi_write(pBd_, (PULONG)p_buffer, p_len, dest_addr, dspNum);

   // Free temporary buffer
   free (p_buffer);

   return (status);
}

/****************************************************************************
evm55x_fifo_read()

  The evm55x_fifo_read function reads data from the fifo
  
    The return value indicates the success of the function call.
    The pBd parameter is the handle returned from a successful
    evm55x_hpi_open call.
    The p_buffer parameter is the address of the buffer that will be
    transferred by the write operation.  This address must be 32bit word
    aligned.
    The p_len parameter is the address of the write len.  The len
    will be updated with the actual transfer len.  The len is the
    number of bytes to be transferred, but must be a multiple of 2 since
    all transfers are 16bit words.
    The dspNum parameter refers to the DSP to access.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_fifo_read(LPVOID pBd_, 
                                   PULONG  pbuffer,
                                   PULONG plen, 
                                   ULONG  dspNum)
{   
	PBoardStruct    pBd = pBd_;
   
    // Make sure pointer to board is valid 
	if (!validBoard(pBd))
		return (FALSE);

    // Check if DSP present
    if (!pBd->dsp[dspNum].isPresent)
    {
      sprintf(printf_str,"DSP %d not present\n", dspNum);
      RedirectPrintf(EVM55XDLL_ERROR, printf_str);
      
      return(FALSE);
    }

	// Read FIFO and return buffer of data
    return fifo_read(pBd, (PFIFO)FIFO_ADDR, pbuffer, plen, dspNum);
}

/****************************************************************************
evm55x_fifo_write()

	The evm55x_fifo_write function writes data to the fifo

	The return value indicates the success of the function call.
	The pBd parameter is the handle returned from a successful
	evm55x_hpi_open call.
	The p_buffer parameter is the address of the buffer that will be
	transferred by the write operation.  This address must be 32bit word
	aligned.
	The p_len parameter is the address of the write len.  The len
	will be updated with the actual transfer len.  The len is the
	number of bytes to be transferred, but must be a multiple of 2 since
	all transfers are 16bit words.
	The dspNum parameter refers to the DSP to access.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_fifo_write(LPVOID pBd_, 
									PULONG pbuffer,
									PULONG plen, 
									ULONG  dspNum)
{   
	PBoardStruct    pBd = pBd_;
   
    // Make sure pointer to board is valid 
	if (!validBoard(pBd))
		return (FALSE);

	// Check if DSP present
    if (!pBd->dsp[dspNum].isPresent)
    {
      sprintf(printf_str,"DSP %d not present\n", dspNum);
      RedirectPrintf(EVM55XDLL_ERROR, printf_str);
      
      return(FALSE);
    }

	// Write buffer of data to FIFO
	return fifo_write(pBd, (PFIFO)FIFO_ADDR, pbuffer, plen, dspNum);
}

/****************************************************************************
evm55x_generate_int()

	The evm55x_generate_int function causes an interrupt on the
	target DSP.

	The return value indicates the success of the function call.
	The pBd parameter is the handle returned from a successful
	evm55x_hpi_open call.

/****************************************************************************/
DLL32_ENTRY BOOL evm55x_generate_int(LPVOID pBd_, PCI2040_DSP_EVENT intr, ULONG dspNum)
{
   BOOL           status = FALSE;
   PBoardStruct   pBd = pBd_;
   ULONG          temp;
   
   // Make sure pointer to board is valid 
   if (!validBoard(pBd))
      return (FALSE);
   
   // Check if DSP present
   if (!pBd->dsp[dspNum].isPresent)
   {
      sprintf(printf_str,"DSP %d not present\n", dspNum);
      RedirectPrintf(EVM55XDLL_ERROR, printf_str);
      
      return(FALSE);
   }
   
   // Get mutex to allow access to this board's HPI interface
   WaitForSingleObject(pBd->hMutex, 0);
   
   // Determine type of interrupt to generate
   switch (intr)
   {
   case DSP_INT:
	   // Generate DSPINT interrupt by writing to HPIC register
		if (pBd->inReset)
		{
		   // If in reset, HPIC_RESET bit should stay low
		   temp = *pBd->dsp[dspNum].pHpic | HPIC_DSPINT;
	       *(pBd->dsp[dspNum].pHpic) = temp;
		}
		else 
		{			  
		   // If not in reset, we must maintain the HPIC_RESET bit high
		   temp = *pBd->dsp[dspNum].pHpic | HPIC_DSPINT | HPIC_RESET;
		   *(pBd->dsp[dspNum].pHpic) = temp;
		}
	    status = TRUE;
	   
	    break;
	   
   case NMI_INT:
	   // Generate NMI by writing to CPLD register bit
	   *((PUSHORT)((char *)pBd->mappedAddr[CPLD_BAR] + (CPLD_CTRL_OFFSET))) |= CPLD_NMI_ENABLE;
	   *((PUSHORT)((char *)pBd->mappedAddr[CPLD_BAR] + (CPLD_CTRL_OFFSET))) &= CPLD_NMI_DISABLE;
	   status = TRUE;
	   break;
	   
   case DSP_INT5:
	   // Generate external interrupt 5 by writing to CPLD register bit
	   *((PUSHORT)((char *)pBd->mappedAddr[CPLD_BAR] + (CPLD_CTRL_OFFSET))) |= CPLD_INT_ENABLE;
	   *((PUSHORT)((char *)pBd->mappedAddr[CPLD_BAR] + (CPLD_CTRL_OFFSET))) &= CPLD_INT_DISABLE;
	   status = TRUE;
	   break;
	   
   default:
	   status = FALSE;
   }
   
   // release mutex
   ReleaseMutex(pBd->hMutex);
   
   return (status);
}

      
/****************************************************************************
evm55x_enable_int()

  The evm55x_enable_int function enables the interrupt specified by the 
  int_mask parameter. This function returns FALSE on failure and TRUE on
  success
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_enable_int(LPVOID pBd_, ULONG intMask)
{
   PPCI2040_HPI_CSR pCsr = NULL;
   PBoardStruct     pBd = pBd_;
   
   if (!validBoard(pBd))
	   return (FALSE);
   
   // Get mutex to allow access to this board's HPI_CSR space
   WaitForSingleObject(pBd->hMutex, 0);
   
   pCsr = (PPCI2040_HPI_CSR) (pBd->mappedAddr[CSR_BAR]);
   
   // Set Master interrupt if not set
   if ( (pCsr->IntMaskSet & MASTERINTENABLE) == 0 )
   {
	   pCsr->IntMaskSet |= MASTERINTENABLE;
   }
   
   // See if int is already set (has not been cleared by DSP)
   if ( (pCsr->IntMaskSet & intMask) == 0 )
   {
	   pCsr->IntMaskSet |= intMask;
   }
   
   // Release mutex
   ReleaseMutex(pBd->hMutex);
   
   return (TRUE);
}


/****************************************************************************
evm55x_disable_int()

  The evm55x_disable_int function disables the interrupt specified by the 
  int_mask parameter. This function returns FALSE on failure and TRUE on
  success
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_disable_int(LPVOID pBd_, ULONG intMask)
{
	PPCI2040_HPI_CSR pCsr = NULL;
	PBoardStruct     pBd = pBd_;
	
	if (!validBoard(pBd))
		return (FALSE);
	
	// Get mutex to allow access to this board's HPI_CSR space
	WaitForSingleObject(pBd->hMutex, 0);
	   
	pCsr = (PPCI2040_HPI_CSR) (pBd->mappedAddr[CSR_BAR]);
	
	// See if int is already set (has not been cleared by DSP)
	if ( (pCsr->IntMaskClear & intMask) == 0 )
	{
		pCsr->IntMaskClear |= intMask;
	}
	
	// Release mutex
	ReleaseMutex(pBd->hMutex);
	
	return (TRUE);
}

/****************************************************************************/
/*                                                                          */
/* evm55x_printf_control() - This function controls the operation of the */
/*                          printf's (error messages) provided by this DLL. */
/*                          By default, all printf's go to the stdio device.*/
/*                          However, it is possible to disable all printf's */
/*                          or redirect them to a user function.            */
/*                                                                          */
/*    enable - When it is TRUE, printf's are enabled to stdio or redirected */
/*             function.                                                    */
/*    func   - Pointer to printf redirection function (NULL is stdio)       */
/*                                                                          */
/*    Returns TRUE.                                                         */
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_printf_control(BOOL enable,PPCI2040_PRINTF_FUNC func)
{
   // Set printf enable flag
   printf_enabled = enable;
   
   // Store printf redirection function
   output_func = func;
   
   return (TRUE);
}

/****************************************************************************/
/*                                                                          */
/* evm55x_input_control() - This function controls the operation of the  */
/*                          input device.                                   */
/*                          By default, all inputs comes from the stdin     */
/*                          device.                                         */
/*                                                                          */
/*    func   - Pointer to input redirection function (NULL is stdin)        */
/*                                                                          */
/*    Returns TRUE.                                                         */
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_input_control(PPCI2040_INPUT_FUNC func)
{   
   // Store printf redirection function
   input_func = func;
   
   return (TRUE);
}

/****************************************************************************/
/*                                                                          */
/* evm55x_dll_revision() - This function returns the DLL's revision info.	*/
/*                                                                          */
/*    revMajor   - Major revision number (X in X.YY)                        */
/*    revMinor   - Minor revision number (YY in X.YY)                       */
/*    buildNum   - Build number                                             */
/*                                                                          */
/*    Returns TRUE.                                                         */
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_dll_revision (PULONG pRevMajor,
   PULONG pRevMinor,
   PULONG pBuildNum)
{
   // Return revision/build information 
   *pRevMajor = DllRevMajor;
   *pRevMinor = DllRevMinor;
   *pBuildNum = DllBuildNum;
   
   return (TRUE);
}

/****************************************************************************
evm55x_read_pci_reg()

  The evm55x_read_pci_reg function reads a value from the PCI2040 device 
  register specified by the pointer pReg
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_read_pci_reg(HANDLE hDevice, PRegister  pReg)
{
	PBoardStruct pBd = NULL;
	ULONG len;
	
    // Make sure pointer to board is valid  
	if ((pBd = getBoard(hDevice)) == NULL)
		return(FALSE);
	
	// Check for valid PCI2040 register offset
	if (!CheckPci2040RegOfs(pReg->Regoff))
		return FALSE;
	
	// Call PCI2040 driver function to read PCI register
	if (!DeviceIoControl(hDevice,
		(ULONG)IOCTL_PCI2040_READ_PCI_REG,
		&pReg->Regoff,
		sizeof(pReg->Regoff),
		&pReg->DataVal,
		sizeof(pReg->DataVal),
		&len,
		NULL))
	{
		sprintf(printf_str,"IOCTL_PCI2040_READ_PCI_REG failed, (%d)\n",
			GetLastError());
		
		RedirectPrintf(EVM55XDLL_ERROR, printf_str);
		
		return (FALSE);
	}
	
	return TRUE;
}

/****************************************************************************
evm55x_write_pci_reg()

  The evm55x_write_pci_reg function writes a value to the PCI2040 device 
  register specified by the pointer pReg
  
    Returns TRUE on succes
    FALSE on failure.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_write_pci_reg(HANDLE hDevice, PRegister  pReg)
{
   PBoardStruct pBd = NULL;
   ULONG len;
   
   // Make sure pointer to board is valid 
   if ((pBd = getBoard(hDevice)) == NULL)
	   return(FALSE);
   
   // Check for valid PCI2040 register offset  
   if (!CheckPci2040RegOfs(pReg->Regoff))
	   return FALSE;
   
   // Call PCI2040 driver function to write PCI register
   if (!DeviceIoControl(hDevice,
	   (ULONG)IOCTL_PCI2040_WRITE_PCI_REG,
	   pReg,
	   sizeof(PCI2040_WRITE_REG),
	   NULL,
	   0,
	   &len,
	   NULL))
   {
	   sprintf(printf_str,"IOCTL_PCI2040_WRITE_CONFIG_REG failed, (%d)\n",
		   GetLastError());
	   
	   RedirectPrintf(EVM55XDLL_ERROR, printf_str);
	   
	   return (FALSE);
   }
   
   return TRUE;
}

/****************************************************************************
*evm55x_read_cpld_reg()
*
*  Parameters:
*		hDevice - Board handle returned from the call to evm55x_open.
*		pReg - pointer to a PCI2040_REG that holds the read value.
*  The evm55x_read_cpld_reg function reads a value from the CPLD register
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_read_cpld_reg(HANDLE hDevice, PRegister  pReg)
{
   PUSHORT cpldBar;
   PBoardStruct pBd = NULL;
   
   // Make sure pointer to board is valid 
   if ((pBd = getBoard(hDevice)) == NULL)
      return(FALSE);
   
   // Check for valid PCI2040 register offset
   if (!CheckCpldRegOfs(pReg->Regoff, REG_READ))
	   return FALSE;

   // Get PCI address of CPLD
   cpldBar = (PUSHORT)(pBd->mappedAddr[CPLD_BAR]);

   // Read CPLD register.  Only 5 LSBs are valid so mask them off.
   pReg->DataVal = *(PUSHORT)((char *)cpldBar + pReg->Regoff) & 0x1F;

   return TRUE;
}

/****************************************************************************
*evm55x_write_cpld_reg()
*
*  The evm55x_write_cpld_reg function writes a value to the CPLD register
*  
*		hDevice - Board handle returned from the call to evm55x_open.
*		pReg - pointer to a PCI2040_REG that holds the value to write
*
*    Returns TRUE on succes
*    FALSE on failure.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_write_cpld_reg(HANDLE hDevice, PRegister  pReg)
{
	PUSHORT cpldBar;
	PBoardStruct pBd = NULL;
	
    // Make sure pointer to board is valid 
	if ((pBd = getBoard(hDevice)) == NULL)
		return(FALSE);
	
    // Check for valid PCI2040 register offset
	if (!CheckCpldRegOfs(pReg->Regoff, REG_WRITE))
		return FALSE;
	
    // Get PCI address of CPLD
	cpldBar = (PUSHORT)(pBd->mappedAddr[CPLD_BAR]);
	
	// Write CPLD register
	*(PWORD)((char *)cpldBar + pReg->Regoff) = (USHORT)pReg->DataVal;
	
	return TRUE;
}

/****************************************************************************
evm55x_set_timeout()

  The evm55x_set_timeout function sets the timeout for the FIFO. FIFO access 
  calls return with an error whenever they block for a multiple of the specified
  timeout value.
  
    Returns TRUE on succes
    FALSE on failure.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_set_timeout(HANDLE hDevice, ULONG timeout)
{
   PBoardStruct pBd = NULL;
   
   // Make sure pointer to board is valid 
   if ((pBd = getBoard(hDevice)) == NULL)
      return (FALSE);

   // Set timeout in board structure
   pBd->timeout = timeout;

   return TRUE;
}

/****************************************************************************
evm55x_dsp_present()

  The evm55x_dsp_present function returns a true if the DSP device specified 
  by dspNum is present on the board.
  
    Returns TRUE if device present
    FALSE if absent
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_dsp_present(HANDLE hDevice, ULONG dspNum)
{
   PBoardStruct pBd = NULL;
   
   // Make sure pointer to board is valid 
   if ((pBd = getBoard(hDevice)) == NULL)
      return (FALSE);

   // Return DSP present indication
   return (pBd->dsp[dspNum].isPresent);   
}

/*****************************************************************************
*  evm55x_cpld_read_all()
*
*	 Description: 
*		Gets a dump of the CPLD registers.
*
*  Parameters:
*		hDevice - Board handle returned from the call to evm55x_open.
*		regs - Pointer to array of shorts to hold register values.
*  Return:
*		TRUE - success.
*     FALSE - failure
*******************************************************************************/
DLL32_ENTRY BOOL evm55x_cpld_read_all(HANDLE hDevice, PUSHORT regs)
{
   PBoardStruct pBd = NULL;
   PUSHORT cpldBar;
   
   // Make sure pointer to board is valid 
   if ((pBd = getBoard(hDevice)) == NULL)
	   return(FALSE);
   
   // Get PCI address of CPLD
   cpldBar = (PUSHORT)(pBd->mappedAddr[CPLD_BAR]);

   // Read all CPLD registers
   regs[0] = *(PUSHORT)((char *)cpldBar + (CPLD_CTRL_OFFSET)) & 0x1F;
   regs[1] = *(PUSHORT)((char *)cpldBar + (CPLD_STAT1_OFFSET)) & 0x1F;
   regs[2] = *(PUSHORT)((char *)cpldBar + (CPLD_STAT2_OFFSET)) & 0x1F;
   regs[3] = *(PUSHORT)((char *)cpldBar + (CPLD_REV_OFFSET)) & 0x1F;
   regs[4] = *(PUSHORT)((char *)cpldBar + (CPLD_SEM0_OFFSET)) & 0x1F;
   regs[5] = *(PUSHORT)((char *)cpldBar + (CPLD_SEM1_OFFSET)) & 0x1F;
   
   return (TRUE);
}

/****************************************************************************/
/*                                                                          */
/* evm55x_user_semaphore_get() - This function tries to acquire the      */
/*    specified user semaphore.                                             */
/*   sem will be filled with the semaphore state                           */
/*    Returns TRUE if acquired                                              */
/*            FALSE if semaphore is not acquired                            */
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_user_semaphore_get(HANDLE hDevice, PCI2040_SEM sem)
{
	PBoardStruct pBd = NULL;
	BOOL status = TRUE;
	Register reg;

    // Make sure pointer to board is valid 	
	if ( (pBd = getBoard(hDevice)) == NULL )
		return (FALSE);
	
	// Check which semaphore is requested
	reg.Regoff = (sem == PCI2040_SEM0) ? 
		(CPLD_SEM0_OFFSET) : (CPLD_SEM1_OFFSET);
	reg.DataVal = 1;
	
	// Attempt to get semaphore
	evm55x_write_cpld_reg(hDevice, &reg);
	
	reg.DataVal = 0;
	
	Sleep(10);
	
	// Read back semaphore.  If 1, then host owns it.  If 0, then DSP owns it.
	evm55x_read_cpld_reg(hDevice, &reg);
	
	status = reg.DataVal;
	
	// Return semaphore value
	return status;
}


/****************************************************************************/
/*                                                                          */
/* evm55x_user_semaphore_release() - This function releases the previously */
/*    acquired semaphore specified by sem.                                  */
/*                                                                          */
/*   returns FALSE if the semaphore has already been released.             */
/*            TRUE if operation is successful                               */
/*                                                                          */
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_user_semaphore_release(HANDLE hDevice, PCI2040_SEM sem)
{
	PBoardStruct pBd = NULL;
	Register reg;
	
    // Make sure pointer to board is valid 	
	if ( (pBd = getBoard(hDevice)) == NULL )
		return (FALSE);
	
	// Check which semaphore to release
	reg.Regoff = (sem == PCI2040_SEM0) ? 
		(CPLD_SEM0_OFFSET) : (CPLD_SEM1_OFFSET);
	
	reg.DataVal = 0;
	
	// Release semaphore
	evm55x_write_cpld_reg(hDevice, &reg);
	
	return (TRUE);
}


/****************************************************************************/
/*                                                                          */
/* evm55x_user_semaphore_wait() - This function waits indefinitely to       */
/*    acquire the semaphore specified by sem.                               */
/*                                                                          */
/*   returns FALSE if opreation fails                                      */
/*            TRUE if successful                                            */
/*                                                                          */
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_user_semaphore_wait(HANDLE hDevice, PCI2040_SEM sem)
{
	PBoardStruct pBd = NULL;
	BOOL status = TRUE;
	Register reg;
	
    // Make sure pointer to board is valid 		
	if ( (pBd = getBoard(hDevice)) == NULL )
		return (FALSE);
	
	// Get register offset based on semaphore
	reg.Regoff = (sem == PCI2040_SEM0) ? (CPLD_SEM0_OFFSET) : (CPLD_SEM1_OFFSET);
	
	reg.DataVal = 1;
	
    // Request semaphore
	evm55x_write_cpld_reg(hDevice, &reg);
	
	reg.DataVal = 0;
	
	// Wait until semaphore is acquired
	while (!reg.DataVal)
		evm55x_read_cpld_reg(hDevice, &reg);
	
   return (status);
}

/*****************************************************************************
*  evm55x_get_event_handle()
*
*	 Description: 
*		Returns the handle to the requested event
*
*  Parameters:
*		hDevice - Board handle returned from the call to evm55x_open.
*		eventNUM - Enumeration represent the event for which the handle is requested.
*  Return:
*		event HANDLE - success.
*     NULL - failure
*******************************************************************************/
DLL32_ENTRY HANDLE evm55x_get_event_handle(HANDLE hDevice, PCI2040_HOST_EVENT eventNum)
{
   PBoardStruct pBd = NULL;
   
   // Make sure pointer to board is valid 	
   if ( (pBd = getBoard(hDevice)) == NULL )
	   return (NULL);
   
   // Validate event number
   if ((eventNum < DSP0_INT) || (eventNum > HPIERROR3_INT) )
	   return (NULL);
   
   // Return event handle
   return (pBd->events[eventNum]);
}

/****************************************************************************/
/*                                                                          */
/* evm55x_get_base_address() - This function returns the specified base		*/
/*    address pointer.                      .                               */
/*                                                                          */
/*   returns NULL if operation fails                                       */
/*            PULONG if successful                                          */
/*                                                                          */
/****************************************************************************/
DLL32_ENTRY PULONG evm55x_get_base_address(HANDLE hDevice, PCI2040_BAR bar_num)
{
	PBoardStruct pBd = NULL;
	
    // Make sure pointer to board is valid 	
	if ( (pBd = getBoard(hDevice)) == NULL )
		return (NULL);
	
	// Valid BAR number
	if ((bar_num < CSR_BAR) || (bar_num > CPLD_BAR) )
		return (NULL);
	
	// Return base address
	return ((PULONG)pBd->mappedAddr[bar_num]);
}

/****************************************************************************
evm55x_read_hpi_reg()

  The evm55x_read_hpi_reg function writes a value to the HPI register
  specified by the reg.
  
    Returns TRUE on succes
    FALSE on failure.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_read_hpi_reg(LPVOID pBd_, PRegister pReg, ULONG dspNum)
{
	PBoardStruct   pBd = pBd_;
	
    // Make sure pointer to board is valid and DSP is present
	if (!validBoard(pBd) || !pBd->dsp[dspNum].isPresent)
		return FALSE;

	// Read selected register
	switch (pReg->Regoff)
	{
	case C55X_HPIC:
		pReg->DataVal = (USHORT)(*(pBd->dsp[dspNum].pHpic));
		break;
		
	case C55X_HPIA:
		if (pBd->inReset)
		{   // If DSP in reset, then HPIC_RESET must be 0
		    *(pBd->dsp[dspNum].pHpic) = HPIC_XADD; //HPIC_XADD = 1

		    // Get address bits 16-19
		    pReg->DataVal = (*(pBd->dsp[dspNum].pHpia) & 0xF )<< 16;
	
			// Reset XADD and keep HPIC_RESET=0
			*(pBd->dsp[dspNum].pHpic) = 0; //HPIC_XADD = 0
		}
		else 
		{  // If DSP is running, then HPIC_RESET must be 1			   
			*(pBd->dsp[dspNum].pHpic) = HPIC_XADD | HPIC_RESET; //HPIC_XADD = 1

            // Get address bits 19..16
		    pReg->DataVal = (*(pBd->dsp[dspNum].pHpia) & 0xF )<< 16;

			// Reset XADD and keep HPIC_RESET=1
			*(pBd->dsp[dspNum].pHpic) = HPIC_RESET; //HPIC_XADD = 0
		}

        // Get address bits 15..0
		pReg->DataVal |= *(pBd->dsp[dspNum].pHpia) & 0xFFFF;

		break;
		
	case C55X_HPID_AUTOINC:
		pReg->DataVal = (USHORT)(*(pBd->dsp[dspNum].pHpid1));
		break;
		
	case C55X_HPID_NOAUTOINC:
		pReg->DataVal = (USHORT)(*(pBd->dsp[dspNum].pHpid2));
		break;
		
	default:
		return FALSE;
		break;
	}
	
	return TRUE;
}

/****************************************************************************
evm55x_write_hpi_reg()

  The evm55x_write_hpi_reg function writes a value to the HPI register
  specified by the reg.
  
    Returns TRUE on success
    FALSE on failure.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_write_hpi_reg(LPVOID pBd_, PRegister pReg, ULONG dspNum)
{
	PBoardStruct   pBd = pBd_;
	ULONG add16_19 = 0;
	
    // Make sure pointer to board is valid and DSP is present
	if (!validBoard(pBd) || !pBd->dsp[dspNum].isPresent)
		return (FALSE);
	
	// Write to selected HPI register
	switch (pReg->Regoff)
	{
	case C55X_HPIC:
        // Check if DSP in reset 
		if (pBd->inReset)
		{  // DSP is in reset, keep HPIC_RESET=0
	    	*(pBd->dsp[dspNum].pHpic) = (ULONG)pReg->DataVal & ~HPIC_RESET;
		}
		else
		{  // DSP is running, keep HPIC_RESET=1
            *(pBd->dsp[dspNum].pHpic) = (ULONG)pReg->DataVal | HPIC_RESET;
		}
		break;
		
	case C55X_HPIA:
		add16_19 = ((pReg->DataVal & 0xf0000) >> 16);
		
		if (pBd->inReset)
		{   // If DSP in reset, then HPIC_RESET must be 0
		    *(pBd->dsp[dspNum].pHpic) = HPIC_XADD; //HPIC_XADD = 1

			// Write address bits 19..16
	    	*(pBd->dsp[dspNum].pHpia)  = add16_19;
	
			// Reset XADD and keep HPIC_RESET=0
			*(pBd->dsp[dspNum].pHpic) = 0; //HPIC_XADD = 0
		}
		else 
		{  // If DSP is running, then HPIC_RESET must be 1			   
			*(pBd->dsp[dspNum].pHpic) = HPIC_XADD | HPIC_RESET; //HPIC_XADD = 1

            // Write address bits 19..16 to HPIA
		    *(pBd->dsp[dspNum].pHpia)  = add16_19;

			// Reset XADD and keep HPIC_RESET=1
			*(pBd->dsp[dspNum].pHpic) = HPIC_RESET; //HPIC_XADD = 0
		}		
		
		// Write address bits 15..0 to HPIA
		*(pBd->dsp[dspNum].pHpia) = (USHORT)pReg->DataVal & 0xFFFF;
		break;
		
	case C55X_HPID_AUTOINC:
		*(pBd->dsp[dspNum].pHpid1) = (ULONG)pReg->DataVal;
		break;
		
	case C55X_HPID_NOAUTOINC:
		*(pBd->dsp[dspNum].pHpid2) = (ULONG)pReg->DataVal;
		break;
		
	default:
		return FALSE;
		break;
	}
	
	return TRUE;
}

/****************************************************************************
evm55x_tbc_reset()

  The evm55x_tbc_reset function holds the TBC in reset
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_tbc_reset(HANDLE hDevice)
{
	PBoardStruct pBd = NULL;
	
	// Get pointer to board structure
	if ((pBd = getBoard(hDevice)) == NULL)
		return(FALSE);
	
	// Bit 2 of CPLD host cntl reg controls TBC reset
	*(PUSHORT)((char *)pBd->mappedAddr[CPLD_BAR] + (CPLD_CTRL_OFFSET)) |= CPLD_TBC_RESET;
	
	return(TRUE);
}

/****************************************************************************
evm55x_tbc_unreset()

  The evm55x_tbc_unreset function removes the TBC from the reset state
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_tbc_unreset(HANDLE hDevice)
{
	PBoardStruct pBd = NULL;
	
	// Get pointer to board structure
	if ((pBd = getBoard(hDevice)) == NULL)
		return(FALSE);
	
	// Bit 2 of cpld host cntl reg control TBC rest
	*(PUSHORT)((char *)pBd->mappedAddr[CPLD_BAR] + (CPLD_CTRL_OFFSET)) &= CPLD_TBC_UNRESET;
	
	return(TRUE);
}

/****************************************************************************
evm55x_read_tbc_reg()

  The evm55x_read_pci_reg function reads a value from the TBC device 
  register specified by the pointer pReg
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_read_tbc_reg(HANDLE hDevice, PRegister  pReg)
{
   PBoardStruct pBd = NULL;
   
   // Make sure pointer to board is valid 
   if ((pBd = getBoard(hDevice)) == NULL)
	   return(FALSE);
   
   // Makes sure TBC register offset is valid for read
   if (!CheckTBCRegOfs(pReg->Regoff, REG_READ))
	   return FALSE;
   
   // Read TBC register
   pReg->DataVal = *((PWORD)(pBd->mappedAddr[TBC_BAR] + (pReg->Regoff<<2)));
   
   return TRUE;
}

/****************************************************************************
evm55x_write_tbc_reg()

  The evm55x_write_tbc_reg function writes a value to the TBC device 
  register specified by the pointer pReg
  
    Returns TRUE on succes
    FALSE on failure.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_write_tbc_reg(HANDLE hDevice, PRegister  pReg)
{
	PBoardStruct pBd = NULL;
	
    // Make sure pointer to board is valid 
	if ((pBd = getBoard(hDevice)) == NULL)
		return(FALSE);
	
	// Makes sure TBC register offset is valid for write
	if (!CheckTBCRegOfs(pReg->Regoff, REG_WRITE))
		return FALSE;

	// Write TBC register
	*((PWORD)(pBd->mappedAddr[TBC_BAR] + (pReg->Regoff<<2))) = (USHORT)pReg->DataVal;
	
	return TRUE;
}

/****************************************************************************
/* fifo_read()
/*
/*  The fifo_read function reads data from the FIFO implemented in DSP memory.
/****************************************************************************/
BOOL fifo_read(PBoardStruct pBd, PFIFO fifo, PULONG  pBuffer, 
                      PULONG pCnt, ULONG  dspNum)
{   
	ULONG           len, i=0, time_cnt;
	PUSHORT         pBuf = (PUSHORT)pBuffer;
	PULONG          takePtr, putPtr;
	ULONG           tempBuf[2];
	ULONG           word32;
	
	// Internal take/put pointers are two locations of tempBuf
	takePtr = tempBuf;
	putPtr = &tempBuf[1];
	
	// Arbitrary assignment to make them equal at this point
	*putPtr = *takePtr;

	// Loop until requested number of words have been read from FIFO
	while (i < *pCnt)
	{
		// Initialize timeout counter
		time_cnt = pBd->timeout;

		// Wait until there is data to read
		while (*takePtr == *putPtr)
		{
			// Get actual takePtr and putPtr values from DSP memory
			len = 4;
			if (!evm55x_hpi_read(pBd, (PULONG)tempBuf, &len, 
				(ULONG)(&fifo->dsp_ch.takePtr), dspNum))
			{
				// If HPI read failed for some reason..
				// Set number of words read so far
				*pCnt = i;

				// Return with failure
				return (FALSE);
			}

            // Swap pointers' MSW/LSW 
			tempBuf[0] = (tempBuf[0] << 16) | (tempBuf[0] >> 16);
			tempBuf[1] = (tempBuf[1] << 16) | (tempBuf[1] >> 16);


            // Check if timeout value was selected
			if (pBd->timeout)
			{
				// Check if no data available yet
				if (*takePtr == *putPtr)
				{
					// Timeout expires when less than 5
					if ( time_cnt < 5 )
					{
						sprintf(printf_str,"DSP %d FIFO timeout!\n", dspNum);
						RedirectPrintf(EVM55XDLL_ERROR, printf_str);
						*pCnt = i;
						return(FALSE);
					}
					else
				    // No timeout yet, decrement timeout counter and wait 5 milliseconds
					{
						time_cnt -= 5;
						Sleep (5);
					}
				}
			}
		}


		// Value is available in FIFO, so read it.
		if ( !evm55x_hpi_read_single(pBd, (PULONG)pBuf, *takePtr, dspNum) )
		{
			*pCnt = i;
			return (FALSE);
		}

		// Increment buffer pointer and value at takePtr
		pBuf++;
		*takePtr += 1;

        // Wrap takePtr if necessary
		if (*takePtr >= (ULONG)&fifo->dsp_ch.data[FIFO_SIZE])
			*takePtr = (ULONG)fifo->dsp_ch.data;;
		
		// Going to write two words into DSP memory
		len = 2;

		// Swap takePtr MSW/LSW before writing to DSP memory
		word32 = (*takePtr>>16) | (*takePtr<<16);

		// Write value of takePtr back to DSP
		if (!evm55x_hpi_write(pBd, (PULONG)&word32, &len,(ULONG)(&fifo->dsp_ch.takePtr), dspNum))
		{
			*pCnt = i;
			return (FALSE);
		}

		// Increment number of words read from FIFO
		++i;
	}
	
    // Set number of words read from FIFO 
    *pCnt = i;

	return (TRUE);
}

/************************************************************************************
/*fifo_write()
/*
/*	The evm55x_fifo_write function writes data to the FIFO implemented in DSP memory.
/***********************************************************************************/
BOOL fifo_write(PBoardStruct pBd, PFIFO fifo, 
                       PULONG pBuffer,PULONG pCnt, ULONG  dspNum)
{   
	ULONG           len, i=0, time_cnt;
	PUSHORT         pBuf = (PUSHORT)pBuffer;
	PULONG          takePtr, putPtr;
	ULONG           tempBuf[2];
	ULONG           word32;

	// Internal take/put pointers are two locations of tempBuf
	takePtr = &tempBuf[0];
	putPtr = &tempBuf[1];
	
	// Arbitrary assignment
	*takePtr = *putPtr + 1;
	
    // Loop until *pCnt values written to FIFO
	while (i < *pCnt)
	{
		// Get timeout value
		time_cnt = pBd->timeout;

		// Wait till there is room to write
		while ( (*putPtr + 1) == *takePtr )
		{
			// Get actual takePtr and putPtr values from DSP memory
			len = 4;
			if (!evm55x_hpi_read(pBd, (PULONG)tempBuf, &len, 
				(ULONG)(&fifo->host_ch.takePtr), dspNum))
			{
				*pCnt = i;
				return (FALSE);
			}

            // Swap pointers' MSW/LSW 
			tempBuf[0] = (tempBuf[0] << 16) | (tempBuf[0] >> 16);
			tempBuf[1] = (tempBuf[1] << 16) | (tempBuf[1] >> 16);

            // Check if timeout value was selected
			if (pBd->timeout)
			{
				// Check if still no room available in FIFO for write
				if ((*putPtr + 1) == *takePtr)
				{
					// Timeout expires when less than 5
					if ( time_cnt < 5 )
					{
						sprintf(printf_str,"DSP %d FIFO timeout!\n", dspNum);
						RedirectPrintf(EVM55XDLL_ERROR, printf_str);
						*pCnt = i;
						return(FALSE);
					}
					else
					{
				        // No timeout yet, decrement timeout counter and wait 5 milliseconds
						time_cnt -= 5;
						Sleep (5);

					}
				}
			}
		}
		
		// Room is available in FIFO for write to DSP memory
		// Write value to DSP memory
		if ( !evm55x_hpi_write_single(pBd, (PULONG)pBuf, *putPtr, dspNum) )
		{
			// If HPI write fails, return current count written
			*pCnt = i;
			return (FALSE);
		}
		
		// Increment local buffer pointer and putPtr value
		pBuf++;
		*putPtr += 1;
		
		// Wrap putPtr if needed
		if (*putPtr >= (ULONG)&fifo->host_ch.data[FIFO_SIZE])
		{
			len = 4;
			if (!evm55x_hpi_read(pBd, (PULONG)tempBuf, &len, 
				(ULONG)(&fifo->host_ch.takePtr), dspNum))
			{
				// If HPI read failed, then return current count written
				*pCnt = i;
				return (FALSE);
			}

            // Swap pointers' MSW/LSW 
			tempBuf[0] = (tempBuf[0] << 16) | (tempBuf[0] >> 16);
			tempBuf[1] = (tempBuf[1] << 16) | (tempBuf[1] >> 16);

			// Get timeout value
			time_cnt = pBd->timeout;

			// Spin until takePtr is out of the way
			while (*takePtr == (ULONG)&fifo->host_ch.data)
			{
				len = 4;
				if (!evm55x_hpi_read(pBd, (PULONG)tempBuf, &len, 
					(ULONG)(&fifo->host_ch.takePtr), dspNum))
				{
					*pCnt = i;
					return (FALSE);
				}
				
                // Swap pointers' MSW/LSW 
		     	tempBuf[0] = (tempBuf[0] << 16) | (tempBuf[0] >> 16);
			    tempBuf[1] = (tempBuf[1] << 16) | (tempBuf[1] >> 16);

				// If timeout value  selected...
				if (pBd->timeout)
				{
					// If takePtr is still at start of FIFO...
					if (*takePtr == (ULONG)&fifo->host_ch.data)
					{
						// Timeout occurrs when time_cnt < 5
						if ( time_cnt < 5 )
						{
							sprintf(printf_str,"DSP %d fifo timeout!\n", dspNum);
							RedirectPrintf(EVM55XDLL_ERROR, printf_str);
							*pCnt = i;
							return(FALSE);
						}
						else
						{
							time_cnt -= 5;
							Sleep (5);

						}
					}
				}
			}

			// Initialize putPtr to start of FIFO
			*putPtr = (ULONG)fifo->host_ch.data;
		}
		
		// Write two words to DSP memory
		len = 2;

		// Swap LSW/MSW of putPtr
		word32 = (*putPtr>>16) | (*putPtr<<16);

		// Write putPtr to DSP memory
		if ( !evm55x_hpi_write(pBd, (PULONG)&word32, &len, (ULONG)(&fifo->host_ch.putPtr), dspNum) )
		{
			*pCnt = i;
			return (FALSE);
		}		
		
		// Increment number of words written to FIFO
		++i;
	}

	// Return number of words written to FIFO
    *pCnt = i;

	return (TRUE);
}

/****************************************************************************/
/* map_spaces()                                                             */
/*                                                                          */
/* This function is responsible for mapping all the address spaces          */ 
/*                                                                          */
/****************************************************************************/
static BOOL map_spaces(PBoardStruct pBd)
{
   PSpaceStruct pS = &Spaces[0];
   
   PULONG pAddr = &pBd->mappedAddr[0];   
   
   // Loop through memory spaces to map them
   while (pS->printStr != NULL)
   {
      if (!map_space(pBd->hDevice, pS->mapIoctlCode, pAddr++))
      {
         sprintf(printf_str,
            "IOCTL_PCI2040_MAP_%s_SPACES failed, (%d)\n", 
            pS->printStr, GetLastError());
         
         RedirectPrintf(EVM55XDLL_ERROR, printf_str);
         return(FALSE);
      }
      
      pS++;
   }

   // Also store CPLD mapped address which is offset from TBC address
   pBd->mappedAddr[CPLD_BAR] = pBd->mappedAddr[TBC_BAR] + 0x80;
   
   return(TRUE);
}

/****************************************************************************/
/* unmap_spaces()                                                           */
/*                                                                          */
/* This function is responsible for unmapping all mapped address spaces     */ 
/*                                                                          */
/****************************************************************************/
static BOOL unmap_spaces(PBoardStruct pBd)
{
	UINT i;
	PSpaceStruct pS = &Spaces[0];
	PULONG pAddr = &pBd->mappedAddr[0];
	
	// Loop through memory spaces and have driver unmap them 
	while (pS->printStr != NULL)
	{
		if (!unmap_space(pBd->hDevice, pS->unmapIoctlCode, pAddr++))
		{
			sprintf(printf_str,
				"IOCTL_PCI2040_UNMAP_%s_SPACES failed, (%d)\n", 
				pS->printStr, GetLastError());
			
			RedirectPrintf(EVM55XDLL_ERROR, printf_str);
			return(FALSE);
		}
		
		pS++;
	}

	// Set unmapped addresses to 0
	for (i=0; i<MAX_SPACES; i++)
		pBd->mappedAddr[i] = 0;
	
	return(TRUE);
}

/****************************************************************************/
/* map_space()                                                              */
/*                                                                          */
/* This function maps the specified address space   .                       */
/****************************************************************************/
static BOOL map_space(HANDLE hDevice, ULONG ioctlCode, PULONG pMappedAddr)
{
	
	ULONG len = 0;
	
	// Low-level Windows driver maps memory spaces into user memory space
	return (DeviceIoControl(hDevice, 
							ioctlCode, 
							NULL, 
							0,
							(LPVOID)pMappedAddr, 
							sizeof(LPVOID), 
							&len,
							NULL));	
}

/****************************************************************************/
/* unmap_space()                                                            */
/*                                                                          */
/* This function unmaps the specified space         .                       */
/****************************************************************************/
static BOOL unmap_space(HANDLE hDevice, ULONG ioctlCode, PULONG pMappedAddr)
{
   
   ULONG len;
   
   // Low-level Windows driver unmaps memory from user memory space
   return (DeviceIoControl(hDevice, 
						  ioctlCode, 
						  (LPVOID)pMappedAddr, 
						  sizeof(LPVOID), 
						  NULL, 
						  0,
						  &len,
						  NULL));
}

/****************************************************************************/
/* hpiCloseAll()                                                            */
/*                                                                          *
/* This function closes all open HPIs.                                      */
/****************************************************************************/
static VOID hpi_close_all()
{
	int  index;
	

	// Loop through all possible boards
	for (index=0; index<MAX_PCI2040_BOARDS; index++)
	{
	    // If board exists, then close it
		if (boards[index].hDevice != NULL)
		{
			if (boards[index].state)
				evm55x_close(boards[index].hDevice);
		}
	}
	
	return;
}


/****************************************************************************
install_vxd_event()                                                      

  This is a Win95 specific function.  It sets up a message event for a
  board and installs the event to the VxD.  Since VxDs can not do this
  in the same manner as NT.
/****************************************************************************/
static BOOL install_vxd_event(PBoardStruct pBd)
{
   HANDLE    h_vxd_event;
   ULONG     len;
   
   // NT
   if (nt_os)
   {
      h_vxd_event = pBd->hEvent;
   }
   else
   {
	  // Windows 9x
      HINSTANCE h_kernel32_dll;
      HANDLE    (WINAPI *pfOpenVxDHandle)(HANDLE);

      h_kernel32_dll = LoadLibrary("kernel32.dll");
      if (!h_kernel32_dll)
      {
         sprintf(printf_str,"Cannot load KERNEL32.DLL\n");
         RedirectPrintf(EVM55XDLL_ERROR, printf_str);
         return FALSE;
      }
      
      pfOpenVxDHandle = (HANDLE (WINAPI *) (HANDLE))
         GetProcAddress( h_kernel32_dll, "OpenVxDHandle" ); 
      if (!pfOpenVxDHandle)
      {
         sprintf(printf_str,"Cannot get address of OpenVxDHandle()\n");
         RedirectPrintf(EVM55XDLL_ERROR, printf_str);
         return FALSE;
      }
      
      h_vxd_event = (*pfOpenVxDHandle)(pBd->hEvent);
      if (!h_vxd_event)
      {
         sprintf(printf_str,"Cannot create VxD (Ring0) event handle\n");
         RedirectPrintf(EVM55XDLL_ERROR, printf_str);
         return FALSE;
      }
   }
   
   if (!DeviceIoControl(pBd->hDevice, 
                        (ULONG)IOCTL_PCI2040_INSTALL_INTERRUPT_EVENT, 
                        &h_vxd_event, 
                        sizeof(HANDLE), 
                        NULL, 
                        0, 
                        &len, 
                        NULL))
   {
      sprintf(printf_str,"IOCTL_PCI2040_INSTALL_INTERRUPT_EVENT failed, (%d)\n",
         GetLastError());
      RedirectPrintf(EVM55XDLL_ERROR, printf_str);
      return(FALSE);
   }
   
   return(TRUE);
}

/****************************************************************************/
/*                                                                          */
/* RedirectPrintf(printf_str) - This function redirects the printf's to     */
/*      either the stdio device, or the user-supplied redirection function. */
/*                                                                          */
/* NOTE: Uses the following global variables:                               */
/*       printf_enabled - TRUE if enabled, FALSE if disabled                */
/*       output_func    - NULL = stdio, not NULL = redirection function     */
/*       printf_str     - String to send to printf                          */
/****************************************************************************/
void RedirectPrintf (char *errType, char *errStr)
{
   // If printf is disabled, just return
   if (!printf_enabled) return;
   
   // If output_func == NULL, just send to stdio device
   if (output_func == NULL)
   {
      printf("%s%s", errType, errStr);
   }
   else
   {
      // Redirect printf to user-supplied redirection function
      if (errType != NULL)
		  (*output_func)(errType);

      (*output_func)(errStr);
   }
}

/****************************************************************************/
/*                                                                          */
/* RedirectStdin() - This function redirects the input from either the      */
/*                 stdin device, or the user-supplied redirection function. */
/*                                                                          */
/* NOTE: Uses the following global variables:                               */
/*       input_func    - NULL = stdio, not NULL = redirection function      */
/****************************************************************************/
static void RedirectStdin(char *pBuf, ULONG *cnt)
{
   // If stdin_func == NULL, just read from stdin device
   if (input_func == NULL)
   {
      // Disabled for time being (cnt = 0)
      *cnt = 0;
   }
   else
   { // Redirect stdin from user-supplied redirection function
      (*input_func)(pBuf);
   }
}

/**************************************************************************** 
/*  initEnvironment()                                                              
/*                                                                              
/*  All global variables and structures are initialized here.
/****************************************************************************/
void initEnvironment()
{
   // Initialize structures if needed
   if ( !boardsInitialized )
   {
      int     ind, i;

      // Set initialized flag and initialize structure
      boardsInitialized = TRUE;
      for ( ind = 0; ind < MAX_PCI2040_BOARDS; ind++ )
      {
         boards[ind].hDevice = INVALID_HANDLE_VALUE;
         boards[ind].state   = CLOSED;
		 boards[ind].inReset = TRUE;
         boards[ind].ioFunc  = NULL;
         boards[ind].outThr  = NULL;
         boards[ind].inThr   = NULL;
         boards[ind].timeout = DEFAULT_TIMEOUT;
         boards[ind].index = -1;
         boards[ind].boardType = -1;
         boards[ind].flashAccess = FALSE;
         
         for (i=0; i<MAX_SPACES; i++)
            boards[ind].mappedAddr[i] = 0;
         
         for (i=0; i<MAX_PCI2040_DSPS; i++)
            boards[ind].dsp[i].isPresent = FALSE;
      }
      
      memset(printf_str, '\0', PRINTF_STR_SIZE);
      memset(writeBuf, '\0', PRINTF_STR_SIZE);

      kernelLoaded = FALSE;

      // Create mutex for mutual exclusion getUnusedBoard
      gMutex = CreateMutex(NULL,FALSE,NULL);
   }
}

/****************************************************************************
/*  internalRead()
/*
/*  This function handles all on-chip reads through the HPI 
/*
/*  Returns
/*  TRUE - success
/*  FALSE - failure
/****************************************************************************/
static BOOL internalRead(LPVOID pBd_, PULONG p_buffer, 
					     PULONG p_len, ULONG src_addr, ULONG dspNum)
{
	ULONG        i;
	ULONG        add16_19 = (src_addr & 0xf0000) >> 16;
	ULONG        cnt = *p_len;
	PUSHORT      pBuf = (PUSHORT)p_buffer;
	PBoardStruct pBd = (PBoardStruct)pBd_;
	
	// Get mutex to allow access to this board's HPI interface
	WaitForSingleObject(pBd->hMutex, 0);
	
	// Setup HPIA register
    if (pBd->inReset)
	{   // If DSP in reset, then HPIC_RESET must be 0
		*(pBd->dsp[dspNum].pHpic) = HPIC_XADD; //HPIC_XADD = 1

        // Write address bits 19..16
	    *(pBd->dsp[dspNum].pHpia)  = add16_19;
	
		// Reset XADD and keep HPIC_RESET=0
		*(pBd->dsp[dspNum].pHpic) = 0; //HPIC_XADD = 0
	}
	else 
	{  // If DSP is running, then HPIC_RESET must be 1			   
		*(pBd->dsp[dspNum].pHpic) = HPIC_XADD | HPIC_RESET; //HPIC_XADD = 1

       // Write address bits 19..16 to HPIA
	    *(pBd->dsp[dspNum].pHpia)  = add16_19;

		// Reset XADD and keep HPIC_RESET=1
		*(pBd->dsp[dspNum].pHpic) = HPIC_RESET; //HPIC_XADD = 0
	}		

	// Write address bits 15..0 to HPIA
    *(pBd->dsp[dspNum].pHpia) = src_addr & 0xffff;

	// Loop to read cnt values from memory
	for (i=0; i<cnt; i++)
	{
		// Check if updated address crosses 64 Kword page
		if (add16_19 != ((src_addr & 0xf0000) >> 16))
		{
			// Update extended address register
			add16_19 = (src_addr & 0xf0000) >> 16;

			if (pBd->inReset)
			{   // If DSP in reset, then HPIC_RESET must be 0
				*(pBd->dsp[dspNum].pHpic) = HPIC_XADD; //HPIC_XADD = 1

				// Write address bits 19..16
				*(pBd->dsp[dspNum].pHpia)  = add16_19;
	
				// Reset XADD and keep HPIC_RESET=0
				*(pBd->dsp[dspNum].pHpic) = 0; //HPIC_XADD = 0
			}
			else 
			{  // If DSP is running, then HPIC_RESET must be 1			   
				*(pBd->dsp[dspNum].pHpic) = HPIC_XADD | HPIC_RESET; //HPIC_XADD = 1

				// Write address bits 19..16 to HPIA
				*(pBd->dsp[dspNum].pHpia)  = add16_19;

				// Reset XADD and keep HPIC_RESET=1
				*(pBd->dsp[dspNum].pHpic) = HPIC_RESET; //HPIC_XADD = 0
			}		

	        // Write HPIA
		    *(pBd->dsp[dspNum].pHpia) = src_addr & 0xffff;
		}

        // Read data from HPID using auto-incremented addresses 
		pBuf[i] = (USHORT)(*(pBd->dsp[dspNum].pHpid1));

		// Increment DSP source address
		src_addr++;
	}
	
	// Release mutex
	ReleaseMutex(pBd->hMutex);
	
	
	return(TRUE);
}

/****************************************************************************
/*   internalWrite()
/*
/*  This function handles all on-chip writes through the HPI 
/*
/*  Returns
/*  TRUE - success
/*  FALSE - failure
/****************************************************************************/
static BOOL internalWrite(LPVOID pBd_, PULONG p_buffer, 
						  PULONG p_len, ULONG dest_addr, ULONG dspNum)
{
	ULONG                      i;
	ULONG add16_19 = (dest_addr & 0xf0000) > 16;
	ULONG                      cnt = *p_len;
	PUSHORT                    pBuf = (PUSHORT)p_buffer;  
	PBoardStruct               pBd = (PBoardStruct)pBd_;
		

	// Get mutex to allow access to this board's HPI interface
	WaitForSingleObject(pBd->hMutex, INFINITE);
	
	// Setup HPIA register
    if (pBd->inReset)
	{   // If DSP in reset, then HPIC_RESET must be 0
		*(pBd->dsp[dspNum].pHpic) = HPIC_XADD; //HPIC_XADD = 1

        // Write address bits 19..16
	    *(pBd->dsp[dspNum].pHpia)  = add16_19;
	
		// Reset XADD and keep HPIC_RESET=0
		*(pBd->dsp[dspNum].pHpic) = 0; //HPIC_XADD = 0
	}
	else 
	{  // If DSP is running, then HPIC_RESET must be 1			   
		*(pBd->dsp[dspNum].pHpic) = HPIC_XADD | HPIC_RESET; //HPIC_XADD = 1

       // Write address bits 19..16 to HPIA
	    *(pBd->dsp[dspNum].pHpia)  = add16_19;

		// Reset XADD and keep HPIC_RESET=1
		*(pBd->dsp[dspNum].pHpic) = HPIC_RESET; //HPIC_XADD = 0
	}	

	// Write address bits 15..0 to HPIA
    *(pBd->dsp[dspNum].pHpia) = dest_addr & 0xFFFF;
	
	// Loop through cnt data values to write to DSP
	for (i=0; i<cnt; i++)
	{
		if (add16_19 != ((dest_addr & 0xf0000) >> 16))
		{
			// Get updated upper address bits
			add16_19 = (dest_addr & 0xf0000) >> 16;

			// Setup HPIA register
			if (pBd->inReset)
			{   // If DSP in reset, then HPIC_RESET must be 0
				*(pBd->dsp[dspNum].pHpic) = HPIC_XADD; //HPIC_XADD = 1

				// Write address bits 19..16
				*(pBd->dsp[dspNum].pHpia)  = add16_19;
	
				// Reset XADD and keep HPIC_RESET=0
				*(pBd->dsp[dspNum].pHpic) = 0; //HPIC_XADD = 0
			}
			else 
			{  // If DSP is running, then HPIC_RESET must be 1			   
				*(pBd->dsp[dspNum].pHpic) = HPIC_XADD | HPIC_RESET; //HPIC_XADD = 1

				// Write address bits 19..16 to HPIA
				*(pBd->dsp[dspNum].pHpia)  = add16_19;

				// Reset XADD and keep HPIC_RESET=1
				*(pBd->dsp[dspNum].pHpic) = HPIC_RESET; //HPIC_XADD = 0
			}

	   		// Write address bits 15..0 to HPIA
			*(pBd->dsp[dspNum].pHpia) = dest_addr & 0xFFFF;
		}

        // Write data to HPID using auto-incremented addresses 
		*(pBd->dsp[dspNum].pHpid1) = (ULONG)pBuf[i];

		// Increment destination address
		dest_addr++;
	}
	
	// Release mutex
	ReleaseMutex(pBd->hMutex);
	
	return(TRUE);
}

/****************************************************************************
/*waitForPciEvent()                                                      
/*
/*   This function listens for interrupt events from the low-level driver.
/*   It re-sends the event once one is received.
/*   This routine runs in its own thread.
/****************************************************************************/
static DWORD WINAPI waitForPciEvent(LPVOID hDevice)
{
   PBoardStruct pBd = NULL;
   
   // Make sure pointer to board is valid 
   if ((pBd = getBoard(hDevice)) == NULL)
      return (FALSE);

   while (TRUE)
   {
	  // Wait for event
      WaitForSingleObject(pBd->hEvent, INFINITE);
 
      // Call event handler
	  pci2040EventHandler(pBd);

	  // If board is closed, then fail 
      if (pBd->state == CLOSED) 
         return 0;
   }
   
   return (TRUE);
}

/****************************************************************************
/*   evm55x_clear_event()
/*
/*   This function clears the event received from the low-level driver
/****************************************************************************/
static BOOL evm55x_clear_event(HANDLE hDevice, HANDLE event)
{
	ULONG len, err;
	
	// Call low-level Windows driver to clear interrupt event
	if (!DeviceIoControl(hDevice, 
		(ULONG)IOCTL_PCI2040_CLEAR_INTERRUPT_EVENT, 
		&event, 
		sizeof(HANDLE), 
		NULL, 
		0, 
		&len, 
		NULL))
	{
		if ((err = GetLastError()) != ERROR_NOT_READY)
		{
			sprintf(printf_str,"IOCTL_PCI2040_CLEAR_INTERRUPT_EVENT failed, (%d)\n",
				err);
			RedirectPrintf(EVM55XDLL_ERROR, printf_str);
		}

		return(FALSE);
	}

	return(TRUE);
}

/****************************************************************************
/*   pci2040EventHandler()
/*
/*   This function handles events coming from the low-level driver
/****************************************************************************/
static void pci2040EventHandler(PBoardStruct pBd)
{
   ULONG dspNum = 0, hpiErrorRpt;
   ULONG intEvent;
   
   // Loop while interrupt events present
   while (getInterruptInfo(pBd->hDevice, &intEvent))
   {  
      // Protect access to HPI CSR
      WaitForSingleObject(pBd->hMutex, 0);
            
      // Decode interrupt and execute response
      if (intEvent & INTDSP0)
      {
         dspNum = 0;
         SetEvent(pBd->events[DSP0_INT]);
      }
      
      if (intEvent & INTDSP1)
      {
         dspNum = 1;
         SetEvent(pBd->events[DSP1_INT]);
      }
      
      if (intEvent & INTDSP2)
      {
         dspNum = 2;
         SetEvent(pBd->events[DSP2_INT]);
      }
      
      if (intEvent & INTDSP3)
      {
         dspNum = 3;
         SetEvent(pBd->events[DSP3_INT]);
      }
      
      if (intEvent & GPINT)
      {
         SetEvent(pBd->events[GP_INT]);
      }
      
      // DSP I/O request
      if (intEvent & INTGPIO2)
      {
		  SetEvent(pBd->events[GPIO2_INT]);
      }
      
      if (intEvent & INTGPIO3)
      {
         SetEvent(pBd->events[GPIO3_INT]);
      }
      
      if (intEvent & HPIERROR)
      {
         hpiErrorRpt = GetHPIErrorDetail(intEvent);

         if (hpiErrorRpt & HPIERR0)
         {
            SetEvent(pBd->events[HPIERROR0_INT]);
         }

         if (hpiErrorRpt & HPIERR1)
         {
            SetEvent(pBd->events[HPIERROR1_INT]);
         }

         if (hpiErrorRpt & HPIERR2)
         {
            SetEvent(pBd->events[HPIERROR2_INT]);
         }

         if (hpiErrorRpt & HPIERR3)
         {
            SetEvent(pBd->events[HPIERROR3_INT]);
         }
      }
      
      ReleaseMutex(pBd->hMutex);
   }
}

/****************************************************************************
/*   getInterruptInfo()
/*
/*   This routine retieves the interrupt event information once it is received.
/*
/****************************************************************************/
static BOOL getInterruptInfo(HANDLE hDevice, PULONG pReg)
{
	ULONG len;

	// Call low-level Windows driver to get interrupt information
	if (!DeviceIoControl(hDevice, 
		(ULONG)IOCTL_PCI2040_GET_INTERRUPT_INFORMATION, 
		NULL, 
		0,
		pReg, 
		sizeof(ULONG), 
		&len, 
		NULL))
	{
		return(FALSE);
	}

	return(TRUE);
}

/****************************************************************************
/*   mem5510Location()
/*
/*   This routine returns HPI_VISIBLE if the supplied address is on-chip or within
/*   the visible external memory space, else it returns a NOT_HPI_VISIBLE.  If
/*   the address is invalid (reserved memory), INVALID_ADDR is returned.
/*
/****************************************************************************/
int  mem5510Location(ULONG addr, PBoardStruct pBd)
{
	int memLoc = HPI_VISIBLE;

	PEvmMemSpace pMem = &rsvdMem[0];
	
	// Loop through reserved memory array to check for match
	while (pMem->endAddr)
	{
		if ((addr >= pMem->startAddr) && (addr <= pMem->endAddr))
		{
			// If address is in reserved memory range, then invalid address
			return INVALID_ADDR;
		}

		// Look at next reserved memory range
		pMem++;
	}

	// Check if memory is visible by HPI at this time
	// NOTE: If DSP is in reset, then only SARAM is available via HPI.
    if (((pBd->inReset) && (addr < SARAM_START_ADDR)) ||
		((pBd->inReset) && (addr > SARAM_END_ADDR))   ||
		(addr > HPI_END_ADDR))
    {	
	   // If address is outside HPI address range, then it is not visible via HPI
	   memLoc = NOT_HPI_VISIBLE;
	}
	
	return memLoc;
}

/****************************************************************************
/*   detectDsps()
/*
/*   This routine detects all installed DSP devices and initiazes their
/*   respective structures.
/****************************************************************************/
static BOOL detectDsps(PBoardStruct pBd)
{
   int i;
   BOOL status = FALSE;
   
   PPCI2040_HPI_CSR pCsr = NULL;
   
   // If board exists...
   if (pBd->hDevice != NULL)
   {   
      // Get mutex to allow access to this board's HPI_CSR space
      WaitForSingleObject(pBd->hMutex, 0);

      // Get PCI2040 CSR addresses
      pCsr = (PPCI2040_HPI_CSR) ((PULONG)pBd->mappedAddr[CSR_BAR]);
      
	  // Loop through all DSPs to see if HPI is implemented 
      for (i = 0; i < MAX_PCI2040_DSPS; i++)
      {
         if ( (pCsr->HPIImplementation & (1 << i)) )
         {
            pBd->dsp[i].isPresent = TRUE;
			
			status = TRUE;

            // Initialize HPI register addresses
            pBd->dsp[i].pHpic  = (PULONG)((char *)pBd->mappedAddr[HPI_BAR] + i*HPI_SPACE_SIZE);
            pBd->dsp[i].pHpid1 = (PULONG)((char *)pBd->dsp[i].pHpic  + HPI_REG_OFFSET);
            pBd->dsp[i].pHpia  = (PULONG)((char *)pBd->dsp[i].pHpid1 + HPI_REG_OFFSET);
            pBd->dsp[i].pHpid2 = (PULONG)((char *)pBd->dsp[i].pHpia  + HPI_REG_OFFSET);
         }
      }

	  // Release mutex
      ReleaseMutex(pBd->hMutex);
   }

   // Return pass/fail indication
   return status;
}


/****************************************************************************
/*   setPCIRegs()
/*
/*   This routine sets all thg PCI configuration registers to the supplied 
/*   values.
/****************************************************************************/
static BOOL setPCIRegs(HANDLE hDevice, PPCI2040_CONFIG_DATA pRegs)
{
   if (!evm55x_store_pci_configuration(hDevice, pRegs)) 
      return (FALSE);
   
   return (TRUE);
}

/****************************************************************************
setPCIDefaults()                                                      

  This routine sets all the PCI configuration registers to their default
  values.
/****************************************************************************/
static BOOL setPCIDefaults(PBoardStruct pBd)
{
   // Get PCI configuration
   if (!evm55x_get_pci_configuration(pBd->hDevice, &pBd->initDefaults.pci_regs))
      return (FALSE);

   // Program GPIO3 as an output. Used to program CPLD (CPLD_TDO)
   pBd->initDefaults.pci_regs.GPBDataDir |= 0x8;
   
   // Store PCI configuration
   if (!evm55x_store_pci_configuration(pBd->hDevice, &pBd->initDefaults.pci_regs)) 
      return (FALSE);
   
   return (TRUE);
}

/****************************************************************************
getBoard()                                                      

  The getBoard function returns a board object using the specified device 
  handle.
/****************************************************************************/
static PBoardStruct getBoard(HANDLE hDevice)
{
   int i;
   
   // Search boards for specified device handld and return its structure
   for (i=0; i<MAX_PCI2040_BOARDS; i++)
   {
      if (boards[i].hDevice == hDevice)
         return (&boards[i]);
   }
   
   return (PBoardStruct)NULL;
}

/****************************************************************************
getUnusedBoard()                                                      

  The getUnusedBoard function returns an unused board object.
/****************************************************************************/
static PBoardStruct getUnusedBoard()
{
   int i;
   
   // Search through boards until unused board found
   for (i=0; i<MAX_PCI2040_BOARDS; i++)
   {
      if (boards[i].hDevice == INVALID_HANDLE_VALUE)
         return (&boards[i]);
   }

   return (PBoardStruct)INVALID_HANDLE_VALUE;
}

/****************************************************************************
validBoard()                                                      

  The validBoard function returns TRUE if the suplied  pointer points to a 
  valid board.
/****************************************************************************/
static BOOL validBoard(PBoardStruct pBd)
{
   int i;
   
   // Scan all board structures to validate board
   for (i=0; i<MAX_PCI2040_BOARDS; i++)
   {
      if (boards[i].hDevice == pBd->hDevice)
         return (TRUE);
   }
   
   return (FALSE);
}

/*****************************************************************************/
/* CheckPci2040RegOfs() -- Verify valid PCI2040 config. register offset.     */
/* Returns 1 if valid, 0 if invalid                                          */
/*****************************************************************************/
static BOOL CheckPci2040RegOfs(ULONG offset)
{
    // Validate PCI2040 configuration register offset 
	if ((offset == 0) || (offset == 2) || (offset == 4) || (offset == 6) ||
	   (offset == 8) || (offset == 9) ||
	   ((offset >= 0xC && offset <= 0xF)) ||
	   (offset == 0x10) ||
	   (offset == 0x14) ||
	   (offset == 0x18) ||
	   (offset == 0x2C) || (offset == 0x2E) ||
	   (offset == 0x34) ||
	   ((offset >= 0x3C) && (offset <= 0x3F)) ||
	   ((offset >= 0x44) && (offset <= 0x48)) ||
	   (offset == 0x4C) || (offset == 0x4F) ||
	   ((offset >= 0x50) && (offset <= 0x52)) ||
	   (offset == 0x54) ||
	   (offset == 0x58) ||
	   ((offset >= 0x5C) && (offset <= 0x5E))) 

	   return (TRUE);

   else 
	   return (FALSE);
}

/*****************************************************************************/
/* CheckCpldRegOfs() -- Verify valid CPLD register offset.                   */
/* Returns 1 if valid, 0 if invalid                                          */
/*****************************************************************************/
static BOOL CheckCpldRegOfs(ULONG offset, BOOL type)
{   
   // Verify valid CPLD register offset	
	if (type == REG_READ) //read
   {
      if ( (offset == 0x00) ||
           (offset == 0x04) ||
           (offset == 0x08) ||
           (offset == 0x0c) ||
           (offset == 0x10) ||
           (offset == 0x14))
         
         return (TRUE);
      else 
      {
         return (FALSE);
      }
   }
   else  // write
   {
      if ( (offset == 0x00) ||
           (offset == 0x10) ||
           (offset == 0x14) )
         
         return (TRUE);
      else 
         return (FALSE);
   }
}

/*****************************************************************************/
/* CheckTBCRegOfs() -- Verify valid TBC register offset.                     */
/* Returns 1 if valid, 0 if invalid                                          */
/*****************************************************************************/
static BOOL CheckTBCRegOfs(ULONG offset, BOOL type)
{
   if (type == REG_READ) //read
   {
      if ( (offset >= 0x00) && (offset <= 0x16) )
         return (TRUE);
      else 
         return (FALSE);
   }
   else //write
   {
      if(  ((offset >= 0x00) && (offset <= 0xf)) || (offset == 0x17) )
         return (TRUE);
      else 
         return (FALSE);
   }
}

/****************************************************************************
/*   waitForCompletion()
/*
/*  This function waits for the communication kernel to complete the data
/*  transfer
/*
/*  Returns
/*  TRUE - success
/*  FALSE - failure
/****************************************************************************/
BOOL waitForCompletion(PBoardStruct pBd, ULONG cnt)
{
	ULONG inProgress = -1;
	ULONG len = 2;

    // Convert cnt to 2 millisecond intervals because Sleep(1) is unpredictable
	cnt >>= 1;

	// Read DSP transfer length to see if it is still in progress
	while (inProgress && cnt)
	{
		(*pBd->intReadFunc)(pBd, &inProgress, &len, COMM_ADDR, 0);
		Sleep(2);
		cnt--;
	}

	// If still in progress, then assume communication kernel is not operating
	if (inProgress)
	{
		RedirectPrintf(EVM55XDLL_ERROR, "**Error: Communication Kernel not Responding.\n");
		return FALSE;
	}

    // If inProgress cleared, then operation was completed
	return TRUE;
}


/****************************************************************************
/*   externalRead()
/*
/*  This function handles all off-chip reads
/*
/*  Returns
/*  TRUE - success
/*  FALSE - failure
/****************************************************************************/
BOOL externalRead(PBoardStruct pBd, PULONG p_buffer, 
                  PULONG p_len, ULONG src_addr, ULONG dspNum)
{
	ExtMemStruct com; 
	ULONG len, readLen = 6;
	ULONG len1 = 2, cnt = *p_len;
	PUSHORT pBuf = (PUSHORT)p_buffer;
	
    // Load communications kernel to haneld external read if not loaded already
	loadCommKernel(pBd);

	// Memory space is external data
	com.memSpace = EXT_DATA_READ << 16;

    // Process cnt number of data words
	while (cnt)
	{
		// Limit each block transfer to COMM_DATA_SIZE 
		len = (cnt > COMM_DATA_SIZE) ? COMM_DATA_SIZE : cnt;

		// Limit each block transfer to within 64 Kword page
		if ((src_addr & 0x7F0000) != ((src_addr+len-1) & 0x7F0000))
		{
           len = 0x10000 - (src_addr & 0xFFFF);
		}
		
		// Set address, write six bytes for length, address and space
		com.len = 0;
		com.addr = com.addr = (src_addr >> 16) | (src_addr << 16);
		(*pBd->intWriteFunc)(pBd, (PULONG)&com, &readLen, COMM_ADDR, 0);

		// Write 2 bytes for length only (up to COMM_DATA_SIZE)
		com.len = len << 16;   // MSW on host = LSW on DSP
		(*pBd->intWriteFunc)(pBd, (PULONG)&com.len, &len1, COMM_ADDR, 0);
		
        // Wait for comm kernel to respond
		if (!waitForCompletion(pBd, DEFAULT_COMM_TIMEOUT))
			return FALSE;
		
		// Read data from comm kernel memory
		(*pBd->intReadFunc)(pBd, (PULONG)pBuf, &len, (ULONG)&(pTxfr->data), 0);
		
		// Increment input buffer ptr and source address and decrement counter
		pBuf += len;
		src_addr += len;
		cnt -= len;
	}
	
	return(TRUE);
}

/****************************************************************************
/*   externalWrite()
/*
/*  This function handles all off-chip writes.
/*
/*  Returns
/*  TRUE - success
/*  FALSE - failure
/****************************************************************************/
BOOL externalWrite(PBoardStruct pBd, PULONG p_buffer, 
					  PULONG p_len, ULONG dest_addr, ULONG dspNum)
{
	ExtMemStruct com; 
	ULONG len;
	ULONG writeLen=COMM_STRUCT_SIZE;
	ULONG len1 = 2, cnt = *p_len;
	PUSHORT pBuf = (PUSHORT)p_buffer;
	ULONG i;
	ULONG waitDelay = DEFAULT_COMM_TIMEOUT;

	
	// Load communications kernel to handle external read if not loaded already
	loadCommKernel(pBd);

	// Memory space is external data
	com.memSpace = EXT_DATA_WRITE << 16;
    
	// Check if Flash memory write and override memory space if applicable
    if ((dest_addr >= FLASH_START_ADDR) && (dest_addr <= FLASH_END_ADDR))
	{
		// If Flash, also increase wait delay to handle longer programming times
		waitDelay = 8000;  
		com.memSpace = EXT_FLASH_WRITE << 16;
	}

    // Process cnt number of data words
	while (cnt)
	{
		// Limit each block transfer to COMM_DATA_SIZE 
		len = (cnt > COMM_DATA_SIZE) ? COMM_DATA_SIZE : cnt;

		// Limit each block transfer to within 64 Kword page
		if ((dest_addr & 0x7F0000) != ((dest_addr+len-1) & 0x7F0000))
		{
           len = 0x10000 - (dest_addr & 0xFFFF);
		}
		
		// Set com structure to configure kernel (length, dest addr, data)
		com.len = 0;
		com.addr = (dest_addr >> 16) | (dest_addr << 16);  // swap MSW/LSW

		// Copy buffer of data into communication kernel data buffer
		for (i=0; i<len; i++)
		{
           com.data[i] = *pBuf++;
		}
		
		// Write communications structure with size equal to 0
		(*pBd->intWriteFunc)(pBd, (PULONG)&com, &writeLen, COMM_ADDR, 0);

		// Write number of words to transfer (write)
		com.len = (len << 16); // swap MSW/LSW
		(*pBd->intWriteFunc)(pBd, (PULONG)&com.len, &len1, COMM_ADDR, 0);
		
		// Wait for comm kernel to respond (8 seconds max to handle Flash)
		if (!waitForCompletion(pBd, waitDelay))
		{
			return FALSE;
		}
		
		// Increment destination address and decrement counter
		dest_addr += len;
		cnt -= len;
	}
	
	return(TRUE);
}
 
/****************************************************************************
*evm55x_flash_erase(HANDLE hDevice, int sector)
*
*  The evm55x_flash_erase erases a portion of flash memory
*  will be to flash memory.
*  
*		hDevice - Board handle returned from the call to evm55x_open.
*		sector - specifies block of flash to erase (0-10 or -1 for entire chip
*
*    Returns TRUE on succes
*    FALSE on failure.
/****************************************************************************/
DLL32_ENTRY BOOL evm55x_flash_erase(HANDLE hDevice, int sector)
{
    PBoardStruct pBd = NULL;
    ExtMemStruct com; 
    ULONG len = 2,writeLen = 6;
	ULONG start = 1<<16;
    
    // Get board structure
	if ((pBd = getBoard(hDevice)) == NULL)
        return(FALSE);
    
	// Load communications kernel to handle Flash erase if not already running
    loadCommKernel(pBd);

	// Initialize com structure for Flash erase
    com.memSpace = EXT_FLASH_ERASE << 16; 
    com.len = 0;
    com.addr = sector << 16;
    
	// Activate comm kernel to perform erase
    (*pBd->intWriteFunc)(pBd, (PULONG)&com, &writeLen, COMM_ADDR, 0);
    (*pBd->intWriteFunc)(pBd, (PULONG)&start, &len, COMM_ADDR, 0);
    
    // Wait for comm kernel to respond
	if (!waitForCompletion(pBd, 20000))
		return FALSE;
    
    return TRUE;
}

/****************************************************************************
setupHpiFunctions()                                                            

  The setupHpiFunctions function sets up the routines used to access the 
  DSP memory based on the existing connection to the board.
  
/****************************************************************************/
static void setupHpiFunctions(PBoardStruct pBd)
{
      // Set board structure pointers to int/ext read/write routines
	  pBd->intReadFunc  = internalRead;
      pBd->intWriteFunc = internalWrite;
      pBd->extReadFunc  = externalRead;
      pBd->extWriteFunc = externalWrite;
}

/****************************************************************************
pciOpen()                                                            

  The pciOpen function handles the details of opening a PCI connection to 
  the C5510 EVM
  
/****************************************************************************/
static void pciOpen(PBoardStruct pBd)
{
   unsigned       thrAddr = 0, i;
   char           deviceName[DEV_NAME_SIZE];
   char           eventName[DEV_NAME_SIZE];
   char	         *event_name = PCI2040_GLOBAL_EVENT_BASE_NAME "????";

   // Initialize device name
   memset(deviceName, '\0', DEV_NAME_SIZE);
   sprintf( deviceName, "\\\\.\\pci2040Dev%03d", pBd->index);
   
   // Create handle for PCI device 
   pBd->hDevice = CreateFile(deviceName,
                           GENERIC_READ | GENERIC_WRITE,
                           pBd->excl ? 0 : FILE_SHARE_READ | FILE_SHARE_WRITE,
                           NULL, 
                           OPEN_EXISTING, 
                           0, 
                           NULL);

   // If PCI device handle creation failed, return error
   if (pBd->hDevice == INVALID_HANDLE_VALUE)
   {
      sprintf(printf_str,"could not open handle to %s, (%d)\n",
         deviceName, GetLastError());
      
      RedirectPrintf(EVM55XDLL_ERROR, printf_str);
      return;
   }
   
   // Set PCI configuration registers to defaults
   setPCIDefaults(pBd);
   
   // Map PCI memory spaces to user memory space
   if (!map_spaces(pBd))
   {
      RedirectPrintf(EVM55XDLL_ERROR, "Unable to map PCI2040 device memory spaces\n");
      CloseHandle(pBd->hDevice);
      pBd->hDevice = INVALID_HANDLE_VALUE;
      return;
   }
   
   // Determine number of DSP on board (1 for 5510 EVM)
   detectDsps(pBd);
   
   // Initialize PCI interface
   evm55x_pci_init(pBd->hDevice, 0);
   
   // Create and install global event  
   memset(eventName, '\0', DEV_NAME_SIZE);
   lstrcpy(eventName, event_name);
   sprintf( eventName + strcspn(eventName, "?"), "%03d", pBd->index);
   
   pBd->hEvent = CreateEvent(NULL, FALSE, FALSE, eventName);

   // Clear previous call to install_vxd_event
   evm55x_clear_event(pBd->hDevice, pBd->hEvent);
   
   if (!install_vxd_event(pBd))
   {
      //CloseHandle(pBd->hDevice);
      //pBd->hDevice = INVALID_HANDLE_VALUE;
      //return;
   }
   
   // Create all other events
   for (i=DSP0_INT; i<=HPIERROR3_INT; i++)
   {
      memset(Eventnames[i], '\0', DEV_NAME_SIZE);
      sprintf( Eventnames[i], _Eventnames[i], pBd->index);
      pBd->events[i] = CreateEvent(NULL, FALSE, FALSE, Eventnames[i]);
   }
   
   pBd->outThr = (HANDLE)_beginthreadex(NULL, 0, waitForPciEvent,(void *)pBd->hDevice, 
      0, &thrAddr);
   
   if ( pBd->outThr == NULL )
   {
      sprintf(printf_str,"Unable to spawn thread -> waitForPciEvent() (%d)\n",
         GetLastError());
      
      RedirectPrintf(EVM55XDLL_ERROR, printf_str);
      
      CloseHandle(pBd->hDevice);
      CloseHandle(pBd->hEvent);
      pBd->hDevice = INVALID_HANDLE_VALUE;
      
      for (i=0; i<MAX_PCI2040_EVENTS; i++)
         CloseHandle(pBd->events[i]);
      
      return;
   }
}

/****************************************************************************
isKernelPresent()                                                            

  Determines if communication kernel is present and running.
  
/****************************************************************************/
BOOL isKernelPresent(PBoardStruct pBd)
{
    ExtMemStruct com; 
    ULONG len = 2, writeLen = 6, cnt = 20;
	ULONG done = -1;
	ULONG start = 0x1ul<<16;

    // Setup comm kernel structure for ext data read
	com.memSpace = EXT_DATA_READ << 16;
    com.len = 0;
	com.addr = (COMM_ADDR >> 16) | (COMM_ADDR << 16);
	
	// Write comm structure in DSP memory
    (*pBd->intWriteFunc)(pBd, (PULONG)&com, &writeLen, COMM_ADDR, 0);

	// Write non-zero value to length field to tell comm kernel to run
    (*pBd->intWriteFunc)(pBd, (PULONG)&start, &len, COMM_ADDR, 0);
    
	// Attempt to read data up to 20 times to see if kernel is running
	// Wait for done to be cleared by kernel or timeout after about 20 ms
	while (done && cnt--)
	{
		len = 2;  // always read two words

		// Read value at COMM_ADDR. It should go to 0 if kernel responded correctly.
		(*pBd->intReadFunc)(pBd, (PULONG)&done, &len, COMM_ADDR, 0);

		// Swap MSW/LSW
		done = (done >> 16) | (done << 16); 

		Sleep(2);
	}
	
	// Return true if kernel was running, false if not
    return (done ? FALSE : TRUE);
}

/****************************************************************************
setCommKernelFile()                                                            

  Creates the filename with path for the communication kernel COFF file.
  
  Requires that com55x.out file be stored in the evm55x bin subdirectory
  if EVM55X_DIR environment variable set or the Windows system directory.

/****************************************************************************/
BOOL setCommKernelFile(void)
{
    char  *evm55x_var;
	FILE  *fin;

    // Check for EVM55X environment variable
	if ((evm55x_var = getenv( "EVM55X_DIR" )) != NULL)
	{  // Environment variable, use comm55x.out in bin subdirectory
	   sprintf(commFile,"%s\\bin\\com55x.out",evm55x_var);
	}
	else
	{  // No environment variable, default to Windows system directory
       commFile[0] = '\0';
       GetSystemDirectory(commFile, COMM_FILE_SIZE);
	   strcat(commFile, "\\com55x.out");
	}
	
	// Attempt to open file (see if it exists)
    if( (fin = fopen( commFile, "rb" )) == NULL )
    {
		// File does not exist
		RedirectPrintf(EVM55XDLL_ERROR, "**Error: Communication Kernel file (com55x.out) not found.\n");
        return(FALSE);
    }
	else 
	{
	   // Close File
	   fclose(fin);

	   // Return successfully
       return(TRUE);
	}
}


/****************************************************************************
/*   loadCommKernel()
/*
/*  This function loads and starts the communications kernel
/*
/*  Returns
/*		none
/****************************************************************************/
BOOL loadCommKernel(PBoardStruct pBd)
{
    FILE  *fin;
    ULONG len = 1;

    // If kernel already running, then return successfully
	if (isKernelPresent(pBd)) 
	{   	
		return TRUE;
	}
    	
	// Open kernel COFF file
    if( (fin = fopen( commFile, "rb" )) == NULL )
    {
        sprintf(printf_str,"The file \"%s\" could not be opened for reading.\n",
            commFile );
        RedirectPrintf(EVM55XDLL_ERROR, printf_str);
        return(FALSE);
    }

	// Reset and unreset DSP
	// NOTE: Requires that EHPIENA input signal be high to enable HPI booting.
	evm55x_reset_dsp(pBd->hDevice,0);
	Sleep(2);
	evm55x_unreset_dsp(pBd->hDevice,0);
	Sleep(2);

    // Load communications kernel
	kernelLoaded = call_cload(pBd->hDevice, pBd, fin, TRUE, 0,0,0,0);
      
    // If kernel loaded successfully, write entry point at reset vector 
	if (kernelLoaded == TRUE)
    {
		// Create HPI boot branch instruction to entry point
		branchInstr = (C55X_BRANCH_OPCODE | (entry_point & 0x00ffffff));
		// Swap high/low words since DSP low address is MSW.
		branchInstr = (branchInstr << 16) | (branchInstr >> 16);
		
	    // Write two words
		len = 2;
		
		// Write reset vector
		if (!(*pBd->intWriteFunc)(pBd, (PULONG)&branchInstr, &len, C5510_HPI_BOOT_ADDR, 0))
		{
			RedirectPrintf(EVM55XDLL_ERROR, "Could not initiate communications kernel code execution.\n");
			kernelLoaded = FALSE;
		}

	    // Run DSP core to execute COFF application
	    evm55x_run_dsp(pBd->hDevice,0);
		Sleep(2);
	}
    
    // Close kernel COFF file
	fclose(fin);
      
    return (kernelLoaded);
}
