/****************************************************************************/
/*	dmatest.c  -  an example to show how to use the DMA to transmit data    */
/*                out of a McBSP. All three LEDs flash when the DMA         */
/*                transfer completes.                                       */
/****************************************************************************/
#include <board.h>
#include <dma55x.h>
#include <mcbsp55x.h>
#include <intr.h>

/****************************************************************************/
/*	defines                                                                 */
/****************************************************************************/

//McBsp 2 setting for AC'97 compliant codec in dual mode
#define WD_PER_FRAME_1			0x0
#define WD_PER_FRAME_4			0x3
#define WD_PER_FRAME 			0x4
 
// set receive control reg values: each frame of receive data has two phases.
// select 16-bit word in phase 1 of the frame.
#define RCR1_VAL				((WD_PER_FRAME_1 << 8) | (WORD_LENGTH_16 << 5))

// select 20-bit word in phase 2 of the frame.
#define RCR2_VAL 				((DUAL_PHASE        << 15) |\
								(WD_PER_FRAME_4     << 8)  |\
								(WORD_LENGTH_20     << 5)  |\
								(NO_COMPAND_MSB_1ST << 3)  |\
								(NO_FRAME_IGNORE    << 2)  |\
								(DATA_DELAY1        << 0))

// set transmit control reg values: each frame of transfer data has two phases.
// select 16-bit word in phase 1 of the frame.
#define XCR1_VAL				((WD_PER_FRAME_1 << 8) | (WORD_LENGTH_16 << 5))

// select 20-bit word in phase 2 of the frame.
#define XCR2_VAL				((DUAL_PHASE        << 15) |\
								(WD_PER_FRAME_4     << 8)  |\
								(WORD_LENGTH_20     << 5)  |\
								(NO_COMPAND_MSB_1ST << 3)  |\
								(NO_FRAME_IGNORE    << 2)  |\
								(DATA_DELAY1        << 0)) 

// set pin control reg values: transmit and receive frame-sync pulses are active
// high. Transmit data is clocked on rising edge of CLKX, while receive data
// is sampled on falling edge of CLKR.
#define PCR_VAL					((FSYNC_POL_HIGH<<3)|(FSYNC_POL_HIGH<<2)|(CLKX_POL_RISING<<1)|(CLKR_POL_FALLING))


#define SPCR1_VAL               0
#define SPCR2_VAL               0

#define SRGR1_VAL	0
#define SRGR2_VAL	0

#define BLINK_DELAY	150
      
#define DMA_BUFSIZE		5
  
/****************************************************************************/
/*	functions                                                               */
/****************************************************************************/     
void do_dma();
void setup_serial_port();
void setup_dma();
interrupt void _rx_isr();

/****************************************************************************/
/*	global variables                                                        */
/****************************************************************************/
u32 txBuf[5];
u32 *pt;
u16 *p16;
volatile u32 value = 0;

/*****************************************************************************/
/* MAIN                                                                      */
/*****************************************************************************/
void main()
{
    //initialize board
    brd_init(160);

    //start DMA
    do_dma();

    // blink all 3 user LEDs forever
	while (1)
	{
		brd_led_toggle(BRD_LED0);
		brd_led_toggle(BRD_LED1);
		brd_led_toggle(BRD_LED2);

		brd_busy_delay(BLINK_DELAY);
	}
}


/****************************************************************************/
/*	interrupt void _rx_isr()                                                */
/*                                                                          */
/*	Parameters:	None                                                        */
/*                                                                          */
/*	Return:		None                                                        */
/*                                                                          */
/****************************************************************************/
interrupt void _rx_isr()
{
	volatile int dummy;
	
    dummy = DRR2(2);
    dummy = DRR1(2);
}


/****************************************************************************/
/*	interrupt void _tx_isr()                                                */
/*                                                                          */
/*	Parameters:	None                                                        */
/*                                                                          */
/*	Return:		None                                                        */
/*                                                                          */
/****************************************************************************/
interrupt void _tx_isr()
{
	volatile int dummy = DMCSR(4);

	if (!(value++ & 0xfffffff)) 
	{
		CPLD_CTRL1_REG ^= 1;
	}  
	
    DMCCR(4) |= (1 << DM_END_PROG);
}
 

/****************************************************************************/
/*	void do_dma()                                                           */
/*                                                                          */
/*	This function sets up the McBSP and the DMA channel and then            */
/*	starts the transfer between the McBSP and DMA  .                        */
/*																			*/
/*	Parameters:	None                                                        */
/*                                                                          */
/*	Return:		None                                                        */
/*                                                                          */
/****************************************************************************/          
void do_dma()
{
	u16 i; 
               
    //stop serial ports
    SPCR1(2)  = 0;
    SPCR2(2)  = 0;
	
	DMCCR(4)   = 0; //stop any current transfer
	
    unhook_interrupt(XINT2_TRAP);
    unhook_interrupt(RINT2_TRAP);
	                 
	//clear any pending interrupts
    IFR0 |= IFR0;
 
 	hook_interrupt(DMAC4_TRAP, _tx_isr);
  	  
    IER0 |= (u16)(1<<DMAC4);
    IER0 |= (u16)(1<<XINT2);
  	
	INTR_GLOBAL_ENABLE;

	DMGCR = 0x0000;

	//reset Tx DMA ch
	DMCSDP(4)  = 0;		// src and dst parameters reg
	DMCICR(4)  = 0;		// interrupt control reg

	DMCSSAL(4) = 0;		// src start address reg (lower part)
	DMCSSAU(4) = 0;		// src start address reg (upper part)
	DMCDSAL(4) = 0;		// dst start address reg (lower part)
	DMCDSAU(4) = 0;		// dst start address reg (upper part)
	
	DMCEN(4)   = 0;		// element number reg
	DMCFN(4)   = 0;		// frame number reg
	DMCEI(4)   = 0;		// element index reg
	DMCFI(4)   = 0;		// frame index reg
                    
    //setup data to transfer 
    pt = txBuf;                  //point to SARAM
	for (i=0; i<DMA_BUFSIZE; i++)
	{
		pt[i] = i;
	}

	setup_dma();

	setup_serial_port();

	//now start McBsp and DMA transfers
    DMA_ENABLE(4);

    // Enable serial port (both TX/RX)
    MCBSP_ENABLE(2, MCBSP_TX);
}

/****************************************************************************/
/*	void setup_serial_port()                                                */
/*																			*/
/*	This function initializes McBSP 2, which is used as dst port            */
/*	for the DMA transfer.													*/
/*                                                                          */
/*	Parameters:	None                                                        */
/*                                                                          */
/*	Return:		None                                                        */
/*                                                                          */
/****************************************************************************/
void setup_serial_port()
{
    SPCR1(2)  = 0;			// serial port control reg 1
    SPCR2(2)  = 0;			// serial port control reg 2

    RCR1(2)  = RCR1_VAL;	// receive control reg 1
    RCR2(2)  = RCR2_VAL;	// receive control reg 2

    XCR1(2)  = XCR1_VAL;	// transmit control reg 1
    XCR2(2)  = XCR2_VAL;	// transmit control reg 2

    SRGR1(2) = 0;			// sample rate generator reg 1
    SRGR2(2) = 0;			// sample rate generator reg 2

    MCR1(2)  = 0;			// multichannel control reg 1
    MCR2(2)  = 0;			// multichannel control reg 2

    PCR(2)   = PCR_VAL;		// pin control reg
	
    RCERA(2) = 0;			// receive channnel enable registers
    RCERB(2) = 0;			// for partition A,B,.....,H
    RCERC(2) = 0;
    RCERD(2) = 0;
    RCERE(2) = 0;
    RCERF(2) = 0;
    RCERG(2) = 0;
    RCERH(2) = 0;
	
    XCERA(2) = 0;			// transmit channel enable registers
    XCERB(2) = 0;			// for partition A,B,.....,H
    XCERC(2) = 0;
    XCERD(2) = 0;
    XCERE(2) = 0;
    XCERF(2) = 0;
    XCERG(2) = 0;
    XCERH(2) = 0;
}


/****************************************************************************/
/*	void setup_dma()                                                        */
/*																			*/
/*	This functions sets up the registers for controlling the channel's      */
/*	context during data transfer in a DMA's channel. The dst port is        */
/*	one of the McBsp while the src port is DARAM.                           */
/*                                                                          */
/*	Parameters:	None                                                        */
/*                                                                          */
/*	Return:		None                                                        */
/*                                                                          */
/****************************************************************************/
void setup_dma()
{
	u32 addr;

	DMGCR = 0x0000;		// global control register

	// Transmit DMA setup
	// bits 12-9 of DMCSDP are dst selection bits. Here DST selects Peripherals as dst.
	// bits 5-2 of DMCSDP are src selection bits. Here SRC selects DARAM as src.
	// bits 1-0 of DMCSDP are data type bit. Here the DMAC makes 32-bit accesses at src & dst.
	DMCSDP(4)  = (DM_RHEA << DM_DST) | (DM_DARAM << DM_SRC) | (DM_DTYPE_32 << DM_DATA_TYPE);

	// send channel interrupt to CPU when all of current frame has been 
	// transferred from src port to dst port.
	DMCICR(4)  = (1 << DM_FRAME);

	// set src address
	addr = ((u32)pt)<<1;		// change word address to form a byte address
	DMCSSAL(4) = (u16)addr;		// load 16 LSBs of byte address
	DMCSSAU(4) = (u16)((addr & 0xff0000)>>16);		// load 8 MSBs of byte address

	// set dst address
	// DXR2 is a McBSP data transmit reg 2
    addr = ((u32)DXR2_ADDR(2))<<1;		// change word address to form a byte address
	DMCDSAL(4) = (u16)addr;			// load 16 LSBs of byte address
	DMCDSAU(4) = (u16)((addr & 0xff0000)>>16);		// load 8 MSBs of byte address
	                      
	//Divide by 2 for double word transfers
	DMCEN(4)   = DMA_BUFSIZE;		// element number reg  
	DMCFN(4)   = 1;					// frame number reg
	DMCEI(4)   = 0;					// 4-byte element index reg
	DMCFI(4)   = 0;					// frame index reg

	// set the channel control reg
	DMCCR(4)  = (DM_NOMOD     << DM_DST_AMODE)	|	// constant address mode for dst
				(DM_POST_INCR << DM_SRC_AMODE)	|	// automatic post increment at src port
				(1 << DM_END_PROG)				|	// end of programmation
				(1 << DM_AUTOINIT)				|	// DAMC reinitializes channel
				(1 << DM_PRIO)					|	// high priority channel
				(DMSYNC_XEVT1 << DM_SYNC);			// McBSP 1 transmit event

	DMCCR(4) |=  (1 << DM_EN);		// channel is enabled
}      
