/*****************************************************************************/
/*****************************************************************************/
/*  fifo.c - TMS320VC5510 EVM FIFO Library Functions                         */
/*                                                                           */
/*  This module provides the routines used to support the HPI				 */
/*  implemented fifo used for HOST/DSP communication.                        */
/*                                                                           */
/*  FUNCTIONS:                                                               */
/*                                                                           */
/*  brd_fifo_read() - reads data from FIFO                                   */
/*  brd_fifo_write() - writes data to FIFO                                   */ 
/*  brd_fifo_reset() - reset both FIFO pointers                              */
/*                                                                           */
/*  STATIC FUNCTIONS:                                                        */
/*                                                                           */
/*                                                                           */
/*  GLOBAL VARIABLES DEFINED                                                 */
/*                                                                           */
/*                                                                           */ 
/*****************************************************************************/

/*---------------------------------------------------------------------------*/
/* INCLUDES AND LOCAL DEFINES                                                */
/*---------------------------------------------------------------------------*/
#include <board.h>

/*---------------------------------------------------------------------------*/
/* typedefs                                                                  */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* DEFINES                                                                   */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* PRAGMAS                                                                   */
/*---------------------------------------------------------------------------*/
#pragma DATA_SECTION(gfifo, ".fifo")

/*---------------------------------------------------------------------------*/
/* GLOBAL VARIABLES                                                          */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* EXTERNAL FUNCTION PROTOTYPES                                              */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* FILE LOCAL (STATIC) VARIABLES                                             */
/*---------------------------------------------------------------------------*/
static volatile FIFO gfifo;

/*---------------------------------------------------------------------------*/
/* FILE LOCAL (STATIC) PROTOTYPES                                            */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* FUNCTIONS                                                                 */
/*---------------------------------------------------------------------------*/

/*****************************************************************************/
/*  s16 brd_fifo_read (s8 *bufPtr, u16 cnt)                                  */
/*                                                                           */
/*  This routine begins a synchronous read from the fifo channel.            */
/*                                                                           */
/*  Parameters:                                                              */
/*  - bufPtr is the location of the data to be written to the FIFO           */
/*  - cnt is the number of bytes of data to be written to the FIFO,          */
/*                                                                           */
/*  Return:                                                                  */
/*  - returns                                                                */
/*      Number of bytes read,                                                */                                                 
/*                                                                           */
/*  Notes:                                                                   */
/*                                                                           */
/*****************************************************************************/
s16 brd_fifo_read(char *bufPtr, u16 cnt)
{
    u16 i;
    
    for (i=0; i<cnt; i++)
    {
		 // wait for fifo to fill
        while (gfifo.host_ch.takePtr == gfifo.host_ch.putPtr);
        
        *bufPtr++ = *((char *)gfifo.host_ch.takePtr++);
        
        if ( gfifo.host_ch.takePtr > (u32)&gfifo.host_ch.data[BRD_FIFO_SIZE-1] )
            gfifo.host_ch.takePtr = (u32)gfifo.host_ch.data;
    }
    
    return cnt;
}

/*****************************************************************************/
/*  s16 brd_fifo_write (s8 *bufPtr, u16 cnt)                                 */
/*                                                                           */
/*  This routine begins a synchronous read from the fifo channel.            */
/*                                                                           */
/*  Parameters:                                                              */
/*  - bufPtr is the location of the data to be written to the FIFO           */
/*  - cnt is the number of bytes of data to be written to the FIFO,          */
/*                                                                           */
/*  Return:                                                                  */
/*      Number of bytes written,                                             */                                                  
/*                                                                           */
/*  Notes:                                                                   */
/*                                                                           */
/*****************************************************************************/
s16 brd_fifo_write(char *bufPtr, u16 cnt)
{
    u16 i;
    
    for (i=0; i<cnt; i++)
    {
		 // wait for fifo to be read
        while ( (gfifo.dsp_ch.putPtr+1) == gfifo.dsp_ch.takePtr);
        
        *((char *)gfifo.dsp_ch.putPtr++) = *bufPtr++;
        
        if ( gfifo.dsp_ch.putPtr > (u32)&gfifo.dsp_ch.data[BRD_FIFO_SIZE-1] )
        {
			 // wait for fifo to be read
            while (gfifo.dsp_ch.takePtr == (u32)gfifo.dsp_ch.data);
            
            gfifo.dsp_ch.putPtr = (u32)gfifo.dsp_ch.data;
        }
    }
    
    return (cnt);
}

/*****************************************************************************/
/*  void brd_fifo_reset(void)                                                */
/*                                                                           */
/*  This routine initializes the fifo                                        */
/*                                                                           */
/*  Parameters:                                                              */
/*  - none                                                                   */
/*                                                                           */
/*  Return:                                                                  */
/*  - none                                                                   */
/*                                                                           */
/*  Notes:                                                                   */
/*                                                                           */
/*****************************************************************************/
void brd_fifo_reset(void)
{
    gfifo.host_ch.takePtr = (u32)gfifo.host_ch.data;
    gfifo.host_ch.putPtr = (u32)gfifo.host_ch.data;
    
    gfifo.dsp_ch.takePtr = (u32)gfifo.dsp_ch.data;
    gfifo.dsp_ch.putPtr = (u32)gfifo.dsp_ch.data;
}
