/*****************************************************************************/
/*****************************************************************************/
/*  sem.c - TMS320VC5510 EVM Semaphore Support Library Functions             */
/*                                                                           */
/*  This module provides the routines used to manipulate the CPLD semaphores */
/*                                                                           */
/*                                                                           */
/*  FUNCTIONS:                                                               */
/*                                                                           */
/*  brd_sem_get() - acquires one of the two CPLD semaphores.                 */
/*  brd_sem_wait() - waits indefinitely to acquire one of the                */
/*                   two CPLD semaphores.                                    */
/*  brd_sem_release() - releases a previously acquired CPLD semaphore        */
/*                                                                           */
/*  STATIC FUNCTIONS:                                                        */
/*                                                                           */
/*                                                                           */
/*  GLOBAL VARIABLES DEFINED                                                 */
/*                                                                           */
/*                                                                           */ 
/*****************************************************************************/

/*---------------------------------------------------------------------------*/
/* INCLUDES AND LOCAL DEFINES                                                */
/*---------------------------------------------------------------------------*/
#include <type.h>
#include <board.h>

/*---------------------------------------------------------------------------*/
/* typedefs                                                                  */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* DEFINES                                                                   */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* PRAGMAS                                                                   */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* GLOBAL VARIABLES                                                          */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* EXTERNAL FUNCTION PROTOTYPES                                              */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* FILE LOCAL (STATIC) VARIABLES                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* FILE LOCAL (STATIC) PROTOTYPES                                            */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* FUNCTIONS                                                                 */
/*---------------------------------------------------------------------------*/

/*****************************************************************************/
/*  s16 brd_sem_get(BrdSem sem)                                              */
/*                                                                           */ 
/*  This routine tries to obtain the specified user semaphore.               */
/*                                                                           */
/*                                                                           */
/*  Parameters:                                                              */
/*      - sem - desired semaphore                                            */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - BRD_OK on success                                                      */
/*  - BRD_ERROR on failure                                                   */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_sem_get(BrdSem sem)
{
    // Check if semaphore 0 is requested and it is available
    if ((sem == BRD_SEM0) && ((CPLD_SEM0_REG & 1) == 0))
    {
        // request semaphore 0
		CPLD_SEM0_REG = 1;

        // verify semaphore ownership
        // if semaphore 0 not owned now, then return error
        if ((CPLD_SEM0_REG & 1)== 0)
        {
           return BRD_ERROR;
        }
		
    }
    // Check if semaphore 1 is requested and it is available
    else if ((sem == BRD_SEM1) && ((CPLD_SEM1_REG & 1) == 0))
    {
        // request semaphore 1
        CPLD_SEM1_REG = 1;

        // verify semaphore ownership
        // if semaphore 1 not owned now, then return error
        if ((CPLD_SEM1_REG & 1) == 0)
        {
           return BRD_ERROR;
        }
    }
    else
	{
        return BRD_ERROR;
	}
    
    return BRD_OK;
}

/*****************************************************************************/
/*  s16 brd_sem_wait(BrdSem sem)                                             */
/*                                                                           */ 
/*  This routine tries to obtain the specified user semaphore. It blocks till*/
/*  the semaphore is acquired.                                               */
/*                                                                           */
/*  Parameters:                                                              */
/*      - sem - desired semaphore                                            */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - BRD_OK on success                                                      */
/*  - BRD_ERROR on failure                                                   */
/*                                                                           */ 
/*  Notes: This routine does not test for availability of required           */
/*         semaphore. The user needs to check for availability before        */
/*         calling this routine.                                             */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_sem_wait(BrdSem sem)
{
    if (sem == BRD_SEM0) 
    {
		CPLD_SEM0_REG = 1;
        
		while ((CPLD_SEM0_REG & 1) == 0);
    }
    else if (sem == BRD_SEM1)
    {
		
        CPLD_SEM1_REG = 1;
        
		while ((CPLD_SEM1_REG & 1) == 0);
    }
    else
	{
        return BRD_ERROR;
	}
    
    return (BRD_OK);
}

/*****************************************************************************/
/*  s16 brd_sem_release(BrdSem sem)                                          */
/*                                                                           */ 
/*  This routine releases the previously acquired semaphore.                 */
/*                                                                           */
/*  Parameters:                                                              */
/*      - sem - semaphore to release                                         */
/*                                                                           */ 
/*  Return:                                                                  */
/*  - BRD_OK on success                                                      */
/*  - BRD_ERROR on failure                                                   */
/*                                                                           */ 
/*  Notes:                                                                   */
/*                                                                           */ 
/*****************************************************************************/
s16 brd_sem_release(BrdSem sem)
{
    // if semaphore 0
    if (sem == BRD_SEM0)
        // set to 0 to release
        CPLD_SEM0_REG = 0;

    // if semphore 1
    else if (sem == BRD_SEM1)
        // set to 0 to release
        CPLD_SEM1_REG = 0;
    else
    // only semaphores 0 and 1 are supported
        return (BRD_ERROR);
    
    return (BRD_OK);
}
