/*****************************************************************************/
/*  flash.h -                                                                */
/*                                                                           */
/*       This module is the device library for the on-board FLASH memory     */
/*                                                                           */
/*  MACRO FUNCTIONS:                                                         */
/*                                                                           */
/*  FUNCTIONS:                                                               */
/*                                                                           */
/*  flash_reset()                                                            */
/*  flash_erase()                                                            */
/*  flash_write()                                                            */
/*  flash_read()                                                             */
/*  flash_write_32()                                                         */
/*  flash_read_32()                                                          */
/*  flash_checksum()                                                         */
/*****************************************************************************/
#ifndef _FLASH_H_
#define _FLASH_H_

/*****************************************************************************/
/* INCLUDES                                                                  */
/*****************************************************************************/
#include "type.h"

/*****************************************************************************/
/* DEFINES                                                                   */
/*****************************************************************************/
#define ALL_SECTORS   -1           //for sector selector in erase
#define MAX_SECTOR    11u          //sectors 0-10

#define FLASH_BASE_ADDR    0x200000ul  //linear base word address

/****************************************************************************/
/* typedefs & enums															*/
/****************************************************************************/

/****************************************************************************/
/* PUBLIC FUNCTION PROTOTYPES                                               */
/****************************************************************************/

/****************************************************************************/
/*  int flash_erase(u16 sector)												*/
/*																			*/
/*  This function erases one or more sectors of the flash.					*/
/*                                                                          */
/* Parameters:                                                              */
/*      int	sector;		//values from -1 for ALL, 0-16 for valid			*/
/*						//sector numbers									*/
/*                                                                          */ 
/*  Return:                                                                 */
/*  - OK success                                                            */
/*  - ERROR failure                                                         */
/*                                                                          */ 
/*  Notes:                                                                  */
/*                                                                          */ 
/****************************************************************************/
int flash_erase(u16 sector);

/****************************************************************************/
/*  int flash_write(u16 *source, u16 *dest, u32 length)                     */
/*                                                                          */
/*  This function writes a block of data into Flash memory. The Flash is    */
/*	addressed with a linear address scheme that addresses the words of the	*/
/*  Flash memory as an offset from the start of the Flash chip. In other    */
/*	words, a dest of 0 will result in a write to sector 0, page 0, address	*/
/*  as viewed by the CPU, will be 0x400000 (0x200000 word address).         */
/*                                                                          */
/*  Parameters:                                                             */
/*		unsigned int *source;												*/
/*		unsigned long dest													*/
/*		unsigned int length													*/
/*                                                                          */
/*  Return:                                                                 */
/*  - OK success                                                            */
/*  - ERROR failure                                                         */
/*                                                                          */
/*  Notes:                                                                  */
/*                                                                          */
/****************************************************************************/
int flash_write(u16 *source, u16 *dest, u32 length);

/****************************************************************************/
/*  int flash_read(u16 *source, u16 *dest, u32 length)                      */
/*                                                                          */
/*  This function reads a block of data from Flash memory. The addressing   */
/*	is the same as in the functions above.					                */
/*                                                                          */
/*  Parameters:                                                             */
/*		unsigned long source												*/
/*		unsigned int length													*/
/*                                                                          */
/*  Return:                                                                 */
/*  - data is read from the Flash and written into the memory addressed by  */
/*		dest																*/
/*  - always returns the length of the block read.							*/
/*		there are no documented errors; however, it is possible to overrun	*/
/*		the border of the flash page window. No checking is done at this	*/
/*		time.		                                                        */
/*                                                                          */
/*  Notes:                                                                  */
/*                                                                          */
/****************************************************************************/
int flash_read(u16 *source, u16 *dest, u32 length);

/****************************************************************************/
/*  int flash_write_32(u32 *source, u32 *dest, u32 length)                  */
/*                                                                          */
/*  This function writes a block of data into Flash memory. The Flash is    */
/*	addressed with a linear address scheme that addresses the words of the	*/
/*  Flash memory as an offset from the start of the Flash chip. In other    */
/*	words, a dest of 0 will result in a write to sector 0, page 0, address	*/
/*  as viewed by the CPU, will be 0x400000 (0x200000 word address).         */
/*                                                                          */
/*  Parameters:                                                             */
/*		u32 *source															*/
/*		u32	*dest															*/
/*      u32 length                                                          */
/*                                                                          */
/*  Return:                                                                 */
/*  - OK success                                                            */
/*  - ERROR failure                                                         */
/*                                                                          */
/*  Notes:                                                                  */
/*                                                                          */
/****************************************************************************/
int flash_write_32(u32 *source, u32 *dest, u32 length);

/****************************************************************************/
/*  int flash_read_32(u32 *source, u32 *dest, u32 length)                   */
/*                                                                          */
/*  This function reads a block of data from Flash memory. The addressing   */
/*	is the same as in the functions above.					                */
/*                                                                          */
/*  Parameters:                                                             */
/*      u32 *source                                                         */
/*      u32 *dest                                                           */
/*      u32 length                                                          */
/*                                                                          */
/*  Return:                                                                 */
/*  - data is read from the Flash and written into the memory addressed by  */
/*		dest																*/
/*  - always returns the length of the block read.							*/
/*		there are no documented errors; however, it is possible to overrun	*/
/*		the border of the flash page window. No checking is done at this	*/
/*		time.		                                                        */
/*                                                                          */
/*  Notes:                                                                  */
/*                                                                          */
/****************************************************************************/
int flash_read_32(u32 *source, u32 *dest, u32 length);

/****************************************************************************/
/*  u32 flash_checksum(u32 *source, u32 length)                             */
/*                                                                          */
/*  This function returns a checksum for the specified memory range         */
/*                                                                          */
/*  Parameters:                                                             */
/*      unsigned int *source;       //linear memory address within the Flash*/
/*		unsigned int length;                                                */
/*                                                                          */
/*  Return:                                                                 */
/*  - checksum value                                                        */
/*  - no errors documented                                                  */
/*                                                                          */
/*  Notes:                                                                  */
/*                                                                          */
/****************************************************************************/
u32 flash_checksum(u32 *source, u32 length);

#endif

