#!/bin/sh
#
## NAME
##      MountAll.sh - Create Mount points below <Rootdir> for all drives
##
## SYNOPSIS
##      MountAll.sh <Rootdir>
##
## PURPOSE
##      Mount all Windows Drives (C:, D:, ...) below the given Rootdir.'
##      This makes the Windows bash execute "cd x:" commands quickly.
##      By also mounting /bin and /tmp, provide a secure POSIX-like environ-
##      ment for the bash (correctly interpret the "#!/bin/sh" sequence
##      in shellscripts).
##
##      <Rootdir>   Any legal directory, which will be created.
##                  This directory will become the Root ("/") for bash.
##                  Below it, empty directories will be created for all
##                  mount points.
##
## EXAMPLE
##      sh MountAll.sh $SNIFF_DIR/ROOTDIR
##
## AUTHOR
##      Martin Oberhuber, (c) 1999 TakeFive Software GmbH
##
## SEE ALSO
##      UmountAll.sh
##

if [ "$SNIFF_DIR" = "" ]; then
    echo "MountAll.sh : you must set SNIFF_DIR before executing this script!"
    exit 1
fi
bin=$SNIFF_DIR/bin

if [ $# != 1 -o "$1" = "-h" ]; then
    $bin/grep '^##' $0 | $bin/cut -c4-
    exit 0
fi

ROOTDIR=${1:?"Missing Rootdir"}
if [ ! -d $ROOTDIR ]; then
    $bin/mkdir -p $ROOTDIR
    if [ $? != 0 ]; then
        echo "Error creating Rootdir on $ROOTDIR -- exiting..."
        exit 1
    fi
fi

echo "MountAll.sh: Mounting Windows Drives for Bash."
echo "MountAll.sh: Use UmountAll.sh to undo."

# make space, since ALL directories would create "not enough files" error
$bin/umount /dev/fd1 2> /dev/null
$bin/umount /dev/st1 2> /dev/null
$bin/umount /bin 2> /dev/null
$bin/umount /tmp 2> /dev/null

# Need to umount Root before moving it to a different place
$bin/umount / 2> /dev/null
$bin/mount $ROOTDIR / 2> /dev/null
cd /

# Make the mount for /bin
test -d bin || $bin/mkdir bin
$bin/mount $SNIFF_DIR/bin /bin

# Make the mount for /tmp
test -d tmp || $bin/mkdir tmp
myTmp="C:/temp"
test "$TMP" != "" && myTmp="$TMP"
test "$TEMP" != "" && myTmp="$TEMP"
test "$TMPDIR" != "" && myTmp="$TMPDIR"
$bin/mount $myTmp /tmp

# Make the mount for all the rest
for i in C D E F G H I J K L M N O P Q R S T U V W X Y Z
do
    $bin/umount /$i 2> /dev/null
    test -d $i || $bin/mkdir $i
    $bin/mount $i: /$i
done
