#!/bin/sh
#
## NAME
##      LinkSniffOver - Create a Project-Level (symlinked) SNiFF+ installation
##
## SYNOPSIS
##      LinkSniffOver.sh <source_dir> <target_dir>
##
## DESCRIPTION
##      This command creates a local SNiFF+ installation by creating 
##      symbolic links to a common SNiFF+ installation, but keeping local
##      changes as local files.
##      You should first create any directories that you need to change
##      locally, or that you extracted from a SNiFF integrations package;
##      Everything that exists as local file will be kept local, while for
##      all missing files symbolic links to the main installation will be
##      made. To clean up the linked SNiFF+ installation, you can use
##      the CleanSniff command.
##
##      Note: the following vital SNiFF+ configuration directories and files
##      will ALWAYS be created as local directories or copied:
##
##      DIRs:  Preferences/FileTypes, Preferences/Platforms, Preferences/VCS
##             bin, config/editor, lib/python/Sniff, make_support
##      FILEs: SitePrefs.sniff, config/SiteMenus.sniff
##
##      Any Error messages printed during Link creation just stem from files
##      or directories already existing locally, so they cannot be overwritten.
##      You can safely ignore these type of error messages.
##
## PARAMETERS
##      <source_dir> Absolute or relative path pointing to the full 
##                   SNiFF+ installatino you want to create symlinks to.
##      <target_dir> Absolute or relative path pointing to the workspace
##                   where you want to create the linked SNiFF installation.
##
## AUTHOR
##      Martin Oberhuber, (c) 1999 TakeFive Software GmbH
##
## SEE ALSO
##      CleanSniff, PackSniffChanges

if [ $# != 2 -o "$1" = "-h" ]; then
    # Print Usage information
    grep '^##' $0 | cut -c4-
    exit 0
fi

dir1=${1:?"Error, missing source"}
dir2=${2:?"Error, missing target"}

curDir=`pwd`
cd $dir1 ; dir1=`pwd`
cd $curDir
cd $dir2 ; dir2=`pwd`

# Create vital stuff
mkdir bin
mkdir -p config/editor
mkdir -p lib/python/Sniff
mkdir make_support
mkdir -p Preferences/VCS
mkdir -p Preferences/FileTypes
mkdir -p Preferences/Platforms
test -f SitePrefs.sniff || cp $dir1/SitePrefs.sniff .
test -f config/SiteMenus.sniff || cp $dir1/config/SiteMenus.sniff config

# Create the Links
echo "Creating symbolic links to $dir1"
echo "Error messages concerning existing files are perfectly normal here."
echo "They just remind you what files you already have locally ..."
for d in `find . -type d -print` ; do
  cd $dir2/$d
  if [ -r $dir1/$d ]; then
	theFiles=`ls -A $dir1/$d`
	if [ "$theFiles" != "" ]; then
		ln -s $dir1/$d/* .
	fi
  fi
done 

cd $curDir
exit 0
