#!/bin/sh
#* MRe
#***********************************************************************
#*                                                                     *
#*   MODULE:  sniffMakeTorPlatforms.sh                                 *
#*   DATE:    2000/04/09                                               *
#*   PURPOSE: Script to generate the MakeSupport bridge files for the  *
#*            Integration of SNiFF+ for Tornado                        *
#*                                                                     *
#*---------------------------------------------------------------------*
# This script does the following:
# Generates Platform description files and make include files
# for the SiFF+ make support for TornadoII
#
# Usage: sniffMakeTorPlatforms.sh
#          <Tormado make folder>
#          <SNiFF+ platforms folder>
#          <SNiFF+ platforms file template>
#          <SNiFF+ make_support folder>
#          <SNiFF+ make_support file template>
#          [ logfile for output ]
#
#----------------------------------------------------------------------*

# DEBUG=0 not Debug output
# DEBUG=1 print names of generated files and value of tha variables CPU and TOOL
# DEBUG=2 same as DEBUG=1 plus "cat"ing the platforms file

if [ "$1" = "-d" ]; then
  DEBUG=1
  shift
else
  DEBUG=0
fi

prog=`basename "$0"`

Usage ()
{
	echo ""
  echo "Usage: $prog <Tormado make folder> <SNiFF+ platforms folder> <SNiFF+ platforms file template> <SNiFF+ make_support folder> <SNiFF+ make_support file template>"
  echo "       <Tormado make folder>"
	echo "            is the pathname of the folder where"
	echo "            Tornado keeps the target platform specific makerules,"
  echo "            usually \"\$WINDBASE/target/h/make\""
  echo "       <SNiFF+ platforms folder>"
	echo "            is the pathname of the folder where"
	echo "            SNiFF+ keeps the target platform specific settings,"
  echo "            usually \"\$SNIFF_DIR/Preferences/Platforms\""
  echo "       <SNiFF+ platforms file template>"
	echo "            is the pathname of the template file for"
	echo "            SNiFF+'s target platform specific settings,"
  echo "            usually \"\$SNIFF_DIR/integrations/TornadoII\""
  echo "       <SNiFF+ make_support folder>"
	echo "            is the pathname of the folder where"
	echo "            SNiFF+ keeps the target make_support files,"
  echo "            usually \"\$SNIFF_DIR/make_support\""
  echo "       <SNiFF+ make_support file template>"
	echo "            is the pathname of the template file for"
	echo "            SNiFF+'s target make_support files,"
  echo "            usually \"\$SNIFF_DIR/integrations/TornadoII\""
  echo "       [logfile for output]"
	echo "            If specified, output will be copied into this file."
  echo "            Usually left empty."
	echo ""
  exit 1
}

# Sanity check
if [ $# -lt 5 ]
then
  echo "$prog: Incorrect amount of parameters, should be 5, you entered $#!"
  Usage
fi

if [ ! -d "$1" ]
then
  echo ""
  echo "$prog: Cannot find Tornado make folder"
  Usage
fi

if [ ! -d "$2" ]
then
  echo ""
  echo "$prog: Cannot find SNiFF+ platforms folder"
  Usage
fi

if [ ! -f "$3" ]
then
  echo ""
  echo "$prog: Cannot find SNiFF+ platform template file"
  Usage
fi

if [ ! -d "$4" ]
then
  echo ""
  echo "$prog: Cannot find SNiFF+ make_support folder"
  Usage
fi

if [ ! -f "$5" ]
then
  echo ""
  echo "$prog: Cannot find SNiFF+ make_support template file"
  Usage
fi

logfile=""
if [ "$6" != "" ]; then
  if [ ! -f "$6" ]; then
    echo "Created logfile on `date`" > "$6" 2>/dev/null
  fi
  if [ -w "$6" ]; then
    logfile="$6"
  fi
fi

# convert_make_to_platforms takes a file and
# generates a Platform description for SNiFF+
# Tornado make file is kept in the variable MAKE_FILENAME
# SNiFF+ platforms file is kept in the variable PLATFORM_FILENAME
# The content of the SNiFF+ platform file is taken from
# the template file located in TEMPLATE_PLATFORM_FILENAME

convert_make_to_platforms ()
{
	DBG_EXEC=gdb${ARCH}
	O_DIR=${P_MAKE}
	T_DIR=${P_MAKE}

	if  [  $DEBUG -gt 1 ] ;
	then
    echo "---------------------------------"
	  echo "File: $PLATFORMS_PATHNAME/$PLATFORMS_FILENAME"
	  cat $PLATFORMS_PATHNAME/$PLATFORMS_FILENAME
    echo "---------------------------------"
	 else
	  # echo -n "." - since echo -n does not work correctly on UNIX - use printf
	  # printf "."
	  printf "\n  $PLATFORMS_FILENAME"
	  test "$logfile" != "" && printf "\n  $PLATFORMS_FILENAME" >> "$logfile"
	fi

	if [ -f "$PLATFORMS_PATHNAME/$PLATFORMS_FILENAME" ]; then
	  rm -f "$PLATFORMS_PATHNAME/$PLATFORMS_FILENAME"
	fi
 	sed -e "/DebuggerExecutable/s/gdbsimnt/$DBG_EXEC/g" \
            -e "/ObjectDir/s/torSIMNTgnu/$O_DIR/g" \
            -e "/PlatformMakefile/s/torSIMNTgnu/$P_MAKE/g" \
            -e "/TargetDir/s/torSIMNTgnu/$T_DIR/g" \
	    "$TEMPLATE_PLATFORM_FILENAME" \
            > "$PLATFORMS_PATHNAME/$PLATFORMS_FILENAME"

  if [ ! -f "$PLATFORMS_PATHNAME/$PLATFORMS_FILENAME" ]; then
	printf ":ERROR!"
	test "$logfile" != "" && printf ":ERROR!" >> "$logfile"
  else
    # also generate an OPTIMIZED makefile for REALTARGET platforms
	if [ "$PLATFORMS_O_FILENAME" != "" ]; then
	  # Optimized version of directories
	  O_O_DIR=${P_O_MAKE}
	  T_O_DIR=${P_O_MAKE}
	  if [ -f "$PLATFORMS_PATHNAME/${PLATFORMS_O_FILENAME}" ]; then
		rm -f "$PLATFORMS_PATHNAME/${PLATFORMS_O_FILENAME}"
	  fi
	  printf " ${PLATFORMS_O_FILENAME}"
	  test "$logfile" != "" && printf " - ${PLATFORMS_O_FILENAME}" >> "$logfile"
	  sed -e "/DebuggerExecutable/s/gdbsimnt/$DBG_EXEC/g" \
          -e "/ObjectDir/s/torSIMNTgnu/$O_O_DIR/g" \
          -e "/PlatformMakefile/s/torSIMNTgnu/$P_O_MAKE/g" \
          -e "/TargetDir/s/torSIMNTgnu/$T_O_DIR/g" \
		  "$TEMPLATE_PLATFORM_FILENAME" \
          > "$PLATFORMS_PATHNAME/$PLATFORMS_O_FILENAME"
	  if [ ! -f "$PLATFORMS_PATHNAME/${PLATFORMS_O_FILENAME}" ]; then
		printf ":ERROR!" 
		test "$logfile" != "" && printf ":ERROR!" >> "$logfile"
	  fi
	fi
  fi
}

# convert_make_to_make_support takes a file and
# generates a make_support file for SNiFF+
# Tornado make file is kept in the variable MAKE_FILENAME
# SNiFF+ make_support file is kept in the variable MAKE_SUPPORT_FILENAME
# The content of the SNiFF+ make_support file is taken from
# the template file located in TEMPLATE_MAKE_SUPPORT_FILENAME
# The values of the variables CPU and TOOL have to be transferred
# from the input file to the output file

convert_make_to_make_support ()
{
  CPU=`grep '^[^a-zA-Z0-9_]*CPU[^a-zA-Z]' \$MAKE_FILENAME | sed 's/[^=]*=[^a-zA-Z]*//'`
  TOOL=`grep '^[^a-zA-Z0-9_]*TOOL[^a-zA-Z]' \$MAKE_FILENAME | sed 's/[^=]*=[^a-zA-Z]*//'`

  if  [  $DEBUG -ne 0 ] ;
  then
    echo "CPU:  \"$CPU\""
    echo "TOOL: \"$TOOL\""
    echo "================================="
  else
    # echo -n "." - since echo -n does not work correctly on UNIX - use printf
	# printf "."
	printf " $MAKE_SUPPORT_FILENAME"
	test "$logfile" != "" && printf " - $MAKE_SUPPORT_FILENAME" >> "$logfile"
  fi

  if [ -f "$MAKE_SUPPORT_PATHNAME/$MAKE_SUPPORT_FILENAME" ]; then
    rm -f "$MAKE_SUPPORT_PATHNAME/$MAKE_SUPPORT_FILENAME"
  fi

  sed -e "/CPU/s/SIMNT/$CPU/g" -e "/TOOL/s/gnu/$TOOL/g" \
	  "$TEMPLATE_MAKE_SUPPORT_FILENAME" \
	  > "$MAKE_SUPPORT_PATHNAME/$MAKE_SUPPORT_FILENAME"

  if [ ! -f "$MAKE_SUPPORT_PATHNAME/$MAKE_SUPPORT_FILENAME" ]; then
	printf ":ERROR!"
	test "$logfile" != "" && printf ":ERROR!" >> "$logfile"
  else
    # also generate an OPTIMIZED makefile for REALTARGET platforms
    if [ "$MAKE_SUPPORT_O_FILENAME" != "" ]; then
	  if [ -f "$MAKE_SUPPORT_PATHNAME/${MAKE_SUPPORT_O_FILENAME}" ]; then
		rm -f "$MAKE_SUPPORT_PATHNAME/${MAKE_SUPPORT_O_FILENAME}"
	  fi
	  printf " ${MAKE_SUPPORT_O_FILENAME}"
	  test "$logfile" != "" && printf " - ${MAKE_SUPPORT_O_FILENAME}" >> "$logfile"
	  sed -e '/^CXX_OPTIM/s,CXX_OPTIM_DEBUG,CXX_OPTIM_NORMAL,g' \
		  -e '/^CC_OPTIM/s,CC_OPTIM_DEBUG,CC_OPTIM_NORMAL,g' \
		  "$MAKE_SUPPORT_PATHNAME/$MAKE_SUPPORT_FILENAME" \
		  > "$MAKE_SUPPORT_PATHNAME/${MAKE_SUPPORT_O_FILENAME}"
	  if [ ! -f "$MAKE_SUPPORT_PATHNAME/${MAKE_SUPPORT_O_FILENAME}" ]; then
		printf ":ERROR!" 
		test "$logfile" != "" && printf ":ERROR!" >> "$logfile"
	  fi
	fi
  fi

}

# get all Tornado make file and generate the corresponding
# Tornado make file is kept in the variable MAKE_FILENAME
# SNiFF+ platform file is kept in the variable PLATFORM_FILENAME
# SNiFF+ platforms filename
# SNiFF+ platform file is generated with the name:
# $MAKE_FILENAME, the trailing "make." is replaced by "tor"
# and the suffix ".sniff" is appended

msg="\nGenerating SNiFF+ Platform Specifications for Tornado BuildSpecs..."
printf "$msg"
test "$logfile" != "" && printf "$msg">> "$logfile"
PLATFORMS_PATHNAME="$2"
TEMPLATE_PLATFORM_FILENAME="$3"
MAKE_SUPPORT_PATHNAME="$4"
TEMPLATE_MAKE_SUPPORT_FILENAME="$5"

if [ $DEBUG -ne 0 ]; then
  echo "PLATFORMS_PATHNAME= $PLATFORMS_PATHNAME"
  echo "TEMPLATE_PLATFORM_FILENAME= $TEMPLATE_PLATFORM_FILENAME"
  echo "MAKE_SUPPORT_PATHNAME= $MAKE_SUPPORT_PATHNAME"
  echo "TEMPLATE_MAKE_SUPPORT_FILENAME= $TEMPLATE_MAKE_SUPPORT_FILENAME"
fi

if  [  $DEBUG -ne 0 ] ;
then
    echo "================================="
fi;

curDir=`pwd`
cd "$1"
for MAKE_FILENAME in `find . -name "make.*"`
do
	CPU_TOOL=`basename "$MAKE_FILENAME" | cut -d. -f2`

	# skip the file make.defaults,
	# this is not a platform related file!
	if [ $CPU_TOOL = "defaults" ] ;
	then
    if  [  $DEBUG -ne 0 ] ;
    then
		  echo ""
		  echo "Skipped file: $MAKE_FILENAME"
    fi;
		continue
	fi;

	PLATFORMS_FILENAME="tor${CPU_TOOL}.sniff"
	P_MAKE="tor${CPU_TOOL}"
	MAKE_SUPPORT_FILENAME="${P_MAKE}.mk"
	case $PLATFORMS_FILENAME in
	torSIM*)
	  # do not generate optimized platform for simulators
	  PLATFORMS_O_FILENAME=""
	  MAKE_SUPPORT_O_FILENAME=""
	  P_O_MAKE=""
	  ;;
	*)
	  PLATFORMS_O_FILENAME="tor${CPU_TOOL}_O.sniff"
	  P_O_MAKE="tor${CPU_TOOL}_O"
	  MAKE_SUPPORT_O_FILENAME="${P_O_MAKE}.mk"
	  ;;
	esac

	# determine, which architecture we have
	# to select the right debugger executable
	# The CPU_TOOL information is extracted out
	# of the MAKE_FILENAME.
	# Due to a trailing compiler name we need
	# to select in the case statement with a
	# trailing wildcard.

	# For unknown architectures or seleted ones
	# no files are generated.
	DONT_GENERATE_STRING=dont_generate
	ARCH=$DONT_GENERATE_STRING

  case $CPU_TOOL in
    SIMHPPA*) ARCH=$DONT_GENERATE_STRING ;;
    SIMNTvc*) ARCH=$DONT_GENERATE_STRING ;;
    SIMNTgnu*) ARCH=simnt ;;
    SIMSPARCSOLARIS*) ARCH=simso ;;
    SIMSPARCSUNOS*) ARCH=simsp ;;
    AM29*) ARCH=am29k ;;
    ARM7TDMI_T*) ARCH=thumb ;;
    ARMARCH4_T*) ARCH=thumb ;;
    ARM*_T*) ARCH=thumb ;;
    ARM*) ARCH=arm ;;
    CPU32*) ARCH=m68k ;;
    I80*) ARCH=i86  ;;
    PENTIUM*) ARCH=i86 ;;
    I960*) ARCH=i960 ;;
    MC68*) ARCH=m68k ;;
    PPC*) ARCH=ppc ;;
    CW4000*) ARCH=mips ;;
    R3000*) ARCH=mips ;;
    R33000*) ARCH=mips ;;
    R33020*) ARCH=mips ;;
    R4000*) ARCH=mips ;;
    R4650*) ARCH=mips ;;
    SPARC*) ARCH=sparc ;;
    *) echo "ERROR -- illegal platform detected in $MAKE_FILENAME!!" ;;
  esac

    if  [  $DEBUG -ne 0 ] ;
	then
	  echo "$PLATFORMS_PATHNAME/$PLATFORMS_FILENAME"
	  echo "$MAKE_SUPPORT_PATHNAME/$MAKE_SUPPORT_FILENAME"
	fi;
	if [ "$ARCH" != "$DONT_GENERATE_STRING" ] ;
	then
	  	# now go and create the files
  	  convert_make_to_platforms
  	else
	  printf "\n  ($PLATFORMS_FILENAME)"
	  test "$logfile" != "" && printf "\n  ($PLATFORMS_FILENAME)" >> "$logfile"
	fi;
	convert_make_to_make_support
done
# end of loop - for MAKE_FILENAME in `find $1 -name "make.*"`

echo ""
echo "Done"
exit 0
