#
# SNiFF+ - General Makefile Bridge for SNiFF+/Tornado on VxWorks
#
# History:
# 01a,17-feb-00,mob  created from merging torBridge-UNIX and Windows
#
# Description:
# get original SNiFF+ definitions from the original SNiFF+ Platform 
# Makefile if necessary. The order of overriding should be as follows:
#
#                1. SNiFF+ host specific definitions
# overridden by  2. Tornado default definitions 
# overridden by  3. Tornado host specific definitions (more fine-grained)
# overridden by  4. Tornado project-type specific definitions
# overridden by  5. Tornado target specific definitions
#                6. Harmonizing SNiFF+ and Tornado settings
# overridden by  7. Platform Makefile, e.g. torPENTIUMgnu.mk
# overridden by  8. Local Project Makefile for project-specific settings.

########################################################################
#  Part I -- SNiFF+ host specific definitions (overridden by Tornado   #
########################################################################

ifneq ($(SNIFF_HOST_TYPE),,)
#----------------------SNiFF+ Host-speceific GENERAL--------------------
# include the standard SNiFF+ Platform Makefile as set by $PLATFORM .
# Rules from host-specific SNiFF makefiles are needed if precompilers
# like lex, yacc, IDL etc. are used.
include $(SNIFF_DIR)/make_support/$(SNIFF_HOST_TYPE).mk
else
ifeq ($(WIND_HOST_TYPE),x86-win32)
#----------------------SNiFF+ Host-speceific WINDOWS--------------------
# this section contains only those settings which are NEEDED by SNiFF
# for having optional settings, you MUST define SNIFF_HOST_TYPE!
SYSTEM = Windows
# tools
RM_INTACT = rm -i
MK_DIR = mkdir -p
MK_SYMLINK =
# path delimiter characters
CLASSPATH_DELIMITER = ;
D = ;
DEVNULL = /nul
# parameters for make
MAKE_PARAMS = --no-print-directory
OWN_MAKE = $(MAKE) $(MFLAGS) $(MAKE_PARAMS)
JAVAC  = $SNIFF_DIR/bin/sniffjavac

else  #----------------------SNiFF+ Host-specific UNIX-------------------

# Additional SNiFF+ Definitions for UNIX Hosts
SYSTEM = Unix
RM_INTACT = rm -i
MK_DIR = mkdir -p
MK_SYMLINK = ln -s
# path delimiter characters
CLASSPATH_DELIMITER = :
D = :
DEVNULL = /dev/null
# parameters for make
MAKE_PARAMS = $(JOBS)
OWN_MAKE = $(MAKE) $(MAKE_PARAMS)
endif  # SNiFF+ Host-specific UNIX
endif  # SNiFF+ Host-specific


# now, override settings which are target-specific and different than
# the host-specific SNiFF default settings. Since we use the target
# linker from Tornado, all linker specific settings must be tornado settings.
SPECIAL_LINK_FLAGS :=
EXE_SUFFIX :=

########################################################################
#  Part II -- Tornado Default Definitions                              #
########################################################################

# SNiFF+ bash doesn't like backslashes so convert from environment var.
WIND_BASE := $(subst \,/,$(WIND_BASE))
TGT_DIR   := $(subst \,/,$(TGT_DIR))
 
# some settings that may be overridden by the local Makefile or Platform
# file - but must be overridden BEFORE including the Bridge because they 
# appear in include statements. Put defaults here.
ifeq ($(TGT_SYSTEM),)
TGT_SYSTEM := vxWorks
endif
ifeq ($(TGT_DIR),)
TGT_DIR := $(WIND_BASE)/target
endif
ifeq ($(OS_MAKE_RELDIR),)
OS_MAKE_RELDIR := h/make
endif

# Because SNiFF+ generates dependencies to standard headers if the
# include browser should show the headers, but in shared projects
# with "Do not generate symbolic links..." the files are not where
# they should, we simply add a VPATH for Header files here.
# Note that this will *not* work for the "new-style" headers from
# stl that don't have the .h extension.
# PROBLEM: if a PWE's directory doesn't exist in the SSWE, this will
# not work!
#vpath %.h $(SNIFF_ShSWS)

# save settings because they will be overridden by defs.default
SNIFF_TOR_CPU  := $(CPU)
SNIFF_TOR_TOOL := $(TOOL)
SNIFF_TGT_DIR  := $(TGT_DIR)
SNIFF_WIND_HOST_TYPE := $(WIND_HOST_TYPE)

# now, we can include the defaults first, then re-set our settings.
include $(TGT_DIR)/$(OS_MAKE_RELDIR)/defs.default
CPU  := $(SNIFF_TOR_CPU)
TOOL := $(SNIFF_TOR_TOOL)

# Because SNiFF+ builds in in the gnuwin32 environment on Windows, differences
# to UNIX are rather small -- BUT the bash doesn't like backslashes...
ifeq ($(SNIFF_WIND_HOST_TYPE),x86-win32)
BIN_DIR:=$(WIND_BASE)/host/$(WIND_HOST_TYPE)/bin
DIRCHAR:=/
# temporarily override WIND_HOST_TYPE to avoid backslashes
WIND_HOST_TYPE := x86-win32gnu
endif  # overriding for Tornado Defaults


########################################################################
#  Part III -- Include Tornado definitions for Target, Host & prjType  #
########################################################################

include $(TGT_DIR)/$(OS_MAKE_RELDIR)/defs.project
# override back because it is modified by defs.project!
TGT_DIR := $(SNIFF_TGT_DIR)
# re-set the host type in case it was set to gnu
WIND_HOST_TYPE := $(SNIFF_WIND_HOST_TYPE)

########################################################################
#  Part IV -- Harmonize SNiFF+ settings to Tornado settings            #
########################################################################

# SNiFF+ always builds directly in the project dir
PRJ_DIR = .

# SYSTYPE form the SNIFF_HOST_TYPE is not interesting for target compile
# so we must override it here to keep the SNiFF+ Platform Makefile general
override SYSTYPE = $(OPTION_DEFINE_MACRO)CPU=$(CPU) $(OPTION_DEFINE_MACRO)OS_VXWORKS

# this is an enhancement over plain SNiFF Platform Makefile!
OS_INCLUDE  = -I$(TGT_DIR)/h

#
# C++ compiler
#
CXX_OUTPUT_FLAG	= $(OPTION_OBJECT_NAME)$@     # default to -o$@
OVERALL_OPTION_CXX = $(OPTION_OBJECT_ONLY)    # default to -c
#CXXFLAGS =              # from Tornado, similar to our ALL_CXXFLAGS
#the CXX_OPTIM_DEBUG is in fact an extension / enhancement to Tornado!
CXX_OPTIM_DEBUG = -g     # don't understand why Tornado doesn't set this anywhere

# include debug info by default!
CXX_OPTIM = $(CXX_OPTIM_DEBUG)

#
# C compiler
#
CC_OUTPUT_FLAG	= $(OPTION_OBJECT_NAME)$@     # default to -o$@
OVERALL_OPTION_CC = $(OPTION_OBJECT_ONLY)     # default to -c
#CFLAGS =                # from Tornado, similar to our ALL_CFLAGS
CC_OPTIM_DEBUG = -g      # don't understand why Tornado doesn't set this anywhere

# include debug info by default!
CC_OPTIM = $(CC_OPTIM_DEBUG)

#
# program for linking an executable
#
# BUGBUG
# We have a doulbe clash here: first, because a SNiFF "executable" is
# mapped to a Tornado "downloadable Application" which is linked by
# $(LD_PARTIAL). Second, because SNiFF also uses LDFLAGS. Because of
# this clash, we will have to redefine the whole LINK_CMD below.
#
override LINK = $(LD_PARTIAL)
#LDFLAGS  = -nostdlib -r -W1,-X     # from Tornado
LINK_OUTPUT_FLAG = $(OPTION_OBJECT_NAME)$@

#
# program for linking relocatable object files
#
# BUGBUG
# another clash: The SNiFF LD is used for partially linked objects
#LD = ld                            # from Tornado
RELINK_OUTPUT_FLAG = $(OPTION_OBJECT_NAME)$@

# Tornado doesn't know how to produce shared libraries and this is very
# different per toolchain. So just leave the GNU options for now.
LD_SHARED = $(CXX)
LD_SHAREDFLAGS  = -G
LD_SHAREDFLAGS_MSWIN = 
SHAREDLIB_OUTPUT_FLAG = $(OPTION_OBJECT_NAME)$@
 
# Archiver
ARFLAGS = rv
LIB_OUTPUT_FLAG = $@

# X11 include files and library -- not available on embedded Systems ;-)
override X11_INCL = 
override X11_LIB  = 

# socket libraries -- handled internally by Tornado ;-)
override SOCKET_LIBS = 


########################################################################
#  Part V -- Harmonize Tornado settings to SNiFF+ settings             #
########################################################################

# resets CC_INCLUDE because we use sniff-generated dependencies, but keep user definitions
# the first line is for SNiFF+ 3.0x VPATH based Makesupport, the second one for
# symbolic link based Makesupport
#override CC_INCLUDE=$(_INHIBIT_LOCAL_INCLUDE_CC) $(VINCLUDE) $(EXTRA_INCLUDE) $(OS_INCLUDE) 
override CC_INCLUDE=$(SNIFF_INCLUDE) $(EXTRA_INCLUDE) $(OS_INCLUDE) 

# expand CFLAGS (Windriver rules) so that it includes other FLAGS like ALL_CFLAGS(SniFF Rules)
# SNIFF_INCLUDE is not eneded here because it is added via the CC_INCLUDE
CFLAGS+=$(OTHER_CFLAGS) $(COMMON_CFLAGS) $(OS_CFLAGS) $(SNIFF_CFLAGS)
# overrides ALL_CFLAGS from general.c.mk with the common value
override ALL_CFLAGS=$(CFLAGS)

# expand C++FLAGS (Windriver rules) so that it includes other FLAGS like ALL_CXXFLAGS(SniFF Rules)
# SNIFF_INCLUDE is not eneded here because it is added via the CC_INCLUDE
C++FLAGS+=$(OTHER_CXXFLAGS) $(COMMON_CXXFLAGS) $(OS_CXXFLAGS) $(CXXFLAGS) $(SNIFF_CFLAGS)
# overrides ALL_CFLAGS from general.c.mk with the common value
override ALL_CXXFLAGS=$(C++FLAGS)

# expand CFLAGS_AS (Windriver rules) so that it includes other FLAGS like ALL_ASMFLAGS(SniFF Rules)
# SNIFF_INCLUDE is not eneded here because it is added via the CC_INCLUDE
CFLAGS_AS+=$(OTHER_ASMFLAGS) $(COMMON_ASMFLAGS) $(OS_ASMFLAGS) $(ASMFLAGS) $(SNIFF_CFLAGS)
# overrides ALL_CFLAGS from general.c.mk with the common value
override ALL_ASMFLAGS=$(CFLAGS_AS)

# expand LD_PARTIAL_FLAGS (Windriver rules) so that it includes other FLAGS like ALL_LDFLAGS(SniFF Rules)
LD_PARTIAL_FLAGS+=$(OTHER_LDFLAGS) $(COMMON_LDFLAGS) $(OS_LDFLAGS)
# overrides ALL_LDFLAGS from general.link.mk with the common value
override ALL_LDFLAGS=$(LDFLAGS) $(LD_PARTIAL_FLAGS)

# SNiFF+ 3.0x VPATH based Make only:
# system include are unkown to gcc and are search in the ssws hierarchy.
#OS_VINCLUDE=$(subst $(SNIFF_PROJECT_DIR),$(OS_SWS_INCLUDE),$(SNIFF_RELATIVE_ROOT_DIR)/$(SNIFF_PROJECT_DIR)$(D)$(SNIFF_ShSWS))
#OS_INCLUDE=$(patsubst %,-I%,$(subst $(D), ,$(OS_VINCLUDE))) 

# platform specific library
OS_LIBS = $(LIBS) # LIBS set in make.(CPU)(HOST) or defs.bsp

# remaps RELINK_CMD as VxWorks has some special settings for relocatables
#       Command to make a relocatable object
override RELINK_CMD = $(LD) $(LD_PARTIAL_FLAGS) -o `basename '$@'` $(OFILES) $(OTHER_OFILES) $(SUB_RELINK_OFILES) $(SUB_LIBS) $(RELINK_OTHER_LIBS)

# remaps LINK_CMD as VxWorks has no executables : modules are loaded in relink format, and no
# system libraries shoulb be included, because they are dynamically linked on loading
#       Command to link an executable
override LINK_CMD = $(LD_PARTIAL) -o '$@.tmp' $(OFILES) $(OTHER_OFILES) $(SUB_RELINK_OFILES) $(SUB_LIBS) $(SUB_LIBS2) $(OTHER_LIBS) $(RELINK_OTHER_LIBS) $(SPECIAL_LINK_FLAGS)&& $(NM) '$@.tmp' | $(MUNCH) >$(OBJECT_DIR)/ctdt.c && $(COMPILE_TRADITIONAL) $(OBJECT_DIR)/ctdt.c -o  $(OBJECT_DIR)/ctdt.o && $(LD_PARTIAL) $(LD_PARTIAL_LAST_FLAGS) -o '$@' '$@.tmp' $(OBJECT_DIR)/ctdt.o $(SPECIAL_LINK_FLAGS) && chmod ugo+x $@ && $(RM) '$@.tmp'

# transfer the various SNIFF subprojects in the end VxWorks Target
MACH_DEP+=$(OTHER_OFILES) $(SUB_RELINK_OFILES) $(SUB_LIBS) $(SUB_LIBS2) $(OTHER_LIBS)

clean ::
	-$(RM) $(OBJECT_DIR)/*.rpo $(OBJECT_DIR)/ctdt.o $(OBJECT_DIR)/ctdt.c $(OBJECT_DIR)/symTbl.c
	-$(RM) prjComps.h prjParams.h prjConfig.c linkSyms.c
	-$(RM) libs.nm libs.size


