#!/bin/sh

#****************************************************************
# Script for installing SNiFF+ integrations.
# Actions are logged in a file called logfile;
# Modified files of target directories are written to backup.
#
# Assumptions:
# * In the same directory where this script resides, there are
#   subdirectories called install.SNIFF_DIR etc. which hold the
#   files to be installed.
#
#.
#. USAGE:
#. * Windows: install [-c] [-d] [-u] [Package]
#. * UNIX: install.sh [-c] [-d] [-u] [Package]
#.
#. -c  ... Configure only: do not copy files but configure only
#. -d  ... Debug mode: print commands only but don't execute them
#. -u  ... UNINSTALL instead of installation
#.
#. If [Package] is given, it points to the SNiFF+ integration that should
#. be installed. This may be either a "tgz" (gzip-compressed tar) file,
#. or a directory with the already-extracted integration.
#. If it is not given, install from the local directory (if valid) or ask 
#. what should be installed.
#.
#. All parameters are optional. The ordering of parameters must be correct!
#.
# History:
# 1.2,   21-Apr-2000, mober   Support for Uninstall, Packages, Space in dirs
# 1.1,   17-Apr-2000, mober   Bugfixes and generalizations; new -c switch
# 1.0,   13-Apr-2000, mober   Adaptions for Windows; Tests run OK!
# 0.99b, 10-Apr-2000, mober   Adaptions for Tornado
# 0.99a, 06-Mar-2000, skogler created
#****************************************************************
MY_VERSION="1.2"

#*****************************************
# some common definitions

greeting="SNiFF+ Integration Install Script v${MY_VERSION}. Copyright (c) 2000, WindRiver (R)"

integration_name="(Unknown)"    # default for external installation

date=`date +"%Y%m%d_%H%M"`
StartFromDir=`pwd`

cd `dirname "$0"`
mydir=`pwd`
prog=`basename "$0"`

exit_noerror=0
exit_usage=1               # user asked for usage information
exit_wrong_package=2       # illegal package (not suitable for installation)
exit_abortinstall=3        # installation aborted by user
exit_wrong_version=4       # versions of integration and target are incompatible
exit_permission_error=5    # could not create backup or other necessary directory

#*******************************************
# Some defaults for this script

PACKTOOL="" # will be set to something if we inistall from an archive.
logfile="$mydir/Setup.log"
installdir=""    # not yet known!

#*****************************************
# Windows / UNIX Specifics

case `uname` in
CYGWIN*) 
  SYSTEM="Windows"
  if [ -f "$mydir/install.SNIFF_DIR/bin/cygpath.exe" ]; then
    CYGPATH="$mydir/install.SNIFF_DIR/bin/cygpath.exe"
  elif [ -f "$mydir/adm/cygpath.exe" ]; then
    CYGPATH="$mydir/adm/cygpath.exe"
  elif [ -f "$mydir/../bin/cygpath.exe" ]; then
    cd "$mydir/../bin"
    CYGPATH="`pwd`/cygpath.exe"
  elif [ -f "$mydir/../../bin/cygpath.exe" ]; then
    cd "$mydir/../../bin"
    CYGPATH="`pwd`/cygpath.exe"
  else
    # hope it's in the path!
    CYGPATH="cygpath.exe"
  fi
  DollarNull=`"$CYGPATH" -u "$0"`
  prog=`basename "$DollarNull"`
  cd `dirname "$DollarNull"`
  mydir=`pwd`
  if [ -f "$mydir/install.SNIFF_DIR/bin/ReadRegistryValue.exe" ]; then
    READ_REGISTRY_VALUE="$mydir/install.SNIFF_DIR/bin/ReadRegistryValue.exe"
  elif [ -f "$mydir/../bin/ReadRegistryValue.exe" ]; then
    cd "$mydir/../bin"
    READ_REGISTRY_VALUE="`pwd`/ReadRegistryValue.exe"
  elif [ -f "$mydir/../../bin/ReadRegistryValue.exe" ]; then
    cd "$mydir/../../bin"
    READ_REGISTRY_VALUE="`pwd`/ReadRegistryValue.exe"
  else
    # hope it's in the path!
    READ_REGISTRY_VALUE="ReadRegistryValue.exe"
  fi
  DIFF="diff -b"
  PREFERRED_SHELL="bash.exe"
  ;;
*) 
  SYSTEM="UNIX"
  DIFF="diff -b"
  PREFERRED_SHELL="sh"
  ;;
esac

#*******************************************
#  message how to use this skript
#*******************************************

usage() {
  grep '#\.' $0 | cut -c4-
  cd "$StartFromDir"
  exit $exit_usage
}

#*******************************************
#  write end message in logfile
#*******************************************

finish() {
  if [ "$RESULT" = "$exit_noerror" ]; then
    TXT="completed successfully"
  elif [ "$RESULT" = "$exit_wrong_package" ]; then
    TXT="ended due to wrong integration directory given"
  elif [ "$RESULT" = "$exit_abortinstall" ]; then
    TXT="ended due to user abort"
  elif [ "$RESULT" = "$exit_wrong_version" ]; then
    TXT="ended due to version clash"
  elif [ "$RESULT" = "$exit_permission_error" ]; then
    TXT="ended due to missing access permissions"
  else
    TXT="ended with unknown error code"
  fi
  if [ "$PACKTOOL" != "" ]; then
    echo ""
    echo "Cleaning up temporary files of extracted archive"
    rm -rf "$installdir"
  fi
  theTime=`date +"%T"`
  theDate=`date +"%d/%m/%Y"`
  msg="
---------------------------------------------
${INSTALL_TXT}ation $TXT
=============================================
Script finished at $theTime on $theDate"
  echo "$msg"
  if [ "$logfile" != "" ]; then
    if [ -w "$logfile" ]; then
      echo "$msg" >> "$logfile"
    fi
  fi
#  if [ "$LOCAL_MOUNTS" != "" ]; then
#    for curMount in $LOCAL_MOUNTS ; do
#      umount $curMount
#    done
#  fi
  cd "$StartFromDir"
  exit $RESULT
}

#*******************************************
# real start of script here

echo ""
echo "$greeting"

#*******************************************
# Parameter evaluation

# If CONFIG_ONLY is set, directly jump from precopy to postinstall
if [ "$1" = "-c" ]; then
  CONFIG_ONLY=1
  ConfigSwitch="-c "
  shift
else
  CONFIG_ONLY="0"
fi

# If DEBUG is set to "echo", don't really copy but only echo what we are doing
if [ "$1" = "-d" ]; then
  DEBUG=echo
  shift
  echo ""
  echo "StartFromDir = $StartFromDir"
  echo "mydir        = $mydir"
  echo "Command was: $mydir/$prog ${ConfigSwitch}-d $*"
else
  DEBUG=""
fi

# If UNINSTALL is set to 1, we should uninstall instead of installing
if [ "$1" = "-u" ]; then
  UNINSTALL=1
  INSTALL_TXT="Uninstall"
  echo "WARNING: UNINSTALL Feature is currently experimental!"
  echo ""
  echo "Uninstall in NOT able to completely restore the state of your"
  echo "system before the last installation; it will merely restore the"
  echo "last backup you made, and do a best-effort downgrade. Normally"
  echo "this should suffice to make your system work as it did before"
  echo "the backup was made."
  echo ""
  echo "If uninstall does not satisfy your needs, you should make a"
  echo "new installation instead, or manually remove files."
  shift
else
  UNINSTALL=0
  INSTALL_TXT="Install"
fi

if [ $# -eq 1 ]; then
  if [ "$1" = "-help" -o "$1" = "--help" -o "$1" = "-h" -o "$1" = "/?" ]; then
    # test for command line args
    usage
  else
    # A package name was given as parameter.
    # If this is a file, unpack it into /tmp and set installdir to there.
    # If this is a directory, set the directory as the installdir.
    # Since directory names may be given relative, cd to the StartFromDir first.
    cd "$StartFromDir"
    if [ "$SYSTEM" = "Windows" ]; then
      thePackage=`"$CYGPATH" -u "$1"`
    else
      thePackage="$1"
    fi
    if [ -d "$thePackage" ]; then
      cd "$thePackage"
      installdir=`pwd`
    elif [ -f "$thePackage" ]; then
      echo ""
      installfile=`basename "$thePackage"`
      case "$installfile" in
      *.tgz|*.TGZ|*.gz|*.GZ|*.tar_gz|*.TAR_GZ) 
        PACKTOOL=tgz ;;
      *) 
        echo "ERROR: Don't know how to unpack your archive $installfile." 
        echo "Currently, only the .tgz (aka .tar.gz) format is supported."
        RESULT=$exit_wrongformat
        finish
      esac
      cd `dirname "$thePackage"`
      installpath="`pwd`/$installfile"
      test "$DEBUG" = "echo" && echo "Package installfile=$installfile, installpath=$installpath"
      num=1
      while [ -f "/tmp/$installfile" -o -d "/tmp/$installfile" ]; do
        newInstallfile="$installfile.$num"
        num=`expr $num + 1`
      done
      installfile="$newInstallfile"
      if [ "$SYSTEM" = "WINDOWS" ]; then
        tmpdir=`"$CYGPATH -w /tmp" | sed 's,[\],/,g'`
      else
        tmpdir=/tmp
      fi
      echo "Extracting package contents into temporary directory"
      echo "  $tmpdir/$installfile"
      mkdir -p "/tmp/$installfile"
      if [ "$?" != "0" ]; then
        echo "ERROR: Cannot create temporary directory!"
        RESULT=$exit_permission_error
        finish
      fi
      cd "/tmp/$installfile"
      gzip -c -d "$installpath" | tar xfv -
      installdir="/tmp/$installfile"
    else
      echo "ERROR: Your specified package"
      echo "  $1"
      echo "does not exist!"
      RESULT=$exit_wrong_package
      finish
    fi
  fi
elif [ $# -gt 2 ]; then
  usage
else
  # No parameter given. If there is not a valid installable package in
  # the local directory (or in the directory where I am myself),
  # we must select_what_to_install.
  if [ -f "$mydir/Version.txt" ]; then
    installdir="$mydir"
  elif [ -f "$StartFromDir/Version.txt" ]; then
    installdir="$StartFromDir"
  else
    select_what_to_install=1
  fi
fi

#*******************************************
# Environment Variable evaluation

# save original SNIFF_DIR Environment variable: it must have precedence over
# Env.Vars automatically found from PATH because of probable Tilde Expansion!
ORIG_SNIFF_DIR="$SNIFF_DIR"

# try to guess correct $SNIFF_DIR setting
if [ -f "$mydir/../../SitePrefs.sniff" ]; then
  # Install Script probably started in $SNIFF_DIR/integrations/integrationName
  test "$DEBUG" = "echo" && echo "Guessing SNIFF_DIR: ../../SitePrefs.sniff found"
  cd "$mydir/../../"
  SNIFF_DIR=`pwd`
  cd "$mydir/.."
  testDirName=`pwd`
  testDirName=`basename "$testDirName"`
  if [ "$testDirName" = "integrations" ]; then
    # install script started in $SNIFF_DIR/integrations/IntegrationName
    installdir="$mydir"
    integration_name=`basename "$installdir"`
    select_what_to_install=0
  fi
elif [ -f "$mydir/../SitePrefs.sniff" ]; then
  # install script started in $SNIFF_DIR/integrations or $SNIFF_DIR/bin
  test "$DEBUG" = "echo" && echo "Guessing SNIFF_DIR: ../SitePrefs.sniff found"
  cd "$mydir/.."
  SNIFF_DIR=`pwd`
elif [ -f "$installdir/../../SitePrefs.sniff" ]; then
  # A package name was given, which is part of the SNiFF+ installation
  test "$DEBUG" = "echo" && echo "Guessing SNIFF_DIR: installdir/../../SitePrefs.sniff found"
  integration_name=`basename $installdir`
  select_what_to_install=0
  cd "$installdir/../.."
  SNIFF_DIR=`pwd`
fi

if [ "$ORIG_SNIFF_DIR" != "" ]; then
  if [ -f "$ORIG_SNIFF_DIR/SitePrefs.sniff" ]; then
    if [ "$SYSTEM" = "Windows" ]; then
      SNIFF_DIR=`"$CYGPATH" -u "$ORIG_SNIFF_DIR" `
      SNIFF_DIR_WB="$ORIG_SNIFF_DIR"
    fi
    test "$DEBUG" = "echo" && echo "Guessing SNIFF_DIR: using Environment: $SNIFF_DIR"
  fi
fi

# Find correct installation directory and cd to it
if [ "$select_what_to_install" = "1" ]; then
  if [ "$installdir" = "" ]; then
    installdir="$SNIFF_DIR/integrations"
  fi
  cd "$installdir"
  dirs=`ls -l | grep "^d" | awk '{print $NF}' | grep -v '^bin\.' | sort -f`
  while [ "1" = "1" ]; do
    # loop until correct installation dir or quit given
    count=1
    echo ""
    echo "Please select a SNiFF+ Integration to install!"
    echo "To install packages not listed here, please specify the package name"
    echo "on the install commandline, like \"$prog newPackage.tgz\"."
    echo "Otherwise, select a SNiFF+ Standard integration here:"
    for dir in $dirs ;
    do
      if [ -f "$dir/Version.txt" ]; then
        #fullname=`cat "$dir/Version.txt"`
        #echo "($count) $fullname"
        echo "  ($count) $dir"
      else
        echo "  ($count) $dir"
      fi
      count=`expr "$count" + 1`
    done
    count=`expr "$count" - 1`
    echo "Please select which integration you want to install by entering its number"
    printf "Or [q] to quit installation: "
    read selection
    case "$selection" in
    q|Q)
      echo "Quit installation on user request."
      RESULT=$exit_abortinstall
      finish
      ;;
    [0-9]*)
      # number given
      integration_name=""
      count=1
      for dir in $dirs 
      do 
        if [ $count -eq $selection ]; then
          integration_name="$dir"
          break
        fi
        count=`expr "$count" + 1`
      done
      if [ "$integration_name" != "" ]; then
        cd "$integration_name"
        installdir=`pwd`
        # correct installdir found, break out of the loop
        break
      fi
      ;;
    esac
    echo ""
    echo "Sorry, selection -->$selection<-- was invalid." 
    echo "Please try again."
  done
fi

# Tell the user what we are finally going to install!
if [ -f "$installdir/Version.txt" ]; then
  integration_name=`cat "$installdir/Version.txt" | cut -d\  -f1`
  integration_version=`cat "$installdir/Version.txt"`
else
  # no new version information?
  integration_version="$integration_name"
fi  
echo ""
echo "${INSTALL_TXT}ing $integration_version"

if [ -f "$installdir/adm/preinstall.sh" ]; then
  . "$installdir/adm/preinstall.sh"
fi

test "$DEBUG" = "echo" && echo "After Preinstall: SNIFF_DIR=$SNIFF_DIR"

# Find out what target directories we need to install into
cd "$installdir"
target_dirs=`ls -ld install.* | grep "^d" | awk '{print $NF}' | sed s,install.,,g`

#*****************************************
# Windows / UNIX Specifics

if [ "$SYSTEM" = "Windows" ] ; then
  installdir=`"$CYGPATH" -w "$installdir"`
  installdir=`echo "$installdir" | sed 's,[\],/,g' `
fi

#*******************************************
# User Input of target, source and options

# Ask the user to enter target directories. this is done in a BIG loop
# which runs until all selections are presented to the user. Because
# the loop is so big, it is not indented ;-)
while [ "1" = "1" ]; do

for curTarget in $target_dirs ; do
  evalstr="echo "\$"$curTarget"
  cur_target_DIR=`eval "$evalstr"`
  test "$SYSTEM" = "Windows" && cur_target_DIR=`"$CYGPATH" -w "$cur_target_DIR" | sed 's,[\],/,g' `
  new_target_DIR=""
  while [ "$new_target_DIR" = "" ]; do
    ERROR=0
    echo ""
    if [ "$UNINSTALL" = "1" ]; then
      echo "Please enter the $curTarget directory from which to uninstall or [q] to quit"
    else
      echo "Please enter the target $curTarget directory for installing or [q] to quit"
    fi
    if [ "$SYSTEM" = "Windows" ]; then
      echo "Please use forward slashes (/) instead of backslashes (\\)!"
    fi
    if [ "$cur_target_DIR" != "" ]; then
      echo "Leave empty for default [$cur_target_DIR]"
    fi
    printf "Target $curTarget: "
    read new_target_DIR
    if [ "$new_target_DIR" = "" ]; then
      new_target_DIR="$cur_target_DIR"
    elif [ "$new_target_DIR" = "q" ]; then
      RESULT=$exit_abortinstall
      finish
    fi
    
    # check correctness of the user-entered $target_DIR
    testBadChars=`echo "$new_target_DIR" | tr "\b" ";" | grep '[;<>?]'`
    if [ "$testBadChars" != "" ]; then
      echo ""
      echo "ERROR: You entered illegal characters in the directory name."
      echo "Note: You must use DELETE instead of the BACKSPACE key here!"
      ERROR=1
    fi
    
    case $curTarget in
    SNIFF_DIR) 
      if [ ! -f "$new_target_DIR/SitePrefs.sniff" ]; then
        echo ""
        echo "ERROR: It seems that"
        echo "  $new_target_DIR"
        echo "is not a correct SNIFF_DIR directory (SitePrefs.sniff is missing)."
        ERROR=1
      elif [ "$UNINSTALL" = "1" ]; then
        # check whether a correct backup directory is there
        bakdir_base=""
        if [ -d "$new_target_DIR/integrations/$integration_name" ]; then
          cd "$new_target_DIR/integrations/$integration_name"
          bakdir_base=`ls -ld backup.* 2>/dev/null \
            | grep '^d' | awk '{print $NF}' \
            | sort -r | head -1 `
        fi
        if [ "$bakdir_base" = "" ]; then
          echo ""
          echo "ERROR: There is no valid backup directory in"
          echo "  $new_target_DIR"
          ERROR=1
        else
          echo ""
          echo "Uninstalling from $bakdir_base"
        fi
      fi
      ;;
    WIND_BASE)
      if [ ! -d "$new_target_DIR/host/resource" ]; then
        echo ""
        echo "ERROR: It seems that"
        echo "  $new_target_DIR"
        echo "is not a correct WIND_BASE directory (host/resource is missing)."
        ERROR=1
      fi
      ;;
    esac

    # WINDOWS: Backslashes look ugly...
    haveBackSlash=`echo "$new_target_DIR" | grep '[\]' `
    haveSlash=`echo "$new_target_DIR" | grep '/' `
    if [ "$haveBackSlash" != "" -o "$haveSlash" = "" ]; then
      echo ""
      echo "This is what I got:"
      tmp_target_DIR=`echo "$new_target_DIR" | sed 's,[\],/,g'`
      echo "  $tmp_target_DIR"
      echo "You may need to use forward slashes instead of backslashes!"
      ERROR=1
    fi

    # try if the target is writable for us
    if [ "$ERROR" != "1" ]; then
      ### MOB BUGFIX -- on some Platforms (particularly Windows w/ bash)
      ### the test -w does not work on directories but plain files only!
      #if [ ! -w "$new_target_DIR" ]; then
      #  echo ""
      #  echo "ERROR: The specified directory"
      #  echo "  $new_target_DIR"
      #  echo "is not writable."
      #  ERROR=1
      #fi
      TOUCH_ERR=`touch "$new_target_DIR/.dummy_writable_test.txt" 2>&1`
      if [ "$TOUCH_ERR" != "" ]; then
        echo "ERROR: The specified directory"
        echo "  $new_target_DIR"
        echo "is not writable."
        ERROR=1
      else
        rm -f "$new_target_DIR/.dummy_writable_test.txt" >/dev/null 2>&1
      fi
    fi
    
    # Error handling
    if [ "$ERROR" = "1" ]; then
      printf "\nRetry, Ignore or Quit [R/i/q] ? R\b"
      read sel
      case $sel in
        i*|I*) ;;
        q*|Q*) RESULT=$exit_abortinstall; finish ;;
        *) new_target_DIR="" ;;    # Retry
      esac
    fi
  done
  # put the $new_target_DIR into the environment
  eval "$curTarget=\"$new_target_DIR\""
done

#*******************************************
# Logfile creation

# only NOW, the SNIFF_DIR is at least guessed and we can thus create the logfile.
# However, we will print the message only after Windows Drive Letters have been
# mounted.
logfile="$SNIFF_DIR/Setup.log"
PERM_ERR=`touch "$logfile" 2>&1`
if [ "$PERM_ERR" != "" ]; then
  logfile="$installdir/Setup.log"
  PERM_ERR=`touch "$logfile" 2>&1`
  if [ "$PERM_ERR" != "" ]; then
    logfile="$mydir/Setup.log"
    PERM_ERR=`touch "$logfile" 2>&1`
    if [ "$PERM_ERR" != "" ]; then
      logfile="/tmp/Setup.log"
      touch "$logfile"
    fi
  fi
fi
echo ""
if [ "$SYSTEM" = "Windows" ]; then
  logfile_W=`"$CYGPATH" -w "$logfile" | sed 's,[\],/,g'`
  echo "Writing logfile to
  $logfile_W..."
  echo ""
  echo "GNUWIN: Checking mount points and converting pathnames..."
else
  echo "Writing logfile to
  $logfile..."
fi
theDate=`date +"%m-%d-%Y"`
theTime=`date +"%H:%M:%S"`
if [ ! -f "$logfile" ]; then
  echo "Created logfile on $theDate at $theTime" > "$logfile"
fi

#*******************************************
# Windows specifics: GNUWIN Mount and Path name conversion

# for every user-entered directory, create a dirname_W for printing
# which contains the Windows-style pathname with FORWARD SLASHES
# and a dirname_WB which contains the Windows-style pathname with BACKSLASHES
logdir=`dirname "$logfile"`
logbase=`basename "$logfile"`
for curTarget in mydir logdir installdir $target_dirs ;
do
  if [ "$SYSTEM" = "Windows" ]; then
    test "$DEBUG" = "echo" && echo ""
    # find out the original setting
    evalstr="curPath_ORIG="\$"${curTarget}"
    eval "$evalstr"
    test "$DEBUG" = "echo" && echo "${curTarget}_ORIG=${curPath_ORIG}"
    # map it into the canonical Windows-style pathname (with backslashes)
    # eliminate trailing backslashes!!!
    curPath_WB=`"$CYGPATH" -w "$curPath_ORIG" | sed 's,[\]*$,,g' `
    #test "$DEBUG" = "echo" && echo "curPath_WB = $curPath_WB"
    # extract the drive letter
    curPath_DRIVE=`echo "$curPath_WB" | cut -c1`
    # and map it into uppercase if necessary
    case $curPath_DRIVE in
    [a-z]) 
      # map the Windows Drive to Uppercase
      do_mount=1
      curPath_DRIVE=`echo "$curPath_DRIVE" | tr 'a-z' 'A-Z'`
      curPath_REST_B=`echo "$curPath_WB" | cut -c2-`
      curPath_WB="$curPath_DRIVE$curPath_REST_B"
      ;;
    [A-Z])
      do_mount=1
      ;;
    *)  # there's not drive letter at all! (started from an UNC pathname)
      do_mount=0
      ;;
    esac
    what_to_mount="$curPath_DRIVE:"
    where_to_mount="/$curPath_DRIVE"
    # check for SPACE character in the UNIX pathname
    has_space=`echo "$curPath_WB" | grep ' '`
    if [ "$has_space" != "" ]; then
      do_mount=1
      # delete everything from the first slash after the last space
      space_Prefix=`echo "$curPath_WB" | sed \
        -e 's,[\],/,g' \
        -e 's,\([ ][^ /]*\)/[^ ]*$,\1,' `
      what_to_mount="$space_Prefix"
      where_to_mount=`echo "/$space_Prefix" | sed -e 's,[:],,g' -e 's,[ ],_,g'`
    fi
    # map the canonical path back into UNIX
    # -- this is what we will work with later in this script
    curPath_UNIX=`"$CYGPATH" -u "$curPath_WB"`
    test "$DEBUG" = "echo" && echo "${curTarget}_UNIX=$curPath_UNIX"
    if [ "$do_mount" = "1" ]; then
      # now, mount the drive if necessary
      must_mount=0
      case $curPath_UNIX in
      //*) # drive not mounted!
        must_mount=1
        ;;
      *) has_space=`echo "$curPath_UNIX" | grep ' '`
         if [ "$has_space" != "" ]; then
           # perhaps, umount the existing drive here?
           must_mount=1
         fi
         ;;
      esac
      if [ "$must_mount" = "1" ]; then
        test "$DEBUG" = "echo" && echo "mount \"$what_to_mount\" \"$where_to_mount\" >/dev/null 2>&1"
        mount "$what_to_mount" "$where_to_mount" >/dev/null 2>&1
        # make sure that the curPath_UNIX_NEW is not cached -- ATTENTION:
        # cd won't work if something is not mounted!!!
        curPath_UNIX_NEW=`"$CYGPATH" -u "$curPath_WB"`
        test "$DEBUG" = "echo" && echo "${curTarget}_UNIX_NEW=$curPath_UNIX_NEW"
        if [ "$curPath_UNIX_NEW" != "$curPath_UNIX" ]; then
          # the mount worked!
          msg="GNUWIN: Mounted "$what_to_mount" on "$where_to_mount" for faster access"
          echo "$msg"
          echo "$msg" >> $logfile
          LOCAL_MOUNTS="$LOCAL_MOUNTS $where_to_mount"
          # reassign original curTarget variable for UNIX
          curPath_UNIX="$curPath_UNIX_NEW"
        fi
      fi
    fi
    # finally, set the real targets to correct canonical settings:
    # first, the curTarget_WB (Windows with backslashes)
    evalstr="${curTarget}_WB=\"$curPath_WB\""
    test "$DEBUG" = "echo" && echo "$evalstr"
    eval "$evalstr"
    # then, Windows with slashes
    curPath_W=`echo "$curPath_WB" | sed 's,[\],/,g'`
    evalstr="${curTarget}_W=\"$curPath_W\""
    test "$DEBUG" = "echo" && echo "$evalstr"
    eval "$evalstr"
    # And, finally UNIX with which we will continue to work!
    evalstr="${curTarget}=\"$curPath_UNIX\""
    test "$DEBUG" = "echo" && echo "$evalstr"
    eval "$evalstr"
  else
    # UNIX: Set the dirname_W = dirname_WB = dirname
    evalstr="${curTarget}_W="\$"${curTarget}"
    eval "$evalstr"
    evalstr="${curTarget}_WB="\$"${curTarget}"
    eval "$evalstr"
  fi
done
logfile="$logdir/$logbase"

# mount the /bin directory unless it exists already: This should only
# be done once we have a correct Windows-Style version of $SNIFF_DIR
if [ "$SYSTEM" = "Windows" ]; then
  if [ ! -f /bin/$PREFERRED_SHELL ]; then
    umount /bin >/dev/null 2>&1
    mount "$SNIFF_DIR_W/bin" /bin >/dev/null 2>&1
    if [ ! -f /bin/sh.exe ]; then
      msg="GNUWIN ERROR: Cannot mount /bin --> Shellscripts may not work!"
      echo "$msg"
      echo "$msg" >> "$logfile"
    else
      msg="GNUWIN: Mounted $SNIFF_DIR_W/bin on /bin"
      echo  "$msg"
      echo "$msg" >> "$logfile"
    fi
  fi
fi

#*******************************************
# Backup, yes or no?

# ask for a backup directory: for uninstalling, this has already been entered!
# It would be better to ask these questions after everything has been mounted,
# because then we know the right xdir_W variables.
if [ "$UNINSTALL" = "1" ] ; then
  # Set bakdir from the bakdir_base only now, because only now we are sure
  # we have correctly mounted paths
  save=0
  bakdir="$SNIFF_DIR/integrations/$integration_name/$bakdir_base"
  bakdir_W="$SNIFF_DIR_W/integrations/$integration_name/$bakdir_base"
else
  cd "$installdir"
  date=`date +"%Y%m%d"`
  save=0
  backup_nonstandard=0
  echo ""
  echo "Checking Backup directory..."
  bakdir="$SNIFF_DIR/integrations/$integration_name/backup.$date"
  bakdir_W="$SNIFF_DIR_W/integrations/$integration_name/backup.$date"
  # find out whether we have permission to create a backup directory
  if [ -d "$bakdir" ]; then
    echo "  WARNING: Backup directory already exists!"
  else
    mkdir -p "$bakdir" 2>/dev/null
    if [ -d "$bakdir" ]; then
      rmdir "$bakdir" 2>/dev/null
    else
      backup_nonstandard=1
      bakdir="$mydir/backup.$date"
      bakdir_W="$mydir_W/backup.$date"
      echo "  WARNING: Backup directory can not be created in its standard location!"
      if [ -d "$bakdir" ]; then
        echo "  WARNING: Backup directory already exists!"
      else
        mkdir -p "$bakdir" 2>/dev/null
        if [ -d "$bakdir" ]; then
          rmdir "$bakdir" 2>/dev/null
        else
          bakdir="/tmp/backup.$date"
          if [ "$SYSTEM" = "Windows" ]; then
            bakdir_W=`"$CYGPATH" -w "$bakdir" | sed 's,[\],/,g'`
          else
            bakdir_W="$bakdir"
          fi
        fi
      fi
    fi
  fi
  echo ""
  echo "Do you want to save files being overwritten during installation"
  echo "to a directory called"
  echo "  $bakdir_W"
  echo ""
  # later, we can allow "o" here to enter a different backup directory
  printf "Yes, no, quit [Y/n/q] ? Y\b"
  read selection
  case $selection in
    n*|N*) ;;
    q*|Q*) RESULT=$exit_abortinstall ; finish
       ;;
    *) save=1
       if [ ! -d "$bakdir" ]; then
         $DEBUG mkdir -p "$bakdir"
         if [ "$?" != 0 ]; then
           echo "ERROR creating backup directory!"
           RESULT=$exit_permission_error
           finish
         fi
       fi
       if [ "$backup_nonstandard" = "1" ]; then
         echo "Note: you should copy your backup directory to its standard location"
         echo "  "\$"SNIFF_DIR/integrations/$integration_name/backup.$date"
         echo "When installation has finished."
       fi
       ;;
  esac
fi

#*******************************************
# Compilation of information screen

# find out whether this is an external install
is_external=1
EXTERNAL="External "
if [ -d "$SNIFF_DIR/integrations/$integration_name" ]; then
  cd "$installdir"
  installdir=`pwd`  # use UNIX Version of filename
  cd "$SNIFF_DIR/integrations/$integration_name"
  curDir=`pwd`      # use UNIX Version of filename
  if [ "$curDir" = "$installdir" ]; then
    is_external=0
    EXTERNAL=""
  elif [ "$SYSTEM" = "Windows" ]; then
    # because of Windows 8.3 pathname shortening, the only possible safe
    # check is through creating a file and checking whether it is there.
    DUMMY="$SNIFF_DIR/integrations/$integration_name/ins_test.111"
    DUMMY2="$installdir/ins_test.111"
    test -f "$DUMMY" && rm -f "$DUMMY"
    test -f "$DUMMY2" && rm -f "$DUMMY2"
    echo "dummy" > "$DUMMY"
    if [ -f "$DUMMY2" ]; then
      is_external=0
      EXTERNAL=""
    fi
    rm -f "$DUMMY"
  fi
fi

cd "$installdir"
echo ""
msg="=============================================
DATE: $theDate(mm-dd-yyyy)
TIME: $theTime(hh-mm-ss)"
if [ "$UNINSTALL" = "1" ]; then
  msg="${msg}
SETUP: About to uninstall $integration_version
---------------------------------------------
Uninstall from
  $bakdir_W"
else
  msg="${msg}
SETUP: About to install $integration_version
---------------------------------------------
${EXTERNAL}Install from
  $installdir_W"
fi
echo "$msg"
logmsg="$msg"

for curTarget in $target_dirs
do
  evalstr="echo "\$"${curTarget}_W"
  tgt_DIR=`eval "$evalstr"`
  msg="${INSTALL_TXT} to "\$"$curTarget
  $tgt_DIR"
  echo ""
  echo "$msg"
  logmsg="${logmsg}\n\n${msg}"
done

if [ "$UNINSTALL" != "1" -a "$save" = "1" ]; then
  msg="Creating Backup Directory in
  $bakdir_W"
  echo ""
  echo "$msg"
  logmsg="${logmsg}\n\n${msg}"
fi

# MASTER Loop for entering directories
cd "$installdir"
printf "\nIs this correct [Y/n/q] ? Y\b"
read correct
case "x$correct" in
  x|xy|xY) break ;;
  xq|xQ) RESULT=$exit_abortinstall
         finish
         ;;
  *) continue ;;
esac
done # with MASTER LOOP

printf "$logmsg" >> "$logfile"

#*******************************************
# Preinstall starts

#
# What we have now:
# *) System specifics:
#    - $SYSTEM is set to "Windows" or "UNIX"
#    - $CYGPATH is set to the cygpath.exe executable if System is "Windows"
#    - $DIFF is set to a diff program that works with cygwin paths
#    - $READ_REGISTRY_VALUE points to the ReadRegistryValue.exe program on Win
#    - $PREFERRED_SHELL is set to the shell program we should use
# *) General Switches
#    - $DEBUG is set to "echo" if debug mode is on (or empty otherwise)
#    - $CONFIG_ONLY is set to 1 if no copying should be made
#    - $UNINSTALL is set to 1 if we should make an uninstall (1 otherwise)
# *) Directories:
#    - $StartFromDir is from where the install.sh was started
#    - $mydir is where this install.sh is located
#    - $installdir is at the installation source
#    - $target_dirs has a list of targets where to install to
#    - $logfile exists and is ready for logging
#    - $save tells us whether to backup or not
#    - $bakdir is the target where to backup files, it exists and is writable
# *) Common switches
#    - $integration_name is the adaptor name to install
#    - $is_external tells us whether we are installing from "outside" the SNiFF Tree
# *) Special cases handled
#    - $SNIFF_DIR is the installation target: correctly set, exists and is writable
#    - $WIND_BASE is correctly set and writable, if it is needed

# At this point, a user-written precopy.sh may be sourced.
if [ -f "$installdir/adm/precopy.sh" ]; then
  . "$installdir/adm/precopy.sh"
fi

# If we configure only (no real copy), no directly go to postinstall, then finish
if [ "$CONFIG_ONLY" = "1" ]; then
  . "$installdir/adm/postinstall.sh"
  RESULT=$exit_noerror
  finish
fi

# In case of external install: copy install.XXX files into target dir 
if [ "$UNINSTALL" != "1" ]; then
  if [ ! -d "$SNIFF_DIR/integrations/$integration_name" ]; then
    # first-time external install! Copy the entire integration directory.
    echo ""
    echo "" >> "$logfile"
    echo "External install: creating"
    echo "External install: creating" >> "$logfile"
    echo "  $SNIFF_DIR_W/integrations/$integration_name"
    echo "  $SNIFF_DIR_W/integrations/$integration_name" >> "$logfile"
    $DEBUG mkdir -p "$SNIFF_DIR/integrations/$integration_name"
    for curFile in `ls "$installdir" ` ; do
      $DEBUG cp -R "$curFile" "$SNIFF_DIR/integrations/$integration_name"
    done
  elif [ "$is_external" = "1" ]; then
    # upgrade external install! Copy the install.* stuff
    for curTarget in $target_dirs
    do
      tgt_DIR="$SNIFF_DIR/integrations/$integration_name/install.$curTarget"
      tgt_DIR_W="$SNIFF_DIR_W/integrations/$integration_name/install.$curTarget"
      if [ -d "$tgt_DIR" ];
      then
        echo ""
        echo "" >> "$logfile"
        echo "External upgrade: removing old"
        echo "External upgrade: removing old" >> "$logfile"
        echo "  $tgt_DIR_W"
        echo "  $tgt_DIR_W" >> "$logfile"
        $DEBUG rm -rf "$tgt_DIR"
      fi
      echo ""
      echo "" >> "$logfile"
      echo "External upgrade: Copying new"
      echo "External upgrade: Copying new" >> "$logfile"
      echo "  $tgt_DIR_W"
      echo "  $tgt_DIR_W" >> "$logfile"
      $DEBUG cp -R "$installdir/install.$curTarget" "$tgt_DIR/.."
    done
    target_dirs="$target_dirs ."
  fi
fi

#
# Install myself into the SNiFF+ bin directory (under a different name)
if [ "$UNINSTALL" != "1" ]; then
  echo "Installing installation program into SNiFF..."
  TARGET="$SNIFF_DIR/bin"
  TARGET_W="$SNIFF_DIR_W/bin"
  TARGET_TXT="SNIFF_DIR/bin"
  SOURCE="$mydir"
  SOURCE_W="$mydir_W"
  BAK="$bakdir/bin"
  BAK_W="$bakdir_W/bin"
  dir="."
  for curExt in sh bat exe ; do
    sfile="install.$curExt"
    if [ "$curExt" = "sh" ]; then
      tfile="sniffinstall"
    else
      tfile="sniffinstall.$curExt"
    fi
    do_copy_file=1
    backup_msg=""
    if [ -f "$TARGET/$tfile" ]; then
      if [ "$DEBUG" = "echo" ]; then
        echo $DIFF "$SOURCE_W/$sfile" "$TARGET_W/$tfile" ">/dev/null" "2>&1"
      fi
      if $DIFF "$SOURCE_W/$sfile" "$TARGET_W/$tfile" >/dev/null 2>&1 ; then
        # file is already up-to-date in the installation: do nothing
        do_copy_file=0
      else
        # file exists: must overwrite, and, perhaps, copy to backup.
        if [ "$save" = "1" ]; then
          if [ ! -d "$BAK" ]; then
            $DEBUG mkdir -p "$BAK"
          fi
          backup_msg="(backup) "
          test "$DEBUG" = "echo" && echo cp -f "$TARGET/$tfile" "$BAK/$tfile" ">/dev/null" "2>&1"
          cp -f "$TARGET/$tfile" "$BAK/$tfile" >/dev/null 2>&1
        fi
        test "$DEBUG" = "echo" && echo rm -f "$TARGET/$tfile" ">/dev/null" "2>&1"
        $DEBUG rm -f "$TARGET/$tfile" >/dev/null 2>&1
      fi
    fi
    if [ "$do_copy_file" = "1" ]; then
      msg="${backup_msg}copying "\$"$TARGET_TXT/$tfile"
      echo "$msg"
      echo "$msg" >> $logfile
      $DEBUG cp "$SOURCE/$sfile" "$TARGET/$tfile"
    fi
  done # with files
  chmod ugo+rx "$SNIFF_DIR/bin/sniffinstall"
fi # NOT uninstall

#*******************************************
# Main install starts

# Now it's the big time: real (un)installation starts!
# This is a loop first over the install.[something] directories,
# then over the files.
echo ""
echo "" >> "$logfile"
echo "Starting main installation..."
echo "Starting main installation..." >> "$logfile"
cd $installdir
for curTarget in $target_dirs
do
  case $curTarget in
  .)
    dirs=`find . -type d -print | sed 's,\./,,' | grep -v '^install.'`
    if [ "$?" != "0" ]; then
      echo "WARNING -- find returned"
      echo "  $dirs"
      echo "Trying $SNIFF_DIR/bin/find instead"
      dirs=`$SNIFF_DIR/bin/find.exe . -type d -print | sed 's,\./,,' | grep -v '^install.'`
    fi
    SOURCE="$installdir"
    SOURCE_W="$installdir_W"
    TARGET="$SNIFF_DIR/integrations/$integration_name"
    TARGET_W="$SNIFF_DIR_W/integrations/$integration_name"
    BAK="$bakdir"
    TARGET_TXT="SNIFF_DIR/integrations/$integration_name"
    ;;
  *)
    dirs=`find install.$curTarget -type d -print | sed "s,install.$curTarget/*,,"`
    if [ "$?" != "0" ]; then
      echo "WARNING -- find returned"
      echo "  $dirs"
      echo "Trying $SNIFF_DIR/bin/find instead"
      dirs=`$SNIFF_DIR/bin/find.exe install.$curTarget -type d -print | sed "s,install.$curTarget/*,,"`
    fi
    SOURCE="$installdir/install.$curTarget"
    SOURCE_W="$installdir_W/install.$curTarget"
    evalstr="echo "\$"$curTarget"
    TARGET=`eval "$evalstr"`
    evalstr="echo "\$"${curTarget}_W"
    TARGET_W=`eval "$evalstr"`
    BAK="$bakdir/$curTarget"
    TARGET_TXT="$curTarget"
    ;;
  esac

  if [ "$UNINSTALL" != "1" ]; then
  for dir in $dirs
  do
    if [ ! -d "$TARGET/$dir" ]; then
      $DEBUG mkdir -p "$TARGET/$dir"
      if [ "$?" != "0" ]; then
        echo "ERROR Creating target directory "\$"$TARGET_TXT/$dir - skipped"
        continue
      fi
    fi
    #echo "ls -l $SOURCE/$dir"
    files=`ls -l "$SOURCE/$dir" | grep "^-" | awk '{print $NF}'`
    #echo $files
    for file in $files
    do
      #if [ "$UNINSTALL" = "1" ]; then
      # We will uninstall only things that were saved in the backup dir.
      #  if [ ! -f "$BAK/$dir/$file" ]; then
      #    # we have no backup, so assume the file was not there before
      #    # installation, or it was the same and thus skipped.
      #    # This is a problem! So we MUST backup also files that are
      #    # unchanged!
      #    msg="Uninstall: removing "\$"$TARGET_TXT/$dir/$file"
      #    echo "$msg"
      #    echo "$msg" >> "$logfile"
      #    $DEBUG rm -f "$TARGET/$dir/$file"
      #  else
      #    msg="Uninstall: restoring "\$"$TARGET_TXT/$dir/$file"
      #    echo "$msg"
      #    echo "$msg" >> "$logfile"
      #    $DEBUG cp -f "$BAK/$dir/$file" "$TARGET/$dir/$file"
      #  fi
      #  # this will skip the rest and continue with the next file
      #  continue
      #fi
      do_copy_file=1
      backup_msg=""
      if [ -f "$TARGET/$dir/$file" ]; then
        if [ "$DEBUG" = "echo" ]; then
          echo $DIFF "$SOURCE_W/$dir/$file" "$TARGET_W/$dir/$file" ">/dev/null" "2>&1"
        fi
        if $DIFF "$SOURCE_W/$dir/$file" "$TARGET_W/$dir/$file" >/dev/null 2>&1 ; then
          # file is already up-to-date in the installation: do nothing
          do_copy_file=0
          #if [ "$save" = "1" ]; then
          #  if [ ! -d "$BAK/$dir" ]; then
          #    $DEBUG mkdir -p "$BAK/$dir"
          #  fi
          #  echo "backup up-to-date "\$"$TARGET_TXT/$dir/$file"
          #  $DEBUG cp -f "$TARGET/$dir/$file" "$BAK/$dir/$file"
          #else
            echo "(no need to copy, up-to-date) "\$"$TARGET_TXT/$dir/$file"
          #fi
        else
          # file exists: must overwrite, and, perhaps, copy to backup.
          if [ "$save" = "1" ]; then
            if [ ! -d "$BAK/$dir" ]; then
              $DEBUG mkdir -p "$BAK/$dir"
            fi
            backup_msg="(backup) "
            test "$DEBUG" = "echo" && echo cp -f "$TARGET/$dir/$file" "$BAK/$dir/$file" ">/dev/null" "2>&1"
            cp -f "$TARGET/$dir/$file" "$BAK/$dir/$file" >/dev/null 2>&1
          fi
          test "$DEBUG" = "echo" && echo rm -f "$TARGET/$dir/$file" ">/dev/null" "2>&1"
          $DEBUG rm -f "$TARGET/$dir/$file" >/dev/null 2>&1
        fi
      fi
      if [ "$do_copy_file" = "1" ]; then
        msg="${backup_msg}copying "\$"$TARGET_TXT/$dir/$file"
        echo "$msg"
        echo "$msg" >> $logfile
        $DEBUG cp "$SOURCE/$dir/$file" "$TARGET/$dir/$file"
      fi
    done # with files
  done # with dirs
  fi #end of INSTALLATION code, beginning of UNINSTALL code
  # all files from install are now restored. Now, restore those files
  # that have been deleted but have not been part of the replacement
  # installation
  if [ "$UNINSTALL" = "1" ]; then
    cd "$BAK"
    # BUGBUG this could produce a commandline overflow
    dirs=`find . -type d -print | sed 's,\./*,,'`
    for dir in $dirs ; do
      if [ ! -d "$TARGET/$dir" ]; then
        echo "Uninstall: restoring direcory "\$"$TARGET_TXT/$dir"
        mkdir -p "$TARGET/$dir"
      fi
      files=`ls -l "$SOURCE/$dir" | grep "^-" | awk '{print $NF}'`
      for file in $files ; do
        if [ ! -f "$TARGET/$dir/$file" ]; then
          msg="Uninstall: restoring "\$"$TARGET_TXT/$dir/$file"
          echo "$msg"
          echo "$msg" >> "$logfile"
          $DEBUG cp "$BAK/$dir/$file" "$TARGET/$dir/$file"
        fi
      done # UNINSTALL files
    done # UNINSTALL dirs
    cd ..
    newname=`echo "$bakdir_base" | sed 's,backup,uninstalled,g'`
    if [ -d "$bakdir_base" ]; then
      msg="$Moving $bakdir_base to $newname"
      echo "$msg"
      echo "$msg" >> "$logfile"
      mv "$bakdir_base" "$newname"
    fi
  fi # UNINSTALL
done # with targets

# source a postinstall file if one exists
if [ -f "$installdir/adm/postinstall.sh" ]; then
  . "$installdir/adm/postinstall.sh"
fi

RESULT=$exit_noerror
finish
exit 0
