#
# PVCSConfigFile.py
#
# this class covers the possible content of the PVCS config files
# Remarks: this class can't read or write a config file, this is only
#          the storage for the content. For read or write operations
#          use the classes PVCSConfigFileReader and PVCSConfigFileWriter
#
# Author: Uwe Stieber
# Copyright (c) 1998, 1999 TakeFive Software, Salzburg
#

# import some system modules
import UserDict
import string
import re
import sys
import os

# import PVCS constants and helper functions
import PVCSConstants
import vcs

class PVCSCommonConfigFile:
	# some class privates
	__validKeys = None			# list of used keys in the config file
								# use it for validation of the keys found
								# by PVCSConfigFileReader
								
	__ArchiveSuffixes = None	# dictionary of parsed archive
								# suffix rules
								
	__Diagnostic = 0			# PVCS diagnostic level
	__Quiet = 1					# PVCS message display
	__SignOn = 0				# PVCS copyright message
	
	__ReferenceDir = None		# ReferenceDir list
	__ReferenceDirFlags = 0		# bitwise OR
								#	0 ... TrunkOnly
								#	1 ... All
								#	2 ... WriteOnly

	# PVCS config file format version
	Version = "Common"
	
	def __init__(self):
		# constructor
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x __init__@B"
		
		# initialize the class variables
		self.__validKeys = []
		self.__ArchiveSuffixes = UserDict.UserDict()
		self.__Diagnostic = 0
		self.__Quiet = 1
		self.__SignOn = 0
		self.__ReferenceDir = []

		self.clearSuffixRules();

		# currently we accept following keys
		self.__validKeys.append(string.upper("ArchiveSuffix"))
		self.__validKeys.append(string.upper("Diagnostic"))
		self.__validKeys.append(string.upper("Quiet"))
		self.__validKeys.append(string.upper("Verbose"))
		self.__validKeys.append(string.upper("SignOn"))
		self.__validKeys.append(string.upper("NoSignOn"))
		self.__validKeys.append(string.upper("ReferenceDir"))

		if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x __init__@B"
		return
				
	def isValidKey(self, aKeyName):
		# method check's, if a key will be accepted by this class
		if PVCSConstants.TEST:
			print "(PVCS Config File)@B@g>>>@x isValidKey (aKeyName = '%s')@B" % aKeyName
			print "(PVCS Config File)\tList of valid Keys: @B@b",
			print self.__validKeys,
			print "@x@B"
		return string.upper(aKeyName) in self.__validKeys
			
	def addSuffixRule(self, aSuffix=None, aRule):
		# add a new suffix rule if the suffix doesn't already
		# exists, otherwise update the rule
		# Remark: the suffix is not a regular expression
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x addSuffixRule@B"
		if aSuffix is None or aSuffix == ".?":
			# the Rule is for all files
			aSuffix = 'undefined'
		if aSuffix == '.':
			# the Rule is for files without a extension
			aSuffix = 'noextension'
		if PVCSConstants.TEST: print "(PVCS Config File)\taSuffix = '@B@b%s@x@B'; aRule = '@B@b%s@x@B'" % (aSuffix, aRule)
		# save the rule
		self.__ArchiveSuffixes.data[aSuffix] = aRule
		if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x addSuffixRule@B"
		return
		
	def removeSuffixRule(self, aSuffix):
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x removeSuffixRule (aSuffix = '%s')@B" % aSuffix
		# remove a suffix rule from th list
		del self.__ArchiveSuffixes.data[aSuffix]
		if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x removeSuffixRule@B"
		return
		
	def getSuffixRule(self, aSuffix):
		# returns the rule for the given Suffix
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x getSuffixRule (aSuffix = '%s')@B" % aSuffix
		if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x getSuffixRule (aRule = '%s')@B" % self.__ArchiveSuffixes.get(aSuffix)
		return self.__ArchiveSuffixes.get(aSuffix)
			
	def findSuffixKey(self, fileSuffix):
		# try to find out, which suffix key from the current list
		# can be used
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x findSuffixKey (fileSuffix = '%s')@B" % fileSuffix
		if self.__ArchiveSuffixes.has_key(fileSuffix):
			# found direct association
			if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x findSuffixKey@B"
			return fileSuffix
		else:
			if fileSuffix == '.':
				# it's file without a extension
				if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x findSuffixKey@B"
				if self.__ArchiveSuffixes.has_key('noextension'):
					return 'noextension'
				else:
					return None
			else:
				# try to find a meta rule
				matchedMetaSuffix = None
				for aSuffix in self.__ArchiveSuffixes.keys():
					# look through all rules which contains a meta character
					if string.find(aSuffix, '?') != -1 and aSuffix != 'undefined' and aSuffix != 'noextension':
						# it's a meta rule
						# translate it to regular expression
						# Note: string.translate / string.maketrans is
						# not useable in this situation
						aRegExPattern = PVCSConstants.makeRegEx(aSuffix)
						# try to match (add IGNORECASE as flag for case insensitiv working on Windows)
						try:
							#print ">>> use regular expression: %s" % aRegExPattern
							flags = 0
							# disabled case insensitiv matching on windows as well,
							# because PVCS didn't do that
							#if sys.platform == "win32":
							#	# I ... IGNORECASE
							#	flags = re.I
							if re.match(aRegExPattern, fileSuffix, flags) != None:
								matchedMetaSuffix = aSuffix
								break
						except re.error:
							if PVCSConstants.TEST: print "(PVCS Config File)\tNot a regular expression: '%s'" % re.escape(aRegExPattern)
				if matchedMetaSuffix == None:
					# no matching suffix found, use finally
					# the suffix for all other files if this key exists
					if self.__ArchiveSuffixes.has_key('undefined'):
						matchedMetaSuffix = 'undefined'
				# send back the matched Meta Suffix String
				if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x findSuffixKey@B"
				return matchedMetaSuffix

	def clearSuffixRules(self):
		# clear all suffix rules
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x clearSuffixRules@B"
		self.__ArchiveSuffixes.clear()
		if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x clearSuffixRules@B"
		return

	def store(self, aSplittedLine):
		# PVCSConfigFileReader found a valid key and
		# sended us the content of the valid line splitted
		# into components inside a list. We have to do the
		# further analysing here.
		# ARCHIVESUFFIX directive ??
		if PVCSConstants.TEST:
			print "(PVCS Config File)@B@g>>>@x store@B"
			print "(PVCS Config File)\tsplitted line = ", aSplittedLine
		if string.upper(aSplittedLine[0]) == "ARCHIVESUFFIX":
			# ArchiveSuffix mode
			if len(aSplittedLine) == 2:
				# if the rule doesn't start with '+' or doesn't contains
				# the meta character '?' we will handle this as special
				# rules, where the resulted archive file doesn't have any
				# extension
				if string.find(aSplittedLine[1], '+') == -1 and string.find(aSplittedLine[1], '?') == -1:
					self.addSuffixRule(aSplittedLine[1], "")
				else:
					# only the rule is given, is valid for all files
					self.addSuffixRule(aRule = aSplittedLine[1])
			else:
				# we should have exactly 3 items
				if len(aSplittedLine) == 3:
					# correct syntax
					self.addSuffixRule(aSplittedLine[1], aSplittedLine[2])
				else:
					# store it as single rules to the database
					for aSourceSuffix in range(1, len(aSplittedLine) - 1):
						self.addSuffixRule(aSplittedLine[aSourceSuffix], aSplittedLine[len(aSplittedLine) - 1])
		if string.upper(aSplittedLine[0]) == "DIAGNOSTIC":
			# Diagnostic level mode
			# we need at least two items
			if len(aSplittedLine) >= 2:
				self.setDiagnostic(int(aSplittedLine[1]))
		if string.upper(aSplittedLine[0]) == "QUIET":
			# set Quiet flag
			self.setQuiet(1)
		if string.upper(aSplittedLine[0]) == "VERBOSE":
			# clear the Quiet flag
			self.setQuiet(0)
		if string.upper(aSplittedLine[0]) == "SIGNON":
			# set the SignOn flag
			self.setSignOn(1)
		if string.upper(aSplittedLine[0]) == "NOSIGNON":
			# clear the SignOn flag
			self.setSignOn(0)
		if string.upper(aSplittedLine[0]) == "REFERENCEDIR":
			try:
				# save the Reference dir
				self.setReferenceDir(aSplittedLine)
			except:
				import traceback
				traceback.print_exc()
		# no further valid keys,
		# end of method reached
		if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x store@B"
		return

	def setReferenceDir(self, aSplittedLine):
		# analyse the ReferenceDir directive and save the reference
		# directory and flags
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x setReferenceDir@B"
		if len(aSplittedLine) == 1:
			# empty referenceDir, delete ReferenceDir list
			self.__ReferenceDir = []
			self.__ReferenceDirFlags = 0
		else:
			# ReferenceDir contains flags and/or directories
			for aEntry in aSplittedLine[1:]:
				if PVCSConstants.TEST:
					print "(PVCS Config File)\taEntry = %s" % aEntry
				if string.upper(aEntry) == "REFERENCEDIR":
					continue
				if string.upper(aEntry) in [ "TRUNKONLY", "ALL", "WRITEPROTECT", "NOWRITEPROTECT" ]:
					# is a flag
					if string.upper(aEntry) == "TRUNKONLY":
						if self.__ReferenceDirFlags & 1 != 0:
							# delete bit 1
							self.__ReferenceDirFlags = self.__ReferenceDirFlags & 2
					if string.upper(aEntry) == "ALL":
						# set bit 1
						self.__ReferenceDirFlags = self.__ReferenceDirFlags | 1
					if string.upper(aEntry) == "WRITEPROTECT":
						# set bit 2
						self.__ReferenceDirFlags = self.__ReferenceDirFlags | 2
					if string.upper(aEntry) == "NOWRITEPROTECT":
						if self.__ReferenceDitFlags & 2 != 0:
							# delete bit 2
							self.__ReferenceDirFlags = self.__ReferenceDirFlags & 1
				else:
					# is a directory
					if string.find(aEntry, "*") != -1:
						# eleminate the '*'
						aEntry = re.sub(r'"', "", aEntry)
						aEntry = re.sub(r'\*', "", aEntry)
					self.__ReferenceDir.append(vcs.normpath(aEntry))
		if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x setReferenceDir@B"
		return

	def clearReferenceDir(self):
		# clear the reference dir list
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x clearReferenceDir@B"
		self.__ReferenceDir = []
		self.__ReferenceDirFlags = 0
		if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x clearReferenceDir@B"
		return

	def getReferenceDirList(self):
		# returns the ReferenceDir list
		if PVCSConstants.TEST: print "(PVCS Config File)@B getReferenceDirList@B"
		return self.__ReferenceDir

	def getReferenceDirFlags(self):
		# returns the ReferenceDir list
		if PVCSConstants.TEST: print "(PVCS Config File)@B getReferenceDirFlags@B"
		return self.__ReferenceDirFlags

	def printReferenceDirList(self):
		# prints out the complete list of known
		# Reference Directories
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x printReferenceDirList@B"
		for aReferenceDir in self.__ReferenceDir:
			print "%s, " % aReferenceDir,
		print ""
		if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x printReferenceDirList@B"
		return

	def printArchiveSuffixList(self):
		# prints out the complete list of known
		# archive suffix rules
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x printArchiveSuffixList@B"
		print "Archive Suffix Rules:"
		for aSuffixKey in self.__ArchiveSuffixes.keys():
			print "%s\t =\t %s" % (aSuffixKey, self.getSuffixRule(aSuffixKey))
		print "<<< end of list"
		if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x printArchiveSuffixList@B"
		return
	
	def setDiagnostic(self, diagnosticLevel):
		# set the diagnostic level
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x setDiagnostic@B"
		self.__Diagnostic = diagnosticLevel
		if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x setDiagnostic@B"
		return

	def getDiagnostic(self):
		# returns the diagnostic level
		if PVCSConstants.TEST: print "(PVCS Config File)@B getDiagnostic@B"
		return self.__Diagnostic
		
	def setQuiet(self, Quiet):
		# set the quiet flag
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x setQuiet@B"
		self.__Quiet = Quiet
		if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x setQuiet@B"
		return

	def getQuiet(self):
		# returns the quiet flag
		if PVCSConstants.TEST: print "(PVCS Config File)@B getQuiet@B"
		return self.__Quiet
		
	def setSignOn(self, SignOn):
		# set the SignOn flag
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x setSignOn@B"
		self.__SignOn = SignOn
		if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x setSignOn@B"
		return

	def getSignOn(self):
		# returns the SignOn flag
		if PVCSConstants.TEST: print "(PVCS Config File)@B getSignOn@B"
		return self.__SignOn
		
	def getArchiveSuffixList(self):
		# returns the suffix rule dictionary
		if PVCSConstants.TEST: print "(PVCS Config File)@B getArchiveSuffixList@B"
		return self.__ArchiveSuffixes.data
		
	def calculateArchiveExtension(self, aFileExtension):
		# calculates the new archive suffix
		if PVCSConstants.TEST: print "(PVCS Config File)@B@g>>>@x calculateArchiveExtension (aFileExtension = '%s')@B" % aFileExtension
		aSuffixKey = self.findSuffixKey(aFileExtension)
		if aSuffixKey != None:
			# start calculating
			aSuffixRule = self.getSuffixRule(aSuffixKey)
			if aSuffixRule != None:
				if string.find(aSuffixRule, '?') != -1 and aSuffixRule[0] != '+':
					# suffix rule is a meta rule
					if (len(aSuffixRule) % 2 == 0):
						rule = aSuffixRule[:len(aSuffixRule)/2]
						replacements = aSuffixRule[len(aSuffixRule)/2:]
					else:
						# replacement doesnt exists
						rule = aSuffixRule
						replacements = None
					newSuffix = ""
					if string.find(rule, '.') == -1 and string.find(aFileExtension, '.') != -1:
						aFileExtension = aFileExtension[1:]
					# walk over the extension
					for aCharIndex in range(len(rule)):
						if rule[aCharIndex] == '?':
							if aCharIndex >= len(aFileExtension):
								if replacements != None:
									try:
										# append a character from replacements
										newSuffix = newSuffix + replacements[aCharIndex]
									except:
										# if aCharIndex is out of range for replacements
										pass
							else:
								newSuffix = newSuffix + aFileExtension[aCharIndex]
						else:
							# append the character from rule
							newSuffix = newSuffix + rule[aCharIndex]
					# check, if there is something behind the part
					# of the extension, which is effected by the rule
					if len(rule) < len(aFileExtension):
						newSuffix = newSuffix + aFileExtension[len(rule):]
					if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x calculateArchiveExtension (Extension = '%s')@B" % ('.' + newSuffix)
					return '.' + newSuffix
				else:
					# suffix rule is a direct replacement or an attachement
					if len(aSuffixRule) > 0 and aSuffixRule[0] == '+':
						# append it
						if aFileExtension != '.':
							newExtension = aFileExtension + aSuffixRule[1:]
						else:
							newExtension = aSuffixRule[1:]
						if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x calculateArchiveExtension (Extension = '%s')@B" % newExtension
						return newExtension
					else:
						# hack to workaround strange suffix calculation of PVCS istelf
						# if specified .h -> .h we got archive "xxx..h" !!!!
						# rule must be .h -> h and we have to add a dot in front of
						if string.find(aSuffixRule, '.') == -1:
							aSuffixRule = '.' + aSuffixRule
						if PVCSConstants.TEST: print "(PVCS Config File)@B@r<<<@x calculateArchiveExtension (Extension = '%s')@B" % aSuffixRule
						return aSuffixRule
			else:
				raise PVCSConstants.PVCSNoSuffixRule
		else:
			raise PVCSConstants.PVCSNoSuffixRule

class PVCSConfigFile(PVCSCommonConfigFile):
	def __init__(self):
		if PVCSConstants.TEST: print "(PVCS 6.0 Config File)@B@g>>>@x __init__@B"
		# call constructor of PVCSCommonConfigFile
		PVCSCommonConfigFile.__init__(self)

		# set version
		self.Version = "6.0"

		if PVCSConstants.TEST: print "(PVCS 6.0 Config File)@B@r<<<@x __init__@B"
		return

	def clearSuffixRules(self):
		# clear all suffix rules and add the PVCS default behavior
		# for all files (ArchivSuffix = ??v___)
		# and file with no extension (ArchivSuffix = ??v___)
		if PVCSConstants.TEST: print "(PVCS 6.0 Config File)@B@g>>>@x clearSuffixRules@B"
		PVCSCommonConfigFile.clearSuffixRules(self)
		self.addSuffixRule(aRule = '??v___')
		self.addSuffixRule(".", '??v___')
		if PVCSConstants.TEST: print "(PVCS 6.0 Config File)@B@r<<<@x clearSuffixRules@B"
		return

class PVCS65ConfigFile(PVCSCommonConfigFile):
	def __init__(self):
		if PVCSConstants.TEST: print "(PVCS 6.5 Config File)@B@g>>>@x __init__@B"
		# call constructor of PVCSCommonConfigFile
		PVCSCommonConfigFile.__init__(self)

		# set version
		self.Version = "6.5"

		if PVCSConstants.TEST: print "(PVCS 6.5 Config File)@B@r<<<@x __init__@B"
		return

	def clearSuffixRules(self):
		# clear all suffix rules and add the PVCS default behavior
		# for all files (ArchivSuffix = ??v___)
		if PVCSConstants.TEST: print "(PVCS 6.5 Config File)@B@g>>>@x clearSuffixRules@B"
		PVCSCommonConfigFile.clearSuffixRules(self)
		self.addSuffixRule(aRule = '+-arc')
		self.addSuffixRule(".", '+.-arc')
		if PVCSConstants.TEST: print "(PVCS 6.5 Config File)@B@r<<<@x clearSuffixRules@B"
		return

